#!/usr/bin/ruby
#
# Für Kaffeine > 2.0.4
#
# S.Buerger - Namenaufbereitung mit Sender, Startdatum, Startzeit und Dauer. Beseitigung/Ersatz von Sonderzeichen (S.Bürger - überarbeitet 2023-05-12), 
#             Zeichensatzfilterung erweitert für Folgebearbeitung mit Script filmschnitt.sh, Pfade angepasst.
#
# Parameter für TV-Brauser Aufnahmesteuerung:
# record {start_year} {start_month} {start_day} {start_hour} {start_minute} {length_minutes} "{isset(channel_name_external_quiet,channel_name)}" "{title}" {production_year} "{episode}" {episode_number}
# remove {start_year} {start_month} {start_day} {start_hour} {start_minute} {length_minutes} "{isset(channel_name_external_quiet,channel_name)}" "{title}" {production_year} "{episode}" {episode_number}
#
# Bitte anpassen falls nötig:
# Zeile 24 Path der kaffeine_db
#
# Zeile 165ff Optionen: Wiederholungen können aufgenommen werden. / Der selbe Titel wird nur ein mal aufgenommen.
#
# Zeile 185ff Hier wird überprüft ob sich der Titel schon auf der Festplatte befindet. Falls gewünscht, bitte Pfad in #187 und #188 anpassen!
 
require 'rubygems'
require 'sqlite3'
require 'time'

# Path zur db muß evt. angepasst werden
db_path = ENV['HOME'] + '/.local/share/kaffeine/sqlite.db'

unless File.readable?(db_path) then
  puts "Kaffeine-DB-Datei ist nicht lesbar: '#{db_path}'"
  exit 1
end


# Parameter prüfen
if ARGV.length < 8 then
  puts "Zu wenig Parameter. Bitte im TV-Browser die Aufnahmesteuerung anpassen."
  exit 2

elsif ARGV[0] == "remove" then
  # Lies die Parameter in Variablen.
  operation, sJahr, sMonat, sTag, sStunde, sMinute, sLaenge, sKanal, sTitel, sProdYear, sEpisode, sEpisode_number= ARGV
 
elsif ARGV[0] == "record" then
  # Vorgabeoperation ist die Aufnahme.
  # Lies die Parameter in Variablen.
  operation, sJahr, sMonat, sTag, sStunde, sMinute, sLaenge, sKanal, sTitel, sProdYear, sEpisode, sEpisode_number= ARGV
 
else
  puts "Die Parameter in der Aufnahmesteuerung müssen angepasst werden!"
  puts ""
  puts " 1. Operation '#{ARGV[0]}'"
  puts " 2. Jahr '#{ARGV[1]}'"
  puts " 3. Monat '#{ARGV[2]}'"
  puts " 4. Tag '#{ARGV[3]}'"
  puts " 5. Stunde '#{ARGV[4]}'"
  puts " 6. Minute '#{ARGV[5]}'"
  puts " 7. Länge '#{ARGV[6]}'"
  puts " 8. Kanal '#{ARGV[7]}'"
  puts " 9. Titel '#{ARGV[8]}'"
  puts "10. ProdJahr '#{ARGV[9]}'" 
  puts "11. Episode '#{ARGV[10]}'"
  puts "12. Epi-Nr. '#{ARGV[11]}'"
  puts ""
  puts "Anzahl der Parameter '#{ARGV.length}'"
  exit 3
end

if md = sKanal.match(/^"(.*)"$/) then
  sKanal = md[1]
 
end

sKanalName=sKanal.gsub("\_", "\-")

sKanalName="#{sKanalName}-dvbt2"

sTitel=sTitel.gsub("\"", "'")

# Titelergänzung
if ARGV[11] and ARGV[10] and ARGV[9] then
   
    sTitel="#{sTitel}_#{sEpisode_number}_#{sEpisode}_#{sProdYear}"
   
elsif ARGV[10] and ARGV[9] then
   
    sTitel="#{sTitel}_#{sEpisode}_#{sProdYear}"
   
elsif ARGV[9] then
   
    sTitel="#{sTitel}_#{sProdYear}"
   
end
   
sTitel="#{sTitel}_#{sKanalName}"

#Zeichen aus dem Titel entfernen
sTitel=sTitel.gsub("\"", "")
sTitel=sTitel.gsub("'", "")
sTitel=sTitel.gsub(/[:,;\[\]\(\)\{\}\§\$\°\%\<\>\?\!]/, "_")
sTitel=sTitel.gsub(" ", "\_")
#sTitel=sTitel.gsub("\&", "und")
sTitel=sTitel.gsub("\+", "und")
sTitel=sTitel.gsub("\/", "-")
sTitel=sTitel.gsub("Ä", "Ae")
sTitel=sTitel.gsub("Ö", "Oe")
sTitel=sTitel.gsub("Ø", "Oe")
sTitel=sTitel.gsub("Ü", "Ue")
sTitel=sTitel.gsub("ä", "ae")
sTitel=sTitel.gsub("ö", "oe")
sTitel=sTitel.gsub("ø", "oe")
sTitel=sTitel.gsub("ü", "ue")
sTitel=sTitel.gsub("ß", "ss")
sTitel=sTitel.gsub("Á", "A")
sTitel=sTitel.gsub("Â", "A")
sTitel=sTitel.gsub("À", "A")
sTitel=sTitel.gsub("á", "a")
sTitel=sTitel.gsub("â", "a")
sTitel=sTitel.gsub("à", "a")
sTitel=sTitel.gsub("É", "E")
sTitel=sTitel.gsub("Ê", "E")
sTitel=sTitel.gsub("È", "E")
sTitel=sTitel.gsub("é", "e")
sTitel=sTitel.gsub("ê", "e")
sTitel=sTitel.gsub("è", "e")
sTitel=sTitel.gsub("\#", "_")

sTitel="#{sTitel}_#{sJahr}-#{sMonat}-#{sTag}_#{sStunde}-#{sMinute}_#{sLaenge}"
sTitel=sTitel.gsub("^_", "")
sTitel=sTitel.gsub(": ", "-")
sTitel=sTitel.gsub(" - ", "-")
sTitel=sTitel.gsub("_-_", "-")
sTitel=sTitel.gsub("_-", "-")
sTitel=sTitel.gsub("-_", "-")
sTitel=sTitel.gsub("___", "_")
sTitel=sTitel.gsub("__", "_")
sTitel=sTitel.gsub("--", "-")

# Umrechnung der Aufzeichnungsdauer von Minuten (tv-browser) nach Stunden und Minuten (kaffeine)
nDauerStunden = sLaenge.to_i / 60
nDauerMinuten = sLaenge.to_i % 60

# Zeitumrechnung auf GMT
# Dies ist notwendig, weil bei der Benutzung von qdbus die Anfangszeiten eine Stunde spaeter sind, als in den Parametern uebergeben
    tBegin=Time.mktime(sJahr, sMonat, sTag, sStunde, sMinute)
    #puts "EingabeZeit", tBegin

    # Differenz zu GMT ermitteln
    zeitstr=tBegin.to_s
    iGMTdiff=zeitstr[20,5].to_i/100

    # Zeit in Datenbank errechnen
    #in Ruby muss man die Zeit in Sekunden abziehen
    tBegin = tBegin-iGMTdiff*3600

# sBegin ist der StartZeitString für die Übergabe an die Datenbank mit sqlite
# sZeit ist der StartZeitString für die Übergabe an die Datenbank mit qdbus org.kde.kaffeine
sBegin = "#{"%04d" % sJahr}-#{"%02d" % sMonat}-#{"%02d" % sTag}T#{"%02d" % sStunde}:#{"%02d" % sMinute}:00ZZ"
sZeit="#{"%04d" % tBegin.year}-#{"%02d" % tBegin.month}-#{"%02d" % tBegin.day}T#{"%02d" % tBegin.hour}:#{"%02d" % tBegin.min}:00ZZ"

# Aufnahmedauer
sDuration = "#{"%02d" % nDauerStunden}:#{"%02d" % nDauerMinuten}:00"

begin
    # Datenbank öffnen
    db = SQLite3::Database.open(db_path)

    # Initialisierung
    id = nil
    rc = false

    # Gibt es den Eintrag der Kaffeine Datenbank schon? (Optional: alle Parameter oder nur den Titel vergleichen)
    # db.execute("select Id from RecordingSchedule where Name = ? and Channel = ? and Begin = ? and Duration = ? and Repeat = ?", sTitel, sKanal, sZeit, sDuration, 0) do |row|
      db.execute("select Id from RecordingSchedule where Name = ?", sTitel) do |row|
        id = row[0].to_i unless row[0].nil?
end

    if id.nil? then
        if operation == 'record' then
            db.execute("select max(Id) from RecordingSchedule") do |row|
                id = row[0].to_i + 1 unless row[0].nil?
            end # select max(id)
            nKanal = 0
            db.execute("select count(*) from Channels where Name = ?", sKanal) do |row|
                nKanal = row[0].to_i unless row[0].nil?
            end # select count(*)  Kanäle mit eingegebenem Namen

            if nKanal == 0 then
                puts "Der Sender '#{sKanal}' ist in kaffeine nicht vorhanden!"
                db.close
                exit 4
            end

# Ist die Aufnahme schon auf Fettplatte?
# S.Bürger: Pfad ggf. anpassen, $(pwd) einsetzen, wenn Script aus diesem Pfad gestartet wurde!
if File.readable?("~/Multimedia/DVBT-Aufnahme/#{sTitel}.m2t") or
            File.readable?("~/Multimedia/DVBT-Aufnahme/#{sTitel}[remux].mpg") then
        puts ""
        puts "Keine erneute Aufnahme!"
      puts "Der Titel '#{sTitel}' befindet sich schon auf der Festplatte."
      puts ""
      exit 5
end


        begin # Beginn Verarbeitung
                   
                    kaf = system("pidof kaffeine")

                    if kaf then #kaffeine läuft
                       
                        rec=system(%Q<qdbus org.kde.kaffeine /Television ScheduleProgram '#{sTitel}' '#{sKanal}' #{sZeit} #{sDuration} 0>)
                       
                    else
                       
            rec=db.execute "INSERT INTO RecordingSchedule(Name, Channel, Begin, Duration, Repeat, Priority, Disabled) VALUES('#{sTitel}', '#{sKanal}', '#{sZeit}', '#{sDuration}', 0, 0, 0)"
                       
                    end

            if rec then
                rc = "OK"
               
            else
                rc = "Fehler beim Insert"
                db.close
                exit 6
            end


            rescue
            rc = "Fehler beim INSERT: #{$!}"
            puts "Fehler beim INSERT: #{$!}"
            db.close
            exit 7
        end

            # Rueckgabe an den TV-Browser.
                        puts ""
            puts "Die Sendung wird aufgenommen:"
                        puts ""
                        puts "Titel: #{sTitel}"
            puts "Kanal: #{sKanal}"
            puts "Beginn: #{sBegin}"
            puts "Dauer: #{sDuration}"
            #puts "Rueckgabewert: #{rc}"

        elsif operation == 'remove' then # id.nil = TRUE (=nicht gefunden), Sendung soll aber entfernt werden
                        puts ""
            puts "Der Eintrag '#{sTitel}' ist in Kaffeine nicht vorhanden!"
            db.close
            exit 8
        end

    else    # id.nil = FALSE (=Sendung gefunden)
                   
        if operation == 'remove' then #(Entferne Eintrag aus db)
                   
                    kaf = system("pidof kaffeine")
                   
                    if kaf then
                       
                        del = system(%Q<qdbus org.kde.kaffeine /Television RemoveProgram #{id}>)
                       
                    else
                       
                        del = db.execute("delete from RecordingSchedule where Name = ? and Channel = ? and Begin = ? and Duration = ? and Repeat = ?", sTitel, sKanal, sZeit, sDuration, 0)
                       
                    end

            if del then
                               
                                puts ""
                puts "Die Sendung '#{sTitel}' wurde gelöscht."                                                               
                       
                        else
                                puts ""
                puts "Fehler! Sendung wurde nicht gelöscht."
                db.closed
                exit 9
            end    
                               
                               
        else  # if id.nil?  Sendung gefunden, soll noch einmal hinzugefügt werden
                   
                        puts ""
                        puts "Keine erneute Aufnahme"
            puts "Die Sendung '#{sTitel}' steht bereits auf der Aufnahmeliste"
            db.close
            exit 10
        end
end
db.close
rescue
    db.close unless db.closed?
    puts "Fehler mit der Kaffeine-DB!"
    exit 20
end
