#!/bin/bash

# umbenennen.sh Version 1.3 vom 02.06.2021
# ========================================

Anzahl_Ebenen=3

# umbenennen.sh konvertiert Filmdateinamen entsprechend den Namensregeln in filmschnitt.sh...
# 1. ohne Aufrufparameter im aktuellen Verzeichnis ohne Unterverzeichnisse
# 2. mit Pfadname als Parameter ab dem Startverzeichnis bis zur ${Anzahl_Ebenen}. Unterverzeichnisebene.
#    Alle mit "XXX_", "0[0-9]_" oder "." beginnenden Pfadnamen werden in allen Ebenen ignoriert, sie müssen direkt als Startverzeichnis angegeben werden!
# 3. mit Videodateiname als Parameter wird diese Datei umbenannt.
# 4. Eine Liste von Pfadnamen oder Videodateinamen wird entsprechend (2.) bzw. (3.) nacheinander bearbeitet
#
# Achtung: lange Laufzeit - ca. 10...12s je Videodatei, also 300...360 Videodateien/Stunde

pushd $(pwd) > /dev/null 2> /dev/null

if [ "$1" != "" ]; then
  cd $1
  if [ $? -ne 0 ]; then
    echo "Fehler, kein gültiges Verzeichnis"
    popd > /dev/null 2> /dev/null
    exit 1
  else
    echo "=> $1"
  fi
fi

function umbenennen_sub2 {
}

function umbenennen_sub {
  # Parameter 1: namenindex 1 ... 3
  # Parameter 2: 0= nächste Ebene, 
  #              1=ist letzte Ebene, nicht tiefer gehen
  if [ $1 -ge 1 -a $1 -le $Anzahl_Ebenen ]; then
    echo -e "#!/bin/bash" > $HOME/umbenennen_s$1.sh
    echo -e "function hier_umbenennen$1 {" >> $HOME/umbenennen_s$1.sh
    echo -e "if [ \$(find \"\$(pwd)/\" -type f -name \"*.mp4\" | wc -l) -eq 0 -a  \$(find \"\$(pwd)/\" -type f -name \"*.m2t\" | wc -l) -eq 0 -a  \$(find \"\$(pwd)/\" -type f -name \"*.ts\" | wc -l) -eq 0 -a  \$(find \"\$(pwd)/\" -type f -name \"*.webm\" | wc -l) -eq 0 ]; then" >> $HOME/umbenennen_s$1.sh
    echo -e "  echo \"Verzeichnis enthält keine Videodateien\"" >> $HOME/umbenennen_s$1.sh
    echo -e "else" >> $HOME/umbenennen_s$1.sh
    echo -e "  find ./ -mindepth 1 -maxdepth 1 -name \"*.mp4\" -exec $HOME/filmschnitt.sh -umbenennen -i '{}' \\;" >> $HOME/umbenennen_s$1.sh
    echo -e "  find ./ -mindepth 1 -maxdepth 1 -name \"*.m2t\" -exec $HOME/filmschnitt.sh -umbenennen -i '{}' \\;" >> $HOME/umbenennen_s$1.sh
    echo -e "  find ./ -mindepth 1 -maxdepth 1 -name \"*.ts\" -exec $HOME/filmschnitt.sh -umbenennen -i '{}' \\;" >> $HOME/umbenennen_s$1.sh
    echo -e "  find ./ -mindepth 1 -maxdepth 1 -name \"*.webm\" -exec $HOME/filmschnitt.sh -umbenennen -i '{}' \\;" >> $HOME/umbenennen_s$1.sh
    echo -e "fi" >> $HOME/umbenennen_s$1.sh
    echo -e "}" >> $HOME/umbenennen_s$1.sh
    echo -e "function verzeichnisauswerten$1 {" >> $HOME/umbenennen_s$1.sh
    echo -e "  initcnt$1=\$(ps | grep -c 'find')" >> $HOME/umbenennen_s$1.sh
    echo -e "  #echo -e -n '[$1] '" >> $HOME/umbenennen_s$1.sh
    echo -e "  cd \"\$1\"" >> $HOME/umbenennen_s$1.sh
    echo -e "  pwd" >> $HOME/umbenennen_s$1.sh
    if [ $2 -ne 1 ]; then
      echo -e "  find ./ -mindepth 1 -maxdepth 1 -type d -exec $HOME/umbenennen_s$(( $1 + 1 )).sh '{}' \\;" >> $HOME/umbenennen_s$1.sh
      echo -e "  while [ \$(ps | grep -c 'find') -gt \$initcnt$1 ]; do" >> $HOME/umbenennen_s$1.sh
      echo -e "    sleep 1" >> $HOME/umbenennen_s$1.sh
      echo -e "  done" >> $HOME/umbenennen_s$1.sh
    fi
    echo -e "  [ \"\${1:2:4}\" != \"XXX_\" -a \"\${1:2:3}\" != \"0[0-9]_\" -a \"\${1:2:1}\" != \".\" ] && hier_umbenennen$1" >> $HOME/umbenennen_s$1.sh
    echo -e "  cd .." >> $HOME/umbenennen_s$1.sh
    echo -e "}" >> $HOME/umbenennen_s$1.sh
    echo -e "#pwd" >> $HOME/umbenennen_s$1.sh
    echo -e "verzeichnisauswerten$1 \"\$1\"" >> $HOME/umbenennen_s$1.sh
    chmod +x $HOME/umbenennen_s$1.sh
  else
    echo "namenindex \"$1\"?"
    exit 1
  fi
}

# ################################################
# ################################################

if [ -z $1 ]; then
  echo "nur im aktuellen Verzeichnis umbenennen..."
  cd "$(pwd)/"
  umbenennen_sub 1 1
  sleep 2s
  $HOME/umbenennen_s1.sh "$(pwd)/"
else
  while [ $# -gt 0 ]; do
    cd $1
    if [ $? -ne 0 ]; then
      echo "Fehler, kein gültiges Verzeichnis"
      #popd > /dev/null 2> /dev/null
      #exit 1
    else
      echo "=> $1"
    fi
    if [ -f $1 ]; then
      echo "nur diese Datei umbenennen..."
      #$HOME/filmschnitt.sh -umbenennen -i "$1"
      find ./ -mindepth 1 -maxdepth 1 -name "$1" -exec $HOME/filmschnitt.sh -umbenennen -i '{}' \\;
    else
      # Hilfsscript für erste bis dritte Verzeichnisebene
      for (( idx=1; idx<=$Anzahl_Ebenen; idx++ )); do
        if [ $idx -eq $Anzahl_Ebenen ]; then
          umbenennen_sub $idx 1
        else
          umbenennen_sub $idx 0
        fi
      done
      echo "im übergebenen Verzeichnis \"$1\" und max. $Anzahl_Ebenen Unterverzeichnisebenen umbenennen..."
      initcnt1=$(ps | grep -c 'find')
      #pwd
      find ./ -mindepth 1 -maxdepth 1 -type d -exec $HOME/umbenennen_s1.sh '{}' \;
      while [ $(ps | grep -c 'find') -gt $initcnt1 ]; do
        sleep 1
      done
      $HOME/umbenennen_s$Anzahl_Ebenen.sh "$(pwd)/"
    fi
    cd ..
    shift
  done
fi

for (( idx=1; idx<=$Anzahl_Ebenen; idx++ )); do
  rm -f $HOME/umbenennen_s$idx.sh
done

popd > /dev/null 2> /dev/null
