#!/bin/bash
## =======================================================================
## filmschnitt.sh - (C) Stefan Bürger, www.stefan-buerger.de

# #########################################################################
# Empfohlebne Verzeichnisstruktur:
#   baseDir -,- logDir                       lokales Laufwerk/RAM-Disk - Logverzeichnis
#            +- srcDir                       beliebiges Laufwerk - Videoquellenverzeichnis, kann Link zu anderer (möglichst lokaler) Festplatte sein
#            |    '- erledigtDir             lokales Laufwerk - Ablage der Quelldatei nach Konvertierlauf, diese Datei kann nach Prüfung des Konvertierergebnisses gelöscht werden
#            +- konvertierenDir/schneidenDir lokales Laufwerk - erstes Zielverzeichnis für Konvertierergebnis bzw. finales Zielverzeichnis, falls kein Backup-Verzeichnis festgelegt ist
#            |    +- bkup[1|2|3]tmp          lokales Laufwerk - temporär für konvertierte Daten vor Übertragung in Backup-Verzeichnis, Videodaten und Unterverzeichnisse werden nach Backup gelöscht
#            |    '- offenDir                lokales Laufwerk - Ablage der konvertierten Datei, wenn der Backup-Ablauf gescheitert ist
#            '- backupDir[1|2|3]             beliebiges Laufwerk - finales Backup-Verzeichnis, kann Link zu Netzlaufwerk sein, muss nicht permanent erreichbar sein
# Benötigte Unterverzeichnisse werden bei Bedarf erzeugt und nach Nutzung wieder entfernt
# 
# Die Pfadnamen sind ab Zeile 216 festgelegt
# -------------------------------------------------------------------------
# Konfigurationsvarianten:
# (default):     Es werden die Parameter aus dem "Standard"-Block ohne Zusatzeinstellungen verwendet: CpuLimitDef=70, VMOD=720p, HEVC+EAC3/AAC, Allaudio, Backup1, Backup2, Backup3.
# -keinbkup:     Konvertierparameter wie Default, aber kein Backup, d.h. konvertierter Film nur in Standard-Zielverzeichnis.
# -low (=272p, mit hevc)         / 
# -dvd (=medium/480p, mit hevc)  / 
# -high (=720p, mit hevc)        /  
# -full (=1080p, mit hevc)       /  
# -max (=2160p, mit hevc)        / 
# -copy:         vereinfachte Auswahl von Standardkonfigurationen, ansonsten wie default
# -subdir "...": Es werden die Parameter aus dem "Standard"-Block mit Zusatzeinstellungen verwendet, bei mir zusätzliches variables Verzeichnis in Backup1, , Backup2 und Backup3.
# -schneiden:    Es werden die Parameter aus dem "Schneiden"-Block verwendet, also nur alle Audio-Streams auf angegebenes Format konvertiert (EAC3), Video-Auflösung unverändert. 
# #########################################################################

CPUtyp="$(lscpu|grep "Vendor ID:"|awk 'BEGIN {val=0} {val=$3} END {print val}')"
CPUarch="$(lscpu|grep "Architecture:"|awk 'BEGIN {val=0} {val=$2} END {print val}')"
CPUmaxGHz="$(lscpu|grep "CPU max MHz:"|awk 'BEGIN {val=0} {val=$4} END {print val}')"
CPUmaxGHz="${CPUmaxGHz%,*}"
CPUmaxGHz="$(echo "$CPUmaxGHz"|awk 'BEGIN {val1=0;val2=0} {val1=$1/1000;val2=int(((val1+0.05)-int(val1))*10);val1=int(val1)} END {print val1","val2"GHz"}')"

source $HOME/livestream-adresse.inc

pcmd="$0"

durchsucheUnterverzeichnisse=1 # 0=keine Duplikate suchen, 1=Arbeitsverzeichnisse, 2=$konvertierenDir-Unterverzeichnisse auch durchsuchen - dauert erheblich länger, je Verzeichnis 10...100s
                               # 3 / 4 = 1 / 2 mit permanenter Zwischenstand-Anzeige. Bei mir sind z.Z. ca. 700 Verzeichnisse/Unterverzeichnisse in jedem Backup-Ziel enthalten,
                               # die Duplikatsuche dauert damit je Backup-Ziel bei 2 und 4 (insgesondere bei USB- bzw. Netzwerkverzeichnissen) mehrere Stunden!
vorherFindeDup=0  # 1=findeDup auch vor Konvertierung ausführen

suche_in_bkup123dir="bkup1dir"  # bei $durchsucheUnterverzeichnisse=2 oder 4: "": in allen bkupXdir-Verzeichnissen suchen | "bkup1dir" oder "bkup2dir" oder "bkup3dir": nur dort suchen

zielCodec="hevc"      # "hevc"/"h264"/"vp9"/"av1"
zielOptimierung="opt" # "std"=Standard, "br"=Bitrate, "opt"=starke Optimierung
zielFormat="720p"     # "272p"/"low"/"480p"/"dvd"/"720p"/"1080p"/"2160p"
CpuLimitDef=75 # Frühjahr/Herbst/Winter meist mäßige/kühle Temperatur, darum höhere Leistung erlauben

einrB="                                "

function einruecken {
numcols=$(( $(/usr/bin/tput cols) - 1 )) ; [ $numcols -lt 200 ] && numcols=200
chkcols=$numcols
}

einruecken

scriptversion="9.6"
scriptdatum="15.03.2024"
scripthdrlen=$(( ${#scriptversion} + ${#scriptdatum} ))
addstr=" "
addstrlen=$(( 17 - $scripthdrlen ))
if [ $addstrlen -ge 1 ]; then for (( idx=0 ; idx<$addstrlen ; idx++ )); do addstr="$addstr " ; done ; fi

scriptversioninfobase="${SETTK}Lauffähig auf Intel-/AMD-PCs und Raspberry PI 5, Aktive CPU: $CPUtyp/$CPUarch, $CPUmaxGHz.$NOCOL
${einrB}${SETTK}Optimierungen, verbesserte Duplikatsuche optional bis 5. Ebene (dauert u.U. mehrere Stunden). Verzeichnis-Reorganisation (\$baseDir=\$HOME/Multimedia/ statt$NOCOL
${einrB}${SETTK}\$HOME/Multimedia/DVBT-Aufnahme). Wieder Mediathekstream-Probleme. Korrekturen, Anpassung von Statusmeldungen, Formatoptionen, Lesefehlertoleranz und Backup-Optionen.$NOCOL"

if [ "$CPUtyp" == "ARM" ]; then
  scriptversioninfobase="$scriptversioninfobase
${einrB}${SETTK}${SETUL}Hinweis:$NOCOL${SETTK} Raspberry PI 5 (4 Kerne, max. 2,4GHz) Konvertierdauer vs. Filmlänge ⭢HEVC bei 75% CPU-Leistung: HIGH/720P ≤0,1x; DVD/480P ≥0,2x; LOW/272P ≥0,4x.$NOCOL
${einrB}         ${SETTK}Verglichen mit einem Intel-PC (4 Kerne, max. 3,3GHz) ist der Raspberry PI 5 (4 Kerne, ARM, max. 2,4GHz) ca. 4-5x langsamer.$NOCOL
${einrB}${SETRT}${SETUL}Achtung:$NOCOL${SETRT} Das Raspberry PI 5-OS (ARM-SW) unterstützt z.Z. nicht libfdk_aac, AAC-Codierung ist daher nur eingeschränkt mit 'aac'-Codec möglich!$NOCOL"
else
  scriptversioninfobase="$scriptversioninfobase
${einrB}${SETTK}${SETUL}Hinweis:$NOCOL${SETTK} Intel/AMD-PC (4 Kerne, max. 3,3GHz) Konvertierdauer vs. Filmlänge ⭢HEVC bei 75% CPU-Leistung: HIGH/720P ≥0,4x; DVD/480P ≥0,9x; LOW/272P ≥2,0x.$NOCOL
${einrB}         ${SETTK}Raspberry PI 5 (ARM, 4 Kerne, max. 2,4GHz) ist ca. 4-5x langsamer.$NOCOL"
fi

versionshistorietxtaktuell="$SETBGWW$SETBLD$pcmd$SETBGSW$NOCOL\n $SETBGGR${SETTK}Aktuell: V$scriptversion - $scriptdatum$NOCOL$SETTK$addstr$scriptversioninfobase$SETBL"

scriptversioninfokomplett="$versionshistorietxtaktuell$NOCOL\r$SETBL 
${einrB}$SETTK${SETUL}Codecs$NOCOL$SETBL AVI/MPG/M2T/M2V/TS/MTK/H264/HEVC/VP9/AV1$SETTK⭢${SETBL}H264/HEVC/VP9/AV1, AAC/AC3/EAC3/MP3/M2A/VORBIS/OPUS$SETTK⭢${SETBL}AAC/AC3/EAC3/MP3/OPUS, $SETTK${SETUL}Container$NOCOL$SETBL TS/M2T/M2V/MP4/M4V/MKV/WEBM$SETTK⭢${SETBL}MP4/WEBM.
${einrB}$SETTK${SETUL}Video:$NOCOL$SETBL ${SETUL}H264$NOCOL ${SETGN}Auf alten Playern abspielbar${SETOR}/Format >480P ineffektiv$SETBL | ${SETUL}VP9$NOCOL ${SETGN}Größe ≤ HEVC/für Streaming${SETOR}/Qualität schlechter/2-5x langsamer/keine Stream-Info.$SETBL
${einrB}       ${SETUL}HEVC$NOCOL ${SETGN}Größe 0,5-0,8x H264/Qualität gut${SETOR}/1,3-1,7x langsamer $SETBL | ${SETUL}AV1$NOCOL ${SETGN}Größe ≤ HEVC/für Streaming/Qualität gut${SETOR}/50-100x langsamer/keine Stream-Info.$SETBL
${einrB}       ${SETTK}Qualität:$NOCOL$SETBL${SETUL}STD$NOCOL ${SETGN}kleinere Dateien${SETOR}/Qualitätsverlust$SETBL ${SETUL}BR$NOCOL ${SETOR}mittlere Größe/evtl. Qualitätsverlust$SETBL ${SETUL}OPT$NOCOL ${SETGN}kaum Qualitätsverlust$SETOR/evtl. größere Dateien$SETBL
${einrB}       ${SETTK}Hinweis:$NOCOL$SETBL Die automatische Qualitätsbewertung bietet keine Info zu Kontrast/Helligkeit des Videostreams, bei Warnhinweisen ist eine Sichtkontrolle sinnvoll, 
${einrB}                Kleine HEVC-Auflösungen sind weniger effektiv, Konvertierung H264$SETTK⭢${SETBL}HEVC ist bei Verkleinerung < Faktor 2 zu vermeiden.
${einrB}$SETTK${SETUL}Audio:$NOCOL$SETBL ${SETUL}AAC/AC3$NOCOL$SETGN Für alte Player/gute Qualität$SETBL | ${SETUL}MP3$NOCOL$SETGN für alte Player${SETOR}/Mono+Stereo$SETBL | ${SETUL}EAC3$NOCOL$SETGN gute Qualität${SETOR}/nicht für alte Player$SETBL | ${SETUL}OPUS$NOCOL$SETGN gute Qualität${SETOR}/keine Stream-Info.$SETBL
${einrB}$SETTK${SETUL}Präferenzen:$NOCOL$SETBL ${SETUL}HiQ-Film$NOCOL ${SETGN}HEVC+≥720P+AAC/EAC3+MP4+OPT${SETBL} | ${SETUL}Doku$NOCOL ${SETGN}H264/HEVC+≤480P+AAC/EAC3+MP4+OPT${SETBL} | ${SETUL}Serie$NOCOL ${SETOR}H264/${SETGN}HEVC+480P+AAC/EAC3+MP4+OPT${SETBL} | ${SETUL}Web$NOCOL ${SETGN}VP9${SETOR}/AV1${SETGN}+≤480P+OPUS+WEBM+OPT$NOCOL"
#$SETBL${einrB}HW-Encode ${SETRT}vorbereitet, aber noch nicht getestet. HW-Encode ist viel schneller, aber Qualität vermutlich schlechter als SW-Standard-Encode.$NOCOL

scriptversion1=${scriptversion%.*}
scripthdrlen1=$(( ${#scriptversion1} + 16 + ${#scriptdatum} ))
addstr1=" "
addstr1len=$(( 30 - $scripthdrlen1 ))
if [ $addstr1len -ge 1 ]; then for (( idx=0 ; idx<$addstr1len ; idx++ )); do addstr1="$addstr1 " ; done ; fi

versionshistorietxt="${SETTK}V$scriptversion1.x / 14.11.2023-$scriptdatum$addstr1$scriptversioninfobase$SETBL
V8.x / 21.02.2022-14.11.2023    Korrekturen (Feintuning/AAC-Behandlung/Fehlerbehebung/Stream-Mapping/Bkup-Ziele/HEVC=Standard/Duplikate/Qualitätsbewertung).
V7.x / 25.02.2021-21.02.2022    Optimierungen, Konvertierproblem-Anzeige, Bitraten-/Streamlängenkontrolle (Bitratenkontrolle einzelner Streams nicht bei VP9/AV1+OPUS). 
V6.x / 18.11.2020-17.02.2021    Audiostreamerkennung, Backup, vertikale Videos, Zeichenersetzung, livestreamadresse.inc, Duplikate, Subdir, Frequenz-/Temperatur-Verlauf.
V5.x / 17.09.2020-15.11.2020    CPU-Leistung/Lastanpassung, Konvertierzeit, laufende Prozesse.
V4.x / 24.06.2018-15.09.2020    Korrekturen, Erweiterungen, CPU-Zeit-/Speicherauslastung, Temperaturbegrenzung, Parametererkennung.
V3.x / 04.05.2018-10.06.2018    Filmdauer, Audiostreams, Vorbereitung für Untertitel.
V2.x / 14.12.2017-28.04.2018    einfache Parametererkennung, Sendernamenfilterung, A/V-Formatbehandlung.
V1                              Manuelle Optionen.$NOCOL"

versionshistorie="$NOCOL$TRENNER\n${SETBL}Script-Historie:\n$versionshistorietxt"

function hilfekurzsyntax {
  if [ $enasubtitle -ge 0 ]; then hilfesubtxt1=" [-u [\"DecSubCmd\" [\"EncSubCmd\"]]]" ; else hilfesubtxt1="" ; fi
  echo -e "$SETBGWS${SETBLD}Parametereingabe$SETBGSW $SETBGGR${SETBL}Ausführliche Hilfe mit \"-h\", optionale Parameter [...], Alternativkonfiguration \"-schneiden|-umbenennen|-subdir...\" als erster Parameter, weitere Parameter in beliebiger Reihenfolge.$SETBGSW$SETBL
'$SETTK\$HOME/${pcmd##*/} [-schneiden|-umbenennen|-subdir \"[/]<Verzeichnis>[/<SubVerz>]\" [\"[/]<SubVerz>\"]] -i \"<Datei>\" [-w \"<neuer Name>\"]$SETBL ${SETTK}[-a <Start>] [-e <Stop>] [-f <Bildrate>] [-o <Offset>]$SETBL ...\n    ... ${SETTK}[-r <Seitenverhältnis>] [-hevc|-h264|-vp9|-av1] [-q std/br/opt/\"<codecspezifisch>\"] [-vertikal[blur]]$SETBL ${SETTK}[-[v]4k|-[v]1080p|-[v]720p|-[v]dvd|-[v]low] [-aac|-[e]ac3|-mp3|-opus]${hilfesubtxt1}$SETBL ...\n    ... ${SETTK}[-s <Audio>] [-n] [-x] [-h] [-l] [-cc] [-d [0|1|2] [0|1]] [-[no]bkup[1|2|3]] [-p <Leistung>]$SETBL ${SETTK}[-zi \"<ffmpeg-Kommandos Quelle>\"] [-zo \"<ffmpeg-Kommandos Ziel>\"] [-zm] [-zl [<level>]]$SETBL'$NOCOL$SETBL"
}

function hilfecmdsyntax {
  hilfekurzsyntax
  echo -e "$SETBLD${SETBGWS}Beispiel 1$SETBGSW $SETBGGR${SETBL}Schneiden+Konvertieren$SETBGSW$SETBL a) \${srcDir}: $SETOR~/${pcmd##*/} -subdir \"test\" -m 720p -i \"[<Pfad>/]TEST.m2t\" -a 13:56 -e 1:59:55$SETBL ⇒ ${SETGN}\${konvertierenDir}/test/TEST_1280x720_HEVC_1h45m59s.mp4$SETBL
$SETBGGR(b dauert mit Vorlaufzeit länger)$SETBGSW$SETUL b) \${srcDir}: $SETOR~/${pcmd##*/} -subdir \"test\" -m 480p -i \"[<Pfad>/]TEST.m2t\" -a 2:00:12 -e 3:29:55$SETBL ⇒ ${SETGN}\${konvertierenDir}/test/TEST_720x480_HEVC_1h29m43s.mp4$SETBL         $NOCOL
$SETBLD${SETBGWS}Beispiel 2$SETBGSW $SETBGGR${SETBL}2-Stufen-Konvertierung$SETBGSW$SETBL a) \${srcDir}:       $SETOR~/${pcmd##*/} -schneiden -i \"[<Pfad>/]TEST.m2t\" -a 13:56 -e 1:59:55$SETBL ⇒ ${SETGE}\${schneidenDir}/TEST_1920x1080_HEVC_1h45m59s.mp4$SETBL
$SETBGGR(Schneiden inklusive Vorlauf     $SETBGSW    \${schneidenDir}: $SETGE~/${pcmd##*/} -subdir \"test\" -m 720p -i \"[<Pfad>/]TEST_1920x1080_HEVC_1h45m59s.mp4\"$SETBL ⇒ ${SETGN}\${konvertierenDir}/test/TEST_1280x720_HEVC_1h45m59s.mp4$SETBL
$SETBGGR 1/40x von Konvertieren, das an- $SETBGSW b) \${srcDir}:       $SETOR~/${pcmd##*/} -schneiden -i \"[<Pfad>/]TEST.m2t\" -a 2:00:12 -e 3:29:55$SETBL ⇒ ${SETGE}\${schneidenDir}/TEST_1920x1080_HEVC_1h29m43s.mp4$SETBL
$SETBGGR schließend weniger langsam ist) $SETBGSW$SETUL    \${schneidenDir}: $SETGE~/${pcmd##*/} -subdir \"test\" -m 480p -i \"[<Pfad>/]TEST_1920x1080_HEVC_1h29m43s.mp4\"$SETBL ⇒ ${SETGN}\${konvertierenDir}/test/TEST_720x480_HEVC_1h29m43s.mp4 $NOCOL$SETBL
$SETBLD${SETBGWS}Konvertierliste$SETBGSW $SETGN~/loop_filmschnitt.sh$SETBL erzeugt im aktuellen Verzeichnis die Datei ${SETGN}convloop_<Aktion>_<Datum>.sh$SETBL mit allen gefundenen Videodateien, die Parameter in der Liste müssen bei Bedarf
angepasst werden. '\$HOME/loop_filmschnitt.sh [Aktion: komplett (default)|schneiden|konvertieren|umbenennen] [Filter/Zielverzeichnis (optional)] [Leistungswert 15...100/max/min (optional)]'
$NOCOL$TRENNER"
}

function warnhinweis {
  echo -e "$SETRT$WARNTR\n!!! VERWENDUNG AUF EIGENE GEFAHR, Original-Videodateien erst nach Prüfung der konvertierten Videodateien löschen !!!\n$WARNTR$NOCOL"
}

# #########################################################################
# für alle Konfigurationen gemeinsame Einstellungen:
pcfg="$1"
if [ "$1" != "" ] && [ $pcfg = "-version" ]; then
  echo "$scriptversion"
  exit 0
fi

nurLokalesZiel=1  # 0=kein Warnhinweis, 1=Warnung bei USB-/Netzlaufwerk, 2=alternative lokale Pfade als Ziel (weniger sicher, wegen NFS/CIFS-Rechteproblem nach Stromausfall 23.3.2022)

rsyncstattcpmv=0  # 0: cp bzw. mv, 1: rsync statt cp/mv

CheckTemp=-1      # (wird bei CpuLimitDef=14 ignoriert) alle Temp/Freq-Checks an (1), ohne Startprüfung (0), reduziert (-1), nur bei hoher Belastung/Temperatur (-2), komplett aus (-3)

fullCmdLine=()
fullCmdLine+=($0)

AufrufStr1="$pcmd Version $scriptversion vom $scriptdatum"
AufrufStr3="$pcmd $@"

#aktmon="$(date +%-m)"
akttemp=0
akttempmax=0
akttempmin=127
_tmpstr="-" # Startwert für nachfolgende ALIVE-Anzeigen
function lebendig {
  if [ "$_tmpstr" = "-" ]; then 
    _tmpstr="\\"
  elif [ "$_tmpstr" = "\\" ]; then
    _tmpstr="|"
  elif [ "$_tmpstr" = "|" ]; then
    _tmpstr="/"
  else
    _tmpstr="-"
  fi
  echo -e -n "$SETBGWW$SETGND $_tmpstr $SETBGSW$NOCOL \r"
}

function checkAktTemp {
  for _idx in {1..40} ; do
    lebendig
    akttemp2=$(echo "scale=0 ; ( $( cat /sys/devices/virtual/thermal/thermal_zone0/temp ) / 1000 )" | bc)
    if [ $_idx -gt 20 ]; then
      sleep 0.5s
      akttemp=$(( $akttemp + $akttemp2 ))
      [ $akttempmax -lt $akttemp2 ] && akttempmax=$akttemp2
      [ $akttempmin -gt $akttemp2 ] && akttempmin=$akttemp2
    else
      [ $akttemp2 -gt 60 ] && sleep 0.5s
    fi
  done
  echo -e -n "  \r"
  akttemp=$(( $akttemp / 20 ))
  # CpuLimitDef=60...75 führt während der Konvertierung (ohne andere Lasten!) an meinem PC zu CPU-Core-Temperatur 80...100°C. Achtung: Die System- und Aux-Temperatur ist
  # deutlich niedriger (35...60°C, Sensoren mit "sudo sensors-detect" aus Paket "lm-sensors" verfügbar machen). Die Prozessorlast wird automatisch lastabhängig weiter reduziert.
  if [ $(echo -e "$AufrufStr3" | grep -c -E ' -p ') -gt 0 ]; then
    initpwrlvlstr="$TRENNER\n$SETBGGR${SETOR}Maximale Leistung mit '-p' geändert, evtl. Reduzierung bei hoher Last. Aktuell $akttemp ($akttempmin...${akttempmax}) °C$SEGBGSW$NOCOL"
  elif [ $akttemp -ge 85 ]; then
    initpwrlvlstr="$TRENNER\n$SETBGGR${SETFLRT}Maximale Leistung wegen sehr hoher Systemtemperatur $akttemp ($akttempmin...${akttempmax}) °C reduziert von $CpuLimitDef%/CPU"
    CpuLimitDef=$(( $(( $CpuLimitDef * 3 )) / 7 )); initpwrlvlstr="$initpwrlvlstr auf $CpuLimitDef%, weitere Reduzierung bei hoher Last.$SEGBGSW$NOCOL"
  elif [ $akttemp -ge 75 ]; then
    initpwrlvlstr="$TRENNER\n$SETBGGR${SETRT}Maximale Leistung wegen hoher Systemtemperatur $akttemp ($akttempmin...${akttempmax}) °C reduziert von $CpuLimitDef%/CPU"
    CpuLimitDef=$(( $(( $CpuLimitDef * 2 )) / 3 )); initpwrlvlstr="$initpwrlvlstr auf $CpuLimitDef%, weitere Reduzierung bei hoher Last.$SEGBGSW$NOCOL"
  elif [ $akttemp -ge 65 ]; then
    initpwrlvlstr="$TRENNER\n$SETBGGR${SETOR}Maximale Leistung wegen erhöhter Systemtemperatur $akttemp ($akttempmin...${akttempmax}) °C reduziert von $CpuLimitDef%/CPU"
    CpuLimitDef=$(( $(( $CpuLimitDef * 5 )) / 6 )); initpwrlvlstr="$initpwrlvlstr auf $CpuLimitDef%, weitere Reduzierung bei hoher Last.$SEGBGSW$NOCOL"
  else
    initpwrlvlstr="$TRENNER\n$SETBGGR${SETGN}Aktuell $akttemp ($akttempmin...${akttempmax}) °C, maximale Leistung $CpuLimitDef%/CPU, Reduzierung bei hoher Last.$SEGBGSW$NOCOL"
  fi
}
[ "${pcfg,,}" != "" -a "${pcfg,,}" != "-zeigepfade" -a "${pcfg,,}" != "-umbenennen" -a $CheckTemp -ne -3 ] && checkAktTemp
# #########################################################################

# Nachfolgende 'min_vbr'-Werte passen für DVBT2. für DVBC2/DVBS2 höhere Bitrate?
min_vbr_uhd_h264=6400 # Mindest-Video-Bitrate bei UHD-Auflösung (H264 zwar unüblich, aber nachfolgende Bewertung basiert auf H264-Werten, andere H264/HEVC/VP9/AV1-Werte werden davon abgeleitet!)
min_abr=125           # Mindest-Audio-Bitrate je Stream

function calcMinVbr {
  # alle mit Faktor <MaxRes> / <ZielRes> errechneten Werte + 100! Schätzwert für eventuelle Konvertiernachteile bei kleineren Bitraten
  min_vbr=$min_vbr_uhd_h264 ; [ $1 -gt 1 ] && min_vbr=$(( $(( $min_vbr / $1 )) + 50 ))
}

finalScript="0_dirlist.sh"                          # finalScript - leerer String oder Scriptdatei im Pfad "bkup{1|2}dir" oder "$HOME", in Verarbeitungsvarianten "myscript" ein- oder ausschalten

# absolute Pfade mit '/' am Ende:
srcDir="$(echo $(findePfad $PWD))/"
baseDir="$HOME/Multimedia/" #DVBT-Aufnahme/"           # Gemeinsamer Basispfad für alle Varianten
logDir="${baseDir}tmp-log/"                         # alle Log-Dateien
# Scripte und temporäre Pfade
konvertierenDir="${baseDir}.VIDEO-AUDIO_konv_tmp/"  # versteckt/unsichtbar - wird bei Konvertierung verwendet
schneidenDir="${baseDir}.VIDEO_geschnitten_tmp/"    # versteckt/unsichtbar - wird bei "-schneiden" verwendet
umbenennenDir="${srcDir}"                           # wird bei "umbenennen" verwendet
offenDir="XXX_Backup_gescheitert/"                  # Ergebnisdatei bei fehlerhaftem Backup-Verlauf, nicht löschen, evtl. erneute Konvertierung nötig
erledigtDir="XXX_Konvertierte_Quellen/"             # erfolgreich konvertierte Quellendatei, kann nach Prüfung der konvertierten Datei gelöscht werden
# Konvertieren- und Schneiden-Pfade...
if [ $nurLokalesZiel -eq 2 ]; then
  # Konvertier-Ziel 1 (2 und 3 nicht verwendet)
  backupDir1K="${baseDir}VIDEO_konvertiert/"      # lokales Laufwerk, muss permanent erreichbar sein
  backupDir2K=""
  backupDir3K=""
  # Schneiden-Ziel 1 (2 und 3 nicht verwendet)
  backupDir1S="${baseDir}VIDEO_geschnitten/"      # lokales Laufwerk, muss permanent erreichbar sein
  backupDir2S=""
  backupDir3S=""
else 
  # Konvertier-Ziele 1 bis 3
  backupDir1K="${baseDir}Archivdisk_16TB-mp4/" # beliebiges Laufwerk, z.B. NAS oder USB-LW an NAS via Link auf Netzwerk-Mountpoint des NAS (ist u.U. zeitweise nicht aktiv)
  backupDir2K="${baseDir}NAS_mp4/"             # beliebiges Laufwerk, z.B. NAS oder USB-LW an NAS via Link auf Netzwerk-Mountpoint des NAS (ist u.U. zeitweise nicht aktiv)
  backupDir3K=""
  # Schneiden-Ziele 1 bis 3
  backupDir1S="${baseDir}VIDEO_geschnitten/"   # Schnittziel temporär, 1 Kopie reicht! # beliebiges Laufwerk, z.B. NAS oder USB-LW an NAS via Link auf Netzwerk-Mountpoint des NAS (ist u.U. zeitweise nicht aktiv)
  backupDir2S=""                               # beliebiges Laufwerk, z.B. NAS oder USB-LW an NAS via Link auf Netzwerk-Mountpoint des NAS (ist u.U. zeitweise nicht aktiv)
  backupDir3S=""                               # beliebiges Laufwerk, z.B. NAS oder USB-LW an NAS via Link auf Netzwerk-Mountpoint des NAS (ist u.U. zeitweise nicht aktiv)
fi
# relative Unterverzeichnisse ohne '/' am Ende:
konvertierenWebSubDir="0007_Web"                # wird bei "Web"-Konvertierung verwendet
konvertierenDVDSubDir="0008_DVD"                # wird bei "DVD"-Konvertierung verwendet
konvertierenAndroidSubDir="0009_Android"        # wird bei "Android"-Konvertierung verwendet

function zeigePfade {
  local i1=0; local i2=0; local i3=0; local i4=0; local fill0str=""; local fill1str=""; local fill2str=""; local fill3str=""
  fill0=$(( 100 - ${#konvertierenDir} - 21 )); [  ${#fill0} -lt 2 ] && fill0=2
  for (( i1=0 ; i1<$fill0 ; i1=i1+1 )); do fill0str="$fill0str "; done
  fill1=$(( 100 - ${#backupDir1K} - 14 )); [  ${#fill1} -lt 2 ] && fill1=2
  for (( i2=0 ; i2<$fill1 ; i2=i2+1 )); do fill1str="$fill1str "; done
  fill2=$(( 100 - ${#backupDir2K} - 14 )); [  ${#fill2} -lt 2 ] && fill2=2
  for (( i3=0 ; i3<$fill2 ; i3=i3+1 )); do fill2str="$fill2str "; done
  fill3=$(( 100 - ${#backupDir3K} - 14 )); [  ${#fill3} -lt 2 ] && fill3=2
  for (( i4=0 ; i4<$fill3 ; i4=i4+1 )); do fill3str="$fill3str "; done

  addopt=""
  if [ "$1" != "schneiden" -a "$1" != "umbenennen" -a "$1" != "verschieben" -a ${#backupDir1K} -eq 0 -o ${#backupDir2K} -eq 0 -o ${#backupDir3K} -eq 0 ]; then
    addopt="\n#  Optionaler Zielpfad:"
    [ ${#backupDir1K} -eq 0 ] && addopt="$addopt\n       pfad1=' -pfad1 $HOME/Multimedia/Archivdisk_16TB-mp4/ '"
    [ ${#backupDir2K} -eq 0 ] && addopt="$addopt\n       pfad2=' -pfad2 $HOME/Multimedia/NAS_mp4/ '"
    [ ${#backupDir3K} -eq 0 ] && addopt="$addopt\n       pfad3=' -pfad3 $HOME/Multimedia/Lokal_mp4/ '"
  fi
  addopt="$addopt\n#  Standard-Aktion: '$1'"
  [ "$1" != "konvertieren" -a "$1" != "umbenennen" -a "$1" != "verschieben" ] && addopt="$addopt (verwendet die 'konvertieren'-Pfade)"
  [ "$1" == "umbenennen" ] && addopt="$addopt (verwendet den Pfad '$umbenennenDir' als Zielpfad)"
  #[ "$1" == "" ] && addopt="$addopt (verwendet ...)"
  echo -e "# Voreingestellte Pfade in filmschnitt.sh ...
#  Quellenpfad:
#      srcDir='$srcDir'
#  Zielpfade für Konvertieren:                                                                       Zielpfade für Schneiden:
#      konvertierenDir='$konvertierenDir'$fill0str schneidenDir='$schneidenDir'
#      bkup1dir='$backupDir1K'$fill1str bkup1dir='$backupDir1S'
#      bkup2dir='$backupDir2K'$fill2str bkup2dir='$backupDir2S'
#      bkup3dir='$backupDir3K'$fill3str bkup3dir='$backupDir3S'
#      Die Zielpfade von bkup1dir, bkup2dir und bkup3dir können mit -pfad1 '...', -pfad2='...' bzw. -pfad3='...' verändert werden!$addopt"
  exit 0
}

[  "${pcfg,,}" = '-zeigepfade' ] && zeigePfade $2 || echo -e -n "$SETBGSW$NOCOL"

# #############################################
function hilfetext {
  echo -e "$scriptversioninfokomplett$NOCOL\n$TRENNER"
  hilfecmdsyntax
  echo -e "$SETBLD${SETBGWW}Linux-Konvertierscript für Audio-Video-Formate - Version $scriptversion - $scriptdatum - Stefan Bürger - https://www.stefan-buerger.de$NOCOL 
$SETBLD${SETBGWS}Basis-Konvertierparameter ('optional'=Gesamter Parameter kann weggelassen werden, Teilparameter:'[<dieser Wert ist optional>]'):$NOCOL$SETBL
-subdir \"[/]<Name>\"|-schneiden|... optional als erster Parameter (!) -subdir: Ziel-Verzeichnis, mit \"/...\" muss Verzeichnis existieren | -schneiden: nur Anfang (-a)/Ende (-e) schneiden |
  ...-umbenennen|-zeigePfade:                                        -umbenennen: Datei entsprechend '${pcmd##*/}'-Zielname-Regeln umbenennen | -zeigePfade: Nur Anzeige der aktiven Zielpfade.
-i <Dateiname>:                    Dateiname mit oder ohne relativen/absoluten Pfad, Dateinamen mit Leer-/Sonderzeichen müssen mit \"...\" eingerahmt werden.
-a <Startzeit>|-e <Endzeit>:       optional, Anfang bzw. Ende [[h:]m:]s[.n], h=0-23, m=0-59, s=0-59, n=\'|0-9, ohne Angabe wird ab Datei-Anfang bzw. bis Datei-Ende konvertiert
$SETBLD${SETBGWS}Zusätzliche Konvertierparameter:$NOCOL$SETBL
-2160p|-1080p|-720p|...|-272p...   optional, Bildgröße (Standard=$VMOD): -2160p =3840x2160, -1080p =1920x1280, -720p =1280x720, -540p =960x540, -480p =720x480, -360p =640x360, -272p =480x272,
...-v2160p|-v1080p|...|-v272p:               Vertikalformate -v4k, -v1080p, -v720p, -v540p, -v480p, -v360p, -v272p
-[v]copy|-h264|-hevc|-vp9|-av1:    optional, Video-Ausgabeformat kopieren oder von Quellenformat nach H.264, HEVC, VP9 oder AV1 ändern (Default=$([ $HevcVideoOut -eq 0 ] && echo -n "H264" || ([ $HevcVideoOut -eq 1 ] && echo -n "HEVC" || ([ $HevcVideoOut -eq 2 ] && echo -n "VP9" || ([ $HevcVideoOut -eq 3 ] && echo -n "AV1" || echo -n "???"))))).
-acopy|-aac|-[e]ac3|-mp3|-opus:    optional, Audio-Ausgabeformat kopieren oder von Quellenformat nach AAC, AC3, EAC3, MP3 oder OPUS ändern (Default=$([ $AacEac3Ac3AudOut -eq 0 ] && echo -n "AAC" || ([ $AacEac3Ac3AudOut -eq 1 ] && echo -n "AC3" || ([ $AacEac3Ac3AudOut -eq 2 ] && echo -n "EAC3" || ([ $AacEac3Ac3AudOut -eq 3 ] && echo -n "MP3" || ([ $AacEac3Ac3AudOut -eq 4 ] && echo -n "OPUS" || echo -n "???")))))).
-q <Wert>:                         optional, Konvertierqualität std, br oder opt (bei HEVC und H264 auch veryslow, slower, slow, medium, fast, faster, veryfast, superfast oder ultrafast)
-vertikal|-vertikalblur:           optional, vertikal (direkt), z.B. v1080p ⇒ v480p | vertikal ⇒ horizontal mit Blur-Rand, z.B. v1080p ⇒ 480p
-s <Wert>|-r <Wert>:               optional, Audiostreams wenn Autoerkennung bzw. default=$XLANG versagt: allaudio, audio[0...6], deu/ger,eng,fra/fre | -r <4zu3|16zu9>: Seitenformat erzwingen
-f <Wert>|-o <Wert>:               optional, Framerate 25000/29970/30000/50000/60000/100000 ±$FPSD, FPS x 1000, z.B. 25015=25,015fps | Offset $MOFS...$POFS [ms]
$SETBLD${SETBGWS}Zieldatei/Ziel-Pfade:$NOCOL$SETBL
-[bkup|pfad][1|2|3] ['<Zielpfad>'] optional, Mit '-bkup*' sind nur die gewählten Backup-Ziele aktiviert (optional mit neuem Zielpfad), mit '-pfad*' nur Pfadänderung. Default: Alle Ziele aktiviert.
-nobkup[1|2|3]:                    optional, Mit '-nobkup*' sind die gewählten Backup-Ziele deaktiviert. Default: Alle Ziele aktiviert.
-w <Ziel-Name>:                    optional, anderer Zieldateiname (Achtung: Angaben zu Dateiendung, Dauer und Codierung werden abgeschnitten und neu erzeugt!)"
  if [ $enasubtitle -ge 0 ]; then
    echo -e "-u:                              optional, Untertitel, 2 optionale Parameter (\"DecSubCmd\", \"EncSubCmd\"),$SETRT noch ungetestet!$SETBL"
  fi
  echo -e "-d [<0|1|2|3|4>]:                  optional, Duplikatsuche - default=$durchsucheUnterverzeichnisse (0=aus/1=Arbeitsverzeichnisse/2=alle Unterverzeichnisse), Suche je Verzeichnis dauert ca. 5...>15s, 
                                                             3/4 entspricht 1/2 mit zusätzlicher Duplikatsuche vor Konvertierung - default=$vorherFindeDup (0=aus/1=aktiv)
$SETBLD${SETBGWS}Leistungsbegrenzung, Parameter für Zusatzinformationen und direkte FFMPEG-Parameter:$NOCOL$SETBL
-p <Wert>:                         optional, -p <Wert>: Leistung in % je CPU, (15...100/min=14%/max=100%), bei min/max erfolgt keine Leistungsreduzierung
-zi|-zo|-zm|-cc:                   optional, -zi/-zo = freie ffmpeg-Kommandos für Quelle/Ziel | -zm = ausführliche Statusmeldungen | -cc = \"closed caption\"-Korrektur
-n|-zl <Log-Level>|-x|-h:          optional, -n = Quellendatei nicht verschieben | -zl = <info (default)/warning/error/panic/quiet> | -x = vorhandene Zieldatei überschreiben | -h = Hilfetext ausgeben
-l:                                optional, permanente Statusausgabe. Dieses Fenster öffnet sich bei Änderungen alle 60...120s im Vordergrund neu (nur sinnvoll, wenn der PC ansonsten ungenutzt ist).
$NOCOL$TRENNER
$SETBLD${SETBGWS}Arbeitsverzeichnisse (bei Bedarf im Script anpassen):$SETBGSW$NOCOL$SETBL\nsrcDir=\"$srcDir\"\n     (aktuelles Verzeichnis oder Dateiname mit relativem/absolutem Pfad)
baseDir=\"$baseDir\"\nlogDir=\"$logDir\"\nkonvertierenDir=\"$konvertierenDir\"\nschneidenDir=\"$schneidenDir\"\numbenennenDir=\"$umbenennenDir\"\noffenDir=\"$offenDir\"
erledigtDir=\"$erledigtDir\"\nbackupDir1K=\"$backupDir1K\"\nbackupDir2K=\"$backupDir2K\"\nbackupDir3K=\"$backupDir3K\"\nbackupDir1S=\"$backupDir1S\"\nbackupDir2S=\"$backupDir2S\"
backupDir3S=\"$backupDir3S\"\n$NOCOL$TRENNER$SETBL"
  warnhinweis
}

# #############################################
[ "${AufrufStr3% -i *}" != "${AufrufStr3##* -i }" ] && AufrufStr2="${SETBL}Aufruf in '$NOCOL$srcDir$SETBL':\n$SETBGWW$SETBLD${AufrufStr3% -i *} \\ $SETBL$SETBGSW$NOCOL\n$SETBGWW$SETBLD-i ${AufrufStr3##* -i } $SETBL$SETBGSW$NOCOL" || AufrufStr2="${SETBL}Aufruf in '$NOCOL$srcDir$SETBL':\n$SETBGWW$SETBLD$AufrufStr3$SETBGSW$NOCOL"

# #########################################################################
# Die folgenden Parameter müssen an die PC-Hardware bzw. das verwendete Abspielgerät und die Abspiel-SW angepasst werden!

interactflag=" -nostdin "

pcfg="${pcfg,,}"
# Nameninitialisierung (leere Strings!)
myscript=""
bkup1dir=""
bkup1dirnam=""
bkup1dirstr=""
bkup2dir=""
bkup2dirnam=""
bkup2dirstr=""
bkup3dir=""
bkup3dirnam=""
bkup3dirstr=""
pcfg2=""
pcfg2_2=""
pcfg3="/"
pcfg2str=""
tmpStrSkip=""
newlinestr="\n                                      "
tmpcol=""

forcets=0           # =1: .ts statt .mp4 oder -webm erzeugen. Nur für "-schneiden" als Zwischenstufe zur Beschleunigung. alle A/V-Daten können so direkt kopiert werden.
abbruchflag=0
CurrLoadLimit=10
closedcaptionkorrektur=0
vencNumCPUs=$(nproc)
# MP4-HEVC/H264:
h264hevcDefaultOption="" # siehe h264hevcDefaultOption="(leer)/veryslow/slow/medium/fast/faster/veryfast/ultrafast" weiter unten, gesonderte Multithreadind-Anweisungen sind nicht nötig!
pix_fmt=" -pix_fmt yuv420p10le" # Encoder-Bittiefe HEVC/H264/vp9/av1: 8-bit=" -pix_fmt yuv420p" (oder leerer String) 
                                #                                    10-bit=" -pix_fmt yuv420p10le" (nicht VP9) => Encodierdauer länger, weniger (neue) Artefakte 
                                #                                    12-bit=" -pix_fmt yuv420p12le" (nicht H264/VP9) => Encodierdauer länger, noch weniger (neue) Artefakte
                                # größere Bittiefe scheint trotz 8-bit-Quellen bei Verkleinerung positiv zu wirken!
# WEBM-VP9:
# "-quality good -cpu-used 0" bringt brauchbare Qualität (etwas schlechter als HEVC/H264) bei ähnlicher Komprimierung (~ 100% HEVC) und langsamer Konvertierung (~ 5...7x HEVC!), 
# "-quality good -cpu-used 1" und "... -cpu-used 2" ist jeweils noch mal 1,5x schneller mit weiter reduzierter Qualität, aber immer noch < 50% Geschwindigkeit von HEVC bei 80...90% der Dateigröße! 
# Bei Werten -cpu-used >= 3 ... 8 wird die Qualität deutlich schlechter bei jeweils stärkerer Komprimierung bis ca. 80% und fast (~ 600...70%) der HEVC-Geschwindigkeit.
# Ohne die erweiterten Optionen - also mit Default-Einstellungen - wird das Konvertierergebnis u.U. sogar größer als bei einer HEVC-Konvertierung!
#vp9OptExt="-quality good -cpu-used 4 -frame-parallel 1 -row-mt 1 -tile-columns $(( $vencNumCPUs - $(( $vencNumCPUs / 2 )) )) -tile-rows $(( $vencNumCPUs / 2 )) -aq-mode 2 -lag-in-frames 25 "
vp9OptExt="-quality good -cpu-used 4 -frame-parallel 0 -row-mt 1 -tile-columns 0 -tile-rows 0 -aq-mode 2 -lag-in-frames 25 "
# WEBM-AV1:
# Die Konvertierqualität erreicht fast HEVC bei stärkerer Komprimierung (~ 70% HEVC) aber allerdings 10...15-facher Konvertierdauer mit Optimierungen bzw. > 20...50-fach ohne Multithreading-Optionen.
#Av1OptExt="-cpu-used 6 -threads 4 -frame-parallel 1 -row-mt 1 -tile-columns $(( $vencNumCPUs - $(( $vencNumCPUs / 2 )) )) -tile-rows $(( $vencNumCPUs / 2 )) -aq-mode 2 -lag-in-frames 25"
Av1OptExt="-cpu-used 6 -threads 4 -frame-parallel 1 -row-mt 1 -tile-columns 0 -tile-rows 0 -aq-mode 2 -lag-in-frames 25 "
#??? Av1OptExt="-aom-params cpu-used=4:frame-parallel=1:row-mt=1:tile-columns=$(( $vencNumCPUs - $(( $vencNumCPUs / 2 )) )):tile-rows=$(( $vencNumCPUs / 2 )):lag-in-frames=25 "
# Video-/Audio-Ausgabe-Codierung - sinnvolle Kombinationen: ---------------
# *  HevcVideoOut: Auswahl 0=H264/1=HEVC/2=VP9/3=AV1, 
# *  AacEac3Ac3AudOut: Auswahl -1=COPY/0=AC3/1=EAC3/2=AAC/3=MP3/4=OPUS, 
# *  vcodecSpecific: 0=Default, 1=Spezifische Konvertierparameter zur Qualitätsverbesserung (bessere Qualität, etwas größere Dateien, erheblich längere Konvertierzeit)
# *  h264hevcDefaultOption: (leer)=Default, slow/fast/... zur einfachen Qualitätsänderung, beeinflusst Konvertierqualität, Dateigröße und Konvertierzeit.
[ -z $zielOptimierung ] && zielOptimierung="opt"
[ "$zielOptimierung" == "" ] && zielOptimierung="opt"
zielCodec="${zielCodec}_$zielOptimierung"   # "hevc_std"/"hevc_br"/"hevc_opt"/"h264_std"/"h264_br"/"h264_opt"/"vp9_std"/"vp9_opt"/"av1_std"/"av1_opt" 
                                            # => ..._opt=starke Optimierung, ..._br=Bitratenvorgabe, ..._std=Standardkonvertierung
function setzeCodierung { # $1=zielCodec, $2=zielOptimierung, $3=AacEac3Ac3AudOut, $4=vcodecSpecific, $5=h264hevcDefaultOption
  zielCodec=${1,,}          # h264 / hevc / vp9 / av1
  zielOptimierung=$2        # std / br / opt
  ONAM3a=${2:0:1}           # 
  case "${zielCodec%_*}" in # 0=H264/1=HEVC/2=VP9/3=AV1
    "av1")  HevcVideoOut=3 ;;
    "vp9")  HevcVideoOut=2 ;;
    "hevc") HevcVideoOut=1 ;;
    *)      HevcVideoOut=0 ;;
  esac
  AacEac3Ac3AudOut=$3       # -1=COPY/0=AC3/1=EAC3/2=AAC/3=MP3/4=OPUS
  vcodecSpecific=$4         # 0=Default, 1=Spezifisch
  h264hevcDefaultOption=$5  # (leer)=Default, slow/fast/...
}

case "${zielCodec,,}" in
  "av1_std")  setzeCodierung "av1" "std"  4 0 ""           ;; # AV1 (Default (leer)/$Av1OptExt)
  "av1_br")   setzeCodierung "av1" "br"   4 1 ""           ;; # AV1 (Default (leer)/$Av1OptExt)
  "av1_opt")  setzeCodierung "av1" "opt"  4 1 "$Av1OptExt" ;; # AV1 (Default (leer)/$Av1OptExt)
  "vp9_std")  setzeCodierung "vp9" "std"  4 0 ""           ;; # VP9 (Default (leer)/$Vp9OptExt)
  "vp9_br")   setzeCodierung "vp9" "br"   4 1 ""           ;; # VP9 (Default (leer)/$Vp9OptExt)
  "vp9_opt")  setzeCodierung "vp9" "opt"  4 1 "$Vp9OptExt" ;; # VP9 (Default (leer)/$Vp9OptExt)
  "h264_std") setzeCodierung "h264" "std" 2 0 "medium"     ;; # H264 (leer=Bitratenvorgabe/.../medium/fast/.../ultrafast) und AAC
  "h264_br")  setzeCodierung "h264" "br"  2 0 ""           ;; # H264 (leer=Bitratenvorgabe/.../medium/fast/.../ultrafast) und AAC
  "h264_opt") setzeCodierung "h264" "opt" 2 1 ""           ;; # H264 (mit zusätzlichen Konvertieroptionen) und AAC
  "hevc_std") setzeCodierung "hevc" "std" 2 0 "medium"     ;; # HEVC (leer=Bitratenvorgabe/.../medium/fast/.../ultrafast) und AAC
  "hevc_br")  setzeCodierung "hevc" "br"  2 0 ""           ;; # HEVC (leer=Bitratenvorgabe/.../medium/fast/.../ultrafast) und AAC
  "hevc_opt") setzeCodierung "hevc" "opt" 2 1 ""           ;; # HEVC (mit zusätzlichen Konvertieroptionen) und AAC
  *)          setzeCodierung "h264" "std" 2 0 "medium"     ;; # H264 (leer=Bitratenvorgabe/.../medium/fast/.../ultrafast) und AAC
esac                # HevcVideoOut: 0=H264, 1=HEVC, 2=VP9, 3=AV1 (noch experimentell/extrem langsam), Video-Ausgabeformat, für ältere HW-/SW-Player auf 0=H264 setzen. 
                    # AacEac3Ac3AudOut: 0=AC3, 1=EAC3, 2=AAC, 3=MP3, für ältere HW-/SW-Player auf 2=AAC setzen. AC3 funktioniert in der Variante 'ac3_fixed' im Gegensdatz zu 'ac3' auch
                    # mit 5.1-Streams. Das lt. MP4-Spec zu bevorzugende AAC-Format funktioniert in 'libfdk_aac' im Gegensdatz zu 'aac' auch mit 5.1-Streams und soll am Besten komprimieren.
                    # vcodecSpecific: HEVC Default-Parameter/spezifische Parameter (bessere Qualität mit an Zielauflösung angepassten Werten, aber langsamer!)
# #########################################################################
# Konfiguratiosvarianten als erster Parameter:

if [ "$pcfg" == "-schneiden" ]; then
  # ===================== Alternativkonfiguration "nur schneiden" =====================
  fullCmdLine+=("$pcfg")
  shift
  durchsucheUnterverzeichnisse=0
  myscript="$finalScript" # myscript - leerer String oder zuletzt auszuführende Scriptdatei im jeweiligen Pfad "bkup{1|2}dir" oder in $HOME
  CpuLimitDef=14
  VMOD="COPY"  # Video wird kopiert. Achtung: Mediatheken reduzieren meistens auf Nicht-Standard-Auflösungen <= 1280x720, die direkt bearbeitet werden sollten.
  forcets=1
  HevcVideoOut=1 ; AacEac3Ac3AudOut=-1 ; vcodecSpecific=0 ; h264hevcDefaultOption="" ; FAVAUD=-1 ; XLANG=deu
  destDir="$schneidenDir"   # Nach Datenverlust (Fehlbedienung durch Maus-Defekt) ist auch hier Backup-Prozedur erforderlich...
  bkup1dir="$backupDir1S"   # NAS via Link auf Netzwerk-Mountpoint des NAS (ist u.U. zeitweise nicht aktiv)
  bkup2dir="$backupDir2S"   # lokales Laufwerk, muss permanent erreichbar sein!
  bkup3dir="$backupDir3S"   # lokales Laufwerk, muss permanent erreichbar sein!
  pcfg3="+" # immer Verzeichnis-Anlegen versuchen
  ChkTemp=-3          # wird bei CpuLimitDef=14 ignoriert
elif [ "$pcfg" == "-umbenennen" ]; then
  # ===================== Alternativkonfiguration "nur umbenennen" (Datei mit nach Script-Regeln neu bearbeitetem Namen umbenannt) =====================
  fullCmdLine+=("$pcfg")
  shift
  durchsucheUnterverzeichnisse=0
  myscript="$finalScript" # myscript - leerer String oder zuletzt auszuführende Scriptdatei im jeweiligen Pfad "bkup{1|2}dir" oder in $HOME
  VMOD="COPY"  # Video wird kopiert. Achtung: Mediatheken reduzieren meistens auf Nicht-Standard-Auflösungen <= 1280x720, die direkt bearbeitet werden sollten.
  HevcVideoOut=1 ; AacEac3Ac3AudOut=-1 ; vcodecSpecific=0 ; h264hevcDefaultOption="" ; FAVAUD=-1 ; XLANG=deu
  destDir="$umbenennenDir"   # kein Backup, nur in aktuellem Verzeichnis bearbeiten!
  bkup1dir=""   # kein Backup, nur in aktuellem Verzeichnis bearbeiten!
  bkup2dir=""   # kein Backup, nur in aktuellem Verzeichnis bearbeiten!
  bkup3dir=""   # kein Backup, nur in aktuellem Verzeichnis bearbeiten!
  ChkTemp=$CheckTemp  # wird bei CpuLimitDef=14 ignoriert
elif [[ "$pcfg" == *@(-android|-websub|-272p|-low|-medium|-480p|-dvd|-dvdsub|-360p|-ldvd|-540p|-hdvd|-high|-720p|-full|-1080p|-max|-2160p|-copy)* ]]; then
  # ===================== Alternativkonfiguration "kleines Format für Mobileinsatz", Parameter ggf. anpassen...    =====================
  # Hinweis: Bei kleinen Zielauflösungen (<=480p) ist wegen des höhereh Audio-Platzanteils die Platzeinsparung mit HEVC zu gering und die Konvertierzeit zu hoch, darum H264!
  #          Bei 480x272 und 720x480 vergrößert jeder Audiostream die Datei um ca. 10-20%, darum habe ich hier bevorzugt  nur einen Audiostream (bevorzugt XLANG) eingestellt.
  fullCmdLine+=("$pcfg")
  shift
  HevcVideoOut=1 ; AacEac3Ac3AudOut=-1; vcodecSpecific=0 ; h264hevcDefaultOption="" ; FAVAUD=-1 ; XLANG=deu
  if [ "$pcfg" == "-android" ]; then VMOD="272p" ; FAVAUD=-2 ; pcfg2="$konvertierenAndroidSubDir" ; pcfg3="+"
  elif [ "$pcfg" == "-webvp9" ]; then VMOD="272p" ; FAVAUD=-2 ; HevcVideoOut=2 ; AacEac3Ac3AudOut=4 ; pcfg2="$konvertierenWebSubDir" ; pcfg3="+"
  elif [ "$pcfg" == "-webav1" ]; then VMOD="272p" ; FAVAUD=-2 ; HevcVideoOut=3 ; AacEac3Ac3AudOut=4 ; pcfg2="$konvertierenWebSubDir" ; pcfg3="+"
  elif [ "$pcfg" == "-dvdsub" ]; then VMOD="480p" ; FAVAUD=-2 ; pcfg2="$konvertierenDVDSubDir" ; pcfg3="+"
  elif [[ "$pcfg" == *@(-low|-272p)* ]]; then VMOD="272p"
  elif [[ "$pcfg" == *@(-ldvd|-360p)* ]]; then VMOD="360p"
  elif [[ "$pcfg" == *@(-dvd|-480p|-medium)* ]]; then VMOD="480p"
  elif [[ "$pcfg" == *@(-hdvd|-540p)* ]]; then VMOD="540p"
  elif [[ "$pcfg" == *@(-high|-720p)* ]]; then VMOD="720p"
  elif [[ "$pcfg" == *@(-full|-1080p)* ]]; then VMOD="1080p"
  elif [[ "$pcfg" == *@(-max|-2160p)* ]]; then VMOD="2160p"
  elif [ "$pcfg" == "-copy" ]; then VMOD="COPY"
  fi
  myscript="$finalScript" # myscript - leerer String oder zuletzt auszuführende Scriptdatei im jeweiligen Pfad "bkup{1|2}dir" oder in $HOME
  destDir="$konvertierenDir/"
  bkup1dir="$backupDir1K"
  bkup2dir="$backupDir2K"
  bkup3dir="$backupDir3K"
  ChkTemp=$CheckTemp
else
  # ===================== Standardkonfiguration inkl. "keinbkup" und "subdir" =====================
  # Video-Ausgabe-Bildgröße: ------------------------------------------------
  VMOD="${zielFormat,,}"  # "VMOD" bestimmt die Bildgröße, HDfull (1080p/1920x1080) ist Standard bei DVBC2/DVBS2/DVBT2, Mediatheken 720p/1280x720 oder weniger.
                      # erlaubte VMOD-Einstellungen: Android (=480x272), DVD (=480p), HDready (=720p), HDfull (=1080p, 2xHDready), UHD (=2160p, 4xHDfull) bzw. Vertikal-Varianten 
                      # Anmerkung zu Android: Die meisten Smartphones haben Displays mit mindestens 1920x1080 PX, aber bei <= 6"/~15cm zählt für mich eher Dateigröße als Auflösung!
                      # Jeder Änderungsschritt, z.B. von 1080, verändert die Konvertierzeit: ->480x272=1/8, ->720x480=480p=1/4, ->1280x720=720p=1/2. Dateigröße je Stunde Spielzeit (H264/HEVC):
                      # 480x272=150...300/---MByte, 720x480=300...600/---MByte, 1280x720=600...1000/350...500MByte, 1920x1080=1,5...2,0/0,7...1,0GByte.
                      # Bei automatischer Parametererkennung wird Hochskalieren verhindert, manuelles Hochskalieren durch Angabe beim Aufruf ist normalerweise nicht sinnvoll.
  # Audio-Stream-Auswahl: ---------------------------------------------------
  #[ $HevcVideoOut -eq 0 ] && AacEac3Ac3AudOut=-1 || AacEac3Ac3AudOut=2
  FAVAUD=-1           # -2=Suche nach erstem "XLANG"-Stream (bei Nicht-Vorhandensein Rückfallwert -1/ALLAUDIO), -1=ALLAUDIO, 0...6=gewählter Audio-Stream
  XLANG=deu           # XLANG wirkt nur mit FAVAUD=-2, Codierung entspr. ISO 639-2 - deu/ger, eng, fra/fre...
  # Leistungsbegrenzung bei erhöhter Temperatur: ----------------------------
  ChkTemp=$CheckTemp  # alle Temp/Freq-Checks an (1), ohne Startprüfung (0), reduziert (-1), nur bei hoher Belastung/Temperatur (-2), komplett aus (-3)
                      # wegen der bei HEVC erheblich längeren Konvertierzeit führen andere (auch vorübergehende) Zusatzlasten zu starker weiterer Reduzierung, zum Test weiter herabschaltbar.
  # -------------------------------------------------------------------------
  # die folgenden Pfade müssen immer erreichbar sein (möglichst auf lokaler Festplatte oder im Fall von logDir auf Ramdisk >= 10MByte):
  destDir="$konvertierenDir"  # Zielverzeichnis
  # -------------------------------------------------------------------------
  # die folgenden Pfade werden zur Sicherung genutzt, wenn sie definiert und erreichbar sind, nicht benötigte Pfade leeren... (bkup{1|2}dir=""):
  fullCmdLine+=("$pcfg")
  if [ "$pcfg" == "-keinbkup" -o "$pcfg" == "-nobkup" ]; then
    # ===================== Variante ohne Backup =====================
    shift
  else
    bkup1dir="$backupDir1K"
    bkup2dir="$backupDir2K"
    bkup3dir="$backupDir3K"
    # ===================== Variante mit Unterverzeichnis =====================
    pcfg2=""
    pcfg2_2=""
    if [ "$pcfg" == "-subdir" ] && [ "${2:0:1}" != "-" ]; then
      fullCmdLine+=("'$2'")
      p2=$2
      pcfgup=""
      if [ "${p2:0:3}" == "../" ]; then
        pcfgup="../"
      elif [ "${p2:0:4}" == "/../" ]; then
        pcfgup="../"
      fi
      [ "${p2:0:1}" == '/' ] && p2=${p2:1}
      [ "${p2:0:3}" == "../" ] && p2=${p2:3}
      if [ -e "${baseDir}Archivdisk_16TB-mp4/$p2/" -a "$bkup1dir" == "" -a "${p2:0:3}" == "000" ] && [[ "${AufrufStr3,,}" != *@(\ -pfad1|\ -bkup1|\ -nobkup1)* ]]; then
        [ "$p2" == "0000_Mediathekview" -a "$pcfgup" == "" ] && bkup1dir="" || bkup1dir="${baseDir}Archivdisk_16TB-mp4/" # meine Sonderlocke für Unterverzeichnisse "000*"
      fi
      if [ -e "${baseDir}NAS_mp4/$p2/" -a "$bkup2dir" == "" -a "${p2:0:3}" == "000" ] && [[ "${AufrufStr3,,}" != *@(\ -pfad2|\ -bkup2|\ -nobkup2)* ]]; then
        [ "$p2" == "0000_Mediathekview" -a "$pcfgup" == "" ] && bkup2dir="" || bkup2dir="${baseDir}NAS_mp4/"             # meine Sonderlocke für Unterverzeichnisse "000*"
      fi
      if [ -e "${baseDir}Lokal_mp4/$p2/" -a "$bkup3dir" == "" -a "${p2:0:3}" == "000" ] && [[ "${AufrufStr3,,}" != *@(\ -pfad3|\ -bkup3|\ -nobkup3)* ]]; then
        bkup3dir="${baseDir}Lokal_mp4/"           # meine Sonderlocke für Unterverzeichnisse "000*"
      fi
      if [ $(echo $p2 | grep -c -E '/') -ge 2 ]; then
        echo -e "${SETRT}Zielverzeichnis '$NOCOL$2$SETRT' enthält mehr als zwei Verzeichnisebenen, Abbruch"
        exit 1
      elif [ "$p2" == "" ] || [ ${#p2} -lt 1 ]; then
        pcfg2=""
        pcfg3=""
        myscript="$finalScript"
      else
        pcfg2="${p2%%/}/"
        pcfg3=${2:0:1}
        if [ "$pcfg3" != "+" -a "$pcfg3" != "/" ]; then
          #pcfg2="$p2/"
          pcfg3="+"
        #else
        #  pcfg2="${p2:1}/"
        fi
        if [ ${#3} -gt 0 -a "${3:0:1}" != "-" ]; then 
          fullCmdLine+=("'$3'")
          if [ "${3:0:1}" != "+" -a "${3:0:1}" != "/" ]; then 
            pcfg2_2="$3/"
            pcfg3_2="+"
          elif [ ${#3} -gt 1 ]; then
            pcfg2_2="${3:1}/"
            pcfg3_2="${3:0:1}"
          else
            pcfg2_2=""
          fi
          shift
        fi
        pcfg2n=$(echo $pcfg2 | sed -e 's/ /_/g;s/,//g;s/_-_/-/g;s/-_/_/g;s/_-/_/g;s/--/_/g;s/__/_/g;s/\/\//\//g')
        [ $filterZeichen -gt 0 ] && pcfg2n=$(echo $pcfg2n | zeichenersetzen)
        if [ "$pcfg2" != "$pcfg2n" ]; then
          pcfg2str="${SETBL}Ziel-Unterverzeichnis '$NOCOL$pcfg2$SETBL' umbenannt nach '$NOCOL$pcfg2n$SETBL'"
          pcfg2=$pcfg2n
        fi
        if [ "$pcfg2_2" != "" ]; then
          pcfg2_2n=$(echo $pcfg2_2 | sed -e 's/ /_/g;s/,//g;s/_-_/-/g;s/-_/_/g;s/_-/_/g;s/--/_/g;s/__/_/g;s/\/\//\//g')
          [ $filterZeichen -gt 0 ] && pcfg2_2n=$(echo $pcfg2_2n | zeichenersetzen)
          if [ "$pcfg2_2" != "$pcfg2_2n" ]; then
            pcfg2str="$pcfg2str\n${SETBL}Ziel-Unterverzeichnis '$NOCOL$pcfg2$pcfg2_2$SETBL' umbenannt nach '$NOCOL$pcfg2_2n$SETBL'"
            pcfg2_2="$pcfg2_2n"
          fi
        fi
        myscript="$finalScript"
        if [ "$bkup1dir" != "" ] && [ ${#pcfg2} -gt 0 ]; then
          if [ "$pcfgup" != "" ]; then
            bkup1dir="${bkup1dir%/}"
            bkup1dir="${bkup1dir%/*}/"
          fi
          if [ "$pcfg3" == "+" ] && [ ! -d "$bkup1dir$pcfg2" ]; then
            bkup1dirnam="$bkup1dir$pcfg2"
            bkup1dirstr="$NOCOL *  ${SETBL}Unterverzeichnis $NOCOL$bkup1dir$pcfg2$SETBL anlegen$NOCOL"
            if [ "$pcfg3_2" == "+" ] && [ ! -d "$bkup1dir$pcfg2$pcfg2_2" ]; then
              bkup1dirnam="$bkup1dir$pcfg2$pcfg2_2"
              bkup1dirstr="$bkup1dirstr\n$NOCOL *  ${SETBL}Unterverzeichnis $NOCOL$bkup1dir$pcfg2$pcfg2_2$SETBL anlegen$NOCOL"
            fi
          fi
          sync
        fi
        if [ "$bkup2dir" != "" ] && [ ${#pcfg2} -gt 0 ]; then
          if [ "$pcfgup" != "" ]; then
            bkup2dir="${bkup2dir%/}"
            bkup2dir="${bkup2dir%/*}/"
          fi
          if [ "$pcfg3" == "+" ] && [ ! -d "$bkup2dir$pcfg2$pcfg2_2" ]; then
            bkup2dirnam="$bkup2dir$pcfg2"
            bkup2dirstr="$NOCOL *  ${SETBL}Unterverzeichnis $NOCOL$bkup2dir$pcfg2$SETBL anlegen$NOCOL"
            if [ "$pcfg3_2" == "+" ] && [ ! -d "$bkup2dir$pcfg2$pcfg2_2" ]; then
              bkup2dirnam="$bkup2dir$pcfg2$pcfg2_2"
              bkup2dirstr="$bkup2dirstr\n$NOCOL *  ${SETBL}Unterverzeichnis $NOCOL$bkup2dir$pcfg2$pcfg2_2$SETBL anlegen$NOCOL"
            fi
          fi
          sync
        fi
        if [ "$bkup3dir" != "" ] && [ ${#pcfg2} -gt 0 ]; then
          if [ "$pcfgup" != "" ]; then
            bkup3dir="${bkup3dir%/}"
            bkup3dir="${bkup3dir%/*}/"
          fi
          if [ "$pcfg3" == "+" ] && [ ! -d "$bkup3dir$pcfg2$pcfg2_2" ]; then
            bkup3dirnam="$bkup3dir$pcfg2"
            bkup3dirstr="$NOCOL *  ${SETBL}Unterverzeichnis $NOCOL$bkup3dir$pcfg2$SETBL anlegen$NOCOL"
            if [ "$pcfg3_2" == "+" ] && [ ! -d "$bkup3dir$pcfg2$pcfg2_2" ]; then
              bkup3dirnam="$bkup3dir$pcfg2$pcfg2_2"
              bkup3dirstr="$bkup3dirstr\n$NOCOL *  ${SETBL}Unterverzeichnis $NOCOL$bkup3dir$pcfg2$pcfg2_2$SETBL anlegen$NOCOL"
            fi
          fi
          sync
        fi
      fi
      shift 2
    else
      pcfg2=""
      pcfg3=""
      myscript="$finalScript"
    fi
  fi
fi

[ "$backupDir1K" != "" ] && _bD1K="$(findePfad $backupDir1K)" || _bD1K=""
[ "$backupDir2K" != "" ] && _bD2K="$(findePfad $backupDir2K)" || _bD2K=""
[ "$backupDir3K" != "" ] && _bD3K="$(findePfad $backupDir3K)" || _bD3K=""
[ "$backupDir1S" != "" ] && _bD1S="$(findePfad $backupDir1S)" || _bD1S=""
[ "$backupDir2S" != "" ] && _bD2S="$(findePfad $backupDir2S)" || _bD2S=""
[ "$backupDir3S" != "" ] && _bD3S="$(findePfad $backupDir3S)" || _bD3S=""
[ "$destDir" != "" ] && _dstD="$(findePfad $destDir)" || _dstD=""
[ "$srcDir" != "" ] && _srcD="$(findePfad $srcDir)" || _srcD=""
[ "$konvertierenDir" != "" ] && _konD="$(findePfad $konvertierenDir)" || _konD=""
[ "$schneidenDir" != "" ] && _schD="$(findePfad $schneidenDir)" || _schD=""

if [ $suche_in_bkup123dir != "" ]; then
  declare -n suche_in_bkup123dir_x="$suche_in_bkup123dir"
  suche_in_bkup123dir="$suche_in_bkup123dir_x"
fi

function clrFilesSub {
  if [ -f "$1" ]; then
    rm -f $1 > /dev/null 2> /dev/null
  fi
}

function killPidSub {
  pidexists=$(ps $1 | grep -c "$2")
  if [ $pidexists -gt 0 ]; then
    kill $1
  fi
}

function clrlogfilessub {
  if [ -n $logpid ] && [ $logpid -gt 0 ]; then
    killPidSub "--pid $logpid" "xmessage"
  fi
  clrFilesSub "$logDir$p1.ffprobe-log"
  clrFilesSub "$logDir$p1.ffprobe-log1"
  clrFilesSub "$logDir$p1.ffprobe-errlog"
  clrFilesSub "$logDir$p1.ffprobe-errlog1"
  clrFilesSub "$logDir$p1.ffmpeg-tmp"
  clrFilesSub "$logDir$p1.ffmpeg-log"
  clrFilesSub "$logDir$p1.ffmpeg-errlog"
  clrFilesSub "$logDir$p1.freq-log"
  if [ -f "$logDir$p1.sh" ]; then
    killPidSub "-e" "$logDir$p1.sh"
    clrFilesSub "$logDir$p1.sh"
    clrFilesSub "${logDir}schiebe_alle_Dateien_${p1}_Sub.sh"
    clrFilesSub "${logDir}schiebe_alle_Dateien_$p1.sh"
  fi
  clrFilesSub "$logDir$p1.status"
}

function clrlogfiles {
  maxAlter=$(( 6 * 60 ))
  maxLogSizeBytes=100000
  find $logDir -mmin +$maxAlter -name "*.sh" -exec rm '{}' \;
  find $logDir -mmin +$maxAlter -name "*.freq-log" -exec rm '{}' \;
  find $logDir -mmin +$maxAlter -name "*.status" -exec rm '{}' \;
  find $logDir -mmin +$maxAlter -name "*.ffprobe-log" -exec rm '{}' \;
  find $logDir -mmin +$maxAlter -name "*.ffprobe-log1" -exec rm '{}' \;
  find $logDir -mmin +$maxAlter -name "*.ffprobe-errlog" -exec rm '{}' \;
  find $logDir -mmin +$maxAlter -name "*.ffprobe-errlog1" -exec rm '{}' \;
  find $logDir -mmin +$maxAlter -name "*.ffmpeg-log" -exec rm '{}' \;
  find $logDir -mmin +$maxAlter -name "*.ffmpeg-errlog" -exec rm '{}' \;
  find $logDir -mmin +$maxAlter -name "*.ffmpeg-tmp" -exec rm '{}' \;
  if [ $( stat -c %s ${logDir}filmschnitt.log ) -gt $maxLogSizeBytes ]; then
    linnum=$( grep -n -m 1 'Temperatur-Durchschnitt\/Maximalwert'  ${logDir}filmschnitt.log)
    linnum=${linnum%%:*}
    if [ "$linnum" != "" ] && [ $linnum -gt 1 ]; then
      if [ "$linnum" -eq "$linnum" ] 2>/dev/null; then
        sed -i "1,${linnum}d" ${logDir}filmschnitt.log
      fi
    fi
  else
    echo -n "" >> ${logDir}filmschnitt.log
  fi
  clrlogfilessub
}

# #########################################################################
# Arbeitsverzeichnisse bei Fehlen:
[ "$srcDir" == "" ] && srcDir="$(echo $(findePfad $PWD))/"
[ "$destDir" == "" ] && destDir="$srcDir"
[ "$logDir" == "" ] && logDir="$srcDir"

# #########################################################################
# Backup-Hilfsparameter nicht ändern:
bkup1script="schiebe_Dateien_nach_bkup1dir.sh"
bkup1tmpdir="bkup1tmp/"
bkup2script="schiebe_Dateien_nach_bkup2dir.sh"
bkup2tmpdir="bkup2tmp/"
bkup3script="schiebe_Dateien_nach_bkup3dir.sh"
bkup3tmpdir="bkup3tmp/"
if [ "$bkup1dir" == "" ] || [ "$bkup1tmpdir" == "" ]; then
  bkup1dir="" ; bkup1tmpdir="" ; bkup1script=""
fi
if [ "$bkup2dir" == "" ] || [ "$bkup2tmpdir" == "" ]; then
  bkup2dir="" ; bkup2tmpdir="" ; bkup2script=""
fi
if [ "$bkup3dir" == "" ] || [ "$bkup3tmpdir" == "" ]; then
  bkup3dir="" ; bkup3tmpdir="" ; bkup3script=""
fi
# !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
# !!! Achtung: Die folgenden Parameter sind risikobehaftet:
HwDecode=0          # 0=SW-Decodierung, 1=HW-Decodierung (nur mit geeigneter Grafik-HW und passenden Treibern möglich!)
                    # Risiko: HOCH, Funktion ist noch nicht geprüft, eventueller Qualitätsunterschied zwischen SW- und HW-Decodierung z.Z. unbekannt, auch Qualitäts-
                    #               unterschiede zwischen HW-Decodern/Encodern der verschiedenen Grafikkarten/-Herstellern sind möglich.
HwEncode=0          # 0=SW-Encodierung, 1=HW-Encodierung (nur mit geeigneter Grafik-HW und passenden Treibern möglich!)
                    # Risiko: HOCH, siehe HwDecode.

# #########################################################################
# die folgenden Parameter und Formeln möglichst nicht verändern:
ChkRam=0            # RAM-Prüfung an (1) oder aus (0), SWAP-Prüfung bleibt wegen kritischem Einfluss bei hoher Speicherbelastung immer an!
zeigeLog=0          # aktuelles Log während Script-Laufzeit mit xmessage in eigenem Fenster anzeigen (wird in eigenem Prozess regelmäßig aktualisiert)
updLogInterval=120  # Update-Intervall [s] für Log-Fenster mit Durchschnittsfrequenz-Info, >=60 = Intervall in Sekunden, 0 = kein Anzeige-Intervall
                    # Achtung mit "zeigeLog=1" und "updLogInterval=60" wird maximal alle 60s ein neues Meldungsfenster im Vordergrund geöffnet!
anzeigeOptimierung=0    # Video-Optimierungsvariante (s/b/o) im Zieldateinamen anzeigen
anzeigeBkupMsg=0    # 0=minimale Anzeige, 1=mit Kopier-Zwischenschritten, 2=komplette Anzeige
nameMitFpsOfs=0         # 0=Ausgabedateiname ohne FPS-/Offset-Änderungswerte, 1=Ausgabedateiname mit FPS-/Offset-Änderungswerten
bkupRm1stLvl=1      # 0=aktuell konvertierte Datei in destDir nicht löschen, 1=aktuell konvertierte Datei nach erfolgreichem Backup in destDir löschen
nameErwBitrateAll=0 # 1=Zielname nur mit Fehlermeldungen, 0=Zielname bleibt unverändert (kein Qualitätshinweis)
nameErwBitrate=1    # 1=Zielname mit Qualitätshinweis, 0=Zielname bleibt unverändert (kein Qualitätshinweis)
CpuTempHighest=95
CpuTempHigher=88
CpuTempMedium=82
CpuTempAboveNormal=75
CurrPwrLimit=33
CurrPwrLimit2=50
HiLoadLimit=90      # maximale CPU-Auslastung vor Testbeginn, wenn ChkTemp=1
RamLoadLimit=95     # maximale RAM-Nutzung vor Testbeginn, wenn ChkRam=1
SwapLoadLimit=50    # maximale SWAP-Nutzung vor Testbeginn (immer, unabhängig von ChkRam)
Laststatuslog=0     # 0=Auflistung der CPU-Last nur in bash-Fenster, 1=Auflistung der CPU-Last durch die einzelnen Tasks in $logDir$p1.ffmpeg-log
Laststatusfulllog=0 # 1=auch Prozesse <0,1% auflisten
CpuPwrKorr=5        # Leistungskorrektur (absolute Prozentpunkte, mein PC zeigt bei CpuLimitDefneu 0...5 Prozentpunkte über dem vorgegebenen Wert CpuLimitDef (mit 
                    # kurzzeitigen größeren Abweichungen in beiden Richtungen). CPU-Frequenz, Temperatur und CPU-Lastrate wird am Ende der Konvertierung angezeigt.
TimProzent=5.0      # Angabe der Toleranzspanne für die Zeitberechnung als Float-Wert [%] mit Dezimal-Trenner '.'
# Zusatzparameter für vertikale Videostreams:
# 1. Boxblur ergibt ~16:9 horizontal mit zusätzlichen breiten verschwommenen Randbereichen links und rechts
# 2. nur 16:9 vertikal
verticalFilterBlur=" -lavfi [0:v]scale=ih*16/9:-1,boxblur=luma_radius=min(h\,w)/20:luma_power=1:chroma_radius=min(cw\,ch)/20:chroma_power=1[bg];[bg][0:v]overlay=(W-w)/2:(H-h)/2,crop=h=iw*9/16 "
verticalFilterSchwarz=" -f lavfi -i color=black "   # erzeugt vertikales Video mit breiten schwarzen Rändern rechts und links

case "${VMOD,,}" in
  "2160p"|"4k"|"uhd")     VMOD="2160p" ; calcMinVbr 1 ;;
  "1080p"|"hdfull")       VMOD="1080p" ; calcMinVbr 4 ;;
  "720p"|"hdready")       VMOD="720p" ; calcMinVbr 9 ;;
  "540p"|"hdvd")          VMOD="540p" ; calcMinVbr 16 ;; # HevcVideoOut=0 ; AacEac3Ac3AudOut=2 ; vcodecSpecific=0 ;;
  "480p"|"dvd")           VMOD="480p" ; calcMinVbr 24 ;; # HevcVideoOut=0 ; AacEac3Ac3AudOut=2 ; vcodecSpecific=0 ;;
  "360p"|"ldvd")          VMOD="360p" ; calcMinVbr 40 ;; # HevcVideoOut=0 ; AacEac3Ac3AudOut=2 ; vcodecSpecific=0 ;;
  "272p"|"android"|"low") VMOD="272p" ; calcMinVbr 65 ;; # HevcVideoOut=0 ; AacEac3Ac3AudOut=3 ; vcodecSpecific=0 ;;
  "copy")                 VMOD="copy" ; calcMinVbr 65 ;;
  "v2160p"|"v4k"|"vuhd")  VMOD="v2160p" ; calcMinVbr 1 ;;
  "v1080p"|"vhdfull")     VMOD="v1080p" ; calcMinVbr 4 ;;
  "v720p"|"vhdready")     VMOD="v720p" ; calcMinVbr 9 ;;
  "v540p"|"vhdvd")        VMOD="v540p" ; calcMinVbr 16 ;; # HevcVideoOut=0 ; AacEac3Ac3AudOut=2 ; vcodecSpecific=0 ;;
  "v480p"|"vdvd")         VMOD="v480p" ; calcMinVbr 24 ;; # HevcVideoOut=0 ; AacEac3Ac3AudOut=2 ; vcodecSpecific=0 ;;
  "v360p"|"vldvd")        VMOD="v360p" ; calcMinVbr 40 ;; # HevcVideoOut=0 ; AacEac3Ac3AudOut=2 ; vcodecSpecific=0 ;;
  "v272p"|"vandroid"|"vlow") VMOD="v272p" ; calcMinVbr 65 ;; # HevcVideoOut=0 ; AacEac3Ac3AudOut=3 ; vcodecSpecific=0 ;;
  "vcopy")                VMOD="vcopy" ; calcMinVbr 65 ;;
  *)                      echo -e "${SETRT}Unbekannter VMOD=\"$VMOD\" - erlaubt: UHD (=2160p), HDfull (=1080p), HDready (=720p), HDVD (=540p), DVD (=480p), LDVD (=360p), Android (=272p), Copy" ; clrlogfilessub ; exit 1 ;;
esac
min_br=$(( $min_vbr + $(( 2 * $min_abr )) )) # Annahme, wird ggf. nachfolgend geändert
# min_br bzw. min_vbr bei HEVC später reduziert
tmpreplacestr="s/ \/ /\//g;s/ fps\,/fps\,/g;s/ tbr\,/tbr\,/g;s/ tbn\,/tbn\,/g;s/ tbc/tbc/g;s/ Hz,/Hz,/g;s/ kb\/s/kb\/s/g"
# -------------------------------------------------------------------------
enasubtitle=-1      # -1=blockiert, 0=aus - kann aktiviert werden, 1:aktiviert
                    # Die standardmäßige Verhinderung mit enasubtitle=0 kann beim Script-Aufruf mit Auswahl "-u" bzw. "Untertitel" überschrieben werden.
                    # !!! Die Übernahme von Untertitel-Streams funktioniert weiterhin nicht - für DVB Teletext (andere Untertitel-Typen konnte ich noch
                    #     nicht testen) wird eine aktuelle ffmpeg-Version (4.1.6 oder höher mit Unterstützung für die verwendeten Untertiteltypen, z.B.
                    #     dvb_teletext) benötigt, aber mit der Festlegung von "-codec:s dvb_teletext" bzw. "-codec:s libzvbi_teletextdec" als Decoder
                    #     und "-codec:s copy" als Encoder wird der vorhandene Subtitlestream nicht übernommen. 
                    #     je 3 Werte: "ffprobe-log Name" "ffprobe-errlog Dec" "ffprobe-errlog Enc"
declare -a subt1str ; subt1str=( "dvb_subtitle" "dvbsub" "dvbsub" ) 
declare -a subt2str ; subt2str=( "dvd_subtitle" "dvdsub" "dvdsub" ) 
declare -a subt3str ; subt3str=( "dvb_teletext" "libzvbi_teletextdec" "" ) #( "dvb_teletext" "libzvbi_teletextdec" "copy" ) 
declare -a subt4str ; subt4str=( "subrip" "subrip" "subrip" ) #( "subrip" "srt" "srt" ) 
declare -a subt5str ; subt5str=( "mov_text" "mov_text" "copy" ) 
declare -a subt6str ; subt6str=( "" "" "" ) 
declare -a subt7str ; subt7str=( "" "" "" ) 
declare -a subt8str ; subt8str=( "" "" "" ) 
declare -a subt9str ; subt9str=( "" "" "" ) 

function Namenfilter {
  # ===== Bei den folgenden Filtern müssen Überschneidungen vermieden werden, ggf. Reihenfolge beachten =====
  # filmschnitt-Formatinformationen (Leerzeichen, doppelte '_'/'-', VCODEC, RES) aus vorhergehenden Läufen und dahinter liegende Teile entfernen
  tmpNamenfilter=$1
  if [ -z ${tmpNamenfilter%%_[-|0-9|A-Z|a-z]*} ]; then tmpNamenfilter=${tmpNamenfilter:1}; fi
  tmpNamenfilter=$(echo $tmpNamenfilter | sed -e 's/^_//;s/,/_/g;s/!/_/g;s/?/_/g;s/_-_/-/g;s/__/_/g;s/--/-/g;s/_-/-/g;s/-_/-/g') #${tmpNamenfilter/__/_}
  tmpNamenfilter="${tmpNamenfilter/ /_}"
  tmpNamenfilter="${tmpNamenfilter/\(/_}"
  tmpNamenfilter="${tmpNamenfilter/\)/_}"
  tmpNamenfilter=$(echo $tmpNamenfilter | sed -e 's/recode_x//g;s/recode//g')
  tmpNamenfilter="${tmpNamenfilter%.m[2|4|k|p][4|t|v]}"   # .m2t/.mp4/.m4v/.mkv
  tmpNamenfilter="${tmpNamenfilter%.ts}"
  tmpNamenfilter="${tmpNamenfilter%.webm}"
  [ $anzeigeOptimierung -ne 0 ] && tmpNamenfilter=${tmpNamenfilter/_[s|b|o]_/_}
  tmpNamenfilter=$(echo $tmpNamenfilter | sed -e 's/_HEVC_/_/g;s/_H264_/_/g;s/_MPEG4_/_/g;s/_VP9_/_/g;s/_AV1_/_/g')
  #tmpNamenfilter=${tmpNamenfilter/[_|-]YouTube[_|-]/_}
  tmpNamenfilter=${tmpNamenfilter/_[1|2][0|1][6|8]0p/}  # 1080p/2160p 
  tmpNamenfilter=${tmpNamenfilter/_[3|4|6|7][2|6|8]0p/}   # 360p/480p/540p/720p
  tmpNamenfilter=$(echo $tmpNamenfilter | sed -e 's/_HEVC//g;s/_H264//g;s/_MPEG4//g;s/_VP9//g;s/_AV1//g;s/_d_/_/g;s/_[1|3][8|9][2|4]0x[1|2][0|1][6|8]0_/_/g;s/_1[0-9][0-9][0-9]x[0-9][0-9][0-9]_/_/g;s/_[1-9][0-9][0-9]x[1-9][0-9][0-9]_/_/g;s/_\.\*_\.\*_/_\.\*_/g;s/\.mp4 /\.\*/g;s/\.m4v /\.\*/g;s/\.m2t /\.\*/g;s/\.ts /\.\*/g;s/\.webm /\.\*/g;s/-$/_/')
  # Kaffeine-Namen-Aufnahmezeitfeld und dahinter liegende Teile entfernen: 20##-[#]#-[#]#_[#]#-[#]#
  tmpNamenfilter="${tmpNamenfilter%_20[0-9][0-9]-*-*_*-*_*}"
  # Mediathek-Namen-Indexfeld (aktuell und alte Variante) und dahinter liegende Teile entfernen: {_|-}########## bzw. {_|-}####-####_
  # oder Hauppauge-Aufnahme-SW-Namen-Zeitfeld und dahinter liegende Teile entfernen: ########_####
  tmpNamenfilter="${tmpNamenfilter%[_|-][0-9][0-9][0-9][0-9][_|-|0-9][0-9][0-9][0-9][0-9|_|-][0-9|_|-]*}"
  # oder Mediathek-Defaultformat -[#]###-#_###### bzw. eigene Konfig Datum/Zeit ####-##-##_##-##
  tmpNamenfilter="${tmpNamenfilter%_20[0-9][0-9]-[0-1][0-9]-[0-3][0-9]_[0-2][0-9]-[0-5][0-9]*}"
  tmpNamenfilter="${tmpNamenfilter%-[1-9][0-9][0-9][0-9]-[0-9]_[0-9][0-9][0-9][0-9][0-9][0-9]*}"
  tmpNamenfilter="${tmpNamenfilter%-[1-9][0-9][0-9]-[0-9]_[0-9][0-9][0-9][0-9][0-9][0-9]*}"
  # filmschnitt-Dauerfeld-Varianten aus vorhergehenden Läufen und dahinter liegende Teile entfernen: #h##m##s
  tmpNamenfilter="${tmpNamenfilter%[_|-][0-2][0-9]h[0-5][0-9]m[0-5][0-9]s*}"
  tmpNamenfilter="${tmpNamenfilter%[_|-][0-2][0-9]h[0-5][0-9]m[0-9]s*}"
  tmpNamenfilter="${tmpNamenfilter%[_|-][0-2][0-9]h[0-9]m[0-5][0-9]s*}"
  tmpNamenfilter="${tmpNamenfilter%[_|-][0-2][0-9]h[0-9]m[0-9]s*}"
  tmpNamenfilter="${tmpNamenfilter%[_|-][0-9]h[0-5][0-9]m[0-5][0-9]s*}"
  tmpNamenfilter="${tmpNamenfilter%[_|-][0-9]h[0-5][0-9]m[0-9]s*}"
  tmpNamenfilter="${tmpNamenfilter%[_|-][0-9]h[0-9]m[0-5][0-9]s*}"
  tmpNamenfilter="${tmpNamenfilter%[_|-][0-9]h[0-9]m[0-9]s*}"
  tmpNamenfilter="${tmpNamenfilter%[_|-][0-9][0-9][0-9]h[0-5][0-9]m[0-5][0-9]s*}"
  tmpNamenfilter="${tmpNamenfilter%[_|-][0-9][0-9][0-9]h[0-5][0-9]m[0-9]s*}"
  tmpNamenfilter="${tmpNamenfilter%[_|-][0-9][0-9][0-9]h[0-9]m[0-5][0-9]s*}"
  tmpNamenfilter="${tmpNamenfilter%[_|-][0-9][0-9][0-9]h[0-9]m[0-9]s*}"
  tmpNamenfilter="${tmpNamenfilter%[_|-][0-9][0-9]h[0-5][0-9]m[0-5][0-9]s*}"
  tmpNamenfilter="${tmpNamenfilter%[_|-][0-9][0-9]h[0-5][0-9]m[0-9]s*}"
  tmpNamenfilter="${tmpNamenfilter%[_|-][0-9][0-9]h[0-9]m[0-5][0-9]s*}"
  tmpNamenfilter="${tmpNamenfilter%[_|-][0-9][0-9]h[0-9]m[0-9]s*}"
  tmpNamenfilter="${tmpNamenfilter%[_|-][0-9][0-9][0-9]m[0-5][0-9]s*}"
  tmpNamenfilter="${tmpNamenfilter%[_|-][0-9][0-9][0-9]m[0-9]s*}"
  tmpNamenfilter="${tmpNamenfilter%[_|-][0-9][0-9]m[0-5][0-9]s*}"
  tmpNamenfilter="${tmpNamenfilter%[_|-][0-9][0-9]m[0-9]s*}"
  tmpNamenfilter="${tmpNamenfilter%[_|-][0-9]m[0-5][0-9]s*}"
  tmpNamenfilter="${tmpNamenfilter%[_|-][0-9]m[0-9]s*}"
  tmpNamenfilter="${tmpNamenfilter%[_|-][0-9][0-9][0-9]Min*}"
  tmpNamenfilter="${tmpNamenfilter%[_|-][0-9][0-9]Min*}"
  tmpNamenfilter="${tmpNamenfilter%[_|-][0-9]Min*}"
  tmpNamenfilter="${tmpNamenfilter%vlc-record-20[1-9][1-9]-[0-1][0-9]-[0-3][0-9]}"
  tmpNamenfilter="${tmpNamenfilter%[_|-]recoded_*}"
  tmpNamenfilter="${tmpNamenfilter%_a_[0-9]-[0-9]*}"
  tmpNamenfilter="${tmpNamenfilter%_a_[0-9][0-9]-[0-9]*}"
  tmpNamenfilter="${tmpNamenfilter%_e_[0-9]-*}"
  tmpNamenfilter="${tmpNamenfilter%_e_[0-9][0-9]-[0-9]*}"
  #tmpNamenfilter="${tmpNamenfilter%[_|-]Staffel[1-9]*}"
  #tmpNamenfilter="${tmpNamenfilter%[_|-][1-9]bis[2-9]von*}" ; tmpNamenfilter="${tmpNamenfilter%[_|-][1-9]bis[0-9][2-9]von*}" ; tmpNamenfilter="${tmpNamenfilter%[_|-][0-9][1-9]bis[0-9][2-9]von*}"
  #tmpNamenfilter="${tmpNamenfilter%[_|-][1-9]bis[2-9]*}" ; tmpNamenfilter="${tmpNamenfilter%[_|-][1-9]und[2-9]*}" ; tmpNamenfilter="${tmpNamenfilter%[_|-][1-9][_|-][0-9|_|-]*}"
  if [ $filterZeichen -gt 0 ]; then tmpNamenfilter=$(echo $tmpNamenfilter | zeichenersetzen); fi
  printf "$tmpNamenfilter"
}

# #########################################################################
# Grafik-/HwDecode-/HwEncode-Fähigkeit prüfen:
# !!! Die folgenden Werte und die darauf zugreifenden Funktionen sind noch nicht getestet bzw. in Bearbeitung !!!
# !!! HwDecode/HwEncode ist z.Z. ungetestet, 
#     genannte mögliche Kommandos entnommen aus aus Dokumenten zu AMD/INTEL/NVIDIA HW-Decode/Encode-Unterstützung in ffmpeg
#     AMD:    Unterstützung in FFMPEG noch nicht enthalten, Linux + AMD-Grafikkartentreiber sind z.Z. unklar!
#     INTEL:  Codec für Encode+Decode ist in aktuellem FFMPEG enthalten, Linux + INTEL-Grafikkartentreiber müsste das auch unterstützen.
#     NVIDIA: Codec für Encode+Decode ist in aktuellem FFMPEG enthalten, Linux + NVIDIA-Grafikkartentreiber müsste das auch unterstützen.
# Alle:   Nach "ffmpeg" optional HwTranscodePre=" -hwaccel_device <num 0 oder 1 ...> " für Auswahl aus mehreren Grafikkarten
# (AMD:    HwTranscodePre="$HwTranscodePre -hwaccel amf "
#          HwDecodeCmd="hevc_amf" oder "h264_amf" ersetzt bei Einstellung "vin" das SW-Codec "hevc" bzw. "h264"
#          HwEncodeCmd="hevc_amf" oder "h264_amf", ersetzt bei Einstellung "vout" das SW-Codec "hevc" bzw. "h264")
# INTEL:  HwTranscodePre="$HwTranscodePre -hwaccel qsv "
#         HwDecodeCmd="hevc_qsv" oder "h264_qsv" ersetzt bei Einstellung "vin" das SW-Codec "hevc" bzw. "h264"
#         HwEncodeCmd="hevc_qsv" oder "h264_qsv" ersetzt bei Einstellung "vout" das SW-Codec "hevc" bzw. "h264" 
# NVIDIA: HwTranscodePre="$HwTranscodePre -hwaccel cuvid "
#         HwDecodeCmd="hevc_cuvid" oder "h264_cuvid" ersetzt bei Einstellung "vin" das SW-Codec "hevc" bzw. "h264" 
#         HwEncodeCmd="hevc_nvenc" oder "h264_nvenc" ersetzt bei Einstellung "vout" das SW-Codec "hevc" bzw. "h264"
if [ $HwEncode -ne 0 ] || [ $HwDecode -ne 0 ]; then
  # Suche nach Grafik-HW mit HW-Decode-/Encode-Fähigkeit
  isXxxVga=$(lspci -vnn 2>/dev/null | grep "VGA compatible controller" -A 1)
  isAmdVga=$(echo "$isXxxVga" | grep -i -c "AMD")
  isIntelVga=$(echo "$isXxxVga" | grep -i -c "INTEL")
  isNvidiaVga=$(echo "$isXxxVga" | grep -i -c "NVIDIA")
  isSupportedVga=$(( $isAtiVga + $isIntelVga + $isNvidiaVga ))
  isGlxMesaDri=$(glxinfo  2>/dev/null | grep -c "OpenGL renderer string: Mesa DRI")                            # ja, wenn isXxxDrv=ja
  isXxxVgaDrv=$(lshw -numeric -C display 2>/dev/null)
  isAmdVgaDrv=0
  isAmdVgaDrv=$(( $isAmdVgaDrv + $(echo "$isXxxVgaDrv" | grep -i -c "configuration: driver=amd") ))
  isIntelVgaDrv=0
  isIntelVgaDrv=$(( $isIntelVgaDrv + $(echo "$isXxxVgaDrv" | grep -i -c "configuration: driver=i965") ))
  isNvidiaVgaDrv=0
  isNvidiaVgaDrv=$(( $isNvidiaVgaDrv + $(echo "$isXxxVgaDrv" | grep -i -c "configuration: driver=nouveau") ))
  isNvidiaVgaDrv=$(( $isNvidiaVgaDrv + $(echo "$isXxxVgaDrv" | grep -i -c "configuration: driver=nvidia") ))
  isSupportedVgaDrv=$(( $isAmdVgaDrv + $isIntelVgaDrv + $isNvidiaVgaDrv ))
  if [ $isSupportedVga -gt 0 ] && [ $isSupportedVgaDrv -gt 0 ]; then
    if [ $isAmdVga -gt 0 ] && [ $isAmdVgaDrv -gt 0 ]; then echo -e "$SETGND${SETBGOK}Mit HW-Beschleunigung (Render/Decode/Encode) kompatible Grafikkarte und Driver von ATI gefunden$NOCOL"
    elif [ $isIntelVga -gt 0 ] && [ $isIntelVgaDrv -gt 0 ]; then echo -e "$SETGND${SETBGOK}Mit HW-Beschleunigung (Render/Decode/Encode) kompatible Grafikkarte und Driver von INTEL gefunden$NOCOL"
    elif [ $isNvidiaVga -gt 0 ] && [ $isNvidiaVgaDrv -gt 0 ]; then echo -e "$SETGND${SETBGOK}Mit HW-Beschleunigung (Render/Decode/Encode) kompatible Grafikkarte und Driver von NVIDIA gefunden$NOCOL"
    else echo -e "$SETRTD${SETBGOK}Keine für HW-Beschleunigung (Render/Decode/Encode) geeignete Grafikkarte / Driver gefunden$NOCOL"; isSupportedVga=0; isSupportedVgaDrv=0
    fi
  else echo -e "$SETRTD${SETBGOK}Keine für HW-Beschleunigung (Render/Decode/Encode) geeignete Grafikkarte / Driver gefunden$NOCOL"; isSupportedVga=0; isSupportedVgaDrv=0
  fi
  if [ $isSupportedVga -gt 0 ] && [ $isSupportedVgaDrv -gt 0 ]; then
    if [ $isGlxMesaDri -gt 0 ]; then echo -e "$SETGND${SETBGOK}Unterstützung von HW-Beschleunigung (Render/Decode/Encode) mit geeigneter Grafikkarte und aktivem Mesa DRI-Treiber möglich$NOCOL" # OpenGL... Mesa bedeutet mögliche Unterstützung von HW-Beschleunigung (Render/Decode/Encode).
    fi
  else HwDecode=0; HwEncode=0
  fi
  if [ $HwEncode -eq 0 ] && [ $HwDecode -eq 0 ]; then
    HwTranscodePre=""
    HwDecodeCmdHevc="hevc"
    HwDecodeCmdH264="h264"
    HwDecodeCmdVP9="libvpx_vp9"
    HwEncodeCmdHevc="hevc"
    HwEncodeCmdH264="h264"
    HwEncodeCmdVP9="libvpx_vp9"
  else
    if [ $HwEncode -ne 0 ] || [ $HwDecode -ne 0 ]; then
      if  [ $isSupportedVga -gt 0 ] && [ $isSupportedVgaDrv -gt 0 ]; then
        HwTranscodePre=" -hwaccel_device 0 "
        if [ $isNvidiaVga -gt 0 ] && [ $isNvidiaVgaDrv -gt 0 ]; then
          HwTranscodePre="$HwTranscodePre  -hwaccel cuvid "
          if [ $HwDecode -ne 0 ]; then
            HwDecodeCmdHevc="hevc_cuvid"
            HwDecodeCmdH264="h264_cuvid"
            HwDecodeCmdVP9="vp9_cuvid"
          fi
          if [ $HwEncode -ne 0 ]; then
            HwEncodeCmdHevc="hevc_vaapi" #"hevc_nvenc"
            HwEncodeCmdH264="h264_vaapi" #"h264_nvenc"
            HwEncodeCmdVP9="vp9_vaapi" #"libvpx_vp9"
          fi
        elif [ $isIntelVga -gt 0 ] && [ $isIntelVgaDrv -gt 0 ]; then
          HwTranscodePre="$HwTranscodePre  -hwaccel qsv "
          if [ $HwDecode -ne 0 ]; then
            HwDecodeCmdHevc="hevc_qsv"
            HwDecodeCmdH264="h264_qsv"
            HwDecodeCmdVP9="libvpx_vp9"
          fi
          if [ $HwEncode -ne 0 ]; then
            HwEncodeCmdHevc="hevc_qsv"
            HwEncodeCmdH264="h264_qsv"
            HwEncodeCmdVP9="libvpx_vp9"
          fi
        elif [ $isAmdVga -gt 0 ] && [ $isAmdVgaDrv -gt 0 ]; then
          HwTranscodePre="$HwTranscodePre  -hwaccel amf "
          if [ $HwDecode -ne 0 ]; then
            HwDecodeCmdHevc="hevc_amf"
            HwDecodeCmdH264="h264_amf"
            HwDecodeCmdVP9="libvpx_vp9"
          fi
          if [ $HwEncode -ne 0 ]; then
            HwEncodeCmdHevc="hevc_amf"
            HwEncodeCmdH264="h264_amf"
            HwEncodeCmdVP9="libvpx_vp9"
          fi
        fi
      else
        HwDecode=0; HwEncode=0
        HwTranscodePre=""
        HwDecodeCmdHevc="hevc"
        HwDecodeCmdH264="h264"
        HwDecodeCmdVP9="libvpx_vp9"
        HwEncodeCmdHevc="hevc"
        HwEncodeCmdH264="h264"
        HwEncodeCmdVP9="libvpx_vp9"
      fi
    fi
  fi
else
  HwTranscodePre=""
  HwDecodeCmdHevc="hevc"
  HwDecodeCmdH264="h264"
  HwDecodeCmdVP9="libvpx_vp9"
  HwEncodeCmdHevc="hevc"
  HwEncodeCmdH264="h264"
  HwEncodeCmdVP9="libvpx_vp9"
fi

if [ $updLogInterval -gt 0 ] && [ $updLogInterval -lt 60 ]; then
  updLogInterval=60
fi

# #########################################################################
# weitere Variablen, Parameter, Formeln und Funktionen: 
x10prozent=$(echo "scale=0 ; ( ( ( $TimProzent * 100 ) + 5 ) / 10 )" | bc) #scale=0 reicht
CpuLimitInit=$CpuLimitDef
CpuFreqMax=$(lscpu|grep "CPU max MHz:"|cut -c22-25)
MeasIter=16         # Anzahl der top-Aufrufe zur Leistungsermittlung/Anzahl der Frequenzmessungen
MeasDelay="0.5"     # Wartezeit zwischen Top-Aufrufen bzw. Frequenzmessungen
chkformatA="$USER"  # Gesamtleistung wird aus $USER und root ermittelt
MemMinimumSizeGb=4  # [GB] Wert benötigter RAM-/Swap-Bereich bei eventueller kurzzeitiger Verdrängung von ffmpeg
RetrySec=60
TempLimit=100
TempDelayLimit=$CpuTempMedium
FreqLimit=0
redlvl=0
NumCPUs=$vencNumCPUs
NumCPUsReal=$NumCPUs
MemMinimumSizeKb=$(( $MemMinimumSizeGb * 1048576 ))
prioselect=18   # 0=Standardpriorität, nicht verwenden, andere Video-Prozesse könnten unterbrochen werden, 19=niedrigste Priorität 
loglevel="fatal" # bei HEVC-Zielformat wird 'warning' eingestellt, H264-Quellen melden oft viele Warnungen, die bei der Konvertierung anscheinend folgenlos sind.
loglevel_hevc="warning"
loglvl="-loglevel $loglevel -stats" # "-loglevel info (default)/warning/error/panic/quiet -stats" info is default
forcemp4=1
ATCMD=
ETCMD=
OFSC=
IFPSC=
QFPSC=
QRESC=
FPSD=2500
POFS=30000
MOFS=-$POFS
ASPDEF="16:9"
subtitlexcmd=""
convstart=""
tmpFlagDuplikatsuche=0
tmpStr2=
tmpStrX=
tmpStrQ=
tmpStrI=
tmpStrU=
tmpStrA=
tmpStrK=
index=0
tmpLen1a=0
tmpLen1b=0
tmpLen2a=0
tmpLen2b=0
insize=0

function map_stream_0_bis_x {
  negtxt=
  STRTYP="$1"
  if [ "${pcfg,,}" != "-umbenennen" ] && [ $anzeigeBkupMsg -ge 1 ]; then echo -e -n "${SETBL}Stream $STRIDX ($STRTYP): $NOCOL$avsIO "; fi
  case "$STRIDX" in
    0)  STRUSE=$MAP0 ;;
    1)  STRUSE=$MAP1 ;;
    2)  STRUSE=$MAP2 ;;
    3)  STRUSE=$MAP3 ;;
    4)  STRUSE=$MAP4 ;;
    5)  STRUSE=$MAP5 ;;
    6)  STRUSE=$MAP6 ;;
    7)  STRUSE=$MAP7 ;;
    8)  STRUSE=$MAP8 ;;
    9)  STRUSE=$MAP9 ;;
    10) STRUSE=$MAP10 ;;
  esac
  if [ "" == "$STRUSE" ]; then
    STRIDX2=-1
    case "$STRTYP" in
      "Video")    STRIDX2=0
                  VSTRN=$STRIDX 
                  ;;
      "Audio")    if [ -n $p4 ] && [ "$p4" != "" ]; then STRIDX2=1; splitavsrc="-i $p1 "
                  elif [ -n $p5 ] && [ "$p5" != "" ]; then STRIDX2=1; splitavsrc="-i $p1 "
                  else STRIDX2=0; splitavsrc=
                  fi
                  if [ 0 -gt $ASTR1N ]; then ASTR1N=$STRIDX
                  elif [ 0 -gt $ASTR2N ]; then ASTR2N=$STRIDX
                  elif [ 0 -gt $ASTR3N ]; then ASTR3N=$STRIDX
                  elif [ 0 -gt $ASTR4N ]; then ASTR4N=$STRIDX
                  elif [ 0 -gt $ASTR5N ]; then ASTR5N=$STRIDX
                  elif [ 0 -gt $ASTR6N ]; then ASTR6N=$STRIDX
                  fi
                  ;;
      "Subtitle") if [ 0 -le $enasubtitle ]; then STRIDX2=-1; negtxt=" nicht"
                  else
                    # ??? evtl. weiteren Shift-Parameter für Subtitle zufügen? 
                    #     $p4 um zweiten optionalen Parameter erweitern, bei Fehlen ersten Wert kopieren
                    #if [ -n $px5 ] && [ "$px5" != "" ]; then
                    #  STRIDX2=2
                    #  infile3="-i $p1 "
                    #else
                      STRIDX2=0
                      infile3=""
                    #fi
                  fi
                  if [ 0 -gt $SSTR1N ]; then SSTR1N=$STRIDX #2
                  elif [ 0 -gt $SSTR2N ]; then SSTR2N=$STRIDX #2
                  elif [ 0 -gt $SSTR3N ]; then SSTR3N=$STRIDX #2
                  elif [ 0 -gt $SSTR4N ]; then SSTR4N=$STRIDX #2
                  fi
                  ;;
      *)          STRIDX2=-1; negtxt=" nicht"
                  ;;
    esac
    if [ $STRIDX2 -ge 0 ]; then
      tmpstr=$(cat $logDir$p1.ffprobe-errlog | grep "Stream #$STRIDX2:$STRIDX" | sed -e "s/    Stream //g;$tmpreplacestr")
      if [ "$negtxt" == " nicht" ]; then tmpcol="$SETRT"; streamakt="übernommen"
      else tmpcol="$SETGN"
      fi
      if [ "${pcfg,,}" != "-umbenennen" ]; then echo -e "$NOCOL$tmpstr ${tmpcol}wird$negtxt $streamakt$NOCOL"
      fi
    fi
    XSTRI=""
    XSTRIS=""
    XSTR=""
    XSTRS=""
    if [ 0 -le $STRIDX2 ]; then XSTR=" -map $STRIDX2:$STRIDX "
    fi
    if [ "$STRTYP" == "Video" ]; then
      if [ "$strvin" != "" ]; then XSTRS=" $strvin ($inresdtxt) ⇒ "; fi
      if [ "$voutd" != "" ]; then XSTRS=" ${XSTRS## } ${voutc%% } ($QRESD) "; XSTR=" $XSTR $voutc "
      fi
      strvin=
    elif [ "$STRTYP" == "Audio" ]; then
      if [ "$strain" != "" ]; then
        XSTRS=" $strain ⇒ "
        if [ "$aoutd" == "copy" ]; then 
          if [ "$strain" == "aaclatm_lc" ] || [ "$strain" == "aaclatm_heaac" ] || [ "$strain" == "aaclatm_lc_heaac" ]; then istAacLatmLc=1; else  istAacLatmLc=2; fi; 
        fi
      fi
      if [ "$aoutd" != "" ]; then XSTRS=" ${XSTRS## } ${aoutc%% } "; XSTR=" $XSTR $aoutc "
      fi
      strain=
    elif [ "$STRTYP" == "Subtitle" ]; then
      if [ "$XSTRDS" != "" ] || [ "$XSTRES" != "" ]; then XSTRS="${XSTRS## }"; XSTRS=" ${XSTRDS## } ${XSTRS%% }  ${XSTRES%% } "
      fi
      XSTRDS=
      XSTRES=
    fi
    case "$STRIDX" in
      0)  MAPI0=$XSTRI ; MAPI0S=$XSTRIS ; MAP0=$XSTR ; MAP0S=$XSTRS ; MAP0T=$STRTYP ;;
      1)  MAPI1=$XSTRI ; MAPI1S=$XSTRIS ; MAP1=$XSTR ; MAP1S=$XSTRS ; MAP1T=$STRTYP ;; 
      2)  MAPI2=$XSTRI ; MAPI2S=$XSTRIS ; MAP2=$XSTR ; MAP2S=$XSTRS ; MAP2T=$STRTYP ;;
      3)  MAPI3=$XSTRI ; MAPI3S=$XSTRIS ; MAP3=$XSTR ; MAP3S=$XSTRS ; MAP3T=$STRTYP ;;
      4)  MAPI4=$XSTRI ; MAPI4S=$XSTRIS ; MAP4=$XSTR ; MAP4S=$XSTRS ; MAP4T=$STRTYP ;;
      5)  MAPI5=$XSTRI ; MAPI5S=$XSTRIS ; MAP5=$XSTR ; MAP5S=$XSTRS ; MAP5T=$STRTYP ;;
      6)  MAPI6=$XSTRI ; MAPI6S=$XSTRIS ; MAP6=$XSTR ; MAP6S=$XSTRS ; MAP6T=$STRTYP ;;
      7)  MAPI7=$XSTRI ; MAPI7S=$XSTRIS ; MAP7=$XSTR ; MAP7S=$XSTRS ; MAP7T=$STRTYP ;;
      8)  MAPI8=$XSTRI ; MAPI8S=$XSTRIS ; MAP8=$XSTR ; MAP8S=$XSTRS ; MAP8T=$STRTYP ;;
      9)  MAPI9=$XSTRI ; MAPI9S=$XSTRIS ; MAP9=$XSTR ; MAP9S=$XSTRS ; MAP9T=$STRTYP ;;
      10) MAPI10=$XSTRI ; MAPI10S=$XSTRIS ; MAP10=$XSTR ; MAP10S=$XSTRS ; MAP10T=$STRTYP ;;
    esac
  fi
  STRTYP=""
} 

function videodef {
  QRESC=
}

function aindef {
  aind=
  ainc=
}

function ainxxx {
  aind="$1"
  ainc="-acodec $1 "
}

function ainaaclatm {
  ainxxx "aac_latm"    # 12.05.2022: Verursacht beim Lesen+Kopieren einiger DVBT2-/Mediathek-Streams (AAC-LATM-LC) von ARTE und ARD Fehler
  #ainxxx "libfdk_aac" #             Verursacht noch mehr Fehler!
}

function ainaac {
  ainxxx "aac_latm"    # 12.05.2022: Verursacht beim Lesen+Kopieren einiger DVBT2-/Mediathek-Streams (AAC-LATM-LC) von ARTE und ARD Fehler
  #[ "$CPUtyp" == "ARM" ] && ainxxx "aac" "" || ainxxx "libfdk_aac"
}

function aineac3 {
  ainxxx "eac3"
}

function ainac3 {
  ainxxx "ac3_fixed"
}

function ainvorbis {
  ainxxx "libvorbis"
}

function ainmp3 {
  ainxxx "mp3"
}

function ainmp2 {
  ainxxx "mp2"
}

function ainopus {
  ainxxx "libopus"
}

function aoutxxx {
  aoutd="$1"
  aoutc="$2-acodec $1 "
  numastreams=$(( $numastreams + 1 ))
  if [ "$aoutd" == "copy" ]; then
    tmpcol="$SETGN"
    streamakt="kopiert"
  else
    tmpcol="$SETOR"
    streamakt="konvertiert"
  fi
  avsIO="$tmpcol$aind ⇒ $aoutd "
}

function aoutcp {
  aoutxxx "copy" ""
}

function aouteac3 {
  aoutxxx "eac3" ""
}

function aoutac3 {
  aoutxxx "ac3_fixed" ""
}

function aoutaac {
  [ "$CPUtyp" == "ARM" ] && aoutxxx "aac" "" || aoutxxx "libfdk_aac" ""
}

function aoutmp3 {
  aoutxxx "mp3" "-ac 2 "
}

function aoutopus {
  aoutxxx "libopus" ""
}

function vindef {
  vind=
  vinc=
}

function vinxxx {
  vind="$1"
  vinc="-vcodec $1 "
}

function vinhevc {
  if [ $HwDecode -eq 1 ] && [ -n $HwDecodeCmdHevc ]; then
    vinxxx "$HwDecodeCmdHevc"
  else
    vinxxx "hevc"
  fi
}

function vinh264 {
  if [ $HwDecode -eq 1 ] && [ -n $HwDecodeCmdH264 ]; then
    vinxxx "$HwDecodeCmdH264"
  else
    vinxxx "h264"
  fi
}

function vinmpeg4 {
  vinxxx "mpeg4"
}

function vinvp8 {
  vinxxx "libvpx"
}

function vinvp9 {
  vinxxx "libvpx-vp9"
}

function vinav1 {
  vinxxx "libaom-av1"
  #vinxxx "libdav1d"
}

function vinmpeg {
  vinxxx "mpeg2video"
}

function voutdef {
  voutd=
  voutc=
}

function voutxxx {
  voutd="$1"
  voutc=" $2 -vcodec $1 $3 "
  if [ "$voutd" == "copy" ]; then
    tmpcol="$SETGN"
    streamakt="kopiert"
  else
    tmpcol="$SETOR"
    streamakt="konvertiert"
  fi
  avsIO="$tmpcol$vind ⇒ $voutd "
}

function voutcp {
  voutxxx "copy" "" ""
}

function vouthevc {
  if [ $HwEncode -eq 1 ] && [ -n $HwEncodeCmdHevc ]; then
    voutxxx "$HwEncodeCmdHevc"
  else
    if [ "$vind" == "hevc" ] && [ "$inresdtxt" == "$QRESD" ] && [ "QRESD" != "" ]; then voutxxx "copy" "" ""
    elif [ "${VMOD,,}" == "copy" ] || [ "${VMOD,,}" == "vcopy" ]; then voutxxx "copy" "" ""
    else
      #  -bsf:v "filter_units=remove_types=6" # lt. Internet-Recherche bei "Closed Caption" als Abhilfe. Ist das auch bei HEVC nötig?
      if [ "$h264hevcDefaultOption" != "" ]; then
        if [ "$h264hevcDefaultOption" != "medium" ]; then
          [ $closedcaptionkorrektur -eq 0 ] && h264hevcDefaultOption="-preset $h264hevcDefaultOption" || h264hevcDefaultOption="-preset $h264hevcDefaultOption -bsf:v \\\"filter_units=remove_types=6\\\"" 
          # ??? Ursache für Verhinderung der AVstream-Idx-Korrektur bei einien Mediathek-Aufzeichnungen?
          # ??? Korrektur funktionierte bis Sommer 2022, seit Herbst Einzelfälle mit V als 2. Stream!
        else
          h264hevcDefaultOption=""
        fi
      else
        [ $vcodecSpecific -eq 1 ] && h264hevcDefaultOption="" || h264hevcDefaultOption=" -b:v $bit_rate -minrate $min_rate -maxrate $max_rate"
      fi
      if [ $vcodecSpecific -eq 1 ]; then
        if [ "$h264hevcDefaultOption" == "" ]; then h264hevcDefaultOption="medium"
        fi
        if [ "$QRESD" == "1280x720" ]; then crfval=26 ; ctuval=32 ; qcompval=0.7 # ; tuval=8
        elif [ "$QRESD" == "1920x1080" ]; then crfval=26 ; ctuval=64 ; qcompval=0.8 # ; tuval=8
        elif [ "$QRESD" == "3840x2160" ]; then crfval=26 ; ctuval=64 ; qcompval=0.8 # ; tuval=8
        else crfval=28 ; ctuval=32 ; qcompval=0.6 # ; tuval=8
        fi
        voutxxx "hevc" "" "-preset $h264hevcDefaultOption -x265-params log-level=$loglevel_hevc:crf=$crfval:ctu=$ctuval:qcomp=$qcompval $pix_fmt " #:log-level=$loglevel:tu=$tuval
      else voutxxx "hevc" "" "$h264hevcDefaultOption -x265-params log-level=$loglevel_hevc $pix_fmt "  #:log-level=$loglevel
      fi
    fi
  fi
}

function vouth264 {
  if [ $HwEncode -eq 1 ] && [ -n $HwEncodeCmdH264 ]; then voutxxx "$HwEncodeCmdH264"
  else
    if [ "$vind" == "h264" -o "$vind" == "mpeg4" ] && [ "$inresdtxt" == "$QRESD" ] && [ "QRESD" != "" ]; then voutxxx "copy" "" ""
    else
      # mehrmals wegen "Closed Caption" Probleme beim Konvertierversuch von Mediathekdateien... avidemux nützt nichts, Ergebnis wird dann größer!
      #  -bsf:v "filter_units=remove_types=6" # lt. Internet-Recherche bei "Closed Caption" als Abhilfe.
      if [ "$h264hevcDefaultOption" != "" ]; then
        #[ "$h264hevcDefaultOption" != "medium" ] && 
        h264hevcDefaultOption="-preset $h264hevcDefaultOption -bsf:v \\\"filter_units=remove_types=6\\\"" 
        #|| h264hevcDefaultOption=""
      else
        [ $vcodecSpecific -eq 1 ] && h264hevcDefaultOption="" || h264hevcDefaultOption=" -b:v $bit_rate -minrate $min_rate -maxrate $max_rate"
      fi
      if [ $vcodecSpecific -eq 1 ]; then
        if [ "$h264hevcDefaultOption" == "" ]; then h264hevcDefaultOption="medium"
        fi
        if [ "$QRESD" == "1280x720" ]; then crfval=21 ; qcompval=0.7
        elif [ "$QRESD" == "1920x1080" ]; then crfval=21 ; qcompval=0.8
        elif [ "$QRESD" == "3840x2160" ]; then crfval=21 ; qcompval=0.8
        else crfval=23 ; qcompval=0.6
        fi
        voutxxx "h264" "" "-preset $h264hevcDefaultOption -x264opts crf=$crfval:qcomp=$qcompval $pix_fmt "
      else voutxxx "h264" "" "$h264hevcDefaultOption $pix_fmt "
      fi
    fi
  fi
}

function voutvp9 {
  #if [ $HwEncode -eq 1 ] && [ -n $HwEncodeCmdVP9 ]; then voutxxx "$HwEncodeCmdVP9"
  #else
    if [ "$vind" == "vp9" ] && [ "$inresdtxt" == "$QRESD" ] && [ "$QRESD" != "" ]; then voutxxx "copy" "" ""
    else
      #  -bsf:v "filter_units=remove_types=6" # lt. Internet-Recherche bei "Closed Caption" als Abhilfe. Ist das auch bei VP9 nötig?
      echo -e "${SETRT}Achtung: Encoder 'libvpx-vp9' ist relativ langsam (2...5x verglichen mit dem HEVC-Encoder) und erlaubt nur 8-Bit-Codierung, z.B. YUV420P8.$NOCOL"
      # "-quality good -cpu-used 0" bringt brauchbare Qualität (etwas schlechter als HEVC/H264) bei stärkerer Komprimierung (~ 75% HEVC), ist aber sehr langsam (~ 6...8 von HEVC), 
      # "-quality good -cpu-used 1" ist etwas weniger langsam (1,5...2x), aber die Qualität schon deutlich schlechter, 
      # "-quality good -cpu-used 2" ist noch mal 1,5x schneller mit noch etwas reduzierterer Qualität, das ist aber immer noch nur 1/2...1/3 Geschwindigkeit von HEVC bei ca. 80% der Dateigröße! 
      # Bei Werten -cpu-used >= 3 ... 8 wird die Qualität deutlich schlechter, erreicht aber mit -cpu-used 8 fast (~ 90%) die HEVC-Geschwindigkeit.
      if [ $vcodecSpecific -eq 1 ]; then voutxxx "libvpx-vp9" "" "-threads $vencNumCPUs $h264hevcDefaultOption " #-b:v $bit_rate -minrate $min_rate -maxrate $max_rate" #-slices 2 -auto-alt-ref 1 $pix_fmt -bsf:v \\\"filter_units=remove_types=6\\\"
      else voutxxx "libvpx-vp9" "" "-threads $vencNumCPUs" #  $pix_fmt
      fi
    fi
  #fi
}

# Die Verwendung von voutav1 ist trotz der lt. Tests gegenüber vp9 besseren Qualität und ca. 10-15% besseren Kompression nicht sinnvoll, 
# da z.Z. die Konvertierzeit noch viel länger ist (lt. Tests ähnlich HEVC "veryslow" oder sogar noch langsamer) 
function voutav1 {
  #if [ $HwEncode -eq 1 ] && [ -n $HwEncodeCmdAV1 ]; then voutxxx "$HwEncodeCmdVP9"
  #else
    if [ "$vind" == "av1" ] && [ "$inresdtxt" == "$QRESD" ] && [ "$QRESD" != "" ]; then voutxxx "copy" "" ""
    else
      #  -bsf:v "filter_units=remove_types=6" # lt. Internet-Recherche bei "Closed Caption" als Abhilfe. Ist das auch bei AV1 nötig?
      echo -e "${SETRT}Achtung: Encoder 'libaom-av1' ist z.Z. noch extrem langsam (30...>100x verglichen mit dem HEVC-Encoder) und damit noch nicht zur Konvertierung geeignet!$NOCOL"
      # Die Konvertierqualität erreicht fast HEVC bei allerdings 10...20-facher Konvertierdauer trotz einiger Optimierungen für Multithreading.
      if [ $vcodecSpecific -eq 1 ]; then
        #voutxxx "libaom-av1" "-strict -2" "-threads $vencNumCPUs $h264hevcDefaultOption -b:v $bit_rate -minrate $min_rate -maxrate $max_rate"
        voutxxx "libaom-av1" "-strict -2" "-crf 30 -b:v 0 $h264hevcDefaultOption " #-bsf:v \\\"filter_units=remove_types=6\\\" $pix_fmt"
      else voutxxx "libaom-av1" "-strict -2" "-threads $vencNumCPUs $pix_fmt"
      fi
    fi
  #fi
}

function getlangSub {
  for STRIDX in {0..6}
  do
    for STRIDX2 in {0..2}
    do
      if [ 0 -gt $ASEL ];then
        LANGSTR=$(cat $logDir$p1.ffprobe-errlog | grep -i -E -c "Stream #$STRIDX2:$STRIDX\($1\): Audio: $2")
        if [ 0 -lt $LANGSTR ]; then echo -e "${SETGN}$LANGSTR: Audiostream $STRIDX2:$STRIDX $3 gefunden$NOCOL"; ASEL=$STRIDX; fi
      fi
    done
  done
}

function getlang {
  if [ $ASEL -lt -1 ]; then
    if [ "$XLANG" != "" ]; then getlangSub "$XLANG" "" "Sprache=\"$XLANG\""; fi
    if [ 0 -gt $ASEL ]; then getlangSub ".*" ".*Hz, 5.1(side)," "Typ=\"5.1(side)\""; fi
    if [ 0 -gt $ASEL ]; then getlangSub ".*" ".*Hz, stereo," "Typ=\"stereo\""; fi
    if [ 0 -gt $ASEL ];then echo -e "kein Audiostream mit \"$XLANG\" oder \"Typ 5.1(side)\" oder \"Typ stereo\" gefunden, Rückfall auf 'allaudio'"; ASEL=-1; fi
  fi
}

function sendermarker {
  [ "$1" == "" ] && tmpp1="$p1" || tmpp1="$1"
  # Marker für Sender, die fehlerhaften AACLATM Audiostream senden - bei ARTE ständig und bei ARD-Sendern oft,  betrifft z.Z.
  # nur DVBT2, Mediathek-Streaming erfolgt noch mit H.264 und AAC. Ggf. Parameter AACLATM verwenden.
  # Die Senderkennung muss im Videodatei-Namen enthalten sein, tvbrowser2kaffeine.rb und TVBrowser-Ansteuerung müssen entsprechend 
  # angepasst werden - tvbrowser2kaffeine.rb via http://wiki.tvbrowser.org/index.php/CapturePlugin_-_Kaffeine#Kaffeine_.3E.3D_2.0.4
  # ca. bei Zeile #88 einfügen: sTitel="#{sTitel}_#{sKanalName}", in Kaffeine und in der TVbrowser-Aufnahmesteuerung müssen die 
  # Sendernamen mit der nachfolgenden Liste übereinstimmen bzw. die folgende Liste angepasst werden, siehe auch livestream_adresse.inc!
  # --------------------------------
  # DVBT2 - in Deutschland Video immer HEVC, 
  #         in Deutschland Audio:
  #           ZDF-Schiene meist AAC/AC3, ARD-/ARTE-Schiene immer AACLATM. 
  #           Der erste Audiostream ist bei ARTE AACLATM-HE und der ARD-Schiene AACLATM-LC, die Codierung wird vor allem bei ARTE als fehlerhaft angezeigt,
  #           aber trotzdem erfolgreich konvertiert. Oft wird der zweite Audiostream - AACLATM_HE oder AACLATM_LC, Audiokommentar oder zweite Sprache - 
  #           von ffmpeg auch als fehlerhaft gemeldet.
  kennung_gefunden=0
  for ((index=0 ; index<${#streamaddr[@]} && $kennung_gefunden==0 ; index=index+$streamaddrsize )); do
    tmpidx=$(($index/$streamaddrsize))
    do_sp_hdl=$(echo "$tmpp1" | grep -c -i "dvbt2")
    if [ 1 -eq $do_sp_hdl ]; then
      do_sp_hdl=$(echo "$tmpp1" | grep -c -i "${streamaddr[$index+$idxSendername_DVBT]}")
      sendername="${streamaddr[$index+$idxSendername_DVBT]}"
    else
      do_sp_hdl=$(echo "$tmpp1" | grep -c -i "${streamaddr[$index+$idxSendername_Mediathek]}")
      sendername="${streamaddr[$index+$idxSendername_Mediathek]}"
      mediathekstream=1
    fi
    if [ "$sendername" != "@@@@@@@@@@" ] && [  0 -lt $do_sp_hdl ]; then kennung_gefunden=1; fi
  done
}

function astreamsel {
  if [ 0 -eq $convaaclatm ]; then specialaaclatm=1
  else
    specialaaclatm=0
    if [ 0 -le $strnr ]; then ASEL=$strnr; fi
  fi
}

function selvideostreamSub {
    a_XXXX=$(cat $logDir$p1.ffprobe-log | grep -c "stream,$STRIDX,$1,")
    if [ $a_XXXX -gt 0 ]; then strvin="$1"; fi
}

function selvideostream {
  if [ "-1" == "$VSTRN" ]; then
    selvideostreamSub "hevc"
    a_HEVC=$a_XXXX
    selvideostreamSub "h265"
    a_H265=$a_XXXX
    selvideostreamSub "h264"
    a_H264=$a_XXXX
    selvideostreamSub "mpeg4"
    a_MP4=$a_XXXX
    selvideostreamSub "vp8"
    a_VP8=$a_XXXX
    selvideostreamSub "vp9"
    a_VP9=$a_XXXX
    selvideostreamSub "av1"
    a_AV1=$a_XXXX
    selvideostreamSub "mpeg2video"
    a_M2V=$a_XXXX
    voutdef
    if [ 1 -eq $a_HEVC ] || [ 1 -eq $a_H265 ] || [ 1 -eq $a_H264 ] || [ 1 -eq $a_MP4  ] || [ 1 -eq $a_VP8  ] || [ 1 -eq $a_VP9  ] || [ 1 -eq $a_AV1  ] || [ 1 -eq $a_M2V  ]; then
      VSTRN=$STRIDX
      b_HEVC=$(( $b_HEVC + $a_HEVC + $a_H265 ))
      b_H264=$(( $b_H264 + $a_H264 ))
      b_MPEG4=$(( $b_MPEG4 + $a_MP4 ))
      b_VP8=$(( $b_VP8 + $a_VP8 ))
      b_VP9=$(( $b_VP9 + $a_VP9 ))
      b_AV1=$(( $b_AV1 + $a_AV1 ))
      b_M2V=$(( $b_M2V + $a_M2V ))
      # vout... wird bei != "copy" weiter unten gesetzt
      if [ $b_HEVC -gt 0 ]; then 
        # 05/2022 und Aktuell (12/2023-01/2024) Problem mit einigen Mediathek-Streams (ARTE und einige ARD-Mediatheken, z.B. MDR): ist zwar H264/AACLATM, aber Festlegung mit vinh264/ainaac... führt zu Fehlerabbruch. Weglassen von vinh264/ainaac... scheint zu helfen
        if [ "$p0" == "" ]; then
          [ $mediathekstream -eq 0 -a ".${p1##*.}" != ".ts" ] && vinhevc || vindef
          #voutcp
        else
          [ $mediathekstream -eq 0 -a ".${p1##*.}" != ".ts" ] && vinhevc || vindef
        fi
      elif [ $b_H264 -gt 0 ]; then
        # 05/2022 und Aktuell (12/2023-01/2024) Problem mit einigen Mediathek-Streams (ARTE und einige ARD-Mediatheken, z.B. MDR): ist zwar H264/AACLATM, aber Festlegung mit vinh264/ainaac... führt zu Fehlerabbruch. Weglassen von vinh264/ainaac... scheint zu helfen
        if [ "$p0" == "" ]; then
          [ $mediathekstream -eq 0 -a ".${p1##*.}" != ".ts" ] && vinh264 || vindef
          #voutcp
        else
          [ $mediathekstream -eq 0 -a ".${p1##*.}" != ".ts" ] && vinh264 || vindef
        fi
      elif [ $b_MPEG4 -gt 0 ]; then vinmpeg4
      elif [ $b_VP8 -gt 0 ]; then vinvp8
      elif [ $b_VP9 -gt 0 ]; then vinvp9
      elif [ $b_AV1 -gt 0 ]; then vinav1
      elif [ $b_M2V -gt 0 ]; then vinmpeg
      else vindef; voutcp
      fi
      if [ "$voutd" != "copy" ]; then
        if [ "$vind" == "vp8" ] || [ "$vind" == "vp9" ] || [ "$vind" == "av1" ] || [ "$vind" == "mpeg2video" ]; then
          #if [ "$vind" == "$voutd" ]; then HevcVideoOut=0; voutcp
          #el
          if [ $HevcVideoOut -eq 0 ]; then HevcVideoOut=0; vouth264
          elif [ $HevcVideoOut -eq 1 ]; then vouthevc
          elif [ $HevcVideoOut -eq 3 ]; then
             [ $vcodecSpecific = 1 ] && h264hevcDefaultOption="$Av1OptExt" || h264hevcDefaultOption=""
             voutav1
          else
             [ $vcodecSpecific = 1 ] && h264hevcDefaultOption="$Vp9OptExt" || h264hevcDefaultOption=""
             voutvp9
          fi
        #elif [ "$inresdtxt" == "$QRESD" ] && [ $HevcVideoOut -eq 0 ] && [ "$vind" == "h264" ]; then voutcp
        #elif [ "$inresdtxt" == "$QRESD" ] && [ $HevcVideoOut -eq 1 ] && [ "$vind" == "hevc" ]; then voutcp
        elif [ $HevcVideoOut -eq 0 ]; then vouth264
        elif [ $HevcVideoOut -eq 1 ]; then vouthevc
        elif [ $HevcVideoOut -eq 3 ]; then
          [ $vcodecSpecific = 1 ] && h264hevcDefaultOption="$Av1OptExt" || h264hevcDefaultOption=""
          voutav1
        else
          [ $vcodecSpecific = 1 ] && h264hevcDefaultOption="$Vp9OptExt" || h264hevcDefaultOption=""
          voutvp9
        fi
      else
        if [ "$vind" == "vp8" ] || [ "$vind" == "vp9" ] || [ "$vind" == "av1" ] || [ "$vind" == "mpeg2video" ]; then
          #if [ "$inresdtxt" == "$QRESD" ]; then HevcVideoOut=0; voutcp
          #el
          if [ $HevcVideoOut -eq 0 ]; then HevcVideoOut=0; vouth264
          elif [ $HevcVideoOut -eq 1 ]; then vouthevc
          elif [ $HevcVideoOut -eq 3 ]; then
            [ $vcodecSpecific = 1 ] && h264hevcDefaultOption="$Av1OptExt" || h264hevcDefaultOption=""
            voutav1
          else
            [ $vcodecSpecific = 1 ] && h264hevcDefaultOption="$Vp9OptExt" || h264hevcDefaultOption=""
            voutvp9
          fi
        elif [ "$vind" == "h264" ] || [ "$vind" == "mpeg4" ]; then
          #if [ "$inresdtxt" == "$QRESD" ]; then HevcVideoOut=0; voutcp
          #el
          if [ $HevcVideoOut -eq 0 ]; then HevcVideoOut=0; vouth264
          elif [ $HevcVideoOut -eq 1 ]; then vouthevc
          elif [ $HevcVideoOut -eq 3 ]; then
            [ $vcodecSpecific = 1 ] && h264hevcDefaultOption="$Av1OptExt" || h264hevcDefaultOption=""
            voutav1
          else
            [ $vcodecSpecific = 1 ] && h264hevcDefaultOption="$Vp9OptExt" || h264hevcDefaultOption=""
            voutvp9
          fi
        elif [ "$vind" == "hevc" ]; then
          #if [ "$inresdtxt" == "$QRESD" ]; then HevcVideoOut=1; voutcp
          #el
          if [ $HevcVideoOut -eq 0 ]; then HevcVideoOut=0; vouth264
          elif [ $HevcVideoOut -eq 1 ]; then vouthevc
          elif [ $HevcVideoOut -eq 3 ]; then
            [ $vcodecSpecific = 1 ] && h264hevcDefaultOption="$Av1OptExt" || h264hevcDefaultOption=""
            voutav1
          else
            [ $vcodecSpecific = 1 ] && h264hevcDefaultOption="$Vp9OptExt" || h264hevcDefaultOption=""
            voutvp9
          fi
        fi
      fi
      map_stream_0_bis_x "Video"
    fi
  fi
}

function getaudiostream {
  strain=
  a_xAAC_LATM_LC=$(cat $logDir$p1.ffprobe-log | grep -c "stream,$STRIDX,aac_latm,AAC LATM (Advanced Audio Coding LATM syntax),LC")
  a_xAAC_LATM_HEAAC=$(cat $logDir$p1.ffprobe-log | grep -c "stream,$STRIDX,aac_latm,AAC LATM (Advanced Audio Coding LATM syntax),HE-AAC")
  if [ -1 -eq $ASEL ] || [ $STRIDX -eq $ASEL ]; then
    if [ 0 -ne $a_xAAC_LATM_LC ]; then a_AAC_LATM_LC=$(( $a_AAC_LATM_LC + 1 ))
    fi #el
    if [ 0 -ne $a_xAAC_LATM_HEAAC ]; then a_AAC_LATM_HEAAC=$(( $a_AAC_LATM_HEAAC + 1 ))
    fi
  fi
  a_AAC_LATM=$(cat $logDir$p1.ffprobe-log | grep -c "stream,$STRIDX,aac_latm")
  a_AAC=$(cat $logDir$p1.ffprobe-log | grep -c "stream,$STRIDX,aac")
  a_AC3=$(cat $logDir$p1.ffprobe-log | grep -c "stream,$STRIDX,ac3")
  a_EAC3=$(cat $logDir$p1.ffprobe-log | grep -c "stream,$STRIDX,eac3")
  a_VORBIS=$(cat $logDir$p1.ffprobe-log | grep -c "stream,$STRIDX,vorbis")
  a_MP3=$(cat $logDir$p1.ffprobe-log | grep -c "stream,$STRIDX,mp3")
  a_MP2=$(cat $logDir$p1.ffprobe-log | grep -c "stream,$STRIDX,mp2")
  a_OPUS=$(cat $logDir$p1.ffprobe-log | grep -c "stream,$STRIDX,opus")
  if [ $a_AAC_LATM -gt 0 ]; then
    if [ $a_xAAC_LATM_LC -gt 0 ] && [ $a_xAAC_LATM_HEAAC -gt 0 ]; then strain="aaclatm_lc_heaac"; a_AAC=0
    elif [ $a_xAAC_LATM_LC -gt 0 ]; then strain="aaclatm_lc"; a_AAC=0
    elif [ $a_xAAC_LATM_HEAAC -gt 0 ]; then strain="aaclatm_heaac"; a_AAC=0
    else
      if [ $a_AAC -gt 0 ]; then strain="aac"; fi
    fi
  elif [ $a_AAC -gt 0 ]; then strain="aac"
  elif [ $a_EAC3 -gt 0 ]; then strain="eac3"
  elif [ $a_AC3 -gt 0 ]; then strain="ac3"
  elif [ $a_VORBIS -gt 0 ]; then strain="vorbis"
  elif [ $a_MP3 -gt 0 ]; then strain="mp3"
  elif [ $a_MP2 -gt 0 ]; then strain="mp2"
  elif [ $a_VORBIS -gt 0 ]; then strain="vorbis"
  elif [ $a_OPUS -gt 0 ]; then strain="opus"
  fi
}

function findaudiostream {
  getaudiostream
  if [ 1 -eq $a_AAC ] || [ 1 -eq $a_AC3 ] || [ 1 -eq $a_EAC3 ] || [ 1 -eq $a_VORBIS ] || [ 1 -eq $a_MP3 ] || [ 1 -eq $a_MP2 ]; then is1staudiostream=$STRIDX; sum_astream=$(( $sum_astream + 1 )); fi
  a_AAC_LATM=0
  a_AAC=0
  a_AC3=0
  a_EAC3=0
  a_VORBIS=0
  a_MP3=0
  a_MP2=0
  a_OPUS=0
}

function selaudiostream {
  # Erkennung von AACLATM-Codiervarianten - reicht für Auto-Erkennung fehlerhafter AACLATM-Streams nicht aus, da nicht alle Sender ständig 
  # fehlerhafte AACLATM-Streams senden. Darum ersatzweise/zusätzlich Auswahl über Sendernamen im Videodateinamen oder Aufrufparameter AACLATM.
  sendermarker
  getaudiostream
  if [ $forcemp4 -eq 1 ]; then AUDXT="${SETGN}$STRIDX=AAC_LATM direkte Konvertierung ${SETRT}"; AUDXN=0
  elif [ 0 -ne $a_xAAC_LATM_HEAAC ]; then AUDXT="${SETFLOR}$STRIDX=AAC_LATM-HEAAC (eventuell decodierbar) ${SETRT}"; AUDXN=2
  elif [ 0 -ne $a_xAAC_LATM_LC ]; then AUDXT="${SETOR}$STRIDX=AAC_LATM-LC (eventuell decodierbar) ${SETRT}"; AUDXN=1
  elif [ 0 -ne $a_AAC ] || [ 0 -ne $a_AC3 ] || [ 0 -ne $a_EAC3 ] || [ 0 -ne $a_VORBIS ] || [ 0 -ne $a_OPUS ] || [ 0 -ne $a_MP3 ] || [ 0 -ne $a_MP2 ]; then AUDXT="${SETGN}$STRIDX=AAC/AC3/EAC3/VORBIS/OPUS/MP3/MP2 (decodierbar) ${SETRT}"; AUDXN=0
  else AUDXT=""; AUDXN=0
  fi
  case "$STRIDX" in
    0)  AUD0T=$AUDXT ; AUD0N=$AUDXN ;;
    1)  AUD1T=$AUDXT ; AUD1N=$AUDXN ;;
    2)  AUD2T=$AUDXT ; AUD2N=$AUDXN ;;
    3)  AUD3T=$AUDXT ; AUD3N=$AUDXN ;;
    4)  AUD4T=$AUDXT ; AUD4N=$AUDXN ;;
    5)  AUD5T=$AUDXT ; AUD5N=$AUDXN ;;
    6)  AUD6T=$AUDXT ; AUD6N=$AUDXN ;;
    7)  AUD7T=$AUDXT ; AUD7N=$AUDXN ;;
    8)  AUD8T=$AUDXT ; AUD8N=$AUDXN ;;
    9)  AUD9T=$AUDXT ; AUD9N=$AUDXN ;;
    10) AUD10T=$AUDXT ; AUD10N=$AUDXN ;;
    *) ;;
  esac
  if [ 1 -eq $a_AAC_LATM ] || [ 1 -eq $a_AAC ] || [ 1 -eq $a_AC3 ] || [ 1 -eq $a_EAC3 ] || [ 1 -eq $a_VORBIS ] || [ 1 -eq $a_MP3 ] || [ 1 -eq $a_MP2 ] || [ 1 -eq $a_OPUS ]; then
    b_AAC_LATM=$(( $b_AAC_LATM + $a_AAC_LATM ))
    b_AAC=$(( $b_AAC + $a_AAC ))
    b_AC3=$(( $b_AC3 + $a_AC3 ))
    b_EAC3=$(( $b_EAC3 + $a_EAC3 ))
    b_VORBIS=$(( $b_VORBIS + $a_VORBIS ))
    b_MP3=$(( $b_MP3 + $a_MP3 ))
    b_MP2=$(( $b_MP2 + $a_MP2 ))
    b_OPUS=$(( $b_OPUS + $a_OPUS ))
    if [ $a_xAAC_LATM_HEAAC -gt 0 ] || [ $a_xAAC_LATM_LC -gt 0 ] || [ $a_AAC_LATM -gt 0 ]; then
      # 05/2022 und Aktuell (12/2023-01/2024) Problem mit einigen Mediathek-Streams (ARTE und einige ARD-Mediatheken, z.B. MDR): ist zwar H264/AAC, aber Festlegung mit vinh264/ainaac... führt zu Fehlerabbruch. Weglassen von vinh264/ainaac... scheint zu helfen
      [ $mediathekstream -eq 0 -a ".${p1##*.}" != ".ts" ] && ainaaclatm || aindef    # 12.05.2022: Verursacht beim Lesen+Kopieren einiger DVBT2-/Mediathek-Streams (AAC-LATM-LC) von ARTE und ARD Fehler
      if [ $ASEL -lt 0 -a 1 -le $a_AAC_LATM_LC -a 1 -le $a_AAC_LATM_HEAAC -a $AacEac3Ac3AudOut -lt 0 ]; then
        # Workaround für AAC-LATM-Problem bei Vorhandensein von LC- und HEAAC-Streams: HEAAC Konvertieren nach AAC statt ACOPY. Achtung: LC muss erster Audiostream sein!
        ASEL=-1 ; AacEac3Ac3AudOut=2 ; aoutaac
      elif [ $ASEL -ge 0 -a 1 -le $a_AAC_LATM_HEAAC -a $AacEac3Ac3AudOut -lt 0 ]; then # && [ 0 -eq $a_AAC_LATM_LC ]
        AacEac3Ac3AudOut=2 ; aoutaac
      elif [ $mediathekstream -eq 1 -a $AacEac3Ac3AudOut -eq -1 -a  $AacEac3Ac3AudOut -lt 0 ]; then
        AacEac3Ac3AudOut=2 ; aoutaac # Problem bei Mediathek-Stream mit AAC-LATM! Versuche Konvertieren In=AAC-LATM -> Out=AAC
      elif [ $forcets -eq 1 ] || [ $AacEac3Ac3AudOut -eq -1 ]; then
        aoutcp
      elif [ $specialaaclatm -ge 1 -a $forcemp4 -eq 0 ]; then
        aoutcp
      else
        if [ $AacEac3Ac3AudOut -eq 0 ]; then aoutac3
        elif [ $AacEac3Ac3AudOut -eq 1 ]; then aouteac3
        elif [ $AacEac3Ac3AudOut -eq 3 ]; then aoutmp3
        elif [ $AacEac3Ac3AudOut -eq 4 ]; then aoutopus
        else aoutaac   # aaclatm wird als Ausgabeformat in mp4-Container nicht unterstützt
        fi
      fi
    elif [ $a_AAC -gt 0 ]; then
      # 05/2022 und Aktuell (12/2023-01/2024) Problem mit einigen Mediathek-Streams (ARTE und einige ARD-Mediatheken, z.B. MDR): ist zwar H264/AAC, aber Festlegung mit vinh264/ainaac... führt zu Fehlerabbruch. Weglassen von vinh264/ainaac... scheint zu helfen
      [ $mediathekstream -eq 0 -a ".${p1##*.}" != ".ts" ] && ainaac || aindef
      #[ $mediathekstream -eq 1 -a $AacEac3Ac3AudOut -eq -1 ] && AacEac3Ac3AudOut=2 # Problem bei Mediathek-Stream mit AAC? Versuche Konvertieren In=AAC-LATM -> Out=AAC
      if [ $forcets -eq 1 ] || [ $AacEac3Ac3AudOut -eq -1 ]; then aoutcp
      elif [ $AacEac3Ac3AudOut -eq 0 ]; then aoutac3
      elif [ $AacEac3Ac3AudOut -eq 1 ]; then aouteac3
      elif [ $AacEac3Ac3AudOut -eq 3 ]; then aoutmp3
      elif [ $AacEac3Ac3AudOut -eq 4 ]; then aoutopus
      else aoutaac   #aoutcp # aaclatm wird als Ausgabeformat in mp4-Container nicht unterstützt
      fi
    elif [ $a_EAC3 -gt 0 ]; then
      aineac3
      #[ $mediathekstream -eq 1 -a $AacEac3Ac3AudOut -eq -1 ] && AacEac3Ac3AudOut=2 # Problem bei Mediathek-Stream mit EAC3? Versuche Konvertieren In=EAC3 -> Out=AAC
      if [ $forcets -eq 1 ] || [ $AacEac3Ac3AudOut -eq -1 ]; then aoutcp
      elif [ $AacEac3Ac3AudOut -eq 0 ]; then aoutac3
      elif [ $AacEac3Ac3AudOut -eq 2 ]; then aoutaac
      elif [ $AacEac3Ac3AudOut -eq 3 ]; then aoutmp3
      elif [ $AacEac3Ac3AudOut -eq 4 ]; then aoutopus
      else aoutcp
      fi
    elif [ $a_AC3 -gt 0 ]; then
      ainac3
      #[ $mediathekstream -eq 1 -a $AacEac3Ac3AudOut -eq -1 ] && AacEac3Ac3AudOut=2 # Problem bei Mediathek-Stream mit AC3? Versuche Konvertieren In=AC3 -> Out=AAC
      if [ $forcets -eq 1 ] || [ $AacEac3Ac3AudOut -eq -1 ]; then aoutcp
      elif [ $AacEac3Ac3AudOut -eq 1 ]; then aouteac3
      elif [ $AacEac3Ac3AudOut -eq 2 ]; then aoutaac
      elif [ $AacEac3Ac3AudOut -eq 3 ]; then aoutmp3
      elif [ $AacEac3Ac3AudOut -eq 4 ]; then aoutopus
      else aoutcp
      fi
    elif [ $a_VORBIS -gt 0 ]; then
      ainvorbis
      if [ $forcets -eq 1 ]; then aoutcp
      else aoutaac
      fi
    elif [ $a_MP3 -gt 0 ]; then
      ainmp3
      if [ $forcets -eq 1 ] || [ $AacEac3Ac3AudOut -eq -1 ]; then aoutcp
      elif [ $AacEac3Ac3AudOut -eq 0 ]; then aoutac3
      elif [ $AacEac3Ac3AudOut -eq 1 ]; then aouteac3
      elif [ $AacEac3Ac3AudOut -eq 2 ]; then aoutaac
      elif [ $AacEac3Ac3AudOut -eq 4 ]; then aoutopus
      else aoutcp
      fi
    elif [ $a_OPUS -gt 0 ]; then
      ainopus
      if [ $forcets -eq 1 ] || [ $AacEac3Ac3AudOut -eq -1 ]; then aoutcp
      elif [ $AacEac3Ac3AudOut -eq 0 ]; then aoutac3
      elif [ $AacEac3Ac3AudOut -eq 1 ]; then aouteac3
      elif [ $AacEac3Ac3AudOut -eq 2 ]; then aoutaac
      elif [ $AacEac3Ac3AudOut -eq 3 ]; then aoutmp3
      else aoutcp
      fi
    elif [ $a_MP2 -gt 0 ]; then
      ainmp2
      if [ $forcets -eq 1 ] || [ $AacEac3Ac3AudOut -eq -1 ]; then aoutcp
      elif [ $AacEac3Ac3AudOut -eq 0 ]; then aoutac3
      elif [ $AacEac3Ac3AudOut -eq 1 ]; then aouteac3
      elif [ $AacEac3Ac3AudOut -eq 2 ]; then aoutaac
      elif [ $AacEac3Ac3AudOut -eq 3 ]; then aoutmp3
      else aoutcp
      fi
    else aindef; aoutcp
    fi
    map_stream_0_bis_x "Audio"
  fi
  a_xAAC_LATM_HEAAC=0
  a_xAAC_LATM_LC=0
  a_AAC_LATM=0
  a_AAC=0
  a_AC3=0
  a_EAC3=0
  a_VORBIS=0
  a_MP3=0
  a_MP2=0
}

function findstream1 {
  streamfound=$(cat $logDir$p1.ffprobe-log | grep -c "stream,$STRIDX,$1")
}

function findstream2 {
  SUBTStr=$(cat $logDir$p1.ffprobe-errlog | grep "$1")
}

#function usestream {
#  # Parameter: $1=Subtitle-Typ, $2=Decoder-Name, $3=Encoder-Name $STRIDX=Stream-Nummer
#  # TBD
#}

function selsubtitlestreamsub {
  if [ "$1" != "" ] && [ $streamfound -eq 0 ]; then
    findstream1 "$1"
    if [ $streamfound -gt 0 ]; then
      findstream2 "$1" # TBD ; usestream "$1" "$2" "$3"
    fi
  fi
}

function selsubtitlestream {
  SUBTStr=""
  streamfound=0
  if [ 1 -eq $enasubtitle ]; then
    selsubtitlestreamsub "${subt1str[0]}" "${subt1str[1]}" "${subt1str[2]}"
    selsubtitlestreamsub "${subt2str[0]}" "${subt2str[1]}" "${subt2str[2]}"
    selsubtitlestreamsub "${subt3str[0]}" "${subt3str[1]}" "${subt3str[2]}"
    selsubtitlestreamsub "${subt4str[0]}" "${subt4str[1]}" "${subt4str[2]}"
    selsubtitlestreamsub "${subt5str[0]}" "${subt5str[1]}" "${subt5str[2]}"
    selsubtitlestreamsub "${subt6str[0]}" "${subt6str[1]}" "${subt6str[2]}"
    selsubtitlestreamsub "${subt7str[0]}" "${subt7str[1]}" "${subt7str[2]}"
    selsubtitlestreamsub "${subt8str[0]}" "${subt8str[1]}" "${subt8str[2]}"
    selsubtitlestreamsub "${subt9str[0]}" "${subt9str[1]}" "${subt9str[2]}"
    if [ "$SUBTStr" != "" ]; then
      IMAPX="" 
      #IMAPX="-map $(echo $SUBTStr | cut -c9-11)"
    fi
    if [ "$subdec" == "" ]; then
      XSTRDS=""
      XSTRD=""
    else
      XSTRD="$IMAPX -codec:s $subdec "
      if [ "$subtitlexcmd" != "" ]; then XSTRD="$STRD$subtitlexcmd "; fi
      XSTRDS=" in= $XSTRD"
    fi
    if [ "$subenc" == "" ]; then XSTRES=""; XSTRE=""
    else
      XSTRE="-codec:s $subenc "
      if [ "$subtitlexcmd2" != "" ]; then XSTRE="$STRE$subtitlexcmd2 "; fi
      XSTRES="$XSTRE"
    fi
    # kein Mapping für Subtitle-Decoder/-Encoder?
    if [ "$XSTRE" != "" ]; then
      XSTRES="out= $XSTRES"
      #map_stream_0_bis_x "Subtitle"
    fi
    case "$STRIDX" in
      1)  SUBD1=$XSTRD ; SUBE1=$XSTRE ; SUBD1S="$XSTRDS" ; SUBE1S="$XSTRES" ;;
      2)  SUBD2=$XSTRD ; SUBE2=$XSTRE ; SUBD2S="$XSTRDS" ; SUBE2S="$XSTRES" ;;
      3)  SUBD3=$XSTRD ; SUBE3=$XSTRE ; SUBD3S="$XSTRDS" ; SUBE3S="$XSTRES" ;;
      4)  SUBD4=$XSTRD ; SUBE4=$XSTRE ; SUBD4S="$XSTRDS" ; SUBE4S="$XSTRES" ;;
      5)  SUBD5=$XSTRD ; SUBE5=$XSTRE ; SUBD5S="$XSTRDS" ; SUBE5S="$XSTRES" ;;
      6)  SUBD6=$XSTRD ; SUBE6=$XSTRE ; SUBD6S="$XSTRDS" ; SUBE6S="$XSTRES" ;;
      7)  SUBD7=$XSTRD ; SUBE7=$XSTRE ; SUBD7S="$XSTRDS" ; SUBE7S="$XSTRES" ;;
      8)  SUBD8=$XSTRD ; SUBE8=$XSTRE ; SUBD8S="$XSTRDS" ; SUBE8S="$XSTRES" ;;
      9)  SUBD9=$XSTRD ; SUBE9=$XSTRE ; SUBD9S="$XSTRDS" ; SUBE9S="$XSTRES" ;;
      10) SUBD10=$XSTRD ; SUBE10=$XSTRE ; SUBD10S="$XSTRDS" ; SUBE10S="$XSTRES" ;;
    esac
  fi
  subdec=
  subenc=
  streamfound=0
}

function getaspect {
  if [ "$QASP" == "" ]; then
    # aspect
    a_4v3=$(cat $logDir$p1.ffprobe-log | grep -c ",4:3,")
    a_13333=$(cat $logDir$p1.ffprobe-log | grep -c ",1.3333,")
    a_16v9=$(cat $logDir$p1.ffprobe-log | grep -c ",16:9,")
    a_17777=$(cat $logDir$p1.ffprobe-log | grep -c ",1.7777,")
    b_4v3=$(( $b_4v3 + $a_4v3 + $a_13333 ))
    b_16v9=$(( $b_16v9 + $a_16v9 + $a_17777 ))
    if [ 1 -le $b_4v3 ]; then IASP="4:3"
    elif [ 1 -le $b_16v9 ]; then IASP="16:9"
    else IASP=
    fi
    if [ "$IASP" == "" ]; then QASP=
    else QASP="-aspect $IASP "
    fi
  fi
}

function get_tval_sec {
  # 
  tval_1=${tval:0:1}
  tval_2=${tval:1:1}
  tval_3=${tval:2:1}
  tval_4=${tval:3:1}
  tval_5=${tval:4:1}
  tval_6=${tval:5:1}
  tval_7=${tval:6:1}
  tval_8=${tval:7:1}
  tval_9=${tval:8:1}
  tval_0=${tval:9:1}
  if [ "$tval_2" == "." ] || [ "$tval_2" == "," ];then
    tval_ss="$tval3$tval4"
    tval_2=
    tval_3=
    tval_4=
    tval_5=
    tval_6=
    tval_7=
    tval_8=
  elif [ "$tval_3" == "." ] || [ "$tval_3" == "," ];then
    tval_ss="$tval4$tval5"
    tval_3=
    tval_4=
    tval_5=
    tval_6=
    tval_7=
    tval_8=
  elif [ "$tval_4" == '.' ] || [ "$tval_4" == ',' ];then
    tval_ss="$tval5$tval6"
    tval_4=
    tval_5=
    tval_6=
    tval_7=
    tval_8=
  elif [ "$tval_5" == '.' ] || [ "$tval_5" == ',' ];then
    tval_ss="$tval6$tval7"
    tval_5=
    tval_6=
    tval_7=
    tval_8=
  elif [ "$tval_6" == '.' ] || [ "$tval_6" == ',' ];then
    tval_ss="$tval7$tval8"
    tval_6=
    tval_7=
    tval_8=
  elif [ "$tval_7" == '.' ] || [ "$tval_7" == ',' ];then
    tval_ss="$tval8$tval9"
    tval_7=
    tval_8=
  elif [ "$tval_8" == '.' ] || [ "$tval_8" == ',' ];then
    tval_ss="$tval9$tval0"
    tval_8=
  elif [ "$tval_9" == '.' ] || [ "$tval_9" == ',' ];then
    tval_ss=$tval0
  else
    tval_ss=0
  fi
  tval_A=$tval_1
  if [ "$tval_2" == ':' ] || [ "$tval_2" == '-' ] || [ "$tval_2" == '_' ];then
    tval_B=$tval_3
    if [ "$tval_4" == ':' ] || [ "$tval_4" == '-' ] || [ "$tval_4" == '_' ];then
      tval_C=$tval_5$tval_6
    else
      tval_B="$tval_B$tval_4"
      tval_C=$tval_6$tval_7
    fi
  else
    tval_A="$tval_A$tval_2"
    tval_B=$tval_4
    if [ "$tval_5" == ':' ] || [ "$tval_5" == '-' ] || [ "$tval_5" == '_' ];then
      tval_C=$tval_6
    else
      tval_B="$tval_B$tval_5"
      tval_C=$tval_7$tval_8
    fi
  fi
  if [ "$tval_C" == "" ]; then
    if [ "$tval_B" == "" ]; then
      if [ "$tval_A" == "" ]; then
        tval_s="0"
      else
        tval_s="$tval_A"
        tval_m="0"
        tval_h="0"
      fi
    else
      tval_s="$tval_B"
      tval_m="$tval_A"
      tval_h="0"
    fi
  else
    tval_s="$tval_C"
    tval_m="$tval_B"
    tval_h="$tval_A"
  fi
  [ "$tval_ss" == "" ] && tval_ss=0
  if [ $tval_ss -eq 0 ]; then
    tval_ss2="04"
  elif [ "$val_ss" == "0[0-9]" ]; then
    tval_ss2="$(( ${tval_ss#0} + 4 ))" ; [ $tval_ss2 -le 9 ] && tval_ss2="0$tval_ss2"
  elif [ $val_ss -le 9 ]; then
    tval_ss2="$(( $(( $tval_ss * 10 )) + 4 ))"
  elif [ $val_ss -gt 95 ]; then
    tval_ss2="00"
    tval_s=$(( $tval_s + 1 ))
    if [ $tval_s -gt 59 ]; then
      tval_s=0
      tval_m=$(( $tval_m + 1 ))
      if [ $tval_m -gt 59 ]; then
        tval_m=0
        tval_h=$(( $tval_h + 1 ))
      fi
    fi
  else
    tval_ss2="$(( $tval_ss + 4 ))"
  fi
  tval_sec=$(( 10#$tval_s + $(( 10#$tval_m * 60 )) + $(( 10#$tval_h * 3600 )) ))
}

function getRSLoad {
  #xxxTotal=$(xxxList=$(free|grep "$1"|cut -c10-20);echo "${xxxList// /}")
  xxxTotal=$(xxxList=$(free|grep "$1"|awk 'BEGIN {val=0} {val=$2} END {print val}');echo "${xxxList// /}"|sed 's/,/\./')
  #xxxFree=$(xxxList=$(free|grep "$1"|cut -c34-44);echo "${xxxList// /}")
  xxxFree=$(xxxList=$(free|grep "$1"|awk 'BEGIN {val=0} {val=$4} END {print val}');echo "${xxxList// /}"|sed 's/,/\./')
  [ $xxxTotal -gt 0 ] && xxxUsed=$(echo "$xxxTotal $xxxFree"|awk '{val=(((($1-$2)*100)/$1)+0.5);print val}'|sed 's/,/\./'); xxxUsed=${xxxUsed%.*} || xxxUsed=0
}

function getRamLoad {
  getRSLoad "Mem:"
  [ $xxxTotal -gt 0 ] || getRSLoad "Speicher:"
  RamLoadTotal=$xxxLoadTotal
  RamLoadFree=$xxxLoadFree
  RamUsed=$xxxUsed
}

function getSwapLoad {
  getRSLoad "Swap:"
  SwapLoadTotal=$xxxTotal
  SwapLoadFree=$xxxFree
  SwapUsed=$xxxUsed
}

function getCpuLoadSub {
  CurrLoadXX=0
  if [ "$1" != "" ]; then
    CurrLoadXX=$(currld=$(cat $logDir$p1.ffmpeg-log|grep "$1"|cut -c49-);echo "${currld//,/}"|awk 'BEGIN {val=0} {val=val+$1} END {print val}')
    CurrLoadXX1=$(( $CurrLoadXX / $(( $MeasIter * $NumCPUs )) ))
    CurrLoadXX2=$(( $CurrLoadXX1 % 10 ))
    CurrLoadXX3=$(( $CurrLoadXX1 / 10 ))
    if [ $CurrLoadXX1 -lt $CurrLoadLimit ]; then #[ $CurrLoadXX3 -eq 0 ] && [ $CurrLoadXX2 -eq 0 ]; then
      if [ $Laststatusfulllog -eq 1 ]; then
        # <= CurrLoadLimit (% * 10)
        if [ $tmppcnt -gt $(( $chkcols - 38 )) ]; then
          tmppcnt=0
          tmpsstr=""
          for (( idx2=0; idx2<$(( $ZeroLoadStr0Size - ${#SETGN} )); idx2++ )); do
            tmpsstr="$tmpsstr "
          done
          ZeroLoadStr="$ZeroLoadStr\n$tmpsstr"
        fi
        tmppcnt=$(( $tmppcnt + ${#1} + 1 ))
        ZeroLoadStr="$ZeroLoadStr $1,"
      fi
    else
      # > CurrLoadLimit (% * 10)
      if [ $tmppcnt2 -gt $(( $chkcols - 38 )) ]; then
        CurrLoadStr="$CurrLoadStr ...$newlinestr            ... $1=$CurrLoadXX3,$CurrLoadXX2% "
        tmppcnt2=0
      else
        CurrLoadStr="$CurrLoadStr$1=$CurrLoadXX3,$CurrLoadXX2% "
      fi
      tmppcnt2=$(( $tmppcnt2 + ${#1} + 6 ))
      setcolcode="${SETGN}"
      if [ $CurrLoadXX1 -ge $CurrLoadAl ]; then
        setcolcode="${SETFLRT}"
      elif [ $CurrLoadXX1 -ge $CurrLoadHi ]; then
        setcolcode="${SETRT}"
      elif [ $CurrLoadXX1 -ge $CurrLoadMed ]; then 
        setcolcode="${SETOR}"
      fi
      tmpstr="$1=$CurrLoadXX3,$CurrLoadXX2%, "
      tmpstrlen=$(( $tmpstrlen + ${#tmpstr} ))
      tmpstr="${setcolcode}${tmpstr}$NOCOL"
      [ "${pcfg,,}" != "-umbenennen" ] && echo -e -n "${tmpstr}"
      if [ $Laststatuslog -eq 1 ]; then
        [ "${pcfg,,}" != "-umbenennen" ] && echo -e -n "${tmpstr}" >> $logDir$p1.ffmpeg-tmp
      fi
      if [ $tmpstrlen -gt $(( $chkcols - 45 )) ]; then
        tmpInd="$newlinestr"
        tmpstrlen=${#tmpInd}
        [ "${pcfg,,}" != "-umbenennen" ] && echo -e -n "$tmpInd"
        [ "${pcfg,,}" != "-umbenennen" ] && echo -e -n "$tmpInd" >> $logDir$p1.ffmpeg-tmp
      fi
      CurrLoad2=$(( $CurrLoad2 + $CurrLoadXX2 ))
      CurrLoad=$(( $CurrLoad + $CurrLoadXX3 ))
    fi
  fi
}

function GetCpuLoadSub2 {
   tmpLoad=$(currld=$(top -b -n $MeasIter -d $MeasDelay|grep "$1" | cut -c49-54);echo "${currld//,/}" | sed 's/\.//g' | awk 'BEGIN {val=0} {val=val+$1} END {print val}')
}

function getCpuLoad {
  sleep 10s # sicherstellen, dass angehaltene/pausierte Prozesse in der Lastabfrage inaktiv sind
  [ $CurrLoadLimit -gt 20 ] && CurrLoadLimit=20 # Anzeige ab Wert (max. 20%), OK unter (Wert * 3,333), Warnung ab (Wert * 3,333), Alarm ab (Wert * 5)
  CurrLoadMed=$(( $CurrLoadLimit * 10 ))
  CurrLoadHi=333 #$(( $CurrLoadLimit * 20 ))
  CurrLoadAl=500 #$(( $CurrLoadLimit * 40 ))
  if [ ${#chkformat[*]} -gt 0 ]; then
    tmpstr="$NOCOL$TRENNER\n${#chkformat[*]} laufende Prozesse, davon >= $(( $CurrLoadLimit / 10 )),$(( $CurrLoadLimit % 10 ))%:"
    tmpstrlen=$(( $tmpstrlen + ${#tmpstr} ))
    tmpstr="${SETBL}${tmpstr}$NOCOL "
    [ "${pcfg,,}" != "-umbenennen" ] && echo -e -n "${tmpstr}"
    if [ $Laststatuslog -eq 1 ]; then
      [ "${pcfg,,}" != "-umbenennen" ] && echo -e -n "${tmpstr}" >> $logDir$p1.ffmpeg-tmp
    fi
    tmppcnt=0
    tmppcnt2=0
    CurrLoad=0
    CurrLoad2=0
    CurrLoadStr=""
    tmpstr=""
    tmpstrlen=38
    if [ $Laststatusfulllog -eq 1 ]; then ZeroLoadStr="${SETGN}<0,1% CPU-Last durch"; ZeroLoadStr0Size=${#ZeroLoadStr}
    else ZeroLoadStr=""; ZeroLoadStr0Size=0
    fi
    _tmpstr="-"
    for (( idx1=0; idx1<${#chkformat[*]}; idx1++ )); do
      tmpchkformat=${chkformat[$idx1]}
      getCpuLoadSub $tmpchkformat
    done
    if [ $Laststatusfulllog -eq 1 ]; then
      if [ ${#ZeroLoadStr} -gt $ZeroLoadStr0Size ]; then
        ZeroLoadStr="${ZeroLoadStr%,}"
        tmpstr="$ZeroLoadStr$NOCOL"
        [ "${pcfg,,}" != "-umbenennen" ] && echo -e "${tmpstr}"
        if [ $Laststatuslog -eq 1 ]; then
          [ "${pcfg,,}" != "-umbenennen" ] && echo -e "${tmpstr}" >> $logDir$p1.ffmpeg-tmp
        fi
      fi
    fi
    CurrLoad=$(($CurrLoad + $(( $CurrLoad2 / 10 )) ))
    CurrLoad2=$(( $CurrLoad2 % 10 ))
    CurrLoadXX1=$(( $(( $CurrLoad * 10 )) + $CurrLoad2 ))
    CurrLoadXX2=$(( $CurrLoadXX1 % 10 ))
    CurrLoadXX3=$(( $CurrLoadXX1 / 10 ))
    if [ $CurrLoadXX3 -ge $CurrPwrLimit2 ]; then
      tmpcol="$SETRT$SETBGAL"
      tmpop=">= $CurrPwrLimit2"
      tmpadd=" hoch"
      if [ $ChkTemp -ge 0 ]; then redlvl=2
      else redlvl=1
      fi
    elif [ $CurrLoadXX3 -ge $CurrPwrLimit ]; then
      tmpcol="$SETOR$SETBGGR"
      tmpop=">=$CurrPwrLimit% / <$CurrPwrLimit2"
      tmpadd=" erhöht"
      if [ $ChkTemp -ge 0 ]; then redlvl=1
      fi
    else
      tmpcol="$SETGN$SETBGGR"
      tmpop="< $CurrPwrLimit"
      tmpadd=""
      redlvl=0
    fi
    tmpstr="$tmpcol$CurrLoadXX3,$CurrLoadXX2% $tmpop% (${NumCPUs} CPUs) CPU-Last$tmpadd$NOCOL"
    if [ $(( $tmppcnt2 + ${#tmpstr} + 38 )) -ge $chkcols ]; then tmpstr="$newlinestr${tmpstr}"
    fi
    [ "${pcfg,,}" != "-umbenennen" ] && echo -e "${tmpstr}"
    [ "${pcfg,,}" != "-umbenennen" ] && echo -e "${tmpstr}" >> $logDir$p1.ffmpeg-tmp
    sync
    if [ "$chkformatA" != "" ]; then
      GetCpuLoadSub2 "root"
      CurrLoadA1=$tmpLoad
      GetCpuLoadSub2 "$chkformatA"
      CurrLoadA2=$tmpLoad
      CurrLoadA=$(( $(( $CurrLoadA1 + $CurrLoadA2 )) / $(( $MeasIter * $NumCPUsReal )) ))
      CurrLoadA=${CurrLoadA//.*/}
    fi
    HiLoad=$(( $CurrLoadA / 10 ))
    tmpcol=""
    tmpop=""
    tmpadd=""
    tmpstr=""
  fi
}

function getCpuFreq {
  # wegen einzelner Abweichungen Mittelwert aus $MeasIter Frequenzwerten
  FreqCPUs=0
  for (( idx4=0; idx4<$MeasIter; idx4++ ))
  do
    if [ "$CPUtyp" == "ARM" ]; then
      tmpFreqCPUs=$(vcgencmd measure_clock arm | cut -c14- | awk 'BEGIN {val=0} {val=$1/(1024*1024)} END {print val}' | sed 's/,/\./')
    else
      tmpFreqCPUs=$(lscpu | grep "CPU MHz" | awk '{print $3}')
    fi
    FreqCPUs=$(( $FreqCPUs + ${tmpFreqCPUs%.*} ))
    sleep $MeasDelay
  done
  FreqCPUs=$(( $FreqCPUs / $MeasIter ))
  MaxFreqCPUs=$(lscpu | grep "CPU max MHz" | awk '{print $4}')
  MaxFreqCPUs=${MaxFreqCPUs%,*}
}

function getCpuTempSub {
  if [ "$CPUtyp" == "ARM" ]; then
    tmpTemp=$(coretemplst=$(vcgencmd measure_temp | cut -c5-9) ; echo "${coretemplst}" | awk 'BEGIN {val=0;cnt=0} {val=val+$1;cnt=cnt+1} END {val=val/cnt;print val}')
  else
    tmpTemp=$(coretemplst=$(sensors | grep "$1" | cut -c16-18) ; echo "${coretemplst//+/}" | awk 'BEGIN {val=0;cnt=0} {val=val+$1;cnt=cnt+1} END {val=val/cnt;print val}')
  fi
}

function getCpuTemp {
  if [ "$CPUtyp" == "ARM" ]; then
    CurrTempMax=$(coretemplst=$(vcgencmd measure_temp | cut -c5-9) ; echo "${coretemplst}" | awk 'BEGIN {max = 0} {if ($1>max) max=$1} END {print max}')
    CurrTempMin=$(coretemplst=$(vcgencmd measure_temp | cut -c5-9) ; echo "${coretemplst}" | awk 'BEGIN {min = 999} {if ($1<min) min=$1} END {print min}')
  else
    CurrTempMax=$(coretemplst=$(sensors | grep Core | cut -c16-18) ; echo "${coretemplst//+/}" | awk 'BEGIN {max = 0} {if ($1>max) max=$1} END {print max}')
    CurrTempMin=$(coretemplst=$(sensors | grep Core | cut -c16-18) ; echo "${coretemplst//+/}" | awk 'BEGIN {min = 999} {if ($1<min) min=$1} END {print min}')
  fi
  getCpuTempSub "Core"
  CurrTempChk=$tmpTemp
}

function calcRedLvlSub2 {
  tmpProzent=$(( $(( $(( $3 * $1 )) + $(( $2 / 2 )) )) / $2 ))
}

function calcRedLvlSub {
  calcRedLvlSub2 $1 $2 $p6
  CpuLimitDefneu=$tmpProzent
}

function calcRedLvl {
  # $redlvl anfangs 0...2 abhängig von CPU-Belastung <25% / >=25...<40% / >=40%
  if [ ${CurrTempChk%%.*} -ge $CpuTempHighest ]; then
    if [ $ChkTemp -ge 0 ]; then redlvl=4 # maximal
    else
      if [ $ChkTemp -eq -1 ]; then redlvl=$(( $redlvl + 2 ))
      elif [ $ChkTemp -eq -2 ]; then redlvl=$(( $redlvl + 1 ))
      fi
    fi
  elif [ ${CurrTempChk%%.*} -ge $CpuTempHigher ]; then
    if [ $ChkTemp -ge 0 ]; then
      if [ $redlvl -le 1 ]; then redlvl=$(( $redlvl + 3 )) # max. redlvl=4!
      else redlvl=$(( $redlvl + 2 ))
      fi
    else
      if [ $ChkTemp -ge -2 ]; then redlvl=$(( $redlvl + 1 )); fi
    fi
  elif [ ${CurrTempChk%%.*} -ge $CpuTempMedium ]; then
    if [ $ChkTemp -ge 0 ]; then redlvl=$(( $redlvl + 2 )); fi
  elif [ ${CurrTempChk%%.*} -ge $CpuTempAboveNormal ]; then
    if [ $ChkTemp -ge 0 ]; then redlvl=$(( $redlvl + 1 )); fi
  fi
  case "$redlvl" in
  "4") calcRedLvlSub 2 5 ;;
  "3") calcRedLvlSub 1 2 ;;
  "2") calcRedLvlSub 5 8 ;;
  "1") calcRedLvlSub 4 5 ;;
  *)   calcRedLvlSub 1 1 ;;
  esac
  if [ $CpuLimitDefneu -ne $p6 ]; then p6=$CpuLimitDefneu; fi
}

function getDateTime {
  currDateTime="$(date +'%d.%m.%Y %H:%M:%S')"
  [ "$convstart" == "" ] && currDurSec=$(echo "scale=0 ; ( $(date +'%s') / 60 )" | bc) || currDurSec=$(echo "scale=0 ; ( ( $(date +'%s') / 60 ) - $currDurSec )" | bc)
}

function printLvLMsg {
  case "$redlvl" in
  "4") colstr="$SETAL2$SETBGAL" ;;
  "3") colstr="$SETAL1$SETBGAL" ;;
  "2") colstr="$SETWA2" ;;
  "1") colstr="$SETWA1" ;;
  "0") colstr="$SETOK" ;;
  *)   colstr="$SETBL" ;;
  esac
  [ "$convstart" == "" ] && convstart="$SETBL${SETBGGR}Start: $SETBGWS$SETBLD$currDateTime$SETBGSW$NOCOL"
  [ "$addstr" == "Vor Konvertierung: " ] && CPULIMITstr="$convstart$NOCOL $colstr$CPULIMITstr$SETBGSW$NOCOL" || CPULIMITstr="$convstart\n$SETBLD${SETBGOK}$addstr$NOCOL $colstr$CPULIMITstr$SETBGSW$NOCOL"
  echo -e "$CPULIMITstr"
  echo -e "$CPULIMITstr" >> $logDir$p1.ffmpeg-tmp
}
  
function indstatusstringsub {
  getDateTime
  if [ $CpuLimitDef -ge 15 ] && [ $CpuLimitDef -le 100 ]; then
    #if [ $SwapLoadFree -lt $MemMinimumSizeKb ]; then
    #  echo -e "Freier SWAP-Bereich ($(( $SwapLoadFree / 1024 ))Mb<$(( $MemMinimumSizeKb / 1024 ))Mb) ist zu klein$NOCOL"
    #  clrlogfiles
    #  exit 1
    #fi  
    if [ $ChkRam -gt 0 ] && [ $RamLoadFree -lt $MemMinimumSizeKb ]; then echo -e "Freier RAM-Bereich ($(( $RamLoadFree / 1024 ))Mb<$(( $MemMinimumSizeKb / 1024 ))Mb) ist eventuell zu klein$NOCOL"; fi
    if [ $HiLoad -gt $CurrPwrLimit ]; then
      if [ $HiLoad -gt $CurrPwrLimit2 ]; then HiLoadStr="" #"HiLoad($HiLoad%) > CurrPwrLimit2($CurrPwrLimit2%): $CurrLoadStr"
      else HiLoadStr="" #"HiLoad($HiLoad%) >  CurrPwrLimit($CurrPwrLimit%): $CurrLoadStr"
      fi
    else HiLoadStr="" #"HiLoad($HiLoad%) <= CurrPwrLimit($CurrPwrLimit%): $CurrLoadStr"
    fi
    if [ $RamUsed -gt $RamLoadLimit ] && [ $SwapUsed -gt $SwapLoadLimit ]; then HiRamStr="" #"RamUsed + SwapUsed > Limit, RAM=$RamUsed%, SWAP=$SwapUsed%"
    elif [ $RamUsed -gt $RamLoadLimit ]; then HiRamStr="" #"RamUsed > Limit, SwapUsed <= Limit, RAM=$RamUsed%, SWAP=$SwapUsed%"
    elif [ $SwapUsed -gt $SwapLoadLimit ]; then HiRamStr="" #"RamUsed <= Limit, SwapUsed > Limit, RAM=$RamUsed%, SWAP=$SwapUsed%"
    else HiRamStr="RamUsed + SwapUsed <= Limit, RAM=$RamUsed%, SWAP=$SwapUsed%"
    fi
    tmpInd="$SETBGSW$NOCOL\n                        "
    if [ ${#HiLoadStr} -gt 0 ]; then HiLoadStr="$tmpInd$HiLoadStr"; fi
    if [ ${#HiRamStr} -gt 0 ]; then HiRamStr="$tmpInd$HiRamStr"; fi
    if [ "${pcfg,,}" != "-umbenennen" ]; then
      if [ "$addstr" == "Vor Konvertierung: " ]; then
        calcRedLvl
        CPUpwrPercent=$p6
        CPUpwrPercentHT=$(( $CPUpwrPercent / $NumCPUs ))
        if [ $(( $CPUpwrPercent % $NumCPUs )) -ge $(( $NumCPUs / 2 )) ]; then CPUpwrPercentHT=$(( $CPUpwrPercentHT + 1 )); fi
        getCpuTemp
        getCpuFreq
        CPULIMIT=" cpulimit -l $CpuLimitDefneu --" 
        CPULIMITstr1="CPU $CurrTempMin...${CurrTempMax}°C, Med=${CurrTempChk}°C, $NumCPUsReal Kerne, CPULIMIT "
        CPULIMITstr2="$SETBGGR$CPUpwrPercent% Leistung verteilt auf $NumCPUs Kerne, Durchschnitt $CPUpwrPercentHT%$SETBGSW$HiLoadStr$HiRamStr"
        if [ $redlvl -eq 0 ]; then CPULIMITstr="${CPULIMITstr1}$p6%, ${FreqCPUs}MHz, $CPULIMITstr2"
        else CPULIMITstr="${CPULIMITstr1}reduziert von $CpuLimitDef%/CPU auf $(( $CpuLimitDefneu / $NumCPUs ))%/CPU, ${FreqCPUs}MHz$tmpInd$CPULIMITstr2"
        fi
      else
        [ "${pcfg,,}" != "-umbenennen" ] && CPULIMITstr="CPU-Temperatur der $NumCPUsReal Kerne=$CurrTempMin...${CurrTempMax}°C, Med=${CurrTempChk}°C, ${FreqCPUs}MHz.$HiRamStr"
      fi
    fi
    printLvLMsg
  elif [ $CpuLimitDef -eq 14 ] || [ $CpuLimitDef -gt 100 ]; then
    [ "${pcfg,,}" != "-umbenennen" ] && getCpuTemp
    [ "${pcfg,,}" != "-umbenennen" ] && getCpuFreq
    [ $CpuLimitDef -eq 14 ] && p6=$(( 14 * $NumCPUs )) || p6=$(( 100 * $NumCPUs ))
    redlvl=0
    [ "${pcfg,,}" != "-umbenennen" ] && CPULIMITstr="CPU-Temperatur der $NumCPUsReal Kerne=$CurrTempMin...${CurrTempMax}°C, Med=${CurrTempChk}°C, ${FreqCPUs}MHz, CPU-Leistung ($(( $p6 / $NumCPUs ))%)"
    printLvLMsg
  else
    echo -e "${SETFLRT}Unzulässiger Leistungswert $CpuLimitDef$NOCOL"
    clrlogfiles
    exit 1
  fi
}

function getXxxLoad {
  getRamLoad
  getSwapLoad
  [ "${pcfg,,}" != "-umbenennen" ] && getCpuLoad
  getCpuFreq
  [ "${pcfg,,}" != "-umbenennen" ] && getCpuTemp
}

function indstatusstring {
  getXxxLoad
  indstatusstringsub
}

function chkHiTemp {
  lebendig
  if [ ${CurrTempChk%.*} -gt $CpuTempHigher ] && [ $ChkTemp -gt 0 ]; then echo "$currDateTime: Warten (${RetrySec}s) wegen hoher CPU-Temperatur ${CurrTempChk}°C"; sleep $(( $RetrySec - $getLoadValsDauer )); fi
}

function tmpTvalSub {
  tmp_dur_h=$(( $tmp_dur / 3600 ))
  tmp_dur_m=$(( $(( $tmp_dur / 60 )) % 60 ))
  [ $tmp_dur_m -le 9 ] && tmp_dur_m="0$tmp_dur_m"
  tmp_dur_s=$(( $tmp_dur % 60 ))
  [ $tmp_dur_s -le 9 ] && tmp_dur_s="0$tmp_dur_s"
}

function tmpTval {
  if [ $tmptim -lt 1 ]; then tmptim=1; fi
  if [ $4 -eq 0 ]; then tmp_dur=$(echo "scale=3 ; ( ( $tmp_dur * 10000 ) / $tmptim )" | bc) # Vorlaufzeit ohne Audio-Konv-Korrektur!
  else tmp_dur=$(echo "scale=3 ; ( ( $tmp_dur * ( 10000 + ( $corrVal * 100 ) ) ) / $tmptim )" | bc)
  fi
  tmp_dur=$(echo "scale=3 ; ( $tmp_dur / ( ( $CpuLimitDefneu + ( $3 ) ) / $NumCPUs ) )" | bc)
  tmp_dur=$(echo "scale=3 ; ( ( $tmp_dur * ( $1 + 100 ) ) / ( $2 + 100 ) )" | bc) ; tmp_dur=$(echo "scale=0 ; ( $tmp_dur / 1 )" | bc)
  tmpTvalSub
}

function tmpET {
  [ $tmp_dur_h -gt 0 ] && tmp_dur_h=${tmp_dur_h#0}
  [ $tmp_dur_m -gt 0 ] && tmp_dur_m=${tmp_dur_m#0}
  tmpendZeit=$(( $aktZeit + $durffprobe + $(( $tmp_dur_h * 3600 )) + $(( $tmp_dur_m * 60 )) ))
  tmpendZeit_h=$(( $(( $tmpendZeit / 3600 )) % 24 ))
  [ $tmpendZeit_h -le 9 ] && tmpendZeit_h="0$tmpendZeit_h"
  tmpendZeit_m=$(echo "scale=0 ; ( ( $tmpendZeit / 60 ) % 60 )" | bc) #scale=0 reicht
  [ $tmpendZeit_m -le 9 ] && tmpendZeit_m="0$tmpendZeit_m"
  [ $(( $aktZeit % 86400 )) -gt $(( $tmpendZeit % 86400 )) ] && tmpaddstr="morgen, " || tmpaddstr=""
}

function infoHdSub2 {
  # $1=(const) Unit
  hdPlatzNum=$(( $hdPlatzNum / 1024 ))
  hdMinNum=$(( $hdMinNum / 1024 ))
  hdUnit="$1"
}

function infoHdSub1 {  # $1=(const) $hdPlatzNum, $2=(const) $hdMinNum
  if [ $1 -ge 102400 ] && [ $2 -ge 102400 ]; then
    infoHdSub2 "M"
    if [ $(( $1 / 1024 )) -ge 102400 ] && [ $(( $2 / 1024 )) -ge 102400 ]; then
      infoHdSub2 "G"
      if [ $(( $1 / 1048576 )) -ge 102400 ] && [ $(( $2 / 1048576 )) -ge 102400 ]; then
        infoHdSub2 "T"
      fi
    fi
  fi
}

function infoHdSub {
# $1=Pfad, $2=Grenzwert [kB], $3=0 (nur Warnung)/1 (nur Warnung/ok)/2 (Warnung/ok/Fehler)/3 (nur Warnung/Fehler)
zielistlokal=1
if [ "$1" != "" ] && [ -d "$1" ]; then
  hdPlatzNum=$(df "$1" | grep '[/dev/|/mnt|/home|//]' | awk 'BEGIN {val=0} END {print $4}')
  hdPlatzDev=$(df "$1" | grep '[/dev/|/mnt|/home|//]' | awk 'BEGIN {val=0} END {print $6}')
  hdPlatzPath=$(df "$1" | grep '[/dev/|/mnt|/home|//]' | awk 'BEGIN {val=0} END {print $1}')
  hdMinNum=$2
  hdUnit="k"
  usbLw=("$(lsblk -do name,tran | grep 'usb' | awk 'BEGIN {val=0} END {print $1}')")
  while [ "${usbLw[$idx]}" != "" ]; do
    lebendig
    usbLwNam+=("/dev/${usbLw[$idx]}1" "/dev/${usbLw[$idx]}2" "/dev/${usbLw[$idx]}3" "/dev/${usbLw[$idx]}4" "/dev/${usbLw[$idx]}5" "/dev/${usbLw[$idx]}6" "/dev/${usbLw[$idx]}7" "/dev/${usbLw[$idx]}8" "/dev/${usbLw[$idx]}9")
    if [ "${usbLw[$idx]}" == "" ]; then break; fi
    (( idx++ ))
  done
  idx=0
  while [ "${usbLwNam[$idx]}" != "" ]; do
    lebendig
    if [ "$nurLokalesZiel" != "" ] && [ $nurLokalesZiel -ne 0 ] && [ "${usbLwNam[$idx]}" == "${hdPlatzPath}" ]; then # "${usbLwNam[$idx]}" == "$hdPlatzPath"
      echo -e "$SETOR$logDirstr'$NOCOL$1${SETOR}' ist USB-Medium und könnte zeitweise nicht verfügbar sein!$NOCOL"
      numzeilen=$(( $numzeilen + 1 ))
      zielistlokal=0
    fi
    if [ "${usbLwNam[$idx]}" == "" ]; then break; fi
    (( idx++ ))
  done
  if [ "$nurLokalesZiel" != "" ] && [ $nurLokalesZiel -ne 0 ] && [ "${hdPlatzPath:0:2}" == "//" ]; then echo -e "$SETOR$logDirstr'$NOCOL$1${SETOR}' ist Netzlaufwerk und könnte zeitweise nicht verfügbar sein!$NOCOL"; numzeilen=$(( $numzeilen + 1 )); zielistlokal=0; fi
  [ $3 -lt 3 -a $zielistlokal -eq 1 ] && echo -e "$SETGN$logDirstr'$NOCOL$1${SETGN}' ist lokale Festplatte.$NOCOL"
  logDirstr="                "
  #if [ $3 -eq 1 -o $3 -eq 2 ]; then echo -e "$SETGN$logDirstr'$NOCOL$1$SETGN' ist erreichbar$NOCOL"; numzeilen=$(( $numzeilen + 1 )); fi
  if [ $hdPlatzNum -lt $2 ]; then
    infoHdSub1 $hdPlatzNum $hdMinNum
    echo -e "${SETAL1}${SETBGOK}${logDirstr}${hdPlatzNum}${hdUnit}B < $hdMinNum${hdUnit}B frei, Abbruch (Pfad: $SETSW$SETBGOK$1$SETAL1$SETBGOK, Device: $SETSW$SETBGOK$hdPlatzDev$SETAL1$SETBGOK, Device-Pfad: $SETSW$SETBGOK$hdPlatzPath$SETAL1$SETBGOK)$NOCOL"
    numzeilen=$(( $numzeilen + 1 ))
    if [ $3 -ge 3 ]; then clrlogfiles; exit 1; fi
  elif [ $hdPlatzNum -lt $(( $2 * 5 )) ]; then
    hdMinNum=$(( 5 * $hdMinNum ))
    infoHdSub1 $hdPlatzNum $hdMinNum
    echo -e "$SETWA1$logDirstr$hdPlatzNum${hdUnit}B < $hdMinNum${hdUnit}B frei (Pfad: $NOCOL$1$SETWA1, Device: $NOCOL$hdPlatzDev$SETWA1, Device-Pfad: $NOCOL$hdPlatzPath$SETWA1)$NOCOL"
    numzeilen=$(( $numzeilen + 1 ))
  elif [ $3 -eq 1 ] || [ $3 -eq 2 ] || [ "${hdPlatzDev:0:2}" == "//" ]; then
    infoHdSub1 $hdPlatzNum $hdPlatzNum
    echo -e "$SETGN$logDirstr$hdPlatzNum${hdUnit}B frei (Pfad: $NOCOL$1$SETGN, Device: $NOCOL$hdPlatzDev$SETGN, Device-Pfad: $NOCOL$hdPlatzPath$SETGN)$NOCOL"
    numzeilen=$(( $numzeilen + 1 ))
  fi
  return 0
else
  if [ "$pcfg3" == '/' -a "$pcfg2" != '' -a $abbruchflag -eq 1 ]; then 
    echo -e "$SETGE$logDirstr'$NOCOL$1$SETGE' soll nicht angelegt werden.$NOCOL"; numzeilen=$(( $numzeilen + 1 ))
    return 0
  else 
    tmppfad="${1%/}"
    tmppfad2="${tmppfad%/*}"
    tmppfad3="${tmppfad2%/*}"
    if [ "$pcfg2" == "" -a ${#tmppfad} -gt 4 -a ! -d "$tmppfad" -a -h "$tmppfad" ] || [ ${#tmppfad2} -gt 4 -a ! -d "$tmppfad2" -a -h "$tmppfad2" ] || [ ${#tmppfad3} -gt 4 -a ! -d "$tmppfad3" -a -h "$tmppfad3" ]; then # Zielpfad inaktiver Link?
      echo -e "$SETRT$logDirstr'$NOCOL$1${SETRT}' ist nicht erreichbar (inaktiver Link - Zielgerät ausgeschaltet oder Unterverzeichnis existiert noch nicht?).$NOCOL"; numzeilen=$(( $numzeilen + 1 ))
      return -1
    elif [ "$pcfg2" == "" -a ${#tmppfad} -gt 4 -a ! -d "$tmppfad" -a -h "$tmppfad" ] || [ ${#tmppfad2} -gt 4 -a ! -d "$tmppfad2" -a ! -f "$tmppfad2" ] || [ ${#tmppfad3} -gt 4 -a ! -d "$tmppfad3" -a ! -f "$tmppfad3" ]; then # Zielpfad/Medium existiert nicht?
      echo -e "$SETRT$logDirstr'$NOCOL$1${SETRT}' ist nicht erreichbar (Medium existiert nicht - Zielgerät ausgeschaltet oder Unterverzeichnis existiert noch nicht?).$NOCOL"; numzeilen=$(( $numzeilen + 1 ))
      return -2
    else
      if [ "$pcfg3" == "/" ]; then
        echo -e "$SETRT$logDirstr'$NOCOL$1${SETRT}' ist nicht erreichbar und soll nicht angelegt werden.$NOCOL"; numzeilen=$(( $numzeilen + 1 ))
        return 0
      else
        echo -e "$SETOR$logDirstr'$NOCOL$1${SETOR}' existiert noch nicht und soll darum neu angelegt werden.$NOCOL"; numzeilen=$(( $numzeilen + 1 ))
        return 1
      fi
    fi
  fi
fi
}

function infoHdH {
  logDirstr="${logDirstr}Homeverzeichnis:"
  abbruchflag=0
  infoHdSub "$HOME" $(( 2 * 1024 * 1024 )) $1 #3  # 2GB sollten im Homeverzeichnis verfügbar sein
  retval=$?
  logDirstr=""
  return $retval
}

function infoHdL {
  logDirstr="${logDirstr}Logpfad:        "
  abbruchflag=0
  infoHdSub "$logDir" 800 $1 #3  # Je Konvertierung ca. 600...800kB (4 Logfiles)
  retval=$?
  logDirstr=""
  return $retval
}

function infoHdQ {
  logDirstr="${logDirstr}Quellenpfad:    "
  abbruchflag=0
  infoHdSub "$srcDir" $(( 512 * 1024 )) $1 #3  # nur Belegungsinfo, keine Schreibaktionen
  retval=$?
  logDirstr=""
  return $retval
}

function infoHdD {
  logDirstr="${logDirstr}Zielpfad$3     "
  abbruchflag=1
  infoHdSub "$2" $(( $Mindestplatz * 1024 * 1024 )) $1 # Je Konvertierung 300...600MB (480p)/850MB...2GB (720p)/2...3GB (1080p)
  retval=$?
  abbruchflag=0
  logDirstr=""
  return $retval
}

function infoHdT {
  logDirstr="${logDirstr}Tmp-Pfad$3     "
  abbruchflag=1
  infoHdSub "$2" $(( $Mindestplatz * 1024 * 1024 )) $1 #3 # Je Konvertierung 300...600MB (480p)/850MB...2GB (720p)/2...3GB (1080p)
  retval=$?
  abbruchflag=0
  logDirstr=""
  return $retval
}

function infoHd {
  numzeilen=0
  # $1 = MeldungslevelLog, $2=MeldungslevelSrc, $3=MeldungslevelDst, $4=MeldungslevelDst2, $5=MeldungslevelDst
  if [ "${p0,,}" == "uhd" ] || [ "${p0,,}" == "vuhd" ]; then Mindestplatz=20
  elif [ "${p0,,}" == "hdfull" ] || [ "${p0,,}" == "vhdfull" ]; then Mindestplatz=10
  elif [ "${p0,,}" == "hdready" ] || [ "${p0,,}" == "vhdready" ]; then Mindestplatz=4
  else Mindestplatz=2
  fi
  echo -e "$NOCOL$TRENNER"
  logDirstr=""
  infoHdH $1 #3
  statusHome=$?
  infoHdL $1 #3
  statusLog=$?
  infoHdQ $2
  statusSrc=$?
  if [ "$bkup1dir" != "" -o "$bkup2dir" != "" -o "$bkup3dir" != "" ]; then infoHdT $3 "$destDir" ":  "; statusTmp=$?; else  infoHdD $3 "$destDir" ":  "; statusTmp=$?; fi
  if [ "$bkup1dir" != "" ]; then infoHdD $4 "$bkup1dir$pcfg2$pcfg2_2" " 1:"; statusDest1=$?; fi
  if [ "$bkup2dir" != "" ]; then infoHdD $4 "$bkup2dir$pcfg2$pcfg2_2" " 2:"; statusDest2=$?; fi
  if [ "$bkup3dir" != "" ]; then infoHdD $4 "$bkup3dir$pcfg2$pcfg2_2" " 3:"; statusDest3=$?; fi
  [ $numzeilen -gt 0 ] && echo -e "$NOCOL$TRENNER"
}

function SubOk {
  echo -e "$SETOK OK$NOCOL"
}

function SubOk2 {
  echo -e -n "$SETOK OK$NOCOL, "
}

function SubErr {
  echo -e "$SETAL1 FEHLER$NOCOL"
}

function SubOkErr {
  if [ $1 -eq $2 ]; then
    [ "$3" = "" ] && SubOk || SubOk2
  else SubErr
  fi
}

function lcsub {
  [ $? -eq 0 ] && lcval=$(( $lcval + 1 ))
}

function prepPermanentCheck {
  if [ $anzeigeBkupMsg -ge 1 ]; then echo -e -n "$TRENNER\n${SETBL}Background-Script zur Ermittlung von Frequenz- und Temperaturverlauf anlegen und starten$NOCOL"; fi
  lcval=0
  lcsub ; echo "#!/bin/bash" > $logDir$p1.sh
  lcsub ; echo    "echo \"CPU-Freqenz-Logging ab \$(date +'%d.%m.%Y %H:%M')\" > $logDir$p1.freq-log" >> $logDir$p1.sh
  lcsub ; echo    "currldlv=0;currldct=0;currf=0;currt=0;altf=0;altt=0;currld=0;tmpt=0;logp=0;pidx=0;strtz=\$(date +'%d.%m.%Y %H:%M')" >> $logDir$p1.sh
  lcsub ; echo    "function getvalsub {" >> $logDir$p1.sh
  lcsub ; echo    "  tmpt=\$(coret=\$(sensors | grep \"\$1\" | cut -c16-20);echo \"\${coret//+/}\" | awk 'BEGIN {v1=0;v2=0} {v1=v1+\$1;v2=v2+1} END {v1=v1/v2;print v1}') ; tmpt=\${tmpt%%.*}" >> $logDir$p1.sh
  lcsub ; echo    "  if [ \$tmpt -ge 10 ] && [ \$tmpt -le 99 ]; then" >> $logDir$p1.sh
  lcsub ; echo    "    tmpt=\" \$tmpt\"" >> $logDir$p1.sh
  lcsub ; echo    "  elif [ \$tmpt -ge 0 ] && [ \$tmpt -le 9 ]; then" >> $logDir$p1.sh
  lcsub ; echo    "    tmpt=\"  \$tmpt\"" >> $logDir$p1.sh
  lcsub ; echo    "  elif [ \$tmpt -lt -1 ] && [ \$tmpt -le -9 ]; then" >> $logDir$p1.sh
  lcsub ; echo    "    tmpt=\" \$tmpt\"" >> $logDir$p1.sh
  lcsub ; echo    "  fi" >> $logDir$p1.sh
  lcsub ; echo    "}" >> $logDir$p1.sh
  lcsub ; echo    "function freqloop {" >> $logDir$p1.sh
  lcsub ; echo    "  currf=\"\$(lscpu | grep \"CPU MHz:\" | cut -c22-25)\"" >> $logDir$p1.sh
  lcsub ; echo    "  getvalsub \"Core \" ; currt=\"\$tmpt\"" >> $logDir$p1.sh
  lcsub ; echo    "  getvalsub \"CPUTIN:\" ; currtcpu=\"\$tmpt\" ; getvalsub \"SYSTIN:\"" >> $logDir$p1.sh
  lcsub ; echo    "  currtsys=\"\$tmpt\" ; getvalsub \"AUXTIN:\" ; currtaux=\"\$tmpt\"" >> $logDir$p1.sh
  lcsub ; echo -e "  echo -e \"\$(date +'%d.%m.%Y %H:%M') CPU MHz=\$currf MHz, Temperatur=\$currt C (Core)/\$currtcpu C (CPU)/\$currtsys C (SYS)/\$currtaux C (AUX)/ \$currld (CPU-Last)\" >> $logDir$p1.freq-log" >> $logDir$p1.sh
  lcsub ; echo    "}" >> $logDir$p1.sh
  lcsub ; echo    "while [ -f \"$logDir$p1.ffprobe-log\" ] || [ -f \"$logDir$p1.ffmpeg-log\" ]; do" >> $logDir$p1.sh 
  lcsub ; echo    "  freqloop" >> $logDir$p1.sh
  lcsub ; echo    "  cpuflv=\$(cat ${logDir}$p1.freq-log | grep \"CPU MHz=\" | cut -c26-29 | awk 'BEGIN {v1=0;v2=0} {v1=v1+1;v2=v2+\$1} END {v2=v2/v1;print v2}')" >> $logDir$p1.sh
  lcsub ; echo    "  cpuflv=\${cpuflv%%.*}" >> $logDir$p1.sh
  lcsub ; echo    "  cputlv=\$(cat ${logDir}$p1.freq-log | grep \"Temperatur=\" | cut -c47-49 | awk 'BEGIN {v1=0;v2=0} {v1=v1+1;v2=v2+\$1} END {v2=v2/v1;print v2}')" >> $logDir$p1.sh
  lcsub ; echo    "  cputlv=\${cputlv%%.*}" >> $logDir$p1.sh
  lcsub ; echo    "  currldct=\$(( \$currldct + 1 ))" >> $logDir$p1.sh
  lcsub ; echo    "  currld=\$(top -b -n 8 -d 0.2|grep \"ffmpeg\"|cut -c47-52|sed 's/,/./g'|sed 's/ //g'|awk 'BEGIN{v1=0} {v1=v1+(\$1/($NumCPUs*8))} END{print v1}')" >> $logDir$p1.sh
  lcsub ; echo    "  currld=\$(echo \"scale=1;(\$currld/1)\"|bc)" >> $logDir$p1.sh
  lcsub ; echo    "  currldlv=\$(echo \"scale=1;(\$currldlv+\$currld)\"|bc)" >> $logDir$p1.sh
  lcsub ; echo    "  if [ $updLogInterval -gt 0 ]; then" >> $logDir$p1.sh
  if [ $zeigeLog -eq 1 ]; then
    lcsub ; echo  "    #if [ \$(( \$currf - \$altf )) -gt 100 ] || [ \$(( \$altf - \$currf )) -gt 100 ] || [ \$(( \$currt - \$altt )) -gt 2 ] || [ \$(( \$altt - \$currt )) -gt 5 ]; then" >> $logDir$p1.sh
    lcsub ; echo  "      if [ -n \$logp ] && [ \$logp -gt 0 ]; then" >> $logDir$p1.sh
    lcsub ; echo  "        pidx=\$(ps --pid \$logp | grep -c \"xmessage\")" >> $logDir$p1.sh
    lcsub ; echo  "        if [ \$pidx -gt 0 ]; then" >> $logDir$p1.sh
    lcsub ; echo  "          kill \$logp" >> $logDir$p1.sh
    lcsub ; echo  "        fi" >> $logDir$p1.sh
    lcsub ; echo  "      fi" >> $logDir$p1.sh
    lcsub ; echo  "      if [ -f \"$logDir$p1.ffmpeg-tmp\" ]; then" >> $logDir$p1.sh
    lcsub ; echo  "        cp --preserve=all -f $logDir$p1.ffmpeg-tmp ${logDir}$p1.status" >> $logDir$p1.sh
    lcsub ; echo  "      fi" >> $logDir$p1.sh
    lcsub ; echo  "      echo -e \"\\n\$strtz ... \$(date +'%H:%M') CPU-Freq: Aktuell=\${currf}MHz, Durchschn.=\${cpuflv}MHz,  CPU-Temp: Aktuell=\$currt°C, Durchschn.=\$cputlv°C, FFMPEG-Pwr Aktuell=\$currld%, Durchschn.=\$(echo \"scale=1;(\$currldlv/\$currldct)\"|bc)% \" >> ${logDir}$p1.status" >> $logDir$p1.sh
    lcsub ; echo  "      more ${logDir}$p1.status | sed -e 's/\ä/ae/g;s/ö/oe/g;s/\ü/ue/g;s/\Ä/Ae/g;s/\Ö/Oe/g;s/\Ü/Ue/g;s/œ/oe/g;s/Œ/Oe/g;s/\ß/ss/g;s/\°/deg/g;s/⇘//g;s/⇛//g;s/\o33//g;s/\[[0|1];[3|4|9][0-9]m//g;s/\[[3|4][0-9]m//g;s/\[[3|4]8;5;[0-2][0-9][0-9]m//g;s/\[[3|4]8;5;[0-9][0-9]m//g;s/\[[3|4]8;5;[0-9]m//g;s/\[[0|5]m//g;s/\[25m//g' | xmessage -timeout 15 -center -file - &" >> $logDir$p1.sh
    lcsub ; echo  "      logp=\$! ; altf=\$currf ; altt=\$currt" >> $logDir$p1.sh
    lcsub ; echo  "    #fi" >> $logDir$p1.sh
  fi
  lcsub ; echo    "    sleep ${updLogInterval}s" >> $logDir$p1.sh
  lcsub ; echo    "  else" >> $logDir$p1.sh
  lcsub ; echo    "    sleep 60" >> $logDir$p1.sh
  lcsub ; echo    "  fi" >> $logDir$p1.sh
  lcsub ; echo    "done" >> $logDir$p1.sh
  lcsub ; echo    "if [ -n \$logp ] && [ \$logp -gt 0 ]; then" >> $logDir$p1.sh
  lcsub ; echo    "  pidx=\$(ps --pid \$logp | grep -c \"xmessage\")" >> $logDir$p1.sh
  lcsub ; echo    "  if [ \$pidx -gt 0 ]; then" >> $logDir$p1.sh
  lcsub ; echo    "    kill \$logp" >> $logDir$p1.sh
  lcsub ; echo    "  fi" >> $logDir$p1.sh
  lcsub ; echo    "fi" >> $logDir$p1.sh
  lcsub ; echo    "rm -f $logDir$p1.ffprobe-* > /dev/null 2> /dev/null" >> $logDir$p1.sh
  lcsub ; echo    "rm -f $logDir$p1.ffmpeg-* > /dev/null 2> /dev/null" >> $logDir$p1.sh
  lcsub ; echo    "rm -f $logDir$p1.sh > /dev/null 2> /dev/null" >> $logDir$p1.sh
  chmod +x $logDir$p1.sh
  #nohup /bin/bash $logDir$p1.sh &>/dev/null &
  nohup $logDir$p1.sh &>/dev/null &
  logpid=$!
  if [ $anzeigeBkupMsg -ge 1 ]; then SubOkErr $(grep "" -c $logDir$p1.sh) $lcval; elif [ $(grep "" -c $logDir$p1.sh) -ne $lcval ]; then echo -e -n "$TRENNER\n${SETBL}Background-Script zur Ermittlung von Frequenz- und Temperaturverlauf anlegen und starten$NOCOL"; SubErr; fi
}

function echoUndLog {
  local tmpstr="$1"
  echo -e "${tmpstr}" ; echo -e "${tmpstr}" >> $logDir$p1.ffmpeg-tmp ; echo -e "${tmpstr}" >> ${logDir}filmschnitt.log
}
        
function felderSchiebenSub { 
  declare -n tmpref="$1"
  declare -n tmpval="$2"
  tmpref="$tmpval"
}

function felderSchieben { # $1=Limit
  if [ $1 -ne 0 ] && [ $1 -le 5 ]; then
    felderSchiebenSub "tmpMAP0" "MAP0"
    felderSchiebenSub "tmpMAP0S" "MAP0S"
    felderSchiebenSub "tmpSUBD0" "SUBD0"
    felderSchiebenSub "tmpSUBD0S" "SUBD0S"
    felderSchiebenSub "tmpSUBE0" "SUBE0"
    felderSchiebenSub "tmpSUBE0S" "SUBE0S"
    for (( idx=0 ; idx<$1 ; idx++ )); do
      idx2=$(( $idx + 1 ))
      felderSchiebenSub "MAP$idx" "MAP$idx2"
      felderSchiebenSub "MAP${idx}S" "MAP${idx2}S"
      felderSchiebenSub "SUBD$idx" "SUBD$idx2"
      felderSchiebenSub "SUBD${idx}S" "SUBD${idx2}S"
      felderSchiebenSub "SUBE$idx" "SUBE$idx2"
      felderSchiebenSub "SUBE${idx}S" "SUBE${idx2}S"
    done
    felderSchiebenSub "MAP$1" "tmpMAP0"
    felderSchiebenSub "MAP${1}S" "tmpMAP0S"
    felderSchiebenSub "SUBD$1" "tmpSUBD0"
    felderSchiebenSub "SUBD${1}S" "tmpSUBD0S"
    felderSchiebenSub "SUBE$1" "tmpSUBE0"
    felderSchiebenSub "SUBE${1}S" "tmpSUBE0S"
  fi
}

function getSizeTime {
  insize=$(ls -l1 "$1" | awk 'END {print int($5/1048576)}')
  intime=$(ls -al1 --time-style=+'%d.%m.%Y/%H:%M' "$1" | awk 'END {print ($6)}')
  insizetime="Datum=$intime, Größe=${insize}MB"
  echo -e "$insizetime"
}

function findDupSub1 {
  sync
  shopt -s nocaseglob 2> /dev/null
  tmpStr2=$(ls -d1 $1$nameAnf2 2> /dev/null)
  shopt -u nocaseglob 2> /dev/null
  tmpStr2=$(echo "$tmpStr2" | sed -e 's/\.mp4 /\.mp4\n/g;s/\.m4v /\.m4v\n/g;s/\.m2t /\.m2t\n/g;s/\.ts /\.ts\n/g;s/\.webm /\.webm\n/g' | grep --exclude-dir="$nameAnf" -ciE "$nameAnf")
  printf "$tmpStr2"
}

function findDupSub2Sub1 {
  echo -e $(echo $tmpStr2 | sed -e 's/\.mp4 /\.mp4\n/g;s/\.m4v /\.m4v\n/g;s/\.m2t /\.m2t\n/g;s/\.ts /\.ts\n/g;s/\.webm /\.webm\n/g' | grep -iE "$nameAnf")
}

function findDupSub2Sub2 {
  read _instr
  local chkcols
  [ "$1" == "" ] && _addstr="$SETBL ($(getSizeTime "$tmpStrI")$SETBL)" || _addstr=""
  _outstr="$_instr$_addstr"
  _addnlx="\n                                                  "
  _addnl="...$_addnlx..."
  chkcols=$(/usr/bin/tput cols) ; [ $chkcols -lt 100 ] && chkcols=100
  if [ ${#_outstr} -gt $(( $chkcols - 5 )) ]; then
    _instr3="${_instr%/*}/"
    _instr3a="${_instr3:0:$(( $chkcols - 20 ))}"
    _instr3b="${_instr3:$(( $chkcols - 20 ))}"
    _instr4="${_instr##*/}"
    _instr4a="${_instr##*/} $_addstr"
    if [ ${#_instr3} -gt $(( $chkcols - 55 )) ]; then
      [ ${#_instr3b} -gt 0 ] && _instr3="$_instr3a$_addnl$_instr3b"
    fi
    if [ ${#_instr4a} -gt $(( $chkcols - 65 )) ]; then
      _instr4="${_instr4:0:$(( $chkcols - 60 ))}${_instr4:$(( $chkcols - 60 ))}$_addnlx$_addstr"
    elif [ ${#_instr4} -gt $(( $chkcols - 65 )) ]; then
      _instr4="${_instr4:0:$(( $chkcols - 60 ))}$_addnl${_instr4:$(( $chkcols - 60 ))} $_addstr"
    else
      _instr4="$_instr4a"
    fi
    _outstr="$_instr3$_addnl$_instr4"
  fi
  echo -e "$_outstr"
}
    
function findDupSub2Sub3 { # $1=tmpLenXa, $2=tmpLenXb, $3=tmpStrI, $4=sendername
  declare -n tmpLenA="$1"
  declare -n tmpLenB="$2"
  tmpLenA=${p1/$4/}
  tmpLenA=${tmpLenA/.mp4/} ; tmpLenA=${tmpLenA/.webm/}
  tmpLenA=${tmpLenA/[-_]HEVC[-_]/_} ; tmpLenA=${tmpLenA/[-_]H264[-_]/_} ; tmpLenA=${tmpLenA/[-_]VP9[-_]/_} ; tmpLenA=${tmpLenA/[-_]AV1[-_]/_} ; tmpLenA=${tmpLenA/[-_]MPEG4[-_]/_}
  tmpLenA=${tmpLenA/[-_]3840x2160[-_]/_} ; tmpLenA=${tmpLenA/[-_]1920x1080[-_]/_} ; tmpLenA=${tmpLenA/[-_]1280x720[-_]/_} ; tmpLenA=${tmpLenA/[-_]960x[0-9][0-9][0-9][-_]/_}
  tmpLenA=${tmpLenA/[-_]720x[0-9][0-9][0-9][-_]/_} ; tmpLenA=${tmpLenA/[-_]480x[0-9][0-9][0-9][-_]/_} ; tmpLenA=${tmpLenA/[-_][12][0-9][0-9][0-9]/_} ; tmpLenA=${tmpLenA/[-_][0-9]h[0-5][0-9]m[0-5][0-9]s/_}
  tmpLenA="${tmpLenA%[_|-]Staffel[1-9]*}"
  tmpLenA="${tmpLenA%[_|-][1-9]bis[2-9]von*}" ; tmpLenA="${tmpLenA%[_|-][1-9]bis[0-9][2-9]von*}" ; tmpLenA="${tmpLenA%[_|-][0-9][1-9]bis[2-9][0-9]von*}" ; tmpLenA="${tmpLenA%[_|-][0-9][1-9]bis[2-9][0-9]von*}"
  tmpLenA="${tmpLenA%[_|-][1-9]bis[2-9]*}"
  tmpLenA="${tmpLenA%[_|-][1-9]und[2-9]*}"
  tmpLenA="${tmpLenA%[_|-][1-9][_|-][0-9|_|-]*}"
  [ ${#sendername1} -gt 0 ] && tmpLenA=$(echo "$tmpLenA" | sed -e "s/$sendername1//g")
  [ ${#sendername2} -gt 0 ] && tmpLenA=$(echo "$tmpLenA" | sed -e "s/$sendername2//g")
  for tmpcnt in {1..10}; do
    tmpLenA=${tmpLenA/[-_][-_]/_}
  done
  tmpLenA=${tmpLenA%[-_]}
  tmpLenA=${tmpLenA##*/}
  tmpLenB=${tmpLenA/[-_]alt[-_][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9]/_}
  for tmpcnt in {1..10}; do
    tmpLenB=${tmpLenB/[-_][-_]/_}
  done
  tmpLenB=${tmpLenB%[-_]}
  tmpLenB=${tmpLenB##*/}
}

function findDupSub2Sub4a {
  if [ $1 -gt $2 ]; then retval=$(( $1 - $2 )); else retval=$(( $2 - $1 )); fi
  echo $retval
}

function findDupSub2Sub4 {
  tmpMaxDiff=3
  tmpMaxDiff2=9
  tmpMinLen=9
  tmpSendername="$sendername"
  sendermarker "$tmpStrI" # gefundener Name ohne Senderkennung
  findDupSub2Sub3 "tmpLen2a" "tmpLen2b" "$tmpStrI" "$sendername"
  tmpDiffa=$(findDupSub2Sub4a ${#tmpLen1a} ${#tmpLen2a})
  tmpDiffb=$(findDupSub2Sub4a ${#tmpLen1b} ${#tmpLen2b})
  sendername="$tmpSendername" # Original-Sendername wird wieder geholt (Prüfung, ob benötigt, noch offen!)
  if [ $tmpDiffa -eq 0 ] || [ $tmpDiffb -eq 0 ]; then retval=3
  elif [ $tmpDiffa -lt $tmpMaxDiff ] && [ $tmpDiffb -lt $tmpMaxDiff ]; then retval=2
  elif [ $tmpDiffa -lt $tmpMaxDiff2 ] && [ $tmpDiffb -lt $tmpMaxDiff2 ]; then retval=1
  elif [ ${#tmpLen1a} -gt $tmpMinLen ] && [ ${#tmpLen2a} -gt $tmpMinLen ] && [ ${#tmpLen1b} -gt $tmpMinLen ] && [ ${#tmpLen2b} -gt $tmpMinLen ]; then retval=0
  else retval=-1
  fi
  return $retval
}

function findDupSub2Sub5 {
  if [ $retval -ge 2 ]; then tmpStrQ="$tmpStrQ${SETTK}\r               $(echo "$tmpStrI" | sed -e 's/^/\Ähnlich Gro\ß-\/Klein\/Umlaut\/Trenner /g' | findDupSub2Sub2)$NOCOL \n"
  else tmpStrQ="$tmpStrQ${SETGR}\r               $(echo "$tmpStrI" | sed -e 's/^/Anders Gro\ß-\/Klein\/Umlaut\/Trenner  /g' | findDupSub2Sub2)$NOCOL \n"
  fi
}

function findDupSub2 {
  sync
  shopt -s nocaseglob 2> /dev/null
  local tmpStr1=($(ls -d1 $1$nameAnf2 2> /dev/null))
  #echo -e $SETOR+1+ $tmpStr1$NOCOL
  shopt -u nocaseglob 2> /dev/null
  idx=0
  tmpDiffa=0
  tmpDiffb=0
  retval=-1
  tmpSendername=
  while [ "${tmpStr1[$idx]}" != "" ] && [ $numNamAnf -lt 9999 ]; do # && [ -f "${tmpStr1[$idx]}" ]
    lebendig
    tmpStr2=${tmpStr1[$idx]}
    #echo -e $SETOR+2+ $tmpStr2$NOCOL
    numNamAnf=$(( $numNamAnf + $(findDupSub1 "$tmpStr2") ))
    tmpStrFunde="$(findDupSub2Sub1)"
    tmpStrFiletype=$(file $tmpStrFunde/ 2> /dev/null)
    findDupSub2Sub3 "tmpLen1a" "tmpLen1b" "$2" "$sendername"
    if [ "$(echo $tmpStrFiletype | awk '{ print $2 }')" == "directory" ]; then # || [ "$( tmpStrFiletype | awk '{ print $3 }')" == "directory" ]; then
      tmpStrI=$(echo "$tmpStrFunde" | grep -iE "$4" )
      findDupSub2Sub4
      if [ ${#tmpStrI} -gt 3 ] && [ $retval -ge 0 ]; then
        if [ "$pcfg2_2" != "" ]; then
          if [ "${pcfg2_2: -1}" == "/" ]; then zielVerz="${pcfg2_2:0: -1}"; else zielVerz="$pcfg2_2"; fi
        elif [ "$pcfg2" != "" ]; then
          if [ "${pcfg2: -1}" == "/" ]; then zielVerz="${pcfg2:0: -1}"; else zielVerz="$pcfg2"; fi
        else
          zielVerz=""
        fi
        zielVerz="${zielVerz##*/}"
        if [ "${tmpStrI: -1}" == "/" ]; then zielRef="${tmpStrI:0: -1}"; else zielRef="$tmpStrI"; fi
        zielRef="${zielRef##*/}"
        if [ "$zielVerz" == "$zielRef" ]; then tmpStrQ="$tmpStrQ${SETBL}\r               $(echo "$tmpStrI" | sed -e 's/^/Zielunterverzeichnis               /g' | findDupSub2Sub2 1)$NOCOL \n"
        else tmpStrQ="$tmpStrQ${SETBL}\r               $(echo "$tmpStrI" | sed -e 's/^/\Ähnliches Unterverzeichnis         /g' | findDupSub2Sub2 1)$NOCOL \n"
        fi
      fi
    else
      tmpp2=${2%$sendername*}; tmpp2=${tmpp2%$namenzusatz*}
      #tmpp2=${tmpp2%_[1-2][0-9][0-9][0-9]*}
      if [ ${#tmpp2} -le 9 ]; then tmpp2=".*$(echo $tmpp2 | sed -e 's/[-_]/\.\*/g;s/\.\.\./\.\*/g;s/\.\*\.\*/\.\*/g').*"
      else tmpp2=".*$(echo $tmpp2 | sed -e 's/[-_][0-9]bis[0-9]von[0-9]/\.\*/g;s/[-_][0-9]und[0-9]von[0-9]/\.\*/g;s/[-_][0-9]bis[0-9]/\.\*/g;s/[-_][0-9]und[0-9]/\.\*/g;s/[-_][0-9][0-9][0-9]/\.\*/g;s/[-_][0-9][0-9]/\.\*/g;s/[-_][0-9]/\.\*/g;s/[-_]/\.\*/g;s/\.\.\./\.\*/g;s/\.\*\.\*/\.\*/g').*"
      fi
      tmpStrI=$(echo "$tmpStrFunde" | grep -E "$2")
      if [ ${#tmpStrI} -gt 3 ]; then tmpStrQ="$tmpStrQ${SETGN}\r               $(echo "$tmpStrI" | sed -e 's/^/Konvertierergebnis                 /g' | findDupSub2Sub2)$NOCOL \n"
      else
        tmpStrI=$(echo "$tmpStrFunde" | grep -iE "$3")
        if [ ${#tmpStrI} -gt 3 ] && [ "${tmpStrI: -4}" == '.m2t' -o "${tmpStrI: -3}" == '.ts' -o "${tmpStrI: -4}" == '.mkv' -o "${tmpStrI: -4}" == '.m2v' -o "${tmpStrI: -4}" == '.m4v' -o "${tmpStrI: -5}" == '.webm' -o "${tmpStrI: -4}" == '.mp4' ]; then
          if [ "$p1pfad" == "$1" ] || [ "${pcfg2##*/}" == "${1%/}" ] || [ "$p1pfad" == "$1$erledigtDir" ] || [ "$p1pfad$erledigtDir" == "$1" ]; then
            if [ $findeDupAlle -gt 0 ]; then tmpStrQ="$tmpStrQ${SETOR}\r               $(echo "$tmpStrI" | sed -e 's/^/Originaldatei                      /g' | findDupSub2Sub2)$NOCOL \n"
            else tmpStrQ="$tmpStrQ${SETGN}\r               $(echo "$tmpStrI" | sed -e 's/^/Originaldatei (wird konvertiert)   /g' | findDupSub2Sub2)$NOCOL \n"
            fi
          else tmpStrQ="$tmpStrQ${SETOR}\r               $(echo "$tmpStrI" | sed -e 's/^/Originaldatei-Kopie                /g' | findDupSub2Sub2)$NOCOL \n"
          fi
        else
          tmpMinLen=9
          tmpStrI=$(echo "$tmpStrFunde" | grep -iE "$tmpp2")
          findDupSub2Sub4
          if [ ${#tmpStrI} -gt 3 ]; then findDupSub2Sub5
          else
            tmpStrI=$(echo "$tmpStrFunde" | grep -iE "$p3")
            findDupSub2Sub4
            if [ ${#tmpStrI} -gt 3 ]; then findDupSub2Sub5
            else
              tmpStrI=$(echo "$tmpStrFunde" | grep -iE "$4")
              findDupSub2Sub4
              if [ ${#tmpStrI} -gt 3 ]; then findDupSub2Sub5; fi
            fi
          fi
        fi
      fi
    fi
    idx=$(( $idx + 1 ))
  done
  if [ "$tmpStrQ" != "" ]; then
    fundeInDir+=("$1")
    if [ $numNamAnf -le $tmpMaxFunde ]; then printf "$tmpStrQ"
    elif [ $numNamAnf -lt 9999 ]; then printf "$tmpStrQ"; numNamAnf=9999
    fi 
  fi 
  tmpStrQ=""
  tmpStrI=
}

function findDupSub3 {
  numNamAnfAlt=$numNamAnf
  numNamAnf=$(( $numNamAnf + $(findDupSub1 "$1") ))
  lsNamAnfSub=$(findDupSub2 "$1" "${ffmpegcmdsub6%.*}$namenzusatz.${ffmpegcmdsub6##*.}" "$p1" "$nameAnf")
}

function findDupSub4Sub1 {
  if [ $durchsucheUnterverzeichnisse -ge 3 ]; then fortschritt="   "
  else
    index=$(( $index + 1 ))
    [ $(( $index % 4 )) -eq 0 ] && fortschritt='-'
    [ $(( $index % 4 )) -eq 1 ] && fortschritt='\\'
    [ $(( $index % 4 )) -eq 2 ] && fortschritt='|'
    [ $(( $index % 4 )) -eq 3 ] && fortschritt='/'
    fortschritt="$SETBGWW$SETGND $fortschritt $SETBGSW$NOCOL"
  fi
  tmphms=$(( $(date +%s) - $tmpzeit ))
  tmph=$(( $tmphms / 3600 ))
  tmpm=$(( $(( $tmphms / 60 )) % 60 )) ; [ $tmpm -le 9 ] && tmpm="0$tmpm"
  tmps=$(( $tmphms % 60 )) ; [ $tmps -le 9 ] && tmps="0$tmps"
  numNamAnfAlt=$numNamAnf
  if [ $numNamAnf -eq 1 ]; then tmpstr1="" ; else tmpstr1="e" ; fi
  tmpPath=$(findePfad "$1")
  if [ $(( $(/usr/bin/tput cols) - 90 )) -lt ${#tmpPath} ]; then tmpPath="${tmpPath:0:$(( $(/usr/bin/tput cols) - 95 ))}*"; fi
  #echo -e -n "\r$SETBGSW$NOCOL$tmpstr\r$SETBL${SETBGGR}               Bisher $numNamAnf Fund$tmpstr1, suche Duplikate in Verzeichnis #$(( $numDirs + 1 )): $(findePfad "$1")/ ...$SETBGSW$NOCOL\r$fortschritt  +${tmph}h${tmpm}m${tmps}s$SETBGSW$NOCOL"
  echo -e -n "\r$SETBGSW$NOCOL$tmpstr\r$SETBL${SETBGGR}               Suche Duplikate in Verzeichnis #$(( $numDirs + 1 )): $tmpPath/ ...$SETBGSW$NOCOL\r$fortschritt  +${tmph}h${tmpm}m${tmps}s$SETBGSW$NOCOL"
  local f1="$(findePfad "$1")"
  local f2=""
  for tmpi2 in "${chkDir[@]}"; do
    f2="$(findePfad "$tmpi2")"
    if [ "$f1" == "$f2" ] || [ "$1" == "$tmpi2" ] || [ "$f1" == "$tmpi2" ] || [ "$1" == "$f2" ]; then return 1; fi
  done
  [ $durchsucheUnterverzeichnisse -ge 3 ] && echo ""
  chkDir+=("$f1")
  numDirs=$(( $numDirs + 1 ))
  findDupSub3 "$f1/"
  if [ $numNamAnf -eq 1 ]; then tmpstr1="" ; else tmpstr1="en" ; fi
  if [ ${#lsNamAnfSub} -gt 35 ]; then 
    echo -e "$lsNamAnfSub" >> ${logDir}filmschnitt.log
    sleep 2s
    findDupStr="\r$SETBGSW$NOCOL$tmpstr\r$lsNamAnfSub\r"
    echo -e "$findDupStr\r$SETBGSW$NOCOL     +${tmph}h${tmpm}m${tmps}s"
  fi
  #return 0
}

function findDupSub4Sub2 {
  if [ "$1" != "" -a -d "$1" ]; then
    findDupSub4Sub1 "$1"
    [ -d "$1$erledigtDir" ] && findDupSub4Sub1 "$1$erledigtDir"
    [ -d "$1$offenDir" ] && findDupSub4Sub1 "$1$offenDir"
    if [ "$pcfg2" != "" -a -d "$1$pcfg2" ]; then
      findDupSub4Sub1 "$1$pcfg2"
      [ -d "$1$pcfg2$erledigtDir" ] && findDupSub4Sub1 "$1$pcfg2$erledigtDir"
      [ -d "$1$pcfg2$offenDir" ] && findDupSub4Sub1 "$1$pcfg2$offenDir"
      if [ "$pcfg2_2" != "" -a -d "$1$pcfg2$pcfg2_2" ]; then
        findDupSub4Sub1 "$1$pcfg2$pcfg2_2"
        [ -d "$1$pcfg2$pcfg2_2$erledigtDir" ] && findDupSub4Sub1 "$1$pcfg2$pcfg2_2$erledigtDir"
        [ -d "$1$pcfg2$pcfg2_2$offenDir" ] && findDupSub4Sub1 "$1$pcfg2$pcfg2_2$offenDir"
      fi
    fi
  fi
}

function findDupSub4 {
  local _p1=$1
  lebendig
  if [ "$1" != "" -a -d "$1" -a $durchsucheUnterverzeichnisse -ge 1 ]; then
    #if [ $durchsucheUnterverzeichnisse -ge 1 ]; then # -a -d "$(findePfad "${_p1##*\*/}")" -a -d "$(findePfad "$_p1")" -a "$(findePfad "${_p1##*\*/}")" == "$(findePfad "$_p1")" ]; then
      findDupSub4Sub2 "$_p1"
      up1a="${_p1%/}"
      up1="${up1a%/*}/"
      if [ "$up1" != "" -a "$up1" != "$baseDir" -a "$up1" != "$(findePfad "$baseDir")" -a "$up1" != "${baseDir%/*}/" -a "$up1" != "$(findePfad "${baseDir%/*}/")" ]; then
        if [ "$_p1" != "$up1" -a ${#up1} -gt 2 -a -d "$up1" ]; then findDupSub4Sub2 "$up1"; fi
      fi
    #fi
  fi
}

function findDupSub5 {
  local _p1=$1
  #local _srcD="$(findePfad $srcDir)"
  local chkDirSub2=()
  if [ "$_p1" != "" -a -d "$_p1" -a "$(findePfad $_p1)" != "$_srcD" -a "$(findePfad $_p1$pcfg2)" != "$_srcD" -a "$(findePfad $_p1$pcfg2$pcfg2_2)" != "$_srcD" ]; then
    findDupSub4 "$_p1"
    if [ "$pcfgup" == "" ] && [ $durchsucheUnterverzeichnisse -eq 2 -o $durchsucheUnterverzeichnisse -eq 4 ]; then
      chkDirSub2="$(find -L "$_p1" -mindepth 1 -maxdepth 5 -type d,l -readable 2> /dev/null | sed 's/$/\//g')"
      chkDirSub2=($chkDirSub2)
      if [ $(echo "$chkDirSub2" | grep -ciE "") -gt 0 ]; then
        if [ "$suche_in_bkup123dir" == "$bkup1dir" -o  "$suche_in_bkup123dir" == "$bkup2dir" -o  "$suche_in_bkup123dir" == "$bkup3dir" ]; then
          if [[ "$_p1" =~ .*"$suche_in_bkup123dir".* ]]; then
            for tmpdirSub2 in "${chkDirSub2[@]}"; do chkDirSub1+=("$tmpdirSub2"); done; unset chkDirSub2
          fi
        elif [ "$suche_in_bkup123dir" == "" ]; then
          for tmpdirSub2 in "${chkDirSub2[@]}"; do chkDirSub1+=("$tmpdirSub2"); done; unset chkDirSub2
        fi
        for tmpdirSub1 in "${chkDirSub1[@]}"; do findDupSub4 "$tmpdirSub1"; done; unset chkDirSub1; chkDirSub1=()
      fi
    elif [ "$pcfgup" == "../" ] && [ $durchsucheUnterverzeichnisse -eq 2 -o $durchsucheUnterverzeichnisse -eq 4 ]; then
      chkDirSub2="$(find -L "$_p1$pcfg2" -mindepth 1 -maxdepth 5 -type d,l -readable 2> /dev/null | sed 's/$/\//g')"
      chkDirSub2=($chkDirSub2)
      if [ $(echo "$chkDirSub2" | grep -ciE "") -gt 0 ]; then
        if [ "$suche_in_bkup123dir" == "$bkup1dir" -o  "$suche_in_bkup123dir" == "$bkup2dir" -o  "$suche_in_bkup123dir" == "$bkup3dir" ]; then
          if [[ "$_p1" =~ .*"$suche_in_bkup123dir".* ]]; then
            for tmpdirSub2 in "${chkDirSub2[@]}"; do chkDirSub1+=("$tmpdirSub2"); done; unset chkDirSub2
          fi
        elif [ "$suche_in_bkup123dir" == "" ]; then
          for tmpdirSub2 in "${chkDirSub2[@]}"; do chkDirSub1+=("$tmpdirSub2"); done; unset chkDirSub2
        fi
        for tmpdirSub1 in "${chkDirSub1[@]}"; do findDupSub4 "$tmpdirSub1"; done; unset chkDirSub1; chkDirSub1=()
      fi
    fi
  fi
}

function findDup {
if [ $durchsucheUnterverzeichnisse -ge 1 ]; then
  chkDirSub2cnt=0
  numDirs=0
  tmpstr=""
  tmpStrQ=""
  numNamAnf=0
  tmpzeit=$(date +%s)
  if [ $durchsucheUnterverzeichnisse -eq 2 -o $durchsucheUnterverzeichnisse -eq 4 ]; then
    if [[ "$suche_in_bkup123dir" == @($bkup1dir|$bkup2dir|$bkup3dir) ]]; then
      chkDirSub2cnt=$(find -L "$suche_in_bkup123dir" -mindepth 1 -maxdepth 5 -type d,l -readable 2> /dev/null | sed 's/$/\//g' | grep -ciE "")
      echoUndLog "${SETOR}komplette Duplikatsuche nur in '$suche_in_bkup123dir' ($chkDirSub2cnt Unterverzeichnisse), ansonsten nur Hauptpfade und ausgewählte Unterverzeichnisse.$NOCOL"
    elif [ "$pcfgup" == "../" -a "$_srcD$pcfgup$pcfg2" != "" ]; then
      echoUndLog "${SETOR}komplette Duplikatsuche nur in Hauptpfaden und ausgewählten Unterverzeichnissen.$NOCOL"
    else
      chkDirSub2cnt=$(find -L "$PWD" -mindepth 1 -maxdepth 5 -type d,l -readable 2> /dev/null | sed 's/$/\//g' | grep -ciE "")
      echoUndLog "${SETGN}komplette Duplikatsuche in allen aktiven Backup-Verzeichnissen und ihren $chkDirSub2cnt Unterverzeichnissen.$NOCOL"
    fi
  fi
  for (( index=1 ; index<$(/usr/bin/tput cols) ; index=index+1 )); do tmpstr="$tmpstr "; done
  findDupSub4 "$srcDir"
  [ "$backupDir1K" != "" ] && findDupSub5 "$backupDir1K"
  if [ "$backupDir2K" != "" -a -d "$backupDir2K" -a "$_bD2K" != "$_bD1K" ]; then
    findDupSub5 "$backupDir2K"
  fi
  if [ "$backupDir3K" != "" -a -d "$backupDir3K" -a "$_bD3K" != "$_bD1K" -a "$_bD3K" != "$_bD2K" ]; then
    findDupSub5 "$backupDir3K"
  fi
  if [ "$backupDir1S" != "" -a -d "$backupDir1S" -a "$_bD1S" != "$_bD1K" ]; then
    findDupSub5 "$backupDir1S"
  fi
  if [ "$backupDir2S" != "" -a -d "$backupDir2S" -a "$_bD2S" != "$_bD1S" -a "$_bD2S" != "$_bD1K" ]; then
    findDupSub5 "$backupDir2S"
  fi
  if [ "$backupDir3S" != "" -a -d "$backupDir3S" -a "$_bD3S" != "$_bD1S" -a "$_bD3S" != "$_bD1K" ]; then
    if [ "$_bD3S" != "$_bD1S" -a "$_bD3S" != "$_bD2S" ]; then
      if [ "$_bD3S" != "$_bD1K" -a "$_bD3S" != "$_bD2K" -a "$_bD3S" != "$_bD3K" ]; then
        findDupSub5 "$backupDir3S"
      fi
    fi
  fi
  if [ "$destDir" != "" -a -d "$destDir" -a "$_dstD" != "$_bD1S" -a "$_dstD" != "$_bD2S" -a "$_dstD" != "$_bD3S" ]; then
    if [ "$_dstD" != "$_bD1K" -a "$_dstD" != "$_bD2K" -a "$_dstD" != "$_bD3K" ]; then
      if [ "$_dstD" != "$_srcD" -a "$(findePfad $destDir$pcfg2)" != "$_srcD" -a "$(findePfad $destDir$pcfg2$pcfg2_2)" != "$_srcD" ]; then
        findDupSub4 "$destDir"
        findDupSub4 "$destDir$bkup1tmpdir"
        findDupSub4 "$destDir$bkup2tmpdir"
      fi
    fi
  fi
  if [ "$konvertierenDir" != "" -a -d "$konvertierenDir" -a "$_konD" != "$_dstD" -a "$_konD" != "$_bD1K" ]; then
    if [ "$_konD" != "$_bD2K" -a "$_konD" != "$_bD3K" -a "$_konD" != "$_bD1S" ]; then
      if [ "$_konD" != "$_bD2S" -a "$_konD" != "$_bD3S" -a "$_konD" != "$_srcD" ]; then
        if [ "$(findePfad $konvertierenDir$pcfg2)" != "$_srcD" -a "$(findePfad $konvertierenDir$pcfg2$pcfg2_2)" != "$_srcD" ]; then
          findDupSub4 "$konvertierenDir"
        fi
      fi
    fi
  fi
  if [ "$schneidenDir" != "" -a -d "$schneidenDir" -a "$_schD" != "$_konD" -a "$_schD" != "$_dstD" -a "$_schD" != "$_bD1K" ]; then
    if [ "$_schD" != "$_bD2K" -a "$_schD" != "$_bD3K" -a "$_schD" != "$_bD1S" ]; then
      if [ "$_schD" != "$_bD2S" -a "$_schD" != "$_bD3S" ]; then
        if [ "$_schD" != "$_srcD" -a "$(findePfad $schneidenDir$pcfg2)" != "$_srcD" -a "$(findePfad $schneidenDir$pcfg2$pcfg2_2)" != "$_srcD" ]; then
          findDupSub4 "$schneidenDir"
        fi
      fi
    fi
  fi
  if [ $durchsucheUnterverzeichnisse -eq 2 -o $durchsucheUnterverzeichnisse -eq 4 ] && [ ${#chkDirSub1[@]} -gt 0 ]; then
    for tmpdirSub1 in "${chkDirSub1[@]}"; do findDupSub4 "$tmpdirSub1"; done
  fi
  chkDirSub1=()
  tmphms=$(( $(date +%s) - $tmpzeit ))
  tmph=$(( $tmphms / 3600 ))
  tmpm=$(( $(( $tmphms / 60 )) % 60 )) ; [ $tmpm -le 9 ] && tmpm="0$tmpm"
  tmps=$(( $tmphms % 60 )) ; [ $tmps -le 9 ] && tmps="0$tmps"
  [ $durchsucheUnterverzeichnisse -ge 3 ] && echo ""
  [ $numNamAnf -gt $tmpMaxFunde ] && numNamAnfAdd=">= " || numNamAnfAdd=""
  if [ $numNamAnf -eq 1 ]; then tmpstr1="" ; else tmpstr1="en" ; fi
  echoUndLog "\r$SETBGSW$NOCOL$tmpstr\r$SETBL${SETBGGR}In $(( $numDirs + 1 )) Verzeichnissen wurde nach Duplikaten (\"$nameAnf2\") gesucht, Dauer der Suche: ${tmph}h${tmpm}m${tmps}s$SETBGSW$NOCOL"
  chkDir=()
fi
sleep 1s
}

function findStream { #$1=StreamTyp 
  tmp_xstr=$1
  tmp_xstr=${tmp_xstr#    Stream }
  if [ "$tmp_xstr" != "" ]; then tmp_xstr=$(echo -e "$tmp_xstr" | sed -e "s/    Stream /$addsp/g"); fi
}

function logStream { # $1=StreamName, $2=StreamInfoString
  if [ "$2" != "" ]; then echoUndLog "$NOCOL      $SETBL$1-Stream:$NOCOL $2 ${SETGN}enthalten"
  else echoUndLog "$NOCOL      $SETRT$1-Stream fehlt"
  fi
}

function splitstr { # $1=string1, $2=string2, $3=string3, $4=Format1, $5=Format2, $6=Addstr
  estrx="$1$2$3"
  if [ ${#estrx} -gt $numcols ]; then
    estrx="$1$2"
    if [ ${#estrx} -gt $numcols ]; then
      estrx="$2$3"
      if [ ${#esfindDupSub2trx} -gt $numcols ]; then
        estrx="$4$1$5$6$4$2$5$6$4$3$5"
      else
        estrx="$4$1$5$6$4$2$3$5"
      fi
    else
      estrx="$4$1$2$5$6$4$3$5"
    fi
  else
    estrx="$4$1$2$3$5"
  fi
  echo -e -n "$estrx"
}

function mvNachXxx { # $1=$p1, $2=$backupDir{1|2}{K|S}, $3=Zielname, $4=Zielpfad
  if [ -d "$2" -a -f "$2$1" ]; then
    if [ "$4" != "" -a -d "$4" -a -f "$4$3" ] || [ "$4" == "" -a -f "$3" ]; then
      echo -e "$NOCOL *  $SETRT'$NOCOL$2$1$SETRT\n    wird nicht nach '$NOCOL$4$3$SETRT' verschoben - Datei existiert schon!$NOCOL"
    else
      echo -e -n "$NOCOL *  ${SETBL}Verschiebe '$NOCOL$2$1$SETBL'\n    nach '$NOCOL$4$3$SETBL' "
      [ "$4" == "" -o -d "$4" ] || mkdir -p "$4"
      local tmpp="$(findePfad $2)"
      tmpp="$(df -l $tmpp 2> /dev/null)"
      if [ "$tmpp" != "" ] || [ $rsyncstattcpmv -eq 0 ]; then # lokales FS => verwende mv
        echo -e -n " - mv "
        mv "$2$1" "$4$3" 2> /dev/null # > /dev/null
        tmpret=$?
      else
        echo -e -n " - rsync\n    "
        rsync -IPat --info=progress2 --remove-source-files "$2$1" "$4$3" 2> /dev/null
        tmpret=$?
        echo -e -n "   "
      fi
      sleep 1s
      SubOkErr $tmpret 0
    fi
  fi
}

function mvNachErledigt { # $1=$p1, $2=$erledigtDir
  mvNachXxx "$1" "$2" "$1" "$2$erledigtDir"
}

function mvNachOffen { # $1=$p1, $2=$offenDir
  mvNachXxx "$1" "$2" "$1" "$2$offenDir"
}

function mvUmbenennen { # $1=$p1, $2=$backupDir{1|2}{K|S}
  if [ "$3" != "" -a "$1" != "$3" ]; then mvNachXxx "$1" "$2" "$3" "$2" ; else mvNachXxx "$1" "$2" "$1-alt-$(date +%s)" "$2" ; fi
}

function bkupSub2 { #$1=destDir, $2=Erweiterung
  cnt1a1=$(ls -1 ./ | grep -c -E "\.$2" 2> /dev/null)
  cnt1a=$(( $cnt1a + $cnt1a1 ))
  if [ $cnt1a1 -gt 0 ]; then
    echo -e -n "$NOCOL *  ${SETBL}Verschiebe '$NOCOL*.$2$SETBL' aus '$NOCOL$PWD$SETBL'\n    nach '$NOCOL$1$SETBL' "
    tmpp="$(findePfad $1)"
    tmpp="$(df -l $tmpp 2> /dev/null)"
    if [ "$tmpp" != "" ] || [ $rsyncstattcpmv -eq 0 ]; then # lokales FS => verwende mv
      echo -e -n " - mv "
      nice -n $prioselect find ./ -mindepth 1 -maxdepth 1 -name "*.$2" -exec mv '{}' $1 \;
    else
      echo -e -n " - rsync\n    "
      nice -n $prioselect find ./ -mindepth 1 -maxdepth 1 -name "*.$2" -exec rsync -IPat --info=progress2 --remove-source-files '{}' $1 \;
    fi
    cnt2a=$(( $cnt2a + $(ls -1 ./ | grep -c -E "\.$2" 2> /dev/null) ))
    cnt3a=$(( $cnt3a + 1 ))
  fi
}

function bkupSub3 { #$1=destDir, $2=destDirNam, $3=bkupscript
  cnt1a=0
  cnt2a=0
  cnt3a=0
  bkupSub2 "$1" "mp4"
  bkupSub2 "$1" "m4v"
  bkupSub2 "$1" "m2t"
  bkupSub2 "$1" "ts"
  bkupSub2 "$1" "webm"
  cnt1=$(( $cnt1 + $cnt1a ))
  cnt2=$(( $cnt2 + $cnt2a ))
  if [ $cnt3a -gt 0 ]; then
    [ $rsyncstattcpmv -ne 0 ] && echo -e -n "   "
    if [ $cnt1a -ge 0 ] && [ $cnt2a -eq 0 ]; then
      SubOk
      [ $anzeigeBkupMsg -gt 1 ] && echo -e -n "$NOCOL *  ${SETBL}Script $NOCOL$3$SETBL löschen "
      clrFilesSub "$3"
      [ $anzeigeBkupMsg -gt 1 ] && SubOkErr $? 0
      sync
      sleep 10s
      if [ "$destDirRm" != "" ]; then
        if [ -d "$destDirRm" ]; then
          tmpSubdir0="$(pwd)"
          tmpSubdir0="${tmpSubdir0%/}"
          tmpSubdir0="${tmpSubdir0##*/}"
          #echo "$destDirRm - $tmpSubdir0"
          if [ -d "$destDirRm$tmpSubdir0" ]; then
            $(sleep 10s ; if [ $(ls -1 $destDirRm$tmpSubdir0 | grep -c -E '') -eq 0 ]; then cd $destDirRm ; rmdir -p "$tmpSubdir0" > /dev/null 2> /dev/null ; fi) &
          fi
          sleep 6s
          tmpSubdir1="${destDirRm%/}"
          tmpSubdir1="${tmpSubdir1%/*}"
          tmpSubdir2="${destDirRm%/}"
          tmpSubdir2="${tmpSubdir2##*/}"
          #echo "$tmpSubdir1 - $tmpSubdir2"
          $(sleep 30s ; if [ $(ls -1 $destDirRm | grep -c -E '') -eq 0 ]; then cd $tmpSubdir1 ; sleep 30s ; rmdir -p "$tmpSubdir2" > /dev/null 2> /dev/null ; fi) &
        fi
      elif [ $(ls -1 ./ | grep -c -E '') -eq 0 ]; then
        [ $anzeigeBkupMsg -gt 1 ] && echo -e -n "$NOCOL *  ${SETBL}Verzeichnis $NOCOL$2$SETBL ist leer, löschen "
        #cd ..
        [ "$2" != "./" -a -d $2 ] && rmdir -p $2 > /dev/null 2> /dev/null
        [ $anzeigeBkupMsg -gt 1 ] && SubOkErr $? 0
      else
        [ $anzeigeBkupMsg -ge 1 ] && echo -e "$NOCOL *  ${SETGE}Verzeichnis $NOCOL$2$SETGE ist nicht leer! ${SETGE}Rest=$cnt2a$NOCOL"
      fi
      sync
    else SubErr
    fi
    sync
    sleep 1s
  fi
}

function bkupSub4 { #$1=bkupziel
  if [ ! -d "$1" ]; then
    [ $anzeigeBkupMsg -gt 1 ] && echo -e -n "$NOCOL *  ${SETBL}Verzeichnis '$NOCOL$1$SETBL' anlegen "
    mkdir -p "$1" 2> /dev/null
    tmpstat=$?
    [ $anzeigeBkupMsg -gt 1 ] && SubOkErr $tmpstat 0
    [ $anzeigeBkupMsg -le 1 -a $tmpstat -gt 0 ] && $(echo -e -n "$NOCOL *  ${SETBL}Verzeichnis '$NOCOL$1$SETBL' anlegen " ; SubOkErr $tmpstat 0)
  else
    [ $anzeigeBkupMsg -gt 1 ] && echo -e "$NOCOL *  ${SETBL}Verzeichnis '$NOCOL$1$SETBL' existiert "
    [ -d "$1" ] && chmod +w "$1"
  fi
  sync
  sleep 1s
}

function bkupSub5 { #$1=String, $2=DateiLink
  [ $anzeigeBkupMsg -eq 1 ] && echo -e "$NOCOL *  ${SETBL}Script '$NOCOL$2$SETBL'$1$NOCOL"
  [ $anzeigeBkupMsg -eq 2 ] && echo -e "$NOCOL *  $SETBGGR${SETBL}Script '$SETWS$2$SETBL'$1:$NOCOL$SETBGSW\n$(echo -e "$(cat $2)" | sed 's/^/    \\033[38;5;123m\\033[48;5;237m/g;s/[^[:alnum:]]*/&\\033[40m\\033[0m\\033[25m...\n    ...\\033[38;5;123m\\033[48;5;237m/480;s/[^[:alnum:]]*/&\\033[40m\\033[0m\\033[25m...\n    ...\\033[38;5;123m\\033[48;5;237m/360;s/[^[:alnum:]]*/&\\033[40m\\033[0m\\033[25m...\n    ...\\033[38;5;123m\\033[48;5;237m/240;s/[^[:alnum:]]*/&\\033[40m\\033[0m\\033[25m...\n    ...\\033[38;5;123m\\033[48;5;237m/120;s/$/\\033[40m\\033[0m\\033[25m/g')\n$TRENNER\r    "
}

function bkupSub { #$1=bkupdir, $2=destDir, $3=filename, $4=myscript, $5=bkuptmpdir, $6=bkupscript, $7=bkupsubdir, $8=bkupsubsubdir, $9=namenzusatz
  if [ "$1" != "" ]; then
    local p78="$7$8"; local p78a="${p78%%/}"; local p78b="${p78a%%/*}"; p78a="${p78a#*/}"
    if [ "$p78" != "" ] && [ "$pcfg3" == '/' ] && [ ! -d "$1$p78" ]; then skipsub=1
    else skipsub=0
    fi
    echo -e "$NOCOL    ${TRENNER:4}"
    # nicht abarbeiten, wenn mehr als eine "filmschnitt.sh"-Instanz läuft!
    if [ "$6" != "" ] && [ "$5" != "" ]; then
      sleep 1s
      if [ $skipsub -eq 0 ]; then
        bkupSub4 "$2$5"
        [ "$p78" != "" ] && bkupSub4 "$2$5$p78"
      fi
      if [ 0 ] && [ $skipsub -eq 0 ] && [ ! -f $2$5$p78$6 ]; then
        [ $anzeigeBkupMsg -gt 1 ] && echo -e -n "$NOCOL *  ${SETBL}Script '$NOCOL$2$5$p78$6$SETBL' anlegen "
        lcval=0
        lcsub ; echo "#!/bin/bash" > "$2$5$p78$6"
        lcsub ; echo "cnt1=0 ; cnt2=0; scrnam=\"$6\"; erldir=$erledigtDir; dstbase=\"$2$5\"; dstbase2=\"$1\"; dstpfad=\"$p78\"" >> "$2$5$p78$6"
        lcsub ; echo "function _sub_ {" >> "$2$5$p78$6"
        lcsub ; echo "  cnt1=\$(( \$cnt1 + \$(find ./ -mindepth 1 -maxdepth 1 -path \"./*\" -type f | grep -c -E \"\.\$1\") ))" >> "$2$5$p78$6"
        lcsub ; echo "  pushd \${dstbase}\${dstpfad} > /dev/null 2> /dev/null" >> "$2$5$p78$6"
        lcsub ; echo "  if [ \$(find ./ -mindepth 1 -maxdepth 1 -path \"./*\" -type f | grep -c -E "\.\$1") -gt 0 ]; then" >> "$2$5$p78$6"
        lcsub ; echo "    find ./ -mindepth 1 -maxdepth 1 -name \"*.\$1\" -exec echo -e \"$NOCOL *  ${SETBL}Datei verschieben aus \\\"$NOCOL\${dstbase}\${dstpfad}$SETBL\\\"$NOCOL\n    ${SETBL}nach \\\"$NOCOL\${dstbase2}\${dstpfad}$SETBL\\\": $SETGN'{}'$NOCOL\" \;" >> "$2$5$p78$6"
        lcsub ; echo "    if [ $anzeigeBkupMsg -ge 1 ]; then" >> "$2$5$p78$6"
        lcsub ; echo "      find ./ -mindepth 1 -maxdepth 1 -name \"*.\$1\" -exec rsync -IPat --info=progress2 --remove-source-files -b --suffix=-alt-\$(date +%s) $2$5\${dstpfad}'{}' \"\${dstbase2}\${dstpfad}\" \;" >> "$2$5$p78$6"
        lcsub ; echo "    else" >> "$2$5$p78$6"
        lcsub ; echo "      find ./ -mindepth 1 -maxdepth 1 -name \"*.\$1\" -exec mv -b --suffix=-alt-\$(date +%s) \${dstbase}\${dstpfad}'{}' \"\${dstbase2}\${dstpfad}\" \;" >> "$2$5$p78$6"
        lcsub ; echo "    fi" >> "$2$5$p78$6"
        lcsub ; echo "  fi" >> "$2$5$p78$6"
        lcsub ; echo "  popd > /dev/null 2> /dev/null" >> "$2$5$p78$6"
        lcsub ; echo "  sleep 1s ; sync ; cnt2=\$(( \$cnt2 + \$(find ./ -mindepth 1 -maxdepth 1 -path \"./*\" -type f | grep -c -E \"\.\$1\") ))" >> "$2$5$p78$6"
        lcsub ; echo "}" >> "$2$5$p78$6"
        lcsub ; echo "" >> "$2$5$p78$6"
        lcsub ; echo "if [ -d \"\${dstbase2}\" ] && [ \"\${dstpfad}\" != \"\" ] && [ \"$pcfg3\" = \"+\" ]; then" >> "$2$5$p78$6"
        lcsub ; echo "  [ -d \"\${dstbase2}\${dstpfad}\" ] && chmod +w \"\${dstbase2}\${dstpfad}\" || mkdir -p \"\${dstbase2}\${dstpfad}\" 2> /dev/null; [ \$? -ne 0 ] && exit -4" >> "$2$5$p78$6"
        lcsub ; echo "fi" >> "$2$5$p78$6"
        lcsub ; echo "if [ -d \"\${dstbase2}\${dstpfad}\" ]; then" >> "$2$5$p78$6"
        lcsub ; echo "  _sub_ \"mp4\" ; _sub_ \"m4v\" ; _sub_ \"m2t\" ; _sub_ \"ts\" ; _sub_ \"webm\"" >> "$2$5$p78$6"
        lcsub ; echo "  #if [ \"${pcfg,,}\" == \"-schneiden\" -a -f \"$srcDir$p1\" ]; then altdatei=\"$srcDir$p1\"" >> "$2$5$p78$6"
        lcsub ; echo "  #elif [ \"${pcfg,,}\" != \"-umbenennen\" -a -f \"$srcDir$p1\" ]; then altdatei=\"$srcDir$p1\"" >> "$2$5$p78$6"
        lcsub ; echo "  #else altdatei=\"\"" >> "$2$5$p78$6"
        lcsub ; echo "  #fi" >> "$2$5$p78$6"
        lcsub ; echo "  #[ \"\$altdatei\" != \"\" ] && (mkdir -p \"\${dstbase2}\${erldir}\" 2> /dev/null; [ \$? -ne 0 ] && exit -3 || mv \"\$altdatei\" \"\${dstbase2}\${erldir}\")" >> "$2$5$p78$6"
        lcsub ; echo "  \$(sleep 5s; cd \"\${dstbase}\"; [ -f \"\${dstbase}\${dstpfad}\${scrnam}\" ] && \$(sleep 3s;rm -f \"\${dstbase}\${dstpfad}\${scrnam}\" > /dev/null 2> /dev/null)) &" >> "$2$5$p78$6"
        lcsub ; echo "  \$(sleep 10s; cd \"\${dstbase}\"; cd ..; rmdir -p \"\${dstbase}\${dstpfad}\" > /dev/null 2> /dev/null) &" >> "$2$5$p78$6"
        lcsub ; echo "fi" >> "$2$5$p78$6"
        lcsub ; echo "[ \$cnt1 -le 0 ] && exit -1" >> "$2$5$p78$6"
        lcsub ; echo "[ \$cnt2 -lt 0 ] && exit -2" >> "$2$5$p78$6"
        lcsub ; echo "[ \$cnt2 -eq 0 ] && exit 0" >> "$2$5$p78$6"
        lcsub ; echo "[ \$cnt2 -lt 127 ] && exit \$cnt2 || exit 127" >> "$2$5$p78$6"
        [ $anzeigeBkupMsg -gt 1 ] && SubOkErr $(grep "" -c $2$5$p78$6) $lcval
        sleep 2s
        chmod +x "$2$5$p78$6"
        [ $anzeigeBkupMsg -ge 1 ] && bkupSub5 ' erzeugt' $2$5$p78$6
      else
        [ $skipsub -eq 0 -a $anzeigeBkupMsg -ge 1 ] && echo -e "$NOCOL *  ${SETBL}Script '$NOCOL$2$5$p78$6$SETBL' existiert$NOCOL"
      fi
    fi
    pushd $2$5$p78 > /dev/null 2> /dev/null
    if [ $skipsub -eq 0 ]; then
      if [ ! -f "$2$3" -a "$9" == "" ]; then
        echo -e "$NOCOL *  ${SETRT}Datei $NOCOL\"$2$3\"$SETRT existiert nicht. "
      elif [ "$9" != "" -a ! -f "$2${3%.*}$9.${3##*.}" ]; then
        echo -e "$NOCOL *  $NOCOL\"$2${3%.*}$9.${3##*.}\"$SETRT existiert nicht. "
      else
        local retval1=0
        local retval2=0
        local retval3=0
        local namenstr2=""
        #local nameadd="${3%.*}$9.${3##*.}"
        #local nameadd="${3%.*}$9-alt-$(date +%s).${3##*.}"
        local nameadd="${3%.*}$9.${3##*.}-alt-$(date +%s)"
        if [ "$9" != "" ]; then
          echo -e -n "$SETGN$SETBGGR"
          #[ -f "$2${3%.*}$9.${3##*.}" ] && mvUmbenennen "${3%.*}$9.${3##*.}" "$2" "$nameadd"
          #[ -f "$2$5$7${3%.*}$9.${3##*.}" ] && mvUmbenennen "${3%.*}$9.${3##*.}" "$2$5$7" "$nameadd"
          #[ -f "$2$5$7$8${3%.*}$9.${3##*.}" ] && mvUmbenennen "${3%.*}$9.${3##*.}" "$2$5$7$8" "$nameadd"
          [ -f "$1$7${3%.*}$9.${3##*.}" ] && mvUmbenennen "${3%.*}$9.${3##*.}" "$1$7" "$nameadd"
          [ -f "$1$7$8${3%.*}$9.${3##*.}" ] && mvUmbenennen "${3%.*}$9.${3##*.}" "$1$7$8" "$nameadd"
          echo -e -n "$SETBGSW$NOCOL"
        fi
        if [ -f "$2$3" ]; then
          $(ln "$2$3" "$2$5$p78${3%.*}$9.${3##*.}" > /dev/null 2> /dev/null); retval2=$?
          if [ $retval2 -eq 0 ]; then
            [ $anzeigeBkupMsg -ge 1 ] && echo -e -n "$NOCOL *  ${SETBL}Verlinke '$NOCOL$2$3$SETBL'\n    nach '$NOCOL$2$5$p78${3%.*}$9.${3##*.}$SETBL'$SETAL1 "
          else
            [ $anzeigeBkupMsg -ge 1 ] && echo -e -n "$NOCOL *  ${SETBL}Link nicht möglich, verschiebe '$NOCOL$2$3$SETBL'\n    nach '$NOCOL$2$5$p78${3%.*}$9.${3##*.}$SETAL1' "
            local tmpp="$(findePfad $2)"
            tmpp="$(df -l $tmpp 2> /dev/null)"
            if [ "$tmpp" != "" ] || [ $rsyncstattcpmv -eq 0 ]; then # lokales FS => verwende cp
              [ $anzeigeBkupMsg -ge 1 ] && echo -e -n " - cp "
              $(cp "$2$3" "$2$5$p78${3%.*}$9.${3##*.}" 2> /dev/null) ; retval2=$? # > /dev/null
            else
              [ $anzeigeBkupMsg -ge 1 ] && echo -e -n " - rsync\n    "
              $(rsync -IPa --info=progress2 "$2$3" "$2$5$p78${3%.*}$9.${3##*.}" 2> /dev/null) ; retval2=$? # > /dev/null  --remove-source-files
            fi
          fi
        elif [ "$9" != "" -a ! -f "$2${3%.*}$9.${3##*.}" ]; then
          $(ln "$2${3%.*}$9.${3##*.}" "$2$5$p78${3%.*}$9.${3##*.}" > /dev/null 2> /dev/null); retval2=$?
          if [ $retval2 -eq 0 ]; then
            [ $anzeigeBkupMsg -ge 1 ] && echo -e -n "$NOCOL *  ${SETBL}Verlinke '$NOCOL$2${3%.*}$9.${3##*.}$SETBL'\n    nach '$NOCOL$2$5$p78$SETBL'$SETAL1 "
          else
            [ $anzeigeBkupMsg -ge 1 ] && echo -e -n "$NOCOL *  ${SETBL}Link nicht möglich, verschiebe '$NOCOL$2${3%.*}$9.${3##*.}$SETBL'\n    nach '$NOCOL$2$5$p78$SETAL1' "
            if [ $rsyncstattcpmv -eq 0 ]; then
              [ $anzeigeBkupMsg -ge 1 ] && echo -e -n " - cp "
              $(cp "$2${3%.*}$9.${3##*.}" "$2$5$p78${3%.*}$9.${3##*.}" 2> /dev/null) ; retval2=$? # > /dev/null
            else
              [ $anzeigeBkupMsg -ge 1 ] && echo -e -n " - rsync\n    "
              $(rsync -IPat --info=progress2 "$2${3%.*}$9.${3##*.}" "$2$5$p78${3%.*}$9.${3##*.}" 2> /dev/null) ; retval2=$? # > /dev/null  --remove-source-files
            fi
          fi
        elif [ "$9" != "" -a -f "$2${3%.*}$9.${3##*.}" ]; then
          [ -d "$destDir$offenDir" ] || mkdir -p "$destDir$offenDir"
          local tmpp="$(findePfad $2)"
          tmpp="$(df -l $tmpp 2> /dev/null)"
          [ $anzeigeBkupMsg -ge 1 ] && echo -e -n "$NOCOL *  ${SETBL}Verschiebe '$NOCOL$2${3%.*}$9.${3##*.}$SETBL'\n    nach '$NOCOL$2$5$p78$SETAL1' "
          if [ "$tmpp" != "" ] || [ $rsyncstattcpmv -eq 0 ]; then # lokales FS => verwende cp
            [ $anzeigeBkupMsg -ge 1 ] && echo -e -n " - cp "
            $(cp "$2${3%.*}$9.${3##*.}" "$2$5$p78${3%.*}$9_$(date +%s).${3##*.}" 2> /dev/null) ; retval3=$? # > /dev/null
          else
            [ $anzeigeBkupMsg -ge 1 ] && echo -e -n " - rsync\n    "
            $(rsync -IPat --info=progress2 "$2${3%.*}$9.${3##*.}" "$2$5$p78${3%.*}$9_$(date +%s).${3##*.}" 2> /dev/null) ; retval3=$? # > /dev/null  --remove-source-files
          fi
          if [ $retval3 -eq 0 ]; then
            [ $anzeigeBkupMsg -ge 1 ] && echo -e "$NOCOL *  ${SETAL1}'$NOCOL$2$5$p78${3%.*}$9.${3##*.}$SETAL1'\n    existiert schon und kann nicht umbenannt/verschoben werden!$SETAL1' "
          else
            $(ln "$2${3%.*}$9.${3##*.}" "$2$5${p78%.*}$9.${3##*.}" > /dev/null 2> /dev/null) ; retval3=$?
            if [ $retval3 -eq 0 ]; then
              [ $anzeigeBkupMsg -ge 1 ] && echo -e -n "$NOCOL *  ${SETBL}Verlinke '$NOCOL$2${3%.*}$9.${3##*.}$SETBL'\n    nach '$NOCOL$2$5$p78${3%.*}$9.${3##*.}$SETBL'$SETAL1 "
            else
              [ $anzeigeBkupMsg -ge 1 ] && echo -e -n "$NOCOL *  ${SETBL}Link nicht möglich, kopiere '$NOCOL${3%.*}$9.${3##*.}$SETBL'\n    nach '$NOCOL$2$5$p78$SETAL1' "
              local tmpp="$(findePfad $2)"
              tmpp="$(df -l $tmpp 2> /dev/null)"
              if [ "$tmpp" != "" ] || [ $rsyncstattcpmv -eq 0 ]; then # lokales FS => verwende cp
                echo -e -n " - cp "
                $(cp --preserve=all -u -p "$2${3%.*}$9.${3##*.}" "$2$5$p78${3%.*}$9.${3##*.}" 2> /dev/null) ; retval3=$? # > /dev/null
              else
                echo -e -n " - rsync\n    "
                $(rsync -IPat --info=progress2 "$2${3%.*}$9.${3##*.}" "$2$5$p78${3%.*}$9.${3##*.}" 2> /dev/null) ; retval3=$? # > /dev/null
              fi
            fi
          fi
        fi
        local retval=$(( $retval1 + $retval2 + $retval3 ))
        if [ $retval -eq 0 ]; then
          if [ $anzeigeBkupMsg -ge 1 ]; then
            [ $rsyncstattcpmv -ne 0 ] && echo -e -n "   "
            SubOk
          fi
          bkupCnt2=$(( $bkupCnt2 + 1 ))
          local tmppcfg2=$pcfg2$pcfg2_2
          tmppcfg2=${tmppcfg2#/}
          tmppcfg2=${tmppcfg2%/}
          if [ ${#tmppcfg2} -gt 0 ] && [ "$tmppcfg2" != "./" ] && [[ "$tmppcfg2" != "./"* ]]; then
            if [ "${tmppcfg2#*/}" != "${tmppcfg2%/*}" ]; then
              cd ../..
            else
              cd ..
            fi
          fi
          echo -e "$NOCOL *  ${SETBL}Zum Verschieben in '$NOCOL$PWD/$SETBL' und Unterverzeichnissen bereitliegende Datei(en):$NOCOL"
          [ $anzeigeBkupMsg -ge 2 -a "$(find ./ -mindepth 0 -maxdepth 2 -path "./[\~\._\-\@0-9a-fA-f]*" -type d)" != "" ] && echo -e "$SETBL$(find ./ -mindepth 0 -maxdepth 2 -path "./[\~\._\-\@0-9a-fA-f]*" -type d | sed -e 's/^/    *  /;s/$/\//')/$NOCOL"
          [ "$(find ./ -mindepth 3 -maxdepth 3 -path "./[\~\._\-\@0-9a-fA-f]*" -type d)" != "" ] && echo -e "$SETRT$(find ./ -mindepth 3 -maxdepth 3 -path "./[\~\._\-\@0-9a-fA-f]*" -type d | sed -e 's/^/    *  (>2 Verzeichnisebenen) /;s/$/\//')/$NOCOL"
          [ "$(find ./ -mindepth 1 -maxdepth 1 -path "./*" -type f)" != "" ] && echo -e "$SETGN$(find ./ -mindepth 1 -maxdepth 1 -path "./*" -type f -printf "%h/%f (%kkB)\n" | grep -E '\.mp4|\.m4v|\.m2t|\.ts|\.webm' | sed -e 's/^/    *  /;s/.\///')$NOCOL"
          [ $anzeigeBkupMsg -ge 1 -a "$(find ./ -mindepth 1 -maxdepth 1 -path "./*" -type f)" != "" ] && echo -e "$SETTK$(find ./ -mindepth 1 -maxdepth 1 -path "./*" -type f -printf "%h/%f (%kkB)\n" | grep -E '\.sh' | sed -e 's/^/    *  /;s/.\///')$NOCOL"
          [ $anzeigeBkupMsg -ge 2 -a "$(find ./ -mindepth 1 -maxdepth 1 -path "./*" -type f)" != "" ] && echo -e "$SETRT$(find ./ -mindepth 1 -maxdepth 1 -path "./*" -type f -printf "%h/%f (%kkB)\n" | grep -Ev '\.mp4|\.m4v|\.m2t|\.ts|\.webm|\.sh' | sed -e 's/^/    *  (unerwartete Datei) /;s/.\///')$NOCOL"
          if [ $(find ./ -mindepth 2 -maxdepth 3 -path "./*" | grep -c "" 2> /dev/null) -gt 0 ]; then
            [ "$(find ./ -mindepth 2 -maxdepth 2 -path "./*" -type f)" != "" ] && echo -e "$SETGN$(find ./ -mindepth 2 -maxdepth 2 -path "./*" -type f -printf "%h/%f (%kkB)\n" | grep -E '\.mp4|\.m4v|\.m2t|\.ts|\.webm' | sed -e 's/^/    *  /;s/.\///;s/\//\/...\n       /')$NOCOL"
            [ $anzeigeBkupMsg -ge 1 -a "$(find ./ -mindepth 2 -maxdepth 2 -path "./*" -type f)" != "" ] && echo -e "$SETTK$(find ./ -mindepth 2 -maxdepth 2 -path "./*" -type f -printf "%h/%f (%kkB)\n" | grep -E '\.sh' | sed -e 's/^/    *  /;s/.\///;s/\//\/...\n       /')$NOCOL"
            [ $anzeigeBkupMsg -ge 2 -a "$(find ./ -mindepth 2 -maxdepth 2 -path "./*" -type f)" != "" ] && echo -e "$SETRT$(find ./ -mindepth 2 -maxdepth 2 -path "./*" -type f -printf "%h/%f (%kkB)\n" | grep -Ev '\.mp4|\.m4v|\.m2t|\.ts|\.webm|\.sh' | sed -e 's/^/    *  (unerwartete Datei) /;s/.\///;s/\//\/...\n        /')$NOCOL"
            if [ $(find ./ -mindepth 3 -maxdepth 4 -path "./*" | grep -c "" 2> /dev/null) -gt 0 ]; then
              [ "$(find ./ -mindepth 3 -maxdepth 4 -path "./*" -type f)" != "" ] && echo -e "$SETGN$(find ./ -mindepth 3 -maxdepth 3 -path "./*" -type f -printf "%h/%f (%kkB)\n" | grep -E '\.mp4|\.m4v|\.m2t|\.ts|\.webm' | sed -e 's/^/    *  /;s/.\///;s/\//\/...\n       /2')$NOCOL"
              [ $anzeigeBkupMsg -ge 1 -a "$(find ./ -mindepth 3 -maxdepth 3 -path "./*" -type f)" != "" ] && echo -e "$SETTK$(find ./ -mindepth 3 -maxdepth 3 -path "./*" -type f -printf "%h/%f (%kkB)\n" | grep -E '\.sh' | sed -e 's/^/    *  /;s/.\///;s/\//\/...\n       /2')$NOCOL"
              [ $anzeigeBkupMsg -ge 2 -a "$(find ./ -mindepth 3 -maxdepth 4 -path "./*" -type f)" != "" ] && echo -e "$SETRT$(find ./ -mindepth 3 -maxdepth 4 -path "./*" -type f -printf "%h/%f (%kkB)\n" | grep -Ev '\.mp4|\.m4v|\.m2t|\.ts|\.webm|\.sh' | sed -e 's/^/    *  (unerwartete Datei) /;s/.\///;s/\//\/...\n        /2')$NOCOL"
            fi
          fi
          cd $tmppcfg2
          # Achtung: Bei mehreren parallel laufenden "filmschnitt.sh" ist hier ein Konflikt möglich, darum folgende Prüfung...
          numinst=$(ps | grep -c "${pcmd##*/}")
          altlogcnt=$logcnt
          logevtcnt=0
          while [ $logcnt -gt 0 ]; do
            echo -e -n "${SETAL1}${logevtcnt}s: Es laufen $(( $logcnt + 1 )) Instanzen von \"${pcmd##*/}\" in Backup-Phase, warte 10s$NOCOL"
            logevtcnt=$(( $logevtcnt + 10 ))
            [ $altlogcnt -eq $logcnt ] && echo -e -n "\r" || echo -e ""
            altlogcnt=$logcnt
            sleep 10
            numinst=$(ps | grep -c "${pcmd##*/}")
            logcnt=$(ls -al $logDir*.ffprobe-log1 | grep -c ".ffprobe-log1")
            [ "$logcnt" == "" ] && logcnt=0
            logcnt2=$(ls -al $logDir/$p1.ffprobe-log1 | grep -c ".ffprobe-log1")
            [ "$logcnt2" == "" ] && logcnt2=0
            logcnt=$(( $logcnt - $logcnt2 ))
          done
          if [ -d "$1" ]; then
            if [ "$pcfg3" == "+" ]; then
              if [ ! -d "$1$7" ] && [ "$7" != "" ]; then
                [ $anzeigeBkupMsg -gt 1 ] && echo -e -n "$NOCOL    ${TRENNER:4}\n *  ${SETBL}Unterverzeichnis $NOCOL$7$SETBL anlegen $NOCOL"
                mkdir -p "$1$7" 2> /dev/null
                status=$?
                [ $anzeigeBkupMsg -gt 1 ] && SubOkErr $status 0 1
                if [ ! -d "$1$7$8" ] && [ "$8" != "" ]; then
                  [ $anzeigeBkupMsg -gt 1 ] && echo -e -n "$NOCOL    ${TRENNER:4}\n *  ${SETBL}Unterverzeichnis $NOCOL$7$8$SETBL anlegen $NOCOL"
                  mkdir -p "$1$7$8" 2> /dev/null
                  status=$?
                  [ $anzeigeBkupMsg -gt 1 ] && SubOkErr $status 0 1
                else 
                  chmod -R +w "$1$7$8"
                  status=0
                fi
              else 
                chmod -R +w "$1$7"
                status=0
              fi
            else
              if [ -d "$1$7" ]; then 
                chmod -R +w "$1$7"
                status=0
                if [ -d "$1$7$8" ]; then 
                  chmod -R +w "$1$7$8"
                  status=0
                else status=-2
                fi
              else status=-1
              fi
            fi
            cnt1=0 ; cnt2=0 ; cnt1a=0 ; cnt2a=0
            if [ $Konvertierergebnis -ge 1024 ]; then echo -e " *  ${SETRT}Ergebnis vermutlich unbrauchbar, Videostream fehlt!$NOCOL"
            elif [ $Konvertierergebnis -ge 512 ]; then echo -e " *  ${SETRT}Ergebnis vermutlich unbrauchbar, Audiostream fehlt!$NOCOL"
            elif [ $Konvertierergebnis -ge 256 ]; then echo -e " *  ${SETRT}Ergebnis vermutlich unbrauchbar, erhebliche Längenabweichung!$NOCOL"
            elif [ $status -eq 0 ]; then
              bkupCnt=$(( $bkupCnt + 1 ))
              cd $2$5 > /dev/null 2> /dev/null
              sleep 1s
              if [ "$p78" != "" -a -d "$p78" ]; then
                cd $2$5$p78 > /dev/null 2> /dev/null
                bkupSub3 "$1$p78" "$p78" "$2$5$p78$6"
                cd $2$5 > /dev/null 2> /dev/null
                rmdir -p "$p78"
              fi
              [ "$pcfgup" != "../" ] && bkupSub3 "$1" "$5" "$2$5$6"
              cd $2 > /dev/null 2> /dev/null
              rmdir -p "$5"
              if [ $Konvertierergebnis -ge 64 ]; then
                echoUndLog " *  ${SETOR}Ergebnis möglicherweise unbrauchbar, manuell prüfen!$NOCOL"
              fi
              if [ $(find ./ -maxdepth 4 -name "*" | grep -c -e "schiebe_Dateien_nach_bkup" 2> /dev/null) -gt 0 ]; then
                lcval=0
                lcsub ; echo "#!/bin/bash" > "${logDir}schiebe_alle_Dateien_$p1.sh"
                lcsub ; echo "if [ -d \"$PWD/\" ]; then" >> "${logDir}schiebe_alle_Dateien_$p1.sh"
                lcsub ; echo "  cd \"$PWD/\"" >> "${logDir}schiebe_alle_Dateien_$p1.sh"
                lcsub ; echo "  #!/bin/bash" > "${logDir}schiebe_alle_Dateien_${p1}_Sub.sh"
                lcsub ; echo "  echo \"\$1\" >> ${logDir}schiebe_alle_Dateien_$p1.sh" >> "${logDir}schiebe_alle_Dateien_${p1}_Sub.sh"
                echo -e "$NOCOL *  ${SETBL}Verschiebe $NOCOL*.mp4$SETBL / $NOCOL*.m4v$SETBL / $NOCOL*.m2t$SETBL / $NOCOL*.ts$SETBL / $NOCOL*.webm$SETBL aus $NOCOL$PWD/*/$SETBL\n    nach $NOCOL$1 ..."
                chmod +x "${logDir}schiebe_alle_Dateien_${p1}_Sub.sh"
                find ./ -maxdepth 4 -name "*schiebe_Dateien_nach_bkup*" -exec "${logDir}schiebe_alle_Dateien_${p1}_Sub.sh" '{}' \;
                echo -e "$NOCOL *  ${SETBL}gefundene Scripte:$NOCOL\n$(cat ${logDir}schiebe_alle_Dateien_$p1.sh|sed '1,3d;s/^/    /g')"
                echo -e "$NOCOL *  ${SETBL}gefundene Scripte werden ausgeführt...$NOCOL"
                lcsub ; echo "  \$( sleep 1s ; [ -f \"${logDir}schiebe_alle_Dateien_$p1.sh\" ] && \$(rm -f \"${logDir}schiebe_alle_Dateien_$p1.sh\" > /dev/null 2> /dev/null)) &" >> "${logDir}schiebe_alle_Dateien_${p1}_Sub.sh"
                lcsub ; echo "  \$( sleep 1s ; [ -f \"${logDir}schiebe_alle_Dateien_${p1}_Sub.sh\" ] && \$(rm -f \"${logDir}schiebe_alle_Dateien_${p1}_Sub.sh\" > /dev/null 2> /dev/null)) &" >> "${logDir}schiebe_alle_Dateien_${p1}_Sub.sh"
                lcsub ; echo "  #sleep 1s" >> "${logDir}schiebe_alle_Dateien_${p1}_Sub.sh"
                lcsub ; echo "fi" >> "${logDir}schiebe_alle_Dateien_$p1.sh"
                chmod +x "${logDir}schiebe_alle_Dateien_$p1.sh"
                ${logDir}schiebe_alle_Dateien_$p1.sh
              fi
              cd $(findePfad "$1")
              if [ "$4" != "" ]; then
                if [ -f "$1$4" ]; then
                  if [ "${p78b:0:3}" == "000" ]; then
                    cd $p78b
                  fi
                  cd $(findePfad $PWD)
                  $1$4 # $PWD # ${1%%/}
                else
                  if [ -f "$HOME/$4" ]; then
                    if [ "${p78b:0:3}" == "000" ]; then
                      cd $p78b
                    fi
                    cd $(findePfad $PWD)
                    $HOME/$4 # $PWD
                  else
                    echo -e "${SETRT}Script \"$4\" nicht gefunden in \"$1$p78\", \"$1\" und \"$HOME/\"!$NOCOL"
                  fi
                fi
              fi
            else echo -e " *  ${SETRT}Zielverzeichnis $NOCOL$1$p78$SETRT existiert nicht! Abbruch der Verschiebesequenz.$NOCOL"
            status=-2
            fi
          else
            echo -e "$NOCOL *  ${SETAL1}$1 ist z.Z. nicht erreichbar!$NOCOL"
            status=-2
          fi
        else
          [ $anzeigeBkupMsg -ge 1 ] && SubErr
          status=-2
        fi
      fi
    else
      echo -e "$NOCOL *  ${SETGE}Sequenzabbruch für '$NOCOL$1$SETGE', '$NOCOL$7$SETGE' existiert nicht und soll nicht angelegt werden.$NOCOL"
      status=0
    fi
    popd > /dev/null 2> /dev/null
  elif [ ! -d "$1" ]; then echo -e "$NOCOL *  ${SETAL1}$1 ist z.Z. nicht erreichbar!$NOCOL"; status=-1
  fi
  return $status
}

function bkupXSub {    
  declare -n bkupXdir="bkup${1}dir"
  declare -n statusX="status${1}"
  declare -n bkupXscript="bkup${1}script"
  declare -n bkupXdirstr="bkup${1}dirstr"
  declare -n bkupXdirnam="bkup${1}dirnam"
  declare -n bkupXtmpdir="bkup${1}tmpdir"
  declare -n destDirXRm="destDir${1}Rm"
  if [ "$bkupXdir" != "" ]; then
    if [ "$bkupXdirstr" != "" ] && [ -d "$bkupXdir" ] && [ ! -d "$bkupXdirstr" ]; then echo -e -n "$NOCOL    ${TRENNER:4}\n$bkupXdirstr" ; mkdir -p "$bkupXdirnam" ; SubOkErr $? 0; fi
    destDirRm="$destDirXRm"
    bkupSub "$bkupXdir" "$destDir" "$ffmpegcmdsub6" "$myscript" "$bkupXtmpdir" "$bkupXscript" "$pcfg2" "$pcfg2_2" "$namenzusatz"
    statusX=$?
    #[ $statusX -ne 0 ] && echo "bkupSub: Fehlercode=$statusX"
    sleep 5s
  fi
}

function bkupZielSub {    
  declare -n bkupXsel="bkup${1}sel"
  declare -n bkupXdir="bkup${1}dir"
  if [ "$bkupXsel" != "" ] && [ $bkupXsel -eq 1 ]; then
    if [ "$bkupXdir" == "" ]; then
      echo -e "${SETRT}Backup-Ziel $1 nicht gesetzt$NOCOL"; bkupXsel=0
    #else
    #  if [ ! -d "$bkupXdir" ]; then
    #    echo -e "${SETRT}Backup-Ziel $1: '$NOCOL$bkupXdir$SETRT' nicht erreichbar$NOCOL"
    #  #else
    #  #  echo -e "${SETGN}Backup-Ziel $1: '$NOCOL$bkupXdir$SETGN' aktiv$NOCOL"
    #  fi
    fi
  else
    bkupXsel=0; echo -e "${SETRT}Backup-Ziel $1: '$NOCOL$bkupXdir$SETRT' deaktiviert$NOCOL"; bkupXdir=""
  fi
}

function getFTMinMedMax {
  cputmpftval=$(cat $logDir$p1.freq-log | grep "$1" | cut -c$2-$3 | awk 'BEGIN {v1=0;v2=9999;v3=0;v4=0} {v1=v1+1;v3=v3+$1;if ($1>v4) v4=$1;if ($1<v2) v2=$1} END {v3=v3/v1;print int(v2) "/" int(v3) "/" int(v4)}')
}

function prepName {
  # Achtung: Reihenfolge wichtig!
  nameAnf=$1
  # entferne Sonderzeichen, Umlaute
  nameAnf=$(echo "$nameAnf" | sed -e 's/#/_/g;s/ /_/g;s/?/\.\*/g;s/!/\.\*/g;s/[ÄäÖöÜüß]/\.\*/g;s/Ae/\.\*/g;s/ae/\.\*/g;s/Oe/\.\*/g;s/oe/\.\*/g;s/Ue/\.\*/g;s/ue/\.\*/g;s/ss/\.\*/g')
  nameAnf=$(echo $nameAnf | sed -e 's/_-_/-/g;s/_-/-/g;s/-_/-/g;s/--/-/g;s/__/_/g')
  # entferne Codec-Infos, Bildformat, Längeninfo
  nameAnf="${nameAnf/_[0-9]h[0-5][0-9]m[0-5][0-9]s/_.*}"
  nameAnf="${nameAnf/_HEVC/_.*}"
  nameAnf="${nameAnf/_H264/_.*}"
  nameAnf="${nameAnf/_MPEG/_.*}"
  nameAnf="${nameAnf/_VP9/_.*}"
  nameAnf="${nameAnf/_AV1/_.*}"
  nameAnf="${nameAnf/_H265/_.*}"
  nameAnf=$(echo "$nameAnf" | sed -e 's/_[1|3][8|9][2|4]0x[1|2][0|1][6|8]0/_\.\*/g;s/_1[0-9][0-9][0-9]x[0-9][0-9][0-9]/_\.\*/g;s/_[1-9][0-9][0-9]x[1-9][0-9][0-9]/_\.\*/g')
  nameAnf=$(echo "$nameAnf" | sed -e 's/[-_][1-2][0-9][0-9][0-9][-_]/_\.\*_/g;s/[-|_]Video[-|_]/_*_/g;s/[-|_]OMU[-|_]/_*_/g')
  for ((index=0 ; index<${#streamaddr[@]} ; index=index+$streamaddrsize )); do
    lebendig
    sendername1=${streamaddr[$index+$idxSendername_DVBT]}
    sendername2=${streamaddr[$index+$idxSendername_Mediathek]}
    if [ "$sendername1" != "@@@@@@@@@@" ] && [ "$sendername1" != "" ] && [[ "${nameAnf}" == *"$sendername1"* ]]; then
      kennung_gefunden=1
      nameAnf="${nameAnf%%$sendername1*}_\.\*"
      break
    fi
    if [ "$sendername2" != "@@@@@@@@@@" ] && [ "$sendername2" != "" ] && [[ "${nameAnf}" == *"$sendername2"* ]]; then
      kennung_gefunden=2
      nameAnf="${nameAnf%%$sendername2*}_\.\*"
      break
    fi
  done
  nameAnf=$( Namenfilter $nameAnf )
  if [ $kennung_gefunden -eq 0 ]; then
    # entferne Mediathek-Endungen
    nameAnf="${nameAnf%%-([0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9])*}"
    nameAnf="${nameAnf%%_([0-9][0-9][0-9][0-9]-[0-9][0-9][0-9][0-9])_*}"
    nameAnf="${nameAnf%%-([0-9][0-9][0-9][0-9][0-9][0-9])_*}"
  fi
  # entferne Erweiterung
  nameAnf="${nameAnf%%.mp4}"
  nameAnf="${nameAnf%%.m4v}"
  nameAnf="${nameAnf%%.m2t}"
  nameAnf="${nameAnf%%.ts}"
  nameAnf="${nameAnf%%.mkv}"
  nameAnf="${nameAnf%%.webm}"
  # entferne Staffel- und Episodennummern und Episodentitel
  nameAnf=${nameAnf/[-_][0-9][0-9]und[0-9][0-9]von[0-9][0-9]*/_\.\*}
  nameAnf=${nameAnf/[-_][0-9]und[0-9][0-9]von[0-9][0-9]*/_\.\*}
  nameAnf=${nameAnf/[-_][0-9]und[0-9]von[0-9][0-9]*/_\.\*}
  nameAnf=${nameAnf/[-_][0-9]und[0-9]von[0-9]*/_\.\*}
  nameAnf=${nameAnf/[-_][0-9][0-9]bis[0-9][0-9]von[0-9][0-9]*/_\.\*}
  nameAnf=${nameAnf/[-_][0-9]bis[0-9][0-9]von[0-9][0-9]*/_\.\*}
  nameAnf=${nameAnf/[-_][0-9]bis[0-9]von[0-9][0-9]*/_\.\*}
  nameAnf=${nameAnf/[-_][0-9]bis[0-9]von[0-9]*/_\.\*}
  nameAnf=${nameAnf/[-_][0-9][0-9]und[0-9][0-9]*/_\.\*}
  nameAnf=${nameAnf/[-_][0-9]und[0-9][0-9]*/_\.\*}
  nameAnf=${nameAnf/[-_][0-9]und[0-9]*/_\.\*}
  nameAnf=${nameAnf/[-_][0-9][0-9]bis[0-9][0-9]*/_\.\*}
  nameAnf=${nameAnf/[-_][0-9]bis[0-9][0-9]*/_\.\*}
  nameAnf=${nameAnf/[-_][0-9]bis[0-9]*/_\.\*}
  nameAnf=${nameAnf/[-_][0-9][0-9]von[0-9][0-9]*/_\.\*}
  nameAnf=${nameAnf/[-_][0-9]von[0-9][0-9]*/_\.\*}
  nameAnf=${nameAnf/[-_][0-9]von[0-9]*/_\.\*}
  nameAnf=${nameAnf/[-_][0-9][0-9]_[0-9][0-9][-_]*/_\.\*_}
  nameAnf=${nameAnf/[-_][0-9]_[0-9][0-9][-_]*/_\.\*_}
  nameAnf=${nameAnf/[-_][0-9]_[0-9][-_]*/_\.\*_}
  nameAnf=${nameAnf/[-_][0-9][-_]/_\.\*_}
  nameAnf=${nameAnf/[-_][0-9][0-9][-_]/_\.\*_}
  nameAnf=${nameAnf/_und_/_\.\*_}
  nameAnf=${nameAnf/_&_/_\.\*_}
  nameAnf="${nameAnf}_.*"
  # entferne doppelte Platzhalter
  nameAnf=$(echo "$nameAnf" | sed -e 's/_-_/-/g;s/_-/-/g;s/-_/-/g;s/--/-/g;s/__/_/g')
  nameAnf="${nameAnf%%[-_]\.\*[a-zA-Z0-9]*}"
  # entferne Wildcards
  nameAnf=$(echo ".*$nameAnf.*" | sed -e 's/_/\.\*/g;s/-/\.\*/g;s/.\*_/.\*/g;s/_.\*/.\*/g;s/.\*-/.\*/g;s/-.\*/.\*/g;s/\.\*\.\*\.\*/\.\*/g;s/\.\*\.\*/\.\*/g;s/\.\*$//;s/^\*//')
  nameAnf2="$(echo "$nameAnf*" | sed -e 's/\.\*/\*/g;s/_/\*/g;s/-/\*/g')"
  nameAnf2="$(echo "$nameAnf2" | sed -e 's/\*\*/\*/g')"
}

## ================================================================================================================
# Konvertiersequenz-Anfang
# Audio-Video Standardoptionen - werden evtl. später durch Auto-Erkennung oder andere Vorgaben ersetzt
if [ "$pcfg" != "-umbenennen" ] && [ "$pcfg" != "" ]; then echo -e "\n$TRENNER\n$SETBL$scriptversioninfokomplett$NOCOL\n$TRENNER"; else  echo -e "\n$TRENNER\n$AufrufStr2\n$TRENNER"; fi
logpid=0
pidexists=0
if [ ! -d "$baseDir" ]; then echo -e "${SETRT}Verzeichnis '$NOCOL$baseDir$SETRT' existiert nicht, Abbruch"; exit 1
elif [ ! -d "$destDir" ]; then echo -e "${SETRT}Verzeichnis '$NOCOL$destDir$SETRT' existiert nicht, Abbruch"; exit 1
elif [ ! -d "${baseDir}tmp-log" ]; then echo -e "${SETRT}Verzeichnis '$NOCOL${baseDir}tmp-log$SETRT' existiert nicht, Abbruch"; exit 1
fi
if [ 0 -eq $# ]; then
  philfe=0
  p0=${VMOD}
  hilfetext
  echo -e "$SETBL$versionshistorie$NOCOL"
  infoHd 2 2 2 2 2
  addstr="Funktionstest: "
  p6=$(( $CpuLimitDef * $NumCPUs ))
  HiLoad=0
  indstatusstring
else
  hilfekurzsyntax
  philfe=0
  aindef
  vindef
  specialaaclatm=0
  numastreams=0
  ONAM=
  p0=
  p1=
  p2=
  px2=
  p3=
  px3=
  p4=
  p5=
  p6=
  HiLoad=0
  ASEL=$FAVAUD
  SSEL=
  VSEL=
  b_HEVC=0
  b_H264=0
  b_MPEG4=0
  b_VP8=0
  b_VP9=0
  b_AV1=0
  b_M2V=0
  b_AAC_LATM=0
  a_AAC_LATM_LC=0
  b_AAC_LATM_LC=0
  a_AAC_LATM_HEAAC=0
  b_AAC_LATM_HEAAC=0
  b_AAC=0
  b_AC3=0
  b_EAC3=0
  b_VORBIS=0
  b_MP3=0
  b_MP2=0
  b_OPUS=0
  MAP0=
  MAP1=
  MAP2=
  MAP3=
  MAP4=
  MAP5=
  MAP6=
  MAP7=
  MAP8=
  MAP9=
  MAP10=
  SUBD1=
  SUBD2=
  SUBD3=
  SUBD4=
  SUBD5=
  SUBD6=
  SUBD6=
  SUBD7=
  SUBD8=
  SUBD9=
  SUBD10=
  SUBD1S=
  SUBD2S=
  SUBD3S=
  SUBD4S=
  SUBD5S=
  SUBD6S=
  SUBD7S=
  SUBD8S=
  SUBD9S=
  SUBD10S=
  SUBE1=
  SUBE2=
  SUBE3=
  SUBE4=
  SUBE5=
  SUBE6=
  SUBE7=
  SUBE8=
  SUBE9=
  SUBE10=
  SUBE1S=
  SUBE2S=
  SUBE3S=
  SUBE4S=
  SUBE5S=
  SUBE6S=
  SUBE7S=
  SUBE8S=
  SUBE9S=
  SUBE10S=
  subdec=
  subenc=
  XSTR=
  XSTRD=
  XSTRE=
  XSTRSD=
  XSTRSE=
  XSTRSDS=
  XSTRSES=
  strvin=
  strain=
  is1staudiostream=-1
  istAacLatmLc=0
  ersetzeDatei=0
  VSTRN=-1
  ASTR1N=-1
  ASTR2N=-1
  ASTR3N=-1
  ASTR4N=-1
  ASTR5N=-1
  ASTR6N=-1
  SSTR1N=-1
  SSTR2N=-1
  SSTR3N=-1
  SSTR4N=-1
  AUD0T=
  AUD1T=
  AUD2T=
  AUD3T=
  AUD4T=
  AUD5T=
  AUD6T=
  AUD7T=
  AUD8T=
  AUD9T=
  AUD10T=
  IASP=
  QASP=
  mediathekstream=0
  findeDupAlle=1
  Konvertierergebnis=0
  tmpct1=0
  tmpca2=0
  tmpca3=0
  bkup1sel=
  bkup2sel=
  bkup3sel=
  statusHome=
  statusLog=
  statusSrc=
  statusTmp=
  statusDest1=
  statusDest2=
  statusDest3=
  echo -n ""  >> ${logDir}filmschnitt.log
  verticalFilter=""
  px=$1
  p6=$(( $CpuLimitDef * $NumCPUs ))
  redlvl=0
  CpuLimitDefneu=$p6
  p0=${VMOD,,}
  convaaclatm=0
  nicht_Schieben=0
  py=$2
  zusatzIn=""
  zusatzOut=""
  tmpMaxFunde=25
  numNamAnf=0
  numNamAnfAlt=0
  p1neu=""
  einruecken
  parcnt=0
  p2dec="000"
  p3dec="000"
  destDirRm=""
  destDir1Rm=""
  destDir2Rm=""
  destDir3Rm=""
  chkDir=()
  chkDirSub1=()
  while [ "$px" != "" ]; do
    lebendig
    px=$1
    py=$2
    pz=$3
    parcnt=$(( $parcnt + 1 ))
    if [[ "${px,,}" == "-hevc"* ]] || [[ "${px,,}" == "-h264"* ]] || [[ "${px,,}" == "-vp9"* ]] || [[ "${px,,}" == "-av1"* ]]; then
      px="${px,,}"
      if [[ "$px" == "-hevc"* ]]; then zielVCodec="hevc"
      elif [[ "$px" == "-h264"* ]]; then zielVCodec="h264"
      elif [[ "$px" == "-vp9"* ]]; then zielVCodec="vp9"
      elif [[ "$px" == "-av1"* ]]; then zielVCodec="av1"
      fi
      if [ "$zielOptimierung" == "" ]; then
        if [[ "$px" == *"_br" ]]; then zielOptimierung="br"
        elif [[ "$px" == *"_std" ]]; then zielOptimierung="std"
        elif [[ "$px" == *"_opt" ]]; then zielOptimierung="opt"
        elif [[ "$px" == "-vp9"* ]] || [[ "$px" == "-av1"* ]]; then zielOptimierung="std"
        elif [[ "$px" == "-hevc"* ]] || [[ "$px" == "-h264"* ]]; then zielOptimierung="opt"
        fi
      else
        px="-${zielVCodec,,}_${zielOptimierung,,}"
      fi
    fi
    px="${px,,}"
    case "$px" in
    # ####################################################
    # Ziel mit 1-2 Unterverzeichnissen (funktioniert nur als erster Parameter!)
      "-subdir") [ $parcnt -gt 1 ] && echo -e "$NOCOL$TRENNER\n${SETOR}'-subdir ...' ist nicht erster Parameter und wird hier ignoriert!$SETBL"
                 # -subdir wurde schon früher bewertet
                 if [ "${py:0:1}" == "-" ]; then shift
                 elif [ "${pz:0:1}" == "-" ]; then shift 2
                 else shift 3
                 fi ;;
    # ####################################################
    # Backupverzeichnis-Auswahl
      "-bkup1"|"-pfad1") fullCmdLine+=("$px")
                [ "${px,,}" != "-pfad1" ] && bkup1sel=1
                if [ "$py" != "" -a "${py:0:1}" != "-" ]; then
                  fullCmdLine+=("'$py'")
                  destDir="${destDir}tmp_$(date +'%y-%m-%d_%H-%M')/"
                  destDir1Rm="$destDir"
                  bkup1dir=$(echo -e "$py" | sed -e 's/^_//;s/^\.\.\./_\.\.\./;s/ /_/g;s/_(/_/g;s/)_/_/g;s/___/-/g;s/__/_/g;s/--/-/g;s/_-/-/g;s/-_/-/g' | zeichenersetzen)
                  bkup1dir="$bkup1dir/"
                  bkup1script="schiebe_Dateien_nach_bkup1dir.sh"
                  bkup1tmpdir="bkup1tmp/"
                  if [ "${py:0:1}" != "/" ]; then
                    [ -d "$baseDir$bkup1dir" ] && echo -e "$NOCOL$TRENNER\n$SETGN\$bkup1dir zeigt auf '$NOCOL$bkup1dir$SETGN' im Basisverzeichnis '$NOCOL$baseDir$SETGN'$NOCOL" || echo -e "$NOCOL$TRENNER\n$SETRT\$bkup1dir zeigt auf in '$NOCOL$baseDir$SETRT'nicht erreichbares '$NOCOL$bkup1dir$SETRT'$NOCOL"
                    bkup1dir="$baseDir$bkup1dir"
                  else
                    [ -d "$bkup1dir" ] && echo -e "$NOCOL$TRENNER\n$SETGN\$bkup1dir zeigt auf '$NOCOL$bkup1dir$SETGN'$NOCOL" || echo -e "$NOCOL$TRENNER\n$SETRT\$bkup1dir zeigt auf nicht erreichbares '$NOCOL$bkup1dir$SETRT'$NOCOL"
                  fi
                  bkup1dirnam="$bkup1dir$pcfg2$pcfg2_2"
                  if [ "$pcfg3" == "+" -a ! -d "$bkup1dir$pcfg2$pcfg2_2" ]; then
                    bkup1dirstr="$NOCOL *  ${SETBL}Unterverzeichnis $NOCOL$bkup1dir$pcfg2$pcfg2_2$SETBL anlegen$NOCOL"
                    if [ "$bkup1dirstr" != "" ] && [ ! -d "$bkup1dir" ] && [ "$statusDest1" != "" ] && [ $statusDest1 -eq 1 ]; then echo -e -n "$bkup1dirstr" ; mkdir -p "$bkup1dirnam" 2> /dev/null ; SubOkErr $? 0; fi
                  fi
                  shift
                fi
                shift ;;
      "-bkup2"|"-pfad2") fullCmdLine+=("$px")
                [ "${px,,}" != "-pfad2" ] && bkup2sel=1
                if [ "$py" != "" -a "${py:0:1}" != "-" ]; then
                  fullCmdLine+=("'$py'")
                  destDir="${destDir}tmp_$(date +'%y-%m-%d_%H-%M')/"
                  destDir2Rm="$destDir"
                  bkup2dir=$(echo -e "$py" | sed -e 's/^_//;s/^\.\.\./_\.\.\./;s/ /_/g;s/_(/_/g;s/)_/_/g;s/___/-/g;s/__/_/g;s/--/-/g;s/_-/-/g;s/-_/-/g' | zeichenersetzen)
                  bkup2dir="$bkup2dir/"
                  bkup2script="schiebe_Dateien_nach_bkup2dir.sh"
                  bkup2tmpdir="bkup2tmp/"
                  if [ "${py:0:1}" != "/" ]; then
                    [ -d "$baseDir$bkup2dir" ] && echo -e "$NOCOL$TRENNER\n$SETGN\$bkup2dir zeigt auf '$NOCOL$bkup2dir$SETGN' im Basisverzeichnis '$NOCOL$baseDir$SETGN'$NOCOL" || echo -e "$NOCOL$TRENNER\n$SETRT\$bkup2dir zeigt auf in '$NOCOL$baseDir$SETRT'nicht erreichbares '$NOCOL$bkup2dir$SETRT'$NOCOL"
                    bkup2dir="$baseDir$bkup2dir"
                  else
                    [ -d "$bkup2dir" ] && echo -e "$NOCOL$TRENNER\n$SETGN\$bkup2dir zeigt auf '$NOCOL$bkup2dir$SETGN'$NOCOL" || echo -e "$NOCOL$TRENNER\n$SETRT\$bkup2dir zeigt auf nicht erreichbares '$NOCOL$bkup2dir$SETRT'$NOCOL"
                  fi
                  bkup2dirnam="$bkup2dir$pcfg2$pcfg2_2"
                  if [ "$pcfg3" == "+" -a ! -d "$bkup2dir$pcfg2$pcfg2_2" ]; then
                    bkup2dirstr="$NOCOL *  ${SETBL}Unterverzeichnis $NOCOL$bkup2dir$pcfg2$pcfg2_2$SETBL anlegen$NOCOL"
                    if [ "$bkup2dirstr" != "" ] && [ ! -d "$bkup2dir" ] && [ "$statusDest2" != "" ] && [ $statusDest2 -eq 1 ]; then echo -e -n "$bkup2dirstr" ; mkdir -p "$bkup2dirnam" 2> /dev/null ; SubOkErr $? 0; fi
                  fi
                  shift
                fi
                shift ;;
      "-bkup3"|"-pfad3") fullCmdLine+=("$px")
                [ "${px,,}" != "-pfad3" ] && bkup3sel=1
                if [ "$py" != "" -a "${py:0:1}" != "-" ]; then
                  fullCmdLine+=("'$py'")
                  destDir="${destDir}tmp_$(date +'%y-%m-%d_%H-%M')/"
                  destDir3Rm="$destDir"
                  bkup3dir=$(echo -e "$py" | sed -e 's/^_//;s/^\.\.\./_\.\.\./;s/ /_/g;s/_(/_/g;s/)_/_/g;s/___/-/g;s/__/_/g;s/--/-/g;s/_-/-/g;s/-_/-/g' | zeichenersetzen)
                  bkup3dir="$bkup3dir/"
                  bkup3script="schiebe_Dateien_nach_bkup3dir.sh"
                  bkup3tmpdir="bkup3tmp/"
                  if [ "${py:0:1}" != "/" ]; then
                    [ -d "$baseDir$bkup3dir" ] && echo -e "$NOCOL$TRENNER\n$SETGN\$bkup3dir zeigt auf '$NOCOL$bkup3dir$SETGN' im Basisverzeichnis '$NOCOL$baseDir$SETGN'$NOCOL" || echo -e "$NOCOL$TRENNER\n$SETRT\$bkup3dir zeigt auf in '$NOCOL$baseDir$SETRT'nicht erreichbares '$NOCOL$bkup3dir$SETRT'$NOCOL"
                    bkup3dir="$baseDir$bkup3dir"
                  else
                    [ -d "$bkup3dir" ] && echo -e "$NOCOL$TRENNER\n$SETGN\$bkup3dir zeigt auf '$NOCOL$bkup3dir$SETGN'$NOCOL" || echo -e "$NOCOL$TRENNER\n$SETRT\$bkup3dir zeigt auf nicht erreichbares '$NOCOL$bkup3dir$SETRT'$NOCOL"
                  fi
                  bkup3dirnam="$bkup3dir$pcfg2$pcfg2_2"
                  if [ "$pcfg3" == "+" -a ! -d "$bkup3dir$pcfg2$pcfg2_2" ]; then
                    bkup3dirstr="$NOCOL *  ${SETBL}Unterverzeichnis $NOCOL$bkup3dir$pcfg2$pcfg2_2$SETBL anlegen$NOCOL"
                    if [ "$bkup3dirstr" != "" ] && [ ! -d "$bkup3dir" ] && [ "$statusDest3" != "" ] && [ $statusDest3 -eq 1 ]; then echo -e -n "$bkup3dirstr" ; mkdir -p "$bkup3dirnam" 2> /dev/null ; SubOkErr $? 0; fi
                  fi
                  shift
                fi
                shift ;;
      "-nobkup1") fullCmdLine+=("$px") ; bkup1sel=0 ; echo -e "$NOCOL$TRENNER\n$SETOR\$bkup1dir wird deaktiviert!$NOCOL" ; bkup1dir=""
                  [ "$bkup2sel" == "" ] && bkup2sel=1
                  [ "$bkup3sel" == "" ] && bkup3sel=1
                  shift ;;
      "-nobkup2") fullCmdLine+=("$px") ; bkup2sel=0 ; echo -e "$NOCOL$TRENNER\n$SETOR\$bkup2dir wird deaktiviert!$NOCOL" ; bkup2dir=""
                  [ "$bkup1sel" == "" ] && bkup1sel=1
                  [ "$bkup3sel" == "" ] && bkup3sel=1
                  shift ;;
      "-nobkup3") fullCmdLine+=("$px") ; bkup3sel=0 ; echo -e "$NOCOL$TRENNER\n$SETOR\$bkup3dir wird deaktiviert!$NOCOL" ; bkup3dir=""
                  [ "$bkup1sel" == "" ] && bkup1sel=1
                  [ "$bkup2sel" == "" ] && bkup2sel=1
                  shift ;;
    # ####################################################
    # Quelle und Zielname
      "-i") fullCmdLine+=("$px")
            if [ "$py" != "" -a "${py:0:1}" != "-" ]; then
              fullCmdLine+=("'$py'")
              p1name="${py##*/}"
              [ "${py%/*}" != "$py" ] && p1Startpfad="${py%/*}/" || p1Startpfad="$py"
              if [ "${p1Startpfad:0:2}" == "./" ]; then
                if [ "${p1Startpfad:2}" != "" ]; then p1Startpfad="$PWD/${p1Startpfad:2}"
                else p1Startpfad="./"
                fi
              elif [ "${p1Startpfad:0:1}" == "~" ]; then p1Startpfad="$HOME${p1Startpfad:1}"
              elif [ "${p1Startpfad:0:1}" != "/" ]; then
                if [ "$p1Startpfad" == "" ] || [ "$p1Startpfad" == "$p1name" ]; then p1Startpfad="$PWD/"
                else p1Startpfad="$PWD/$p1Startpfad"
                fi
              fi
              p1pfad="$p1Startpfad"
              realerpfad="$(findePfad $PWD)"
              realerpfad="${realerpfad%/}"
              if [ ${#p1pfad} -gt 2 ] && [ "$p1pfad" != "" ] && [ "$p1pfad" != "./" ] && [ "$p1pfad" != "." ] && [ "$p1pfad" != "$p1name" ] && [ "${p1pfad%/}" != "$PWD" ] && [ "${p1pfad%/}" != "$realerpfad" ]; then
                cd $(findePfad $p1pfad)
                echo -e "$NOCOL$TRENNER\n${SETBL}Verzeichniswechsel nach$NOCOL $PWD/"
              fi
              p1pfad="$(findePfad $PWD)/"
              p1="$p1name"
              shift 2
              if [ ! -e "$p1" ] && [ -e "$p1.m2t" ]; then
                p1="$p1.m2t"
              elif [ ! -e "$p1" ] && [ -e "$p1.ts" ] ; then
                p1="$p1.ts"
              fi
            fi
            ;;
      "-w") fullCmdLine+=("$px")
            if [ "$py" != "" -a "${py:0:1}" != "-" ]; then 
              fullCmdLine+=("'$py'")
              py=$(echo -e "$py" | sed -e 's/^_//;s/^\.\.\./_\.\.\./;s/ /_/g;s/_(/_/g;s/)_/_/g;s/___/-/g;s/__/_/g;s/--/-/g;s/_-/-/g;s/-_/-/g' | zeichenersetzen) 
              [ ${#py} -gt 0 ] && p1neu="$py"
              shift
            fi
            shift ;;
    # ####################################################
    # Video-Schnittangaben
      "-a") fullCmdLine+=("$px")
            if [ "$py" != "" -a "${py:0:1}" != "-" ]; then 
              fullCmdLine+=("$py")
              p2=$py; 
              if [ "${py%.*}" != "${py#*.}" ]; then
                p2dec="${py#*.}"
                if [ $p2dec -lt 10 ]; then
                  p2dec=$(( $p2dec * 100 ))
                elif [ $p2dec -lt 100 ]; then
                  p2dec=$(( $p2dec * 10 ))
                elif [ $p2dec -lt 100 ]; then
                  p2dec=${p2dec:0:3}
                fi
              else
                p2dec=0
              fi
              shift;
            fi
            shift ;;
      "-e") fullCmdLine+=("$px")
            if [ "$py" != "" -a "${py:0:1}" != "-" ]; then 
              fullCmdLine+=("$py")
              [ "$py" != "0" ] && p3=$py
              if [ "${py%.*}" != "${py#*.}" ]; then
                p3dec="${py#*.}"
                if [ $p3dec -lt 10 ]; then
                  p3dec=$(( $p3dec * 100 ))
                elif [ $p3dec -lt 100 ]; then
                  p3dec=$(( $p3dec * 10 ))
                elif [ $p3dec -lt 100 ]; then
                  p3dec=${p3dec:0:3}
                fi
              else
                p3dec=0
              fi
              shift; fi
            shift ;;
    # ####################################################
    # Video-Encoder-Einstellungen
      "-av1_std") ONAM3a="s" ; fullCmdLine+=("$px") ; HevcVideoOut=3 ; AacEac3Ac3AudOut=4 ; vcodecSpecific=0 ; h264hevcDefaultOption="" ; shift ;;       # AV1 (Default (leer)/$Av1OptExt)
      "-av1_br") ONAM3a="b" ; fullCmdLine+=("$px") ; HevcVideoOut=3 ; AacEac3Ac3AudOut=4 ; vcodecSpecific=1 ; h264hevcDefaultOption="" ; shift ;;       # AV1 (Default (leer)/$Av1OptExt)
      "-av1_opt") ONAM3a="o" ; fullCmdLine+=("$px") ; HevcVideoOut=3 ; AacEac3Ac3AudOut=4 ; vcodecSpecific=1 ; h264hevcDefaultOption="$Av1OptExt" ; shift ;; # AV1 (Default (leer)/$Av1OptExt)
      "-h264_std") ONAM3a="s" ; fullCmdLine+=("$px") ; HevcVideoOut=0 ; AacEac3Ac3AudOut=2 ; vcodecSpecific=0 ; h264hevcDefaultOption="medium" ; shift ;; # H264 (leer=Bitratenvorgabe/.../medium/fast/.../ultrafast) und AAC
      "-h264_br") ONAM3a="b" ; fullCmdLine+=("$px") ; HevcVideoOut=0 ; AacEac3Ac3AudOut=2 ; vcodecSpecific=0 ; h264hevcDefaultOption="" ; shift ;;       # H264 (leer=Bitratenvorgabe/.../medium/fast/.../ultrafast) und AAC
      "-h264_opt") ONAM3a="o" ; fullCmdLine+=("$px") ; HevcVideoOut=0 ; AacEac3Ac3AudOut=2 ; vcodecSpecific=1 ; h264hevcDefaultOption="" ; shift ;;       # H264 (mit zusätzlichen Konvertieroptionen) und AAC
      "-hevc_std") ONAM3a="s" ; fullCmdLine+=("$px") ; HevcVideoOut=1 ; AacEac3Ac3AudOut=-1 ; vcodecSpecific=0 ; h264hevcDefaultOption="medium" ; shift ;; # HEVC (leer=Bitratenvorgabe/.../medium/fast/.../ultrafast) und AAC
      "-hevc_br") ONAM3a="b" ; fullCmdLine+=("$px") ; HevcVideoOut=1 ; AacEac3Ac3AudOut=-1 ; vcodecSpecific=0 ; h264hevcDefaultOption="" ; shift ;;       # HEVC (leer=Bitratenvorgabe/.../medium/fast/.../ultrafast) und AAC
      "-hevc_opt") ONAM3a="o" ; fullCmdLine+=("$px") ; HevcVideoOut=1 ; AacEac3Ac3AudOut=-1 ; vcodecSpecific=1 ; h264hevcDefaultOption="" ; shift ;;       # HEVC (mit zusätzlichen Konvertieroptionen) und AAC
      "-vp9_std") ONAM3a="s" ; fullCmdLine+=("$px") ; HevcVideoOut=2 ; AacEac3Ac3AudOut=4 ; vcodecSpecific=0 ; h264hevcDefaultOption="" ; shift ;;       # VP9 (Default (leer)/$Vp9OptExt)
      "-vp9_br") ONAM3a="b" ; fullCmdLine+=("$px") ; HevcVideoOut=2 ; AacEac3Ac3AudOut=4 ; vcodecSpecific=1 ; h264hevcDefaultOption="" ; shift ;;       # VP9 (Default (leer)/$Vp9OptExt)
      "-vp9_opt") ONAM3a="o" ; fullCmdLine+=("$px") ; HevcVideoOut=2 ; AacEac3Ac3AudOut=4 ; vcodecSpecific=1 ; h264hevcDefaultOption="$Vp9OptExt" ; shift ;; # VP9 (Default (leer)/$Vp9OptExt)
      "-m") fullCmdLine+=("$px") ; 
            if [ "$py" != "" -a "${py:0:1}" != "-" ]; then
              fullCmdLine+=("$py")
              case "${py,,}" in
              "copy"|"vcopy") p0=${py,,} ; VMOD=$p0 ;;
              "uhd"|"2160p"|"4k") p0="2160p" ; VMOD=$p0  ;;
              "vuhd"|"v2160p"|"v4k") p0="v2160p" ; VMOD=$p0 ;;
              "hdfull"|"1080p"|"max") p0="1080p" ; VMOD=$p0 ;;
              "vhdfull"|"v1080p") p0="v1080p" ; VMOD=$p0 ;;
              "hdready"|"720p"|"high") p0="720p" ; VMOD=$p0 ;;
              "vhdready"|"v720p") p0="v720p" ; VMOD=$p0 ;;
              "hdvd"|"540p") p0="540p" ; VMOD=$p0 ;;
              "vhdvd"|"v540p") p0="v540p" ; VMOD=$p0 ;;
              "dvd"|"480p"|"medium") p0="480p" ; VMOD=$p0 ;;
              "vdvd"|"v480p") p0="v480p" ; VMOD=$p0 ;;
              "ldvd"|"360p") p0="360p" ; VMOD=$p0 ;;
              "vldvd"|"v360p") p0="v360p" ; VMOD=$p0 ;;
              "android"|"low"|"272p") p0="272p" ; VMOD=$p0 ;;
              "vandroid"|"vlow"|"v272p") p0="v272p" ; VMOD=$p0 ;;
              "default") p0=${VMOD,,} ; VMOD=$p0 ;;
              *) ;;
              esac
              shift 2  
            fi
            ;;
      "-uhd"|"-2160p"|"-4k") fullCmdLine+=("$px") ; p0="2160p" ; VMOD=$p0 ; shift ;;
      "-vuhd"|"-v2160p"|"-v4k") fullCmdLine+=("$px") ; p0="v2160p" ; VMOD=$p0 ; shift ;;
      "-hdfull"|"-1080p"|"-max") fullCmdLine+=("$px") ; p0="1080p" ; VMOD=$p0 ; shift ;;
      "-vhdfull"|"-v1080p") fullCmdLine+=("$px") ; p0="v1080p" ; VMOD=$p0 ; shift ;;
      "-hdready"|"-720p"|"-high") fullCmdLine+=("$px") ; p0="720p" ; VMOD=$p0 ; shift ;;
      "-vhdready"|"-v720p") fullCmdLine+=("$px") ; p0="v720p" ; VMOD=$p0 ; shift ;;
      "-hdvd"|"-540p") fullCmdLine+=("$px") ; p0="540p" ; VMOD=$p0 ; shift ;;
      "-vhdvd"|"-v540p") fullCmdLine+=("$px") ; p0="v540p" ; VMOD=$p0 ; shift ;;
      "-dvd"|"-480p"|"-medium") fullCmdLine+=("$px") ; p0="480p" ; VMOD=$p0 ; shift ;;
      "-vdvd"|"-v480p") fullCmdLine+=("$px") ; p0="v480p" ; VMOD=$p0 ; shift ;;
      "-ldvd"|"-360p") fullCmdLine+=("$px") ; p0="360p" ; VMOD=$p0 ; shift ;;
      "-vldvd"|"-v360p") fullCmdLine+=("$px") ; p0="v360p" ; VMOD=$p0 ; shift ;;
      "-android"|"-low"|"-272p") fullCmdLine+=("$px") ; p0="272p" ; VMOD=$p0 ; shift ;;
      "-vandroid"|"-vlow"|"-v272p") fullCmdLine+=("$px") ; p0="v272p" ; VMOD=$p0 ; shift ;;
      "-copy"|"-vcopy") fullCmdLine+=("$px") ; p0=${px,,} ; p0=${p0:1} ; VMOD=$p0 ; shift ;;
    # ####################################################
    # Video-Hochformatvarianten und Qualitätseinstellungen
      "-f") fullCmdLine+=("$px")
            if [ "$py" != "" -a "${py:0:1}" != "-" ]; then fullCmdLine+=("$py") ; p4=$py; shift; fi
            shift ;;
      "-o") fullCmdLine+=("$px")
            #pz=$3
            if [ "$py" != "" -a "${py:0:1}" != "-" ]; then
              fullCmdLine+=("$py")
              p5=$py
              shift 2
              #if [ "$pz" != "" -a "${pz:0:1}" != "-" ]; then
              #  px5=$pz
              #  shift
              #else
              #  px5=$p5
              #fi
            fi
            ;;
      "-q") fullCmdLine+=("$px")
            if [ "$py" != "" -a "${py:0:1}" != "-" ]; then
              fullCmdLine+=("$py")
              if [ "${py,,}" == "std" ]; then
                ONAM3a="s"
                [ $HevcVideoOut = 0 ] && vcodecSpecific=0 ; h264hevcDefaultOption="medium"
                [ $HevcVideoOut = 1 ] && vcodecSpecific=0 ; h264hevcDefaultOption="medium"
                [ $HevcVideoOut = 2 ] && vcodecSpecific=0 ; h264hevcDefaultOption=""
                [ $HevcVideoOut = 3 ] && vcodecSpecific=0 ; h264hevcDefaultOption=""
              elif [ "${py,,}" == "br" ]; then
                ONAM3a="b" ; 
                [ $HevcVideoOut = 0 ] && vcodecSpecific=0 ; h264hevcDefaultOption=""
                [ $HevcVideoOut = 1 ] && vcodecSpecific=0 ; h264hevcDefaultOption=""
                [ $HevcVideoOut = 2 ] && vcodecSpecific=1 ; h264hevcDefaultOption=""
                [ $HevcVideoOut = 3 ] && vcodecSpecific=1 ; h264hevcDefaultOption=""
              elif [ "${py,,}" == "opt" ]; then
                ONAM3a="o" ; 
                [ $HevcVideoOut = 0 ] && vcodecSpecific=1 ; h264hevcDefaultOption=""
                [ $HevcVideoOut = 1 ] && vcodecSpecific=1 ; h264hevcDefaultOption=""
                [ $HevcVideoOut = 2 ] && vcodecSpecific=1 ; h264hevcDefaultOption="$Vp9OptExt"
                [ $HevcVideoOut = 3 ] && vcodecSpecific=1 ; h264hevcDefaultOption="$Av1OptExt"
              else h264hevcDefaultOption=${py,,}
              fi
              shift 2
            fi
            ;;
      "-r") fullCmdLine+=("$px")
            if [ "$py" != "" -a "${py:0:1}" != "-" ]; then
              fullCmdLine+=("$py")
              case "${py,,}" in
              "4zu3") IASP="4:3" ; QASP="-aspect $IASP " ;;
              "16zu9") IASP="16:9" ; QASP="-aspect $IASP " ;;
              *) IASP= ; QASP= ;;
              esac
              shift 2 
            fi
            ;;
      "-vertikal") fullCmdLine+=("$px") ; verticalFilter="$verticalFilterSchwarz" ; shift ;;
      "-vertikalblur") fullCmdLine+=("$px") ; verticalFilter="$verticalFilterBlur" ; shift ;;
    # ####################################################
    # Audio-Encoder-Einstellungen
      "-aac") fullCmdLine+=("$px") ; AacEac3Ac3AudOut=2 ; shift ;;
      "-aaclatm")fullCmdLine+=("$px") ;  ASEL=-1 ; AacEac3Ac3AudOut=-1 ; forcemp4=0 ; specialaaclatm=1 ; shift ;;
      "-acopy") fullCmdLine+=("$px") ; AacEac3Ac3AudOut=-1 ; shift ;;
      "-ac3") fullCmdLine+=("$px") ; AacEac3Ac3AudOut=0 ; shift ;;
      "-allaudio") fullCmdLine+=("$px") ; ASEL=-1 ; shift ;;
      "-audio0"|"-audio1"|"-audio2"|"-audio3"|"-audio4"|"-audio5"|"-audio6") fullCmdLine+=("$px") ; px=${px,,} ; ASEL=${px:1} ; shift ;;
      "-convaaclatm"|"-conv") fullCmdLine+=("$px") ; convaaclatm=1 ; forcemp4=0 ; specialaaclatm=0 ; shift ;;
      "-deu"|"-ger"|"-eng"|"-fra"|"-fre") fullCmdLine+=("$px") ; ASEL=-2 ; XLANG=${px:1} ; shift ;;
      "-eac3") fullCmdLine+=("$px") ; AacEac3Ac3AudOut=1 ; shift ;;
      "-mp3") fullCmdLine+=("$px") ; AacEac3Ac3AudOut=3 ; shift ;;
      "-opus") fullCmdLine+=("$px") ; AacEac3Ac3AudOut=4 ; shift ;;
      "-s") fullCmdLine+=("$px")
            if [ "$py" != "" -a "${py:0:1}" != "-" ]; then
              fullCmdLine+=("$py")
              case "${py,,}" in
              "allaudio") ASEL=-1 ;;
              "audio0"|"audio1"|"audio2"|"audio3"|"audio4"|"audio5"|"audio6") ASEL=${py: -1} ;;
              "deu"|"ger"|"eng"|"fra"|"fre")  ASEL=-2 ; XLANG=${py,,} ;;
              "aaclatm") ASEL=-1 ; AacEac3Ac3AudOut=-1 ; forcemp4=0 ; specialaaclatm=1 ;;
              "acopy") ASEL=-1 ; AacEac3Ac3AudOut=-1 ;;
              "convaaclatm"|"conv") convaaclatm=1 ; forcemp4=0 ; specialaaclatm=0 ;;
              *) ASEL=$FAVAUD ;;
              esac
              shift 2
            fi
            ;;
    # ####################################################
    # Untertitel
      "-u") fullCmdLine+=("$px") ; 
            if [ $enasubtitle -ge 0 ]; then enasubtitle=1; fi
            if [ "${py:0:1}" != "-" ]; then subtitlexcmd=$py; fullCmdLine+=("$py") ; shift
            else subtitlexcmd=""
            fi
            if [ "${pz:0:1}" != "-" ]; then subtitlexcmd2=$pz; fullCmdLine+=("$pz") ; shift
            else subtitlexcmd2=""
            fi
            shift ;;
    # ####################################################
    # Sonstige Kommandos
      "-cc")  fullCmdLine+=("$px") ; closedcaptionkorrektur=1 ; shift ;;
      "-d") fullCmdLine+=("$px")
            if [ "$py" == "0" ] || [ "$py" == "1" ] || [ "$py" == "2" ] || [ "$py" == "3" ] || [ "$py" == "4" ]; then 
              fullCmdLine+=("$py")
              durchsucheUnterverzeichnisseOrig=$durchsucheUnterverzeichnisse
              durchsucheUnterverzeichnisse=$py
              if [ $durchsucheUnterverzeichnisse -eq 0 ]; then echo -e "$NOCOL$TRENNER\n${NOCOL}Keine Duplikatsuche aktiviert, ggf. mit '-d [1|2]' für Arbeitsverzeichnisse bzw. in allen Unterverzeichnissen' aufrufen$SETBL"
              elif [ $durchsucheUnterverzeichnisse -eq 1 ]; then echo -e "$NOCOL$TRENNER\n${NOCOL}Duplikatsuche in Arbeitsverzeichnissen, ggf. mit '-d [0|2]' für nicht-Suche bzw. Suche in allen Unterverzeichnissen' aufrufen$SETBL"
              elif [ $durchsucheUnterverzeichnisse -eq 2 ]; then echo -e "$NOCOL$TRENNER\n${NOCOL}Duplikatsuche in Arbeits- und Unterverzeichnissen, ggf. mit '-d [0|1]' für nicht-Suche bzw. Suche in Arbeitsverzeichnissen aufrufen$SETBL"
              elif [ $durchsucheUnterverzeichnisse -eq 3 ]; then echo -e "$NOCOL$TRENNER\n${NOCOL}Duplikatsuche in Arbeitsverzeichnissen (mit Anzeige aller durchsuchten Verzeichnispfade)$SETBL"; vorherFindeDup=1
              elif [ $durchsucheUnterverzeichnisse -eq 4 ]; then echo -e "$NOCOL$TRENNER\n${NOCOL}Duplikatsuche in Arbeits- und Unterverzeichnissen (mit Anzeige aller durchsuchten Verzeichnispfade)\n${SETOR}Achtung: Die Duplikatsuche wird wahrscheinlich mehrere Stunden dauern!$SETBL"; vorherFindeDup=1
              fi
            else
              if [ $durchsucheUnterverzeichnisse -eq 0 ]; then echo -e "$NOCOL$TRENNER\n${NOCOL}Standardeinstellung: Keine Duplikatsuche aktiviert, ggf. mit '-d [1|2]' für Arbeitsverzeichnisse mit allen Unterverzeichnissen' aufrufen$SETBL"
              elif [ $durchsucheUnterverzeichnisse -eq 1 ]; then echo -e "$NOCOL$TRENNER\n${NOCOL}Standardeinstellung: Duplikatsuche in Arbeitsverzeichnissen, ggf. mit '-d [0|2]' für nicht-Suche bzw. Suche in allen Unterverzeichnissen' aufrufen$SETBL"
              elif [ $durchsucheUnterverzeichnisse -eq 2 ]; then echo -e "$NOCOL$TRENNER\n${NOCOL}Standardeinstellung: Duplikatsuche in Arbeits- und Unterverzeichnissen, ggf. mit '-d [0|1]' für nicht-Suche bzw. Suche in Arbeitsverzeichnissen aufrufen$SETBL"
              elif [ $durchsucheUnterverzeichnisse -eq 3 ]; then echo -e "$NOCOL$TRENNER\n${NOCOL}Standardeinstellung: Duplikatsuche in Arbeitsverzeichnissen (mit Anzeige aller durchsuchten Verzeichnispfade)$SETBL"; vorherFindeDup=1
              elif [ $durchsucheUnterverzeichnisse -eq 4 ]; then echo -e "$NOCOL$TRENNER\n${NOCOL}Standardeinstellung: Duplikatsuche in Arbeits- und Unterverzeichnissen (mit Anzeige aller durchsuchten Verzeichnispfade)\n${SETOR}Achtung: Die Duplikatsuche wird wahrscheinlich mehrere Stunden dauern!$SETBL"; vorherFindeDup=1
              fi
            fi
            if [ "$pz" == "0" ] || [ "$pz" == "1" ]; then 
              vorherFindeDupOrig=$vorherFindeDup
              vorherFindeDup=$pz
              if [ $vorherFindeDup -eq 0 ]; then echo -e "${NOCOL}Keine Duplikatsuche vor Konvertierung$SETBL"
              elif [ $vorherFindeDup -eq 1 ]; then echo -e "${NOCOL}Duplikatsuche vor Konvertierung wird aktiviert$SETBL"
              fi
            else
              if [ $vorherFindeDup -eq 0 ]; then echo -e "${NOCOL}Standardeinstellung: Keine Duplikatsuche aktiviert, ggf. mit '-d [1|2|3|4] 1' für Duplikatsuche auch vor Konvertierung$SETBL"
              elif [ $vorherFindeDup -eq 1 ]; then echo -e "${NOCOL}Standardeinstellung: Duplikatsuche in Arbeitsverzeichnissen, ggf. mit '-d [1|2|3|4] 0' für keine Duplikatsuche auch vor Konvertierung$SETBL"
              fi
            fi
            if [ "${py:0:1}" != "-" ]; then
              shift
              [ "${pz:0:1}" != "-" ] && fullCmdLine+=("$pz") ; shift
            fi
            shift ;;
      "-h") fullCmdLine+=("$px") ; philfe=1 ; shift ;;
      "-l") fullCmdLine+=("$px") ; zeigeLog=1 ; updLogInterval=120 ; shift ;;
      "-n") fullCmdLine+=("$px") ; nicht_Schieben=1 ; shift ;;
      "-p") fullCmdLine+=("$px")
            if [ "$py" != "" ] && [ "${py:0:1}" != "-" ] && [ "${py,,}" == "max" ]; then
              p6=$(( 100 * $NumCPUs ))
              RamUsed=0
              CurrTempChk=0.0
              FreqCPUs=$FreqLimit
              redlvl=0
              CpuLimitDefneu=$p6
              CpuLimitDef=101
            elif [ "$py" != "" ] && [ "${py:0:1}" != "-" ] && [ "${py,,}" == "min" ]; then
              p6=$(( 14 * $NumCPUs ))
              RamUsed=0
              CurrTempChk=0.0
              FreqCPUs=$FreqLimit
              redlvl=0
              CpuLimitDefneu=$p6
              CpuLimitDef=14
            elif [ "$py" != "" ] && [ "${py:0:1}" != "-" ] && [ -z "${py##[0-9]*}" ] && [ $py -ge 10 ] && [ $py -le 100 ]; then
              p6=$(( $py * $NumCPUs ))
              RamUsed=0
              CurrTempChk=0.0
              FreqCPUs=$FreqLimit
              redlvlpy=0
              CpuLimitDefneu=$p6
              CpuLimitDef=$py
            fi
            [ "$py" != "" -a "${py:0:1}" != "-" ] && fullCmdLine+=("$py") ; shift
            shift ;;
      "-x") fullCmdLine+=("$px") ; ersetzeDatei=1; shift ;;
      #"-ze") loglvl="" ; shift ;;
      "-zi") fullCmdLine+=("$px")
             if [ "$py" != "" ]; then # Achtung: der nächste Wert (Einzelparameter oder in ' eingerahmte Parametergruppe) wird immer als zusatzIn-Parameter übernommen!
               fullCmdLine+=("'$py'")
               #[ "${py:0:1}" != "-" ] && zusatzIn="-$py" || 
               zusatzIn="$py"
               shift
             fi
             shift ;;
      "-zl") fullCmdLine+=("$px")
             if [ "$py" != "" -a "${py:0:1}" != "-" ]; then
               fullCmdLine+=("$py")
               #loglevel="$py"; loglvl="-loglevel $loglevel -stats"; 
               loglevel_hevc="$py"; shift
             else
               #loglevel="info"; loglvl="-loglevel info -stats"; 
               loglevel_hevc="info"
             fi
             shift
             ;;
      "-zm") fullCmdLine+=("$px") ; anzeigeBkupMsg=2; shift ;;
      "-zo") fullCmdLine+=("$px")
             if [ "$py" != "" ]; then # Achtung: der nächste Wert (Einzelparameter oder in ' eingerahmte Parametergruppe) wird immer als zusatzOut-Parameter übernommen!
               fullCmdLine+=("'$py'")
               #[ "${py:0:1}" != "-" ] && zusatzOut="-$py" || 
               zusatzOut="$py"
               shift
             fi
             shift ;;
    # ####################################################
    # Unbekannter Parameter
      *)    echo -e "${SETFLRT}Unbekannter Parameter \"$px\"$NOCOL" ; clrlogfiles ; exit -125 ;;
    esac
    px=$1
    py=$2
  done
  fullCmdLineStr="${fullCmdLine[@]}"
  echoUndLog "$NOCOL$TRENNER\n${SETBL}Aufruf in '$NOCOL$srcDir$SETBL':"
  [ $(( ${#fullCmdLineStr} + 10 )) -ge $chkcols ] && echoUndLog "$SETBGWW$SETBLD${fullCmdLineStr% -i *} \\ $SETBL$SETBGSW$NOCOL\n$SETBGWW$SETBLD-i ${fullCmdLineStr##* -i } $NOCOL$SETBGSW" || echoUndLog "$SETBGWW$SETBLD$fullCmdLineStr $NOCOL$SETBGSW"
  if [ $anzeigeBkupMsg -ge 1 ]; then
    echo -e -n "$SETBGWS${SETBLD}Video-Ziel: ${zielVCodec^^} / ${zielOptimierung^^}, Audio-Ziel: $( [ $AacEac3Ac3AudOut -eq -1 ] && echo -e -n "COPY" || ( [ $AacEac3Ac3AudOut -eq 0 ] && echo -e -n "AC3" || ( [ $AacEac3Ac3AudOut -eq 1 ] && echo -e -n "EAC3" || ( [ $AacEac3Ac3AudOut -eq 2 ] && echo -e -n "AAC" || ( [ $AacEac3Ac3AudOut -eq 3 ] && echo -e -n "MP3" || ( [ $AacEac3Ac3AudOut -eq 4 ] && echo -e -n "OPUS" || echo -e -n "???" ) ) ) ) ) ) $NOCOL$SETBGSW\n"
  fi
  echoUndLog "$TRENNER"
  #; fi
  if [ $philfe -eq 1 ]; then hilfetext; fi
  if [ $CpuLimitDef -eq 101 ]; then
    p6=$(( 100 * $NumCPUs ))
    RamUsed=0
    CurrTempChk=0.0
    FreqCPUs=$FreqLimit
    redlvl=0
    CpuLimitDefneu=$p6
  elif [ $CpuLimitDef -eq 14 ]; then
    p6=$(( 14 * $NumCPUs ))
    RamUsed=0
    CurrTempChk=0.0
    FreqCPUs=$FreqLimit
    redlvl=0
    CpuLimitDefneu=$p6nbekannt
  fi
  if [ "$pcfg" != "-umbenennen" ]; then
    if [ "$bkup1sel" == "" -a "$bkup2sel" == "" -a "$bkup3sel" == "" ]; then
      bkup1sel=1; bkup2sel=1; bkup3sel=1
    fi
    if [ "$bkup1dir" != "" ] || [ "$bkup2dir" != "" ] || [ "$bkup3dir" != "" ]; then
      [ "$bkup1dir" != "" ] && bkup1pfad="$bkup1dir$pcfg2$pcfg2_2" || bkup1pfad=""
      [ "$bkup2dir" != "" ] && bkup2pfad="$bkup2dir$pcfg2$pcfg2_2" || bkup2pfad=""
      [ "$bkup3dir" != "" ] && bkup3pfad="$bkup3dir$pcfg2$pcfg2_2" || bkup3pfad=""
      if [ "$bkup1pfad" != "" ] && [ "$bkup2pfad" != "" ] && [ "$bkup1pfad" == "$bkup2pfad" ]; then 
        echo -e "$SETOR\$bkup1dir=$NOCOL\"$bkup1dir$pcfg2$pcfg2_2\" ${SETOR}und \n\$bkup2dir=$NOCOL\"$bkup2dir$pcfg2$pcfg2_2\"\n${SETOR}zeigen auf selbes Zielverzeichnis, darum wird \$bkup2dir deaktiviert!$NOCOL"
        bkup2dir=""
        bkup2pfad=""
      fi
      if [ "$bkup1pfad" != "" ] && [ "$bkup3pfad" != "" ] && [ "$bkup1pfad" == "$bkup3pfad" ]; then 
        echo -e "$SETOR\$bkup1dir=$NOCOL\"$bkup1dir$pcfg2$pcfg2_2\" ${SETOR}und \n\$bkup3dir=$NOCOL\"$bkup3dir$pcfg2$pcfg2_2\"\n${SETOR}zeigen auf selbes Zielverzeichnis, darum wird \$bkup3dir deaktiviert!$NOCOL"
        bkup3dir=""
        bkup3pfad=""
      fi
      if [ "$bkup2pfad" != "" ] && [ "$bkup3pfad" != "" ] && [ "$bkup2pfad" == "$bkup3pfad" ]; then 
        echo -e "$SETOR\$bkup2dir=$NOCOL\"$bkup2dir$pcfg2$pcfg2_2\" ${SETOR}und \n\$bkup3dir=$NOCOL\"$bkup2dir$pcfg2$pcfg2_2\"\n${SETOR}zeigen auf selbes Zielverzeichnis, darum wird \$bkup3dir deaktiviert!$NOCOL"
        bkup3dir=""
        bkup3pfad=""
      fi
    fi
    echo -e "$NOCOL$TRENNER"
    [ "$bkup1dir" != "" ] && bkupZielSub 1
    [ "$bkup2dir" != "" ] && bkupZielSub 2
    [ "$bkup3dir" != "" ] && bkupZielSub 3
  fi
  infoHd 3 0 0 0 3 #3 3 3
  if [ "$bkup1dirstr" != "" ] && [ ! -d "$bkup1dir" ] && [ $bkup1sel -eq 1 ] && [ "$statusDest1" != "" ] && [ $statusDest1 -eq 1 ]; then echo -e -n "$bkup1dirstr" ; mkdir -p "$bkup1dirnam" 2> /dev/null ; SubOkErr $? 0; fi
  if [ "$bkup2dirstr" != "" ] && [ ! -d "$bkup2dir" ] && [ $bkup2sel -eq 1 ] && [ "$statusDest2" != "" ] && [ $statusDest2 -eq 1 ]; then echo -e -n "$bkup2dirstr" ; mkdir -p "$bkup2dirnam" 2> /dev/null ; SubOkErr $? 0; fi
  if [ "$bkup3dirstr" != "" ] && [ ! -d "$bkup3dir" ] && [ $bkup3sel -eq 1 ] && [ "$statusDest3" != "" ] && [ $statusDest3 -eq 1 ]; then echo -e -n "$bkup3dirstr" ; mkdir -p "$bkup3dirnam" 2> /dev/null ; SubOkErr $? 0; fi
  if [ "$bkup1dirstr" != "" ] || [ "$bkup2dirstr" != "" ] || [ "$bkup3dirstr" != "" ]; then echo -e "$TRENNER"; fi
  if [ "$CpuLimitDefneu" == "" ]; then CpuLimitDefneu=$CpuLimitDef; fi
  if [ -z $p6 ]; then p6=$(( $CpuLimitDef * $NumCPUs )); fi
  [ "${p1:0:2}" = './' ] && p1="${p1:2}"
  [ -z "${p1##*/*}" ] && $( path="${p1%/*}/" ; p1=${p1##*/} )
  p1p=$p1
  if [ "${pcfg,,}" != "-umbenennen" ]; then
    p1=$(echo -e "$p1p" | sed -e 's/^_//;s/^\.\.\./_\.\.\./;s/ /_/g;s/_(/_/g;s/)_/_/g;s/___/-/g;s/__/_/g;s/--/-/g;s/_-/-/g;s/-_/-/g;s/_nicht_konvertierbar//;s/_Bitratenhinweis_[V|G]//;s/_Bitratenwarnung_[V|G]//;s/_Bitratenalarm_[V|G]//;s/_Audiowarnung_A[1-5]//;s/_Audioalarm_A1//' | zeichenersetzen)
    [ "$pcfg2str" != "" ] && echo -e "$pcfg2str"
  else
    p1=$(echo -e "$p1p" | sed -e 's/^_//;s/^\.\.\./_\.\.\./;s/ /_/g;s/_(/_/g;s/)_/_/g;s/___/-/g;s/__/_/g;s/--/-/g;s/_-/-/g;s/-_/-/g' | zeichenersetzen)
    [ "$pcfg2str" != "" ] && echo -e "$pcfg2str"
  fi
  if [ -f "$erledigtDir$p1" ]; then
    echo -e -n "${SETBL}'$SETWS$erledigtDir$p1$SETBL' wird für erneute Bearbeitung zurückgeholt "
    tmpp="$(findePfad $PWD)"
    tmpp="$(df -l $tmpp 2> /dev/null)"
    if [ "$tmpp" != "" ] || [ $rsyncstattcpmv -eq 0 ]; then # lokales FS => verwende mv
      echo -e -n " - mv "
      mv "$erledigtDir$p1" "$p1" 2> /dev/null # > /dev/null
      tmpret=$?
    else
      echo -e -n " - rsync\n    "
      rsync -IPat --info=progress2 --remove-source-files "$erledigtDir$p1" "$p1" 2> /dev/null # > /dev/null
      tmpret=$?
      echo -e -n "   "
    fi
    SubOkErr $tmpret 0
  elif [ -f "${p1p%.*}_nicht_konvertierbar.${p1p##*.}" ]; then
    echo -e -n "${SETBL}'$SETWS${p1p%.*}_nicht_konvertierbar.${p1p##*.}$SETBL'\nstatt ${SETWS}'$p1'$SETBL wird konvertiert"
    p1="${p1p%.*}_nicht_konvertierbar.${p1p##*.}"
    p1p=$p1
    SubOk
  elif [ -f "${p1%.*}_nicht_konvertierbar.${p1##*.}" ]; then
    echo -e -n "${SETBL}'$SETWS${p1%.*}_nicht_konvertierbar.${p1##*.}$SETBL'\nstatt ${SETWS}'$p1'$SETBL wird konvertiert"
    p1="${p1%.*}_nicht_konvertierbar.${p1##*.}"
    p1p=$p1
    SubOk
  elif [ -f "$erledigtDir$p1p" ]; then
    echo -e -n "${SETBL}'$SETWS$erledigtDir$p1p$SETBL' wird zurückgeholt,\numbenannt nach ${SETWS}'$p1'$SETBL und konvertiert"
    tmpp="$(findePfad $PWD)"
    tmpp="$(df -l $tmpp 2> /dev/null)"
    if [ "$tmpp" != "" ] || [ $rsyncstattcpmv -eq 0 ]; then # lokales FS => verwende mv
      echo -e -n " - mv "
      mv "$erledigtDir$p1p" "$p1" 2> /dev/null # > /dev/null
      tmpret=$?
    else
      echo -e -n " - rsync\n    "
      rsync -IPat --info=progress2 --remove-source-files "$erledigtDir$p1p" "$p1" 2> /dev/null # > /dev/null
      tmpret=$?
      echo -e -n "   "
    fi
    p1p=$p1
    SubOkErr $tmpret 0
  elif [ -f "$erledigtDir${p1p%.*}_nicht_konvertierbar.${p1p##*.}" ]; then
    echo -e -n "${SETBL}'$SETWS$erledigtDir${p1p%.*}_nicht_konvertierbar.${p1p##*.}$SETBL'\nstatt ${SETWS}'$p1'$SETBL wird zurückgeholt und konvertiert"
    tmpp="$(findePfad $PWD)"
    tmpp="$(df -l $tmpp 2> /dev/null)"
    if [ "$tmpp" != "" ] || [ $rsyncstattcpmv -eq 0 ]; then # lokales FS => verwende mv
      echo -e -n " - mv "
      mv "$erledigtDir${p1p%.*}_nicht_konvertierbar.${p1p##*.}" "./" 2> /dev/null # > /dev/null
      tmpret=$?
    else
      echo -e -n " - rsync\n    "
      rsync -IPat --info=progress2 --remove-source-files "$erledigtDir${p1p%.*}_nicht_konvertierbar.${p1p##*.}" "./" 2> /dev/null # > /dev/null
      tmpret=$?
      echo -e -n "   "
    fi
    SubOkErr $tmpret 0
    p1="${p1p%.*}_nicht_konvertierbar.${p1p##*.}"
    p1p=$p1
  elif [ -f "$erledigtDir${p1%.*}_nicht_konvertierbar.${p1##*.}" ]; then
    echo -e -n "${SETBL}'$SETWS$erledigtDir${p1%.*}_nicht_konvertierbar.${p1##*.}$SETBL'\nstatt ${SETWS}'$p1'$SETBL wird zurückgeholt und konvertiert"
    tmpp="$(findePfad $PWD)"
    tmpp="$(df -l $tmpp 2> /dev/null)"
    if [ "$tmpp" != "" ] || [ $rsyncstattcpmv -eq 0 ]; then # lokales FS => verwende mv
      echo -e -n " - mv "
      mv "$erledigtDir${p1%.*}_nicht_konvertierbar.${p1##*.}" "./" 2> /dev/null # > /dev/null
      tmpret=$?
    else
      echo -e -n " - rsync\n    "
      rsync -IPat --info=progress2 --remove-source-files "$erledigtDir${p1%.*}_nicht_konvertierbar.${p1##*.}" "./" 2> /dev/null # > /dev/null
      tmpret=$?
      echo -e -n "   "
    fi
    SubOkErr $tmpret 0
    p1="${p1%.*}_nicht_konvertierbar.${p1##*.}"
    p1p=$p1
  fi
  if [ "$p1" != "$p1p" ]; then
    if [ -f "$p1" ] && [ -f "$p1p" ]; then
      echo -e -n "$SETRT'$SETWS$p1p$SETRT' und '$SETWS$p1$SETRT' existieren, unklare Auswahl, Abbruch "
      SubErr
      clrlogfiles
      exit -13
    elif [ ! -f "$p1p" ]; then
      echo -e -n "${SETRT}'$SETWS$p1p$SETRT' existiert nicht$NOCOL"
      if [ -f "$p1" ]; then
        echo -e -n "${SETBL}, aber\n'$SETWS$p1$SETBL' existiert $NOCOL"
        SubOk
      else
        echo -e -n "$SETRT,\n '$SETWS$p1$SETRT' existiert auch nicht $NOCOL"
        SubErr
        clrlogfiles
        exit -12
      fi
    else
      [ "$pcfg2str" != "" ] && pcfg2str="$pcfg2str\n"
      echo -e -n "${SETBL}'$NOCOL$p1p$SETBL'\numbenannt nach '$NOCOL$p1$SETBL' $NOCOL"
      mv "$p1p" "$p1" > /dev/null 2> /dev/null
      if [ $? -eq 0 ]; then
        SubOk
      else
        SubErr
        echo -e "${SETRT}ffmpeg kann nicht mit ' ' in Dateinamen umgehen$NOCOL"
        clrlogfiles
        exit -11
      fi
    fi
  fi
  if [ "$p1" == "" ]; then addstr="Test/kein Name, "; indstatusstring
  elif [ ! -f "$p1" ]; then addstr="${SETRT}Name '$p1' unbekannt, "; indstatusstring
  else
    vorprep=$(date +'%s')
    logpid=0
    clrlogfiles
    # -------------------------------------------------------------------------
    if [ "${pcfg,,}" != "-umbenennen" ]; then
      sync
      sleep 10
      chkformat=()  
      for (( idx0=0; idx0<MeasIter; idx0++ )); do
        lebendig
        top -b -n 1 -d 0.1|sed '1,7d' >> $logDir$p1.ffmpeg-log
        cat $logDir$p1.ffmpeg-log|sed '1,7d'|cut -c70-89|sed 's/^[0-9]//g;s/^ //g;s/^\///g;s/\:.*//g;s/\/.*//g;s/\_.*//g;s/\-.*//g'|sort|uniq > $logDir$p1.ffmpeg-log1
        sleep $MeasDelay
      done
      while read -r line; do lebendig ; chkformat+=( "$line" ); done  < $logDir$p1.ffmpeg-log1
      sleep 3s
      clrFilesSub "$logDir$p1.ffmpeg-log1"
    fi
    # -------------------------------------------------------------------------
    if [ "${pcfg,,}" != "-umbenennen" ]; then
      echo -e "$initpwrlvlstr"
      if [ $CpuLimitDef -ge 15 ] && [ $CpuLimitDef -le 100 ]; then
        getLoadValsStart=$(date +%s)
        getXxxLoad
        getLoadValsStop=$(date +%s)
        getLoadValsDauer=$(( $getLoadValsStop - $getLoadValsStart ))
        getDateTime
        #if [ $ChkRam -eq 0 ]; then
        #  RamUsed=0
        #fi
        if [ $ChkTemp -eq 0 ]; then
          CurrTempChk=0.0
          FreqCPUs=$FreqLimit
          if [ "$redlvl" == "" ]; then redlvl=0; fi
          CpuLimitDefneu=$p6
        fi
        wartesubstr="$SETRT$currDateTime: Warnung wegen"
        if [ $RamUsed -gt $RamLoadLimit ] && [ $ChkRam -gt 0 ]; then echo -e "$wartesubstr hoher RAM-Last $RamUsed% > $RamLoadLimit%$NOCOL"; fi
        if [ $SwapUsed -gt $SwapLoadLimit ]; then echo -e "$wartesubstr hoher SWAP-Last $SwapUsed% > $SwapLoadLimit%$NOCOL"; fi
        if [ $FreqCPUs -lt $FreqLimit ] && [ $ChkTemp -gt 0 ]; then echo -e "$wartesubstr lastbedingt zu geringer CPU-Frequenz ${FreqCPUs}MHz$NOCOL"; fi
        if [ ${CurrTempChk%.*} -gt $TempLimit ] && [ $ChkTemp -gt 0 ]; then echo -e "$wartesubstr lastbedingt zu hoher CPU-Temperatur $CurrTempChk°C$NOCOL"; fi
        chkHiTemp
        getCpuTemp
        chkHiTemp
        getCpuTemp
        chkHiTemp
        getCpuTemp
        chkHiTemp
      else
        getXxxLoad
        if [ $ChkRam -eq 0 ]; then RamUsed=0; fi
        if [ $ChkTemp -le 0 ]; then
          CurrTempChk=0.0
          FreqCPUs=$FreqLimit
          if [ "$redlvl" == "" ]; then redlvl=0; fi
          CpuLimitDefneu=$p6
        fi
        if [ $SwapLoadFree -lt $MemMinimumSizeKb ]; then echo "Geringer verfügbarer SWAP-Bereich ($(( $SwapLoadFree / 1024 ))Mb < $(( $MemMinimumSizeKb / 1024 ))Mb), zur Vermeidung von eventuellem Lockup SWAP vergrößern!"; fi
        if [ $ChkRam -gt 0 ] && [ $RamLoadFree -lt $MemMinimumSizeKb ]; then echo "Verfügbarer RAM-Bereich ($(( $RamLoadFree / 1024 ))Mb<$(( $MemMinimumSizeKb / 1024 ))Mb) ist eventuell zu klein"; fi
        if [ $RamUsed -gt $RamLoadLimit ] && [ $ChkRam -gt 0 ]; then echo "Hohe RAM-Last $RamUsed% > $RamLoadLimit"; fi
        if [ $SwapUsed -gt $SwapLoadLimit ]; then echo "Hohe SWAP-Last $SwapUsed% > $SwapLoadLimit"; fi
        if [ $HiLoad -gt $HiLoadLimit ]; then echo -e "Hohe CPU-Last $HiLoad% > $HiLoadLimit%\n($CurrLoadStr)"; fi
        if [ $FreqCPUs -lt $FreqLimit ] && [ $ChkTemp -gt 0 ]; then echo "Geringe CPU-Frequenz ${FreqCPUs}MHz"; fi
        if [ ${CurrTempChk%.*} -gt $TempLimit ] && [ $ChkTemp -gt 0 ]; then echo "Hohe CPU-Temperatur $CurrTempChk°C"; fi
      fi
    else
      getXxxLoad
      RamUsed=0
      CurrTempChk=0.0
      FreqCPUs=$FreqLimit
      redlvl=0
      CpuLimitDefneu=$p6
    fi
    p6=$CpuLimitDefneu
    addstr="Vor Konvertierung: "
    indstatusstringsub
    case "${p0,,}" in
      "2160p"|"4k"|"uhd")     VMOD="2160p" ; shift ;;
      "1080p"|"hdfull")       VMOD="1080p" ; shift ;;
      "720p"|"hdready")       VMOD="720p" ; shift ;;
      "540p"|"hdvd")          VMOD="540p" ; shift ;;
      "480p"|"dvd")           VMOD="480p" ; shift ;;
      "360p"|"ldvd")          VMOD="360p" ; shift ;;
      "android"|"low"|"272p") VMOD="272p" ; shift ;;
      "v2160p"|"v4k"|"vuhd")  VMOD="v2160p" ; shift ;;
      "v1080p"|"vhdfull")     VMOD="v1080p" ; shift ;;
      "v720p"|"vhdready")     VMOD="v720p" ; shift ;;
      "v540p"|"vhdvd")        VMOD="v540p" ; shift ;;
      "v480p"|"vdvd")         VMOD="v480p" ; shift ;;
      "v360p"|"vldvd")        VMOD="v360p" ; shift ;;
      "vandroid"|"vlow"|"v272p") VMOD="v272p" ; shift ;;
      "copy"|"vcopy")         VMOD="copy" ; shift ;;
      *)                      ;;
    esac
    # automatische AV-Parametererkennung versuchen...
    durprep=$(( $(date +'%s') - $vorprep ))
    vorffprobe=$(date +'%s')
    if [ "$p2" == "" ]; then p2=0
    #elif [ "${p2:0:1}" != "[0-9]" ]; then p2=0
    fi
    ffprobe -of csv -show_streams -read_intervals ${p2}%+1200 "$p1"  > $logDir$p1.ffprobe-log1 2> $logDir$p1.ffprobe-errlog #/dev/null
    tr -s \\n ' ' < $logDir$p1.ffprobe-log1 > $logDir$p1.ffprobe-log
    tmp_dstr="$(cat $logDir$p1.ffprobe-errlog | grep -E 'Duration:')"
    tval_delta_h=$(echo "$tmp_dstr"|cut -c13-14) ; [ "$tval_delta_h" == "0[0-9]" ] && tval_delta_h=${tval_delta_h:1}
    tval_delta_m=$(echo "$tmp_dstr"|cut -c16-17) ; [ "$tval_delta_m" == "0[0-9]" ] && tval_delta_m=${tval_delta_m:1}
    tval_delta_s=$(echo "$tmp_dstr"|cut -c19-20) ; [ "$tval_delta_s" == "0[0-9]" ] && tval_delta_s=${tval_delta_s:1}
    tval_delta_ss=$(echo "$tmp_dstr"|cut -c21-23) ; 
    if [ "$tval_delta_ss" != ".[0-9][0-9]" ]; then
      tval_delta_ss=0
      tval_delta_ss2=0
    else
      tval_delta_ss2=$(( "${tval_delta_ss:1}" / 100 ))
      tval_delta_ss="${tval_delta_ss:1}0"
    fi
    if [ "$tval_delta_h" == 'N/' ] || [ "$tval_delta_m" == 'N/' ] || [ "$tval_delta_s" == 'N/' ]; then
      if [ "$p3" == "" ]; then 
        echo -e "$SETBGRT${SETWS}Zeitmarkenfehler in \'$p1\' - Bearbeitung nicht möglich, Abbruch!$SETBGSW$NOCOL"
        mkdir -p $offenDir
        mvNachOffen $p1 $srcDir
        clrlogfiles
        exit -127
      else
        tval=$p2
        get_tval_sec
        p2o=$tval_sec
        tval=$p3
        get_tval_sec
        p3o=$tval_sec
        tval_delta=$p3o #$(( $p3o - $p2o ))
        tval_delta_h=$(( $tval_delta / 3600 ))
        tval_delta_m=$(( $(( $tval_delta / 60 )) % 60 ))
        tval_delta_s=$(( $tval_delta % 60 ))
        echo -e "$SETBGWS${SETRT}Zeitmarkenfehler in \'$p1\', trotzdem Konvertierung (${p3o}s-${p2o}s, Länge mit Vorlauf: ${tval_delta}s=$tval_delta_h:$tval_delta_m:$tval_delta_s.$tval_delta_ss) versuchen...$SETBGSW$NOCOL"
      fi
    else
      tval_delta_ss=$(echo "$tmp_dstr"|cut -c22-23|sed 's/\,//;s/ //g')
      tval_delta=$(( $(( 10#$tval_delta_h * 3600 )) + $(( 10#$tval_delta_m * 60 )) + 10#$tval_delta_s + $tval_delta_ss2 ))
    fi
    p3i=$tval_delta
    if [ "$p3" == "" ]; then p3="$tval_delta_h:$tval_delta_m:$tval_delta_s.$tval_delta_ss"; fi
    tval=$p2
    get_tval_sec
    p2o=$tval_sec
    tval=$p3
    get_tval_sec
    p3o=$tval_sec
    if [ $p2o -gt $p3i ] || [ $p2o -gt $p3o ] || [ $p3i -lt $p3o ]; then
      [ $p2o -gt $p3i ] && echo -e "${SETRT}$p2i -gt $p3i, Startzeit > Filmdauer!$NOCOL"
      [ $p2o -gt $p3o ] && echo -e "${SETRT}$p2o -gt $p3o, Startzeit > Endzeit!$NOCOL"
      [ $p3i -lt $p3o ] && echo -e "${SETRT}$p3i -lt $p3o, Stopzeit > Endzeit!$NOCOL"
      clrlogfiles
      exit -126
    fi
    tmp_brstr=${tmp_dstr#*, bitrate: }
    tmp_brval=$(echo "$tmp_brstr" | cut -c1-5)
    tmp_brval=${tmp_brval/ /}
    tmp_brstr=$(echo "$tmp_brstr" | cut -c5-)
    tmp_brstr=${tmp_brstr/ /}
    if [ "$tmp_brstr" == "Mb/s" ]; then tmp_brstr="kb/s"; tmp_brval=$(( $tmp_brval * 1024 )); fi
    sleep 3s
    clrFilesSub "$logDir$p1.ffprobe-log1"
    durffprobe=$(( $(date +'%s') - $vorffprobe ))
    if [ -f "$p1" ]; then
      ## ----------------------------------------------------------------------------------------------------------------
      # Videostream-Auflösung und -FPS erkennen
      b_ResStr=$(ffprobe "$p1" 2>&1 >/dev/null | grep " Video: ")
      b_ResStr1=$(echo $b_ResStr | grep -o ", [0-9][0-9][0-9][0-9]x[0-9][0-9][0-9][0-9]" | sed -e 's/\, //g;s/ //g')
      [ "$b_ResStr1" == "" ] && b_ResStr1=$(echo $b_ResStr | grep -o ", [0-9][0-9][0-9][0-9]x[0-9][0-9][0-9]" | sed -e 's/\, //g;s/ //g')
      [ "$b_ResStr1" == "" ] && b_ResStr1=$(echo $b_ResStr | grep -o ", [0-9][0-9][0-9]x[0-9][0-9][0-9]" | sed -e 's/\, //g;s/ //g')
      if [ "$b_ResStr1" == "" ]; then inresdtxt=; QRESD=
      else inresdtxt="$b_ResStr1"; QRESD="$b_ResStr1"
      fi
      b_3840x2160=$(cat $logDir$p1.ffprobe-log | grep -c "3840,2160")
      b_1920x1080=$(cat $logDir$p1.ffprobe-log | grep -c "1920,1080")
      b_1280x720=$(cat $logDir$p1.ffprobe-log | grep -c "1280,720")
      b_720x576=$(cat $logDir$p1.ffprobe-log | grep -c "720,576")
      b_720x384=$(cat $logDir$p1.ffprobe-log | grep -c "720,384")
      b_25FPS=$(cat $logDir$p1.ffprobe-log | grep -c ",25/1,")
      b_29FPS97=$(cat $logDir$p1.ffprobe-log | grep -c ",29.97/1,")
      b_30FPS=$(cat $logDir$p1.ffprobe-log | grep -c ",30/1,")
      b_50FPS=$(cat $logDir$p1.ffprobe-log | grep -c ",50/1,")
      b_60FPS=$(cat $logDir$p1.ffprobe-log | grep -c ",60/1,")
      b_100FPS=$(cat $logDir$p1.ffprobe-log | grep -c ",100/1,")
      if [ $b_25FPS -gt 0 ]; then FPSDEF=25000; infpsval=25
      elif [ $b_29FPS97 -gt 0 ]; then FPSDEF=29970; infpsval=29.970
      elif [ $b_30FPS -gt 0 ]; then FPSDEF=30000; infpsval=30
      elif [ $b_50FPS -gt 0 ]; then FPSDEF=50000; infpsval=50
      elif [ $b_60FPS -gt 0 ]; then FPSDEF=60000; infpsval=60
      elif [ $b_100FPS -gt 0 ]; then FPSDEF=100000; infpsval=100
      else FPSDEF=; infpsval=unbekannt
      fi
      if [ "" == "$FPSDEF" ]; then FPSDEF=25000; FPSDIFF=$(( $FPSDEF * $FPSD / 1000 )); fps_val=
      else FPSDIFF=$(( $FPSDEF * $FPSD / 1000 )); fps_val=$(echo "scale=3 ; ( $FPSDEF / 1000 )" | bc)
      fi
      RFPSL=$(( $FPSDEF - $FPSDIFF ))
      RFPSH=$(( $FPSDEF + $FPSDIFF ))
      if [ "$p1neu" == "" ]; then ONAM1=$( Namenfilter $p1 ); p1orig="$ONAM1"
      else
        p1orig=$( Namenfilter $p1 )
        sendermarker
        if [ $kennung_gefunden -eq 0 ] || [[ "$p1neu" == *"-HD-dvbt2"* ]] || [[ "$p1neu" == *"-HD"* ]]; then p1neu="${p1neu}"; else p1neu=$(echo "${p1neu}_${sendername}"|sed 's/__/_/g'); fi
        ONAM1="$p1neu"
      fi
      ONAM2=
      # Start- und Stopzeit, FPS, Offset, Ausgabedateiname
      if [ -n $p2 ] && [ "$p2" != "" ]; then tval=$p2; ATCMD="-ss $p2 "; get_tval_sec; px2="$tval_h:$tval_m:$tval_s.$tval_ss"
      else tval_sec=0
      fi
      tval_start=$tval_sec
      if [ -n $p3 ] && [ "$p3" != "" ]; then tval=$p3; ETCMD="-to $p3 "; get_tval_sec; px3="$tval_h:$tval_m:$tval_s.$tval_ss2"
      else tval_sec=0
      fi
      tval_stop=$tval_sec
      if [ $tval_stop -lt $tval_start ]; then echo -e "${SETRT}$tval_stop -le $tval_start, Stopzeit < Startzeit!$NOCOL"; clrlogfiles; exit -124
      fi
      if [ $tval_stop -eq 0 ]; then
        if [ $tval_start -ne 0 ]; then tval_stop=$tval_delta; tval_delta=$(( $tval_delta - $tval_start ))
        else tval_stop=$tval_delta
        fi
      else
        if [ $tval_start -eq 0 ]; then tval_delta=$tval_stop
        else tval_delta=$(( $tval_stop - $tval_start ))
        fi
      fi
      tval_delta_i=$tval_delta
      tval_delta_h=$(( $(( $tval_delta / 3600 )) % 24 )) ; tval_delta_m=$(( $(( $tval_delta / 60 )) % 60 )) ; tval_delta_s=$(( $tval_delta % 60 ))
      [ $tval_delta_m -lt 10 ] && tval_delta_m="0$tval_delta_m"
      [ $tval_delta_s -lt 10 ] && tval_delta_s="0$tval_delta_s"
      if [ $tval_delta -gt 0 ]; then tval_delta_string="_${tval_delta_h}h${tval_delta_m}m${tval_delta_s}s"
      else tval_delta_string=""
      fi
      # Audiooffset- und FPS-Korrektur zu Konvertierparametern verarbeiten  
      if [ -n $p4 ] && [ "$p4" != "" ]; then
        if [ $p4 -lt $RFPSL ] || [ $RFPSH -lt $p4 ]; then echo "FPS=$p4 ungültig ($RFPSL...$RFPSH)"
        else
          fpsnewval=`perl -E "say $p4 / 1000"`
          QFPSD=$fpsnewval
          QFPSC="-r $fpsnewval "
          if [ $nameMitFpsOfs -gt 0 ]; then ONAM2=_${ONAM2}_f_$p4; fi
        fi
      else fpsnewval="unverändert"
      fi
      if [ -n $p5 ] && [ "$p5" != "" ]; then
        p5a=`perl -E "say abs($p5)"`
        if [ $p5a -le $POFS ]; then
          offsetval=`perl -E "say $p5 / 1000"`
          OFSC="-itsoffset $offsetval "
          if [ $nameMitFpsOfs -gt 0 ]; then ONAM2=_${ONAM2}_o_$p5; fi
        else
          echo "Offset=$p5 ungültig \(-$POFS...$POFS\), ignoriert"
        fi
      else OFSC=; offsetval=0
      fi
      # Zielauflösung
      QRESC=
      convVertical=""
      case ${VMOD,,} in
        "v2160p"|"v1080p"|"v720p"|"v540p"|"v480p"|"v360p"|"v272p"|"vuhd"|"vhdfull"|"vhdready"|"vhdvd"|"vdvd"|"vldvd"|"vandroid"|"vcopy") convVertical="$verticalFilter" ;; 
        *)                                                     ;;
      esac
      # alle mit Faktor <MaxRes> / <ZielRes> errechneten Werte + 100! Schätzwert für eventuelle Konvertiernachteile bei kleineren Bitraten
      case "${VMOD,,}" in
        "2160p"|"v2160p"|"uhd"|"vuhd")       QRESD="3840x2160" ; calcMinVbr 1 ; min_br=$(( $min_vbr + $(( 2 * $min_abr )) )) ;;
        "1080p"|"v1080p"|"hdfull"|"vhdfull") QRESD="1920x1080" ; calcMinVbr 4 ; min_br=$(( $min_vbr + $(( 2 * $min_abr )) )) ;;
        "720p"|"v720p"|"hdready"|"vhdready") QRESD="1280x720" ; calcMinVbr 9 ; min_br=$(( $min_vbr + $(( 2 * $min_abr )) )) ;;
        "540p"|"v540p"|"hdvd"|"vhdvd")       QRESD="960x540" ; calcMinVbr 16 ; min_br=$(( $min_vbr + $(( 2 * $min_abr )) )) ;;
        "480p"|"v480p"|"dvd"|"vdvd")         QRESD="720x480" ; calcMinVbr 24 ; min_br=$(( $min_vbr + $(( 2 * $min_abr )) )) ;;
        "360p"|"v360p"|"ldvd"|"vldvd")       QRESD="640x360" ; calcMinVbr 24 ; min_br=$(( $min_vbr + $(( 2 * $min_abr )) )) ;;
        "272p"|"v272p"|"android"|"vandroid") QRESD="480x272" ; calcMinVbr 65 ; min_br=$(( $min_vbr + $(( 2 * $min_abr )) )) ;;
        "copy"|"vcopy")       videodef
                              case "${inresdtxt,,}" in
                                "3840x2160")  calcMinVbr 1 ; min_br=$(( $min_vbr + $(( 2 * $min_abr )) )) ;;
                                "1920x1080")  calcMinVbr 4 ; min_br=$(( $min_vbr + $(( 2 * $min_abr )) )) ;;
                                "1280x720")   calcMinVbr 9 ; min_br=$(( $min_vbr + $(( 2 * $min_abr )) )) ;;
                                "960x540")    calcMinVbr 16 ; min_br=$(( $min_vbr + $(( 2 * $min_abr )) )) ;;
                                "720x480")    calcMinVbr 24 ; min_br=$(( $min_vbr + $(( 2 * $min_abr )) )) ;;
                                "640x360")    calcMinVbr 44 ; min_br=$(( $min_vbr + $(( 2 * $min_abr )) )) ;;
                                "480x272")    calcMinVbr 65 ; min_br=$(( $min_vbr + $(( 2 * $min_abr )) )) ;;
                                *)            calcMinVbr 65 ; min_br=$(( $min_vbr + $(( 2 * $min_abr )) )) ;;
                              esac ;;
        *)         echo -e "${SETRT}Unbekannter VMOD=\"$VMOD\" - erlaubt: UHD (=2160p), HDfull (=1080p), HDready (=720p), HDVD (=540p), DVD (=480p), LDVD (=360p), Android (=272p), Copy/Vcopy" ; 
                   clrlogfiles ; exit 1 ;;
      esac
      if [ "${VMOD,,}" == "copy" ] || [ "${VMOD,,}" == "vcopy" ]; then
        if [ $b_HEVC -gt 0 ]; then ONAM3="_HEVC" ; min_vbr=$(( $(( $min_vbr * 7 )) / 10 )) ;  min_br=$(( $(( $min_br * 11 )) / 20 ))
        elif [ $b_H264 -gt 0 ]; then ONAM3="_H264" ; min_br=$(( $(( $min_br * 11 )) / 20 ))
        elif [ $b_MPEG4 -gt 0 ]; then ONAM3="_MPEG4" ; min_vbr=$(( $min_vbr / 2 )) ;  min_br=$(( $(( $min_br * 11 )) / 20 ))
        elif [ $b_VP9 -gt 0 ]; then ONAM3="_VP9" ; min_abr=$(( $(( $min_abr * 2 )) / 5 )) ; min_vbr=$(( $(( $min_vbr * 3 )) / 5 )) ;  min_br=$(( $(( $min_br * 9 )) / 20 ))
        elif [ $b_AV1 -gt 0 ]; then ONAM3="_AV1" ; min_abr=$(( $(( $min_abr * 2 )) / 5 )) ; min_vbr=$(( $min_vbr / 2 )) ;  min_br=$(( $(( $min_br * 8 )) / 20 ))
        else ONAM3="_xxx"
        fi
        min_br=$(( $(( $min_br * 8 )) / 20 ))
      elif [ $HevcVideoOut -eq 3 ]; then
        ONAM3="_AV1"
        #vcodecSpecific=1
        min_abr=$(( $(( $min_abr * 2 )) / 5 ))
        min_vbr=$(( $min_vbr / 2 ))
        case "${VMOD,,}" in
          "2160p"|"v2160p"|"1080p"|"v1080p"|"720p"|"v720p"|"uhd"|"vuhd"|"hdfull"|"vhdfull"|"hdready"|"vhdready") min_br=$(( $(( $min_br * 8 )) / 20 )) ;;
          "480p"|"v480p"|"540p"|"v540p"|"360p"|"v360p"|"dvd"|"vdvd"|"hdvd"|"vhdvd"|"ldvd"|"vldvd") min_br=$(( $(( $min_br * 10 )) / 20 )) ;;
          "272p"|"v272p"|"android"|"vandroid") min_br=$(( $(( $min_br * 12 )) / 20 )) ;;
          *)                    ;;
        esac
      elif [ $HevcVideoOut -eq 2 ]; then
        ONAM3="_VP9"
        #vcodecSpecific=1
        min_abr=$(( $(( $min_abr * 2 )) / 5 ))
        min_vbr=$(( $(( $min_vbr * 3 )) / 5 ))
        case "${VMOD,,}" in
          "2160p"|"v2160p"|"1080p"|"v1080p"|"720p"|"v720p"|"uhd"|"vuhd"|"hdfull"|"vhdfull"|"hdready"|"vhdready") min_br=$(( $(( $min_br * 9 )) / 20 )) ;;
          "480p"|"v480p"|"540p"|"v540p"|"360p"|"v360p"|"dvd"|"vdvd"|"hdvd"|"vhdvd"|"ldvd"|"vldvd") min_br=$(( $(( $min_br * 11 )) / 20 )) ;;
          "272p"|"v272p"|"android"|"vandroid") min_br=$(( $(( $min_br * 13 )) / 20 )) ;;
          *)                    ;;
        esac
      elif [ $HevcVideoOut -eq 1 ]; then
        ONAM3="_HEVC"
        min_vbr=$(( $(( $min_vbr * 7 )) / 10 ))
        case "${VMOD,,}" in
          "2160p"|"v2160p"|"1080p"|"v1080p"|"720p"|"v720p"|"uhd"|"vuhd"|"hdfull"|"vhdfull"|"hdready"|"vhdready") min_br=$(( $(( $min_br * 10 )) / 20 )) ;;
          "480p"|"v480p"|"540p"|"v540p"|"360p"|"v360p"|"dvd"|"vdvd"|"hdvd"|"vhdvd"|"ldvd"|"vldvd") min_br=$(( $(( $min_br * 12 )) / 20 )) ;;
          "272p"|"v272p"|"android"|"vandroid") min_br=$(( $(( $min_br * 15 )) / 20 )) ;;
          *)                    ;;
        esac
      else
        ONAM3="_H264"
        case "${VMOD,,}" in
          "2160p"|"v2160p"|"1080p"|"v1080p"|"720p"|"v720p"|"uhd"|"vuhd"|"hdfull"|"vhdfull"|"hdready"|"vhdready") min_br=$(( $(( $min_br * 14 )) / 20 )) ;;
          "480p"|"v480p"|"540p"|"v540p"|"360p"|"v360p"|"dvd"|"vdvd"|"hdvd"|"vhdvd"|"ldvd"|"vldvd") min_br=$(( $(( $min_br * 18 )) / 20 )) ;;
          "272p"|"v272p"|"android"|"vandroid") min_br=$(( $(( $min_br * 22 )) / 20 )) ;;
          *)                    ;;
        esac
      fi
      # audiostream - XLANG
      sum_astream=0
      if [ $ASEL -eq -2 ]; then getlang; fi
      if [ $ASEL -lt -1 ]; then
        for STRIDX in {1..10}; do
          lebendig
          findaudiostream
          if [ $ASEL -le -1 ] && [ $ASEL -ge -2 ]; then ASEL=$is1staudiostream; fi
        done
      fi
      # finde alle gültigen Streams
      [ "$min_vbr" != "" -a "$min_vbr" != "9999" ] && min_br=$(( $min_vbr + $(( $sum_astream * $min_abr )) ))
      bit_rate=$(( $(( $min_br * 1000 )) / 1 ))
      min_rate=$(( $(( $min_br * 1 )) / 8 ))
      max_rate=$(( $(( $min_br * 4 )) / 1 ))
      for STRIDX in {0..10}; do
        if [ 0 -gt $VSTRN ]; then selvideostream; getaspect; fi
      done
      for STRIDX in {0..10}; do
        lebendig
        if [ -1 -eq $ASEL ]; then
          if [ 0 -gt $ASTR1N ] || [ 0 -gt $ASTR2N ] || [ 0 -gt $ASTR3N ] || [ 0 -gt $ASTR4N ] || [ 0 -gt $ASTR5N ] || [ 0 -gt $ASTR6N ]; then selaudiostream; fi
          specialaaclatm=1
        else
          if [ $ASEL -eq $STRIDX ]; then selaudiostream; fi
        fi
      done
      for STRIDX in {0..10}; do
        lebendig
        if [ 0 -gt $SSTR1N ] || [ 0 -gt $SSTR2N ] || [ 0 -gt $SSTR3N ] || [ 0 -gt $SSTR4N ]; then selsubtitlestream; fi
      done
      # Audio-Konfigurationssonderoptionen entsprechend Vorgabe für bestimmte Sender oder entsprechend manueller Auswahl
      if [ 1 = $convaaclatm ]; then
        #Audiostreams in MP4-Container einbetten (kann bei AACLATM-Audiostreams zu unbrauchbarem Ergebnis führen)
        specialaaclatm=0
      elif [ 1 = $specialaaclatm ]; then
        echo -n "" #Audiostreams in TS-Container einbetten
      elif [ 1 -le $a_AAC_LATM_LC ] || [ 1 -le $a_AAC_LATM_HEAAC ]; then # && [ 0 -eq $a_AAC_LATM_HEAAC ]; then
        strnr=-1
        astreamsel
      #elif [ 0 -eq $a_AAC_LATM_LC ] && [ 1 -le $a_AAC_LATM_HEAAC ]; then specialaaclatm=0
      else 
        specialaaclatm=0
      fi
      if [ "${pcfg,,}" == "-umbenennen" ] || [ "${VMOD,,}" == "copy" ] || [ "${VMOD,,}" == "vcopy" ]; then
        if [ $b_HEVC -gt 0 ]; then ONAM3="_HEVC"
        elif [ $b_H264 -gt 0 ]; then ONAM3="_H264"
        elif [ $b_MPEG4 -gt 0 ]; then ONAM3="_MPEG4"
        elif [ $b_VP9 -gt 0 ]; then ONAM3="_VP9"
        elif [ $b_AV1 -gt 0 ]; then ONAM3="_AV1"
        else ONAM3="_xxx"
        fi
      fi
      [ "${pcfg,,}" != "-umbenennen" ] && echo -e "$SETBGGR${SETBL}Zielbitrate für Auflösung $SETBGWS$SETBLD$QRESD$SETBGGR$SETBL und Video-Codec $SETBGWS$SETBLD${ONAM3:1}$SETBGGR$SETBL: $SETBGWS$SETBLD ≥ ${min_vbr}kbps$SETBGSW$NOCOL"
      sinc="$SUBD0$SUBD1$SUBD2$SUBD3$SUBD4$SUBD5$SUBD6"
      if [ "$QRESD" != "" ]; then
        QRESC="-s:0 $QRESD "
        if [ $voutd = "copy" ] || [ $voutd = "vcopy" ] || [ $voutd = "" ]; then
          if [ $b_H264 -gt 0 ] || [ $b_MPEG4 -gt 0 ]; then
            if [ ${p0,,} = "copy" ] || [ ${p0,,} = "vcopy" ]; then vindef; voutcp
            elif [ "$inresdtxt" == "$QRESD" ] && [ "$convVertical" == "" ]; then voutcp
            else
              echo -e "${SETOR}Achtung: Wegen Bildgrößenänderung ist Umcodieren statt Kopieren nötig.$NOCOL"
              if [ $HevcVideoOut -eq 2 ]; then
                [ $vcodecSpecific -eq 1 ] && h264hevcDefaultOption="$Vp9OptExt" || h264hevcDefaultOption=""
                voutvp9
              elif [ $HevcVideoOut -eq 3 ]; then
                [ $vcodecSpecific -eq 1 ] && h264hevcDefaultOption="$Av1OptExt" || h264hevcDefaultOption=""
                voutav1
              elif [ $HevcVideoOut -eq 1 ]; then vouthevc
              else vouth264
              fi
            fi
          fi
        fi
      fi
      # Überschreiben der Video-Eingangskonfiguration, wenn Audiooffset- oder FPS-Korrektur ausgewählt sind
      if [ -z $QFPSC ] || [ -z $OFSC ]; then
        echo -n "" #kein AV-sync
      else
        if [ $b_AV1 -gt 0 ] || [ $b_VP9 -gt 0 ] || [ $b_HEVC -gt 0 ] || [ $b_H264 -gt 0 ] || [ $b_MPEG4 -gt 0 ]; then
          if [ "${p0,,}" -eq "vcopy" ] || [ "${p0,,}" -eq "copy" ]; then vindef
          else
            [ $b_AV1 -gt 0 ] && vinav1
            [ $b_VP9 -gt 0 ] && vinvp9
            [ $b_HEVC -gt 0 ] && vinhevc
            [ $b_H264 -gt 0 ] && vinh264
            [ $b_MPEG4 -gt 0 ] && vinh264
          fi
        fi
        #AV-sync
        if [ "${p0,,}" -eq "vcopy" ] || [ "${p0,,}" -eq "copy" ]; then
          if [ "$inresdtxt" == "$QRESD" ]; then voutcp
          elif [ $HevcVideoOut -eq 2 ]; then
            [ $vcodecSpecific -eq 1 ] && h264hevcDefaultOption="$Vp9OptExt" || h264hevcDefaultOption=""
            voutvp9
          elif [ $HevcVideoOut -eq 3 ]; then
            [ $vcodecSpecific -eq 1 ] && h264hevcDefaultOption="$Av1OptExt" || h264hevcDefaultOption=""
            voutav1
          elif [ $HevcVideoOut -eq 1 ]; then vouthevc
          else vouth264
          fi
        else
          if [ "$inresdtxt" == "$QRESD" ]; then voutcp
          elif [ $HevcVideoOut -eq 2 ]; then
            [ $vcodecSpecific -eq 1 ] && h264hevcDefaultOption="$Vp9OptExt" || h264hevcDefaultOption=""
            voutvp9
          elif [ $HevcVideoOut -eq 3 ]; then
            [ $vcodecSpecific -eq 1 ] && h264hevcDefaultOption="$Av1OptExt" || h264hevcDefaultOption=""
            voutav1
          elif [ $HevcVideoOut -eq 1 ]; then vouthevc
          else vouth264
          fi
        fi
      fi
      # Auswahl des Zielcontainers abhängig von als problematisch erkanntem Audio-Quellenformat
      if [ "${pcfg,,}" == "-umbenennen" ]; then ONAM4=.${p1##*.}
      #elif [ $specialaaclatm -eq 1 -a $b_AAC_LATM -ge 1 -a $forcemp4 -eq 0 ] || [ $forcets -eq 1 ]; then echo -e "TS-Container verwenden "; ONAM4=.ts
      elif [ $HevcVideoOut -eq 2 ] || [ $HevcVideoOut -eq 3 ]; then ONAM4=.webm
      else ONAM4=.mp4
      fi
      if [ "$QRESD" == "" ]; then #[ "$QRESC" == "" ]; then
        [ $anzeigeOptimierung -eq 0 ] && ONAM3=$ONAM3$tval_delta_string$ONAM4
        [ $anzeigeOptimierung -ne 0 ] && ONAM3=${ONAM3}_$ONAM3a$tval_delta_string$ONAM4
      else
        [ $anzeigeOptimierung -eq 0 ] && ONAM3=_$QRESD$ONAM3$tval_delta_string$ONAM4
        [ $anzeigeOptimierung -ne 0 ] && ONAM3=_$QRESD${ONAM3}_$ONAM3a$tval_delta_string$ONAM4
      fi
      echo -e "${SETBL}" >> $logDir$p1.ffmpeg-tmp
      if [ "${pcfg,,}" != "-umbenennen" ]; then
        tmpxbrvalv="$(cat $logDir$p1.ffprobe-errlog | grep -E ' Video: ')"
        #if [[ "$tmpxbrvalv" == *"\]: Video: "* ]]; then 
        tmpxbrvalv=${tmpxbrvalv%%\[*\]: Video*}
        #elif [[ "$tmpxbrvalv" == *"\): Video: "* ]]; then 
        tmpxbrvalv=${tmpxbrvalv%%(*): Video*}
        #else tmpxbrvalv=0
        #fi
        tmpxbrvalv=${tmpxbrvalv#*:}
        [ $tmpxbrvalv -ge 1 -a $tmpxbrvalv -le 9 ] && felderSchieben $tmpxbrvalv #|| felderSchieben 0
        echo -e "Automatische Audio-Video-Parametererkennung:" >> $logDir$p1.ffmpeg-tmp
        streamstrx=""
        streamstra="| 0:$SUBD0S$MAP0S $SUBE0S|"
        streamstra="$(echo "$streamstra" | sed -e 's/  */ /g')"
        streamstrb=" 1:$SUBD1S$MAP1S $SUBE1S| 2:$SUBD2S$MAP2S $SUBE2S| 3:$SUBD3S$MAP3S $SUBE3S| 4:$SUBD4S$MAP4S $SUBE4S| 5:$SUBD5S$MAP5S $SUBE5S|"
        streamstrb="$(echo "$streamstrb" | sed -e 's/  */ /g;s/ 1: |//g;s/ 2: |//g;s/ 3: |//g;s/ 4: |//g;s/ 5: |//g')"
        streamstrc=" 6:$SUBD6S$MAP6S $SUBE6S| 7:$SUBD7S$MAP7S $SUBE7S| 8:$SUBD8S$MAP8S $SUBE8S| 9:$SUBD9S$MAP9S $SUBE9S| 10:$SUBD10S$MAP10S $SUBE10S|"
        streamstrc="$(echo "$streamstrc" | sed -e 's/  */ /g;s/ 6: |//g;s/ 7: |//g;s/ 8: |//g;s/ 9: |//g;s/ 10: |//g')"
        streamstrlen=$(( ${#streamstra} + ${#streamstrb} + ${#streamstrc} ))
        streamstry="               "
        streamstrz="Stream Mapping "
        if [ $chkcols -ge $(( $streamstrlen + ${#streamstrz} )) ]; then
          for (( idx=0 ; idx<$streamstrlen ; idx++ )); do
            streamstrx="-$streamstrx"
          done
          streamstr="$SETBL$streamstry$streamstrx\n$streamstrz$streamstra$streamstrb$streamstrc\n$streamstry$streamstrx$NOCOL"
        else
          streamstrb="|$streamstrb"
          if [ ${#streamstra} -ge $(( ${#streamstrb} + ${#streamstrc} )) ]; then
            streamstrlen=${#streamstra}
            for (( idx=0 ; idx<$streamstrlen ; idx++ )); do
              streamstrx="-$streamstrx"
            done
            streamstr="$SETBL$streamstry$streamstrx\n$streamstrz$streamstra\n$streamstry$streamstrx\n$streamstry$streamstrb$streamstrc\n$streamstry$streamstrx$NOCOL"
          else
            streamstrlen=$(( ${#streamstrb} + ${#streamstrc} ))
            if [ $chkcols -ge $(( $streamstrlen + ${#streamstrz} )) ]; then
              for (( idx=0 ; idx<$streamstrlen ; idx++ )); do
                streamstrx="-$streamstrx"
              done
              streamstr="$SETBL$streamstry$streamstrx\n$streamstrz$streamstra\n$streamstry$streamstrx\n$streamstry$streamstrb$streamstrc\n$streamstry$streamstrx$NOCOL"
            else
              streamstrc="|$streamstrc"
              if [ ${#streamstra} -ge ${#streamstrb} ] && [ ${#streamstra} -ge ${#streamstrc} ]; then streamstrlen=${#streamstra}
              elif [ ${#streamstrb} -ge ${#streamstrc} ]; then streamstrlen=${#streamstrb}
              else streamstrlen=${#streamstrc}
              fi
              for (( idx=0 ; idx<$streamstrlen ; idx++ )); do
                streamstrx="-$streamstrx"
              done
              streamstr="$SETBL$streamstry$streamstrx\n$streamstrz$streamstra\n$streamstry$streamstrx\n$streamstry$streamstrb\n$streamstry$streamstrx\n$streamstry$streamstrc\n$streamstry$streamstrx$NOCOL"
            fi
          fi
        fi
        echo -e "$streamstr" ; echo -e "$streamstr" >> $logDir$p1.ffmpeg-tmp
      fi
      actin=""
      actout=""
      if [ "$vind" == "" ] && [ "$voutd" == "" ]; then actin="recode"; actout="to default"
      elif [ "$vind" == "" ]; then
        if [ "$voutd" == "copy" ]; then actin="copy"; actout=""
        else actin="recode"; actout="to $voutd"
        fi
      elif [ "$voutd" == "" ]; then actin="recode $vind"; actout="to default"
      else actin="recode $vind"; actout="to $voutd"
      fi
      echo -e "Video-Codec $actin $actout" >> $logDir$p1.ffmpeg-tmp
      actin=""
      actout=""
      if [ "$aind" == "" ] && [ "$aoutd" == "" ]; then actin="recode"; actout="to default"
      elif [ "$aind" == "-copy_unknown " ]; then actin="copy_unknown"; actout=""
      elif [ "$aind" == "" ]; then
        if [ "$aoutd" == "copy" ]; then actin="copy"; actout=""
        else actin="recode"; actout="to $aoutd"
        fi
      elif [ "$aoutd" == "" ]; then actin="recode $aind"; actout="to default"
      else actin="recode $aind"; actout="to $aoutd"
      fi
      echo -e "Audio-Codec $actin $actout" >> $logDir$p1.ffmpeg-tmp
      if [ "$QRESD" == "" ]; then QREST="unverändert"
      else QREST=$QRESD
      fi
      ONAMx=$ONAM1$ONAM2$ONAM3
      ffmpegcmdsub0="nice -n $prioselect $CPULIMIT "
      [ "$zusatzIn" != "" ] && zusatzIn=" $zusatzIn "
      [ "$zusatzOut" != "" ] && zusatzOut=" $zusatzOut "
      if [ "$convVertical" == "" ]; then
        ffmpegcmdsub1="ffmpeg $interactflag $loglvl $HwTranscodePre $vinc$sinc$splitavsrc"
        ffmpegcmdsub2="$IFPSC$OFSC$ainc-i $path$p1 $zusatzIn "
        if [ "$infile3" == "" ]; then ffmpegcmdsub3=""; ffmpegcmdsub3str=""
        else ffmpegcmdsub3="$infile3 "; ffmpegcmdsub3str="$ffmpegcmdsub3"
        fi
        ffmpegcmdsub4a="$QASP $MAP0$SUBE0$MAP1$SUBE1"
        ffmpegcmdsub4b="$MAP2$SUBE2$MAP3$SUBE3$MAP4$SUBE4$MAP5$SUBE5$MAP6$SUBE6$MAP7$SUBE7$MAP8$SUBE8$MAP9$SUBE9$MAP10$SUBE10"
        ffmpegcmdsub4="$ffmpegcmdsub4a$ffmpegcmdsub4b"
        ffmpegcmdsub4str="$ffmpegcmdsub4a $ffmpegcmdsub4b"
      else
        ffmpegcmdsub0=""
        ffmpegcmdsub1="ffmpeg $interactflag " #$loglvl $HwTranscodePre $vinc$sinc$splitavsrc"
        ffmpegcmdsub2=" -i $path$p1 $zusatzIn "
        ffmpegcmdsub3=""
        ffmpegcmdsub3str=""
        ffmpegcmdsub4="$convVertical"
        ffmpegcmdsub4str="$ffmpegcmdsub4"
      fi
      ffmpegcmdsub5="$ATCMD$ETCMD$QRESC$QFPSC $destDir"
      ffmpegcmdsub6="$ONAMx"
      ffmpegcmd=$(echo "$ffmpegcmdsub0 $ffmpegcmdsub1$ffmpegcmdsub2$ffmpegcmdsub3$ffmpegcmdsub4 $zusatzOut $ffmpegcmdsub5$ffmpegcmdsub6" | sed -e 's/  */ /g')
      ffmpegcmds1=$(echo -e " $ffmpegcmdsub0 $ffmpegcmdsub1$ffmpegcmdsub2 " | sed -e 's/  */ /g'); ffmpegcmds1l=${#ffmpegcmds1}
      ffmpegcmds2=$(echo -e " $ffmpegcmdsub3str$ffmpegcmdsub4str $zusatzOut " | sed -e 's/  */ /g'); ffmpegcmds2l=${#ffmpegcmds2}
      ffmpegcmds3=$(echo -e " $ffmpegcmdsub5$ffmpegcmdsub6 " | sed -e 's/  */ /g'); ffmpegcmds3l=${#ffmpegcmds3}
      if [ $ffmpegcmds1l -ge $(( $numcols -15 )) ]; then
        ffmpegcmds1="$SETBGGR$SETTK${ffmpegcmds1% -i *} \\ $SETBGSW$NOCOL \n          $SETBGGR$SETTK -i ${ffmpegcmds1##* -i } \\ $SETBGSW$NOCOL "
      else
        ffmpegcmds1="$SETBGGR$SETTK${ffmpegcmds1}\\ $SETBGSW$NOCOL "
      fi
      if [ $ffmpegcmds2l -ge $(( $numcols -15 )) ]; then
        ffmpegcmds2="          $SETBGGR$SETTK${ffmpegcmds2% -map 0:1 *} \\ $SETBGSW$NOCOL \n          $SETBGGR$SETTK -map 0:1 ${ffmpegcmds2##* -map 0:1 }\\ $SETBGSW$NOCOL "
      else
        ffmpegcmds2="          $SETBGGR$SETTK${ffmpegcmds2}\\ $SETBGSW$NOCOL "
      fi
      if [ $ffmpegcmds3l -ge $(( $numcols -15 )) ]; then
        ffmpegcmds3="          $SETBGGR$SETTK${ffmpegcmds3%\/*}/... $SETBGSW$NOCOL \n          $SETBGGR$SETTK ...${ffmpegcmds3##*/}$SETBGSW$NOCOL "
      else
        ffmpegcmds3="          $SETBGGR$SETTK$ffmpegcmds3$SETBGSW$NOCOL "
      fi
      ffmpegcmdstr=$(echo -e "$ffmpegcmds1\n$ffmpegcmds2\n$ffmpegcmds3")
      prepName $ffmpegcmdsub6
      if [ "${pcfg,,}" == "-umbenennen" ]; then
        clrlogfiles
        if [ "$p1" != "$ONAMx" ]; then
          mv "$p1" "$ONAMx" > /dev/null 2> /dev/null
          [ $? -eq 0 ] && echo -e "$SETBLD${SETBGOK}Umbenennen beendet, neuer Name '$ONAMx'$NOCOL\n$TRENNER\n" || echo -e "$SETBLD${SETBGOK}Fehler bei Umbenennen von '$p1' nach '$ONAMx'$NOCOL\n$TRENNER\n"
        else echo -e "${SETOR}Datei wird nicht umbenannt, Zieldateiname == Quelldateiname!$NOCOL"
        fi
      else
        substr=""
        if [ -f "$destDir$ONAMx" ] || [ -f "$destDir$pcfg2$ONAMx" ] || [ -f "$destDir$pcfg2$pcfg2_2$ONAMx" ] ||
           [ -f "$destDir$ONAM1$ONAM2${ONAM3%.*}\*$ONAM4" ] || [ -f "$destDir$pcfg2$ONAM1$ONAM2${ONAM3%.*}\*$ONAM4" ] || [ -f "$destDir$pcfg2$pcfg2_2$ONAM1$ONAM2${ONAM3%.*}\*$ONAM4" ]; then
          existiertInDest=1
          substr="    DEST="
          [ -f "$destDir$ONAMx" ] && substr="$substr\"$destDir\"  "
          if [ "$pcfg2" != "" ] && [ -f "$destDir$pcfg2$ONAMx" -o -f "$destDir$pcfg2$ONAM1$ONAM2${ONAM3%.*}\*$ONAM4" ]; then substr="$substr\"$destDir$pcfg2\"  "; fi
          if [ "$pcfg2" != "" ] && [ "$pcfg2_2" != "" ]; then substr="$substr\"$destDir$pcfg2$pcfg2_2\""; fi
        else existiertInDest=0
        fi
        if [ -f "$bkup1dir$pcfg2$ONAMx" ] || [ -f "$bkup1dir$pcfg2$pcfg2_2$ONAMx" ] || [ -f "$bkup1dir$pcfg2$ONAM1$ONAM2${ONAM3%.*}\*$ONAM4" ] || [ -f "$bkup1dir$pcfg2$pcfg2_2$ONAM1$ONAM2${ONAM3%.*}\*$ONAM4" ]; then
          existiertInBkup1=1
          if [ "$substr" == "" ]; then substr="    BKUP1=\"$bkup1dir$pcfg2$pcfg2_2\""
          else substr="$substr\n    BKUP1=\"$bkup1dir$pcfg2$pcfg2_2\""
          fi
        else existiertInBkup1=0
        fi
        if [ -f "$bkup2dir$pcfg2$ONAMx" ] || [ -f "$bkup2dir$pcfg2$pcfg2_2$ONAMx" ] || [ -f "$bkup2dir$pcfg2$ONAM1$ONAM2${ONAM3%.*}\*$ONAM4" ] || [ -f "$bkup2dir$pcfg2$pcfg2_2$ONAM1$ONAM2${ONAM3%.*}\*$ONAM4" ]; then
          existiertInBkup2=1
          if [ "$substr" == "" ]; then substr="    BKUP2=\"$bkup2dir$pcfg2$pcfg2_2\""
          else substr="$substr\n    BKUP2=\"$bkup2dir$pcfg2$pcfg2_2\""
          fi
        else existiertInBkup2=0
        fi
        if [ -f "$bkup3dir$pcfg2$ONAMx" ] || [ -f "$bkup3dir$pcfg2$pcfg2_2$ONAMx" ] || [ -f "$bkup3dir$pcfg2$ONAM1$ONAM2${ONAM3%.*}\*$ONAM4" ] || [ -f "$bkup3dir$pcfg2$pcfg2_2$ONAM1$ONAM2${ONAM3%.*}\*$ONAM4" ]; then
          existiertInBkup3=1
          if [ "$substr" == "" ]; then substr="    BKUP3=\"$bkup3dir$pcfg2$pcfg2_2\""
          else substr="$substr\n    BKUP3=\"$bkup3dir$pcfg2$pcfg2_2\""
          fi
        else existiertInBkup3=0
        fi
        if [ "$p1" == "$ONAMx" ]; then
          echoUndLog "${SETOR}Datei wird nicht konvertiert, Zieldateiname == Quelldateiname!$NOCOL"
          sleep 2s
          findDup
          echo -e "\n$NOCOL$TRENNER"
          clrlogfiles
          exit 1
        elif [ $ersetzeDatei -eq 0 ] && [ $(( $existiertInDest + $existiertInBkup1 + $existiertInBkup2 + $existiertInBkup3 )) -gt 0 ]; then
          nameadd="${ONAMx}-alt-$(date +%s)"
          echo -e -n "$SETGN$SETBGGR"
          #[ -f $destDir$ONAMx ] && mvUmbenennen "$ONAMx" "$destDir" "$nameadd"
          #[ -f $destDir$pcfg2$ONAMx ] && mvUmbenennen "$ONAMx" "$destDir$pcfg2" "$nameadd"
          #[ -f $destDir$pcfg2$pcfg2_2$ONAMx ] && mvUmbenennen "$ONAMx" "$destDir$pcfg2$pcfg2_2" "$nameadd"
          [ -f $bkup1dir$ONAMx ] && mvUmbenennen "$ONAMx" "$bkup1dir" "$nameadd"
          [ -f $bkup1dir$pcfg2$ONAMx ] && mvUmbenennen "$ONAMx" "$bkup1dir$pcfg2" "$nameadd"
          [ -f $bkup1dir$pcfg2$pcfg2_2$ONAMx ] && mvUmbenennen "$ONAMx" "$bkup1dir$pcfg2$pcfg2_2" "$nameadd"
          [ -f $bkup2dir$ONAMx ] && mvUmbenennen "$ONAMx" "$bkup2dir" "$nameadd"
          [ -f $bkup2dir$pcfg2$ONAMx ] && mvUmbenennen "$ONAMx" "$bkup2dir$pcfg2" "$nameadd"
          [ -f $bkup2dir$pcfg2$pcfg2_2$ONAMx ] && mvUmbenennen "$ONAMx" "$bkup2dir$pcfg2$pcfg2_2" "$nameadd"
          [ -f $bkup3dir$ONAMx ] && mvUmbenennen "$ONAMx" "$bkup3dir" "$nameadd"
          [ -f $bkup3dir$pcfg2$ONAMx ] && mvUmbenennen "$ONAMx" "$bkup3dir$pcfg2" "$nameadd"
          [ -f $bkup3dir$pcfg2$pcfg2_2$ONAMx ] && mvUmbenennen "$ONAMx" "$bkup3dir$pcfg2$pcfg2_2" "$nameadd"
          echo -e -n "$SETBGSW$NOCOL"
        elif [ $ersetzeDatei -eq 1 ] && [ $(( $existiertInDest + $existiertInBkup1 + $existiertInBkup2 + $existiertInBkup3 )) -gt 0 ]; then
          echo -e -n "$NOCOL$TRENNER\n$SETRT$ONAMx bzw. $ONAM1$ONAM2${ONAM3%.*}*$ONAM4 existiert schon in \n$substr$SETGN, $SETGND$SETBGOK Alte Datei wird gelöscht.$NOCOL"
          clrFilesSub "$destDir$ONAMx"
          clrFilesSub "$bkup1dir$pcfg2$ONAMx"
          clrFilesSub "$bkup2dir$pcfg2$ONAMx"
          clrFilesSub "$bkup3dir$pcfg2$ONAMx"
        fi
        if [ $(( $(( $CpuLimitDefneu / $NumCPUs )) + $(( $CpuPwrKorr )) )) -lt 100 ]; then preStr4="...$(( $(( $CpuLimitDefneu / $NumCPUs )) + $(( $CpuPwrKorr )) ))"
        elif [ $CpuPwrKorr -ge 0 ] && [ $(( $(( $CpuLimitDefneu / $NumCPUs )) + $(( $CpuPwrKorr )) )) -eq 100 ]; then preStr4="...100"; CpuPwrKorr=$(( 100 - $(( $CpuLimitDefneu / $NumCPUs )) ))
        else preStr4=""; CpuPwrKorr=0
        fi
        prepPermanentCheck
        [ "$p2" == "" ] && KonvStrA="" || KonvStrA=", Start=$NOCOL$px2$SETBL"
        [ "$p3" == "" ] && KonvStrB="" || KonvStrB=", Ende=$NOCOL$px3$SETBL"
        [ "$p4" == "" ] && KonvStrC="" || KonvStrC=", FPS=$NOCOL$p4$SETBL"
        [ "$p5" == "" ] && KonvStrD="" || KonvStrD=", OFS=$NOCOL$p5$SETBL"
        sizetimep1=$(getSizeTime "$p1")
        KonvStr1="${SETBL}Konvertierung von '$NOCOL$path$p1$SETBL'$NOCOL\n                  $SETBL($NOCOL$sizetimep1$SETBL$KonvStrA$KonvStrB$KonvStrC$KonvStrD, $NOCOL$tmp_brval$tmp_brstr$SETBL)\nnach $SETBGWS$SETBLD$pcfgup$pcfg2$pcfg2_2$ONAMx$NOCOL$SETBL\n$NumCPUs-Kern-${CpuFreqMax}MHz-CPU, CPU-Frequenz beim Start=$NOCOL${FreqCPUs}MHz$SETBL, Leistungswert/CPU=$NOCOL$(( $(( $CpuLimitDefneu / $NumCPUs )) - $(( $CpuPwrKorr )) ))$preStr4%$SETBL, Filmlänge "
        if [ $tval_delta -eq 0 ]; then echo -e "${KonvStr1}unbekannt$NOCOL\n$TRENNER" >> $logDir$p1.ffmpeg-tmp
        else KonvStr="${KonvStr1}(h:m:s) $NOCOL$tval_delta_h:$tval_delta_m:$tval_delta_s.$tval_delta_ss$NOCOL\n$TRENNER"
        fi
        # Konvertierung ausführen
        logpid=0
        #if [ $philfe -eq 1 ]; then 
          KonvStr="$TRENNER\n$SETBLD${SETBGWS}ffmpegcmd:$NOCOL$ffmpegcmdstr$NOCOL\n$TRENNER\n$KonvStr"
        #else
        #  KonvStr="$TRENNER\n$SETBL$KonvStr"
        #fi
        echo -e "${KonvStr}" ; echo -e "${KonvStr}" >> $logDir$p1.ffmpeg-tmp ; echo -e "${KonvStr}" >> ${logDir}filmschnitt.log
        if [ ! -d "$destDir" ]; then
          echo -e -n " *  ${SETBL}Verzeichnis $NOCOL$destDir$SETBL anlegen$SETAL1"
          mkdir -p "$destDir" 2> /dev/null
          SubOkErr $? 0
        fi
        if [ $vorherFindeDup -ne 0 ]; then
          numcols=$(/usr/bin/tput cols) ; [ $numcols -lt 100 ] && numcols=100
          echoUndLog "$SETBGGR${SETBL}Suche Dateien mit ähnlichem Namen in realen Pfaden (\"$nameAnf2\"):$SETBGSW$NOCOL"
          saveFlag=$durchsucheUnterverzeichnisse
          findeDupAlleSav=$findeDupAlle
          findeDupAlle=0
          [ $durchsucheUnterverzeichnisse -ge 3 ] && durchsucheUnterverzeichnisse=3 || durchsucheUnterverzeichnisse=1  # suche in Arbeitsverzeichnissen
          sleep 2s
          nameAnf2="$(echo "$nameAnf2" | sed -e 's/\*\*/\*/g')"
          tmpNameAnf2="$nameAnf2"
          findDup
          if [ "$p1neu" != "" ]; then
            prepName $p1orig
            [ "$tmpNameAnf2" != "$nameAnf2" ] && (echoUndLog "$SETBGGR${SETBL}Suche Dateien mit ähnlichem Namen in realen Pfaden (\"$nameAnf2\"):$SETBGSW$NOCOL"; findDup)
          fi
          echoUndLog "$NOCOL$TRENNER"
          NameAnf2=$tmpNameAnf2
          findeDupAlle=$findeDupAlleSav
          durchsucheUnterverzeichnisse=$saveFlag
        fi
        # #########################################################################
        $ffmpegcmd # !!! das Kommando "cpulimit" vor "ffmpeg" wirkt wie Start eines Sub-Prozesses mit "... &" !!!
        # #########################################################################
        sync
        tmpcnt=2
        while [ $tmpcnt -gt 0 ]; do
          lebendig ; sleep 10s ; if [ "$(lsof +d $destDir 2> /dev/null | grep -iE "$ffmpegcmdsub6" | grep -iEc "ffmpeg")" -gt 0 -o "$(lsof +d $srcDir 2> /dev/null | grep -iE "$p1" | grep -iEc "ffmpeg")" -gt 0 ]; then break; else tmpcnt=$(( $tmpcnt - 1 )); fi
        done
        tmpcnt=240
        while [ "$(lsof +d $destDir 2> /dev/null | grep -iE "$ffmpegcmdsub6" | grep -iEc "ffmpeg")" -gt 0 -o "$(lsof +d $srcDir 2> /dev/null | grep -iE "$p1" | grep -iEc "ffmpeg")" -gt 0 -a $tmpcnt -gt 0 ]; do
          lebendig ; sleep 30s ; tmpcnt=$(( $tmpcnt - 1 ))
        done
        einruecken
        bkupCnt=0
        bkupCnt2=0
        bkupLimCnt=0
        for bkupXdir in "$bkup1dir" "$bkup2dir" "$bkup3dir"; do
          if [ "$bkupXdir" != "" ] && [ "$pcfg2" != "" ] && [ "$pcfg3" == "+" ]; then bkupLimCnt=$(( $bkupLimCnt + 1 ))
          elif [ "$bkupXdir" != "" ] && [ "$pcfg2" != "" ] && [ -d "$bkupXdir$pcfg2" ]; then bkupLimCnt=$(( $bkupLimCnt + 1 ))
          elif [ "$bkupXdir" != "" ] && [ "$pcfg2" == "" ] ; then bkupLimCnt=$(( $bkupLimCnt + 1 ))
          fi
        done
        echo -e "\n$TRENNER\n" > $logDir$p1.ffmpeg-tmp
        if [ $(lsof +d $srcDir 2> /dev/null | grep -iE "$p1" | grep -iEc "ffmpeg") -gt 0 ]; then
          if [ $(lsof +d $destDir 2> /dev/null | grep -iE "$ffmpegcmdsub6" | grep -iEc "ffmpeg") -gt 0 ]; then
            echo -e "$p1 und $ffmpegcmdsub6 ist noch von FFMPEG geöffnet, warten..."
          else
            echo -e "$p1 ist noch von FFMPEG geöffnet, warten..."
          fi
        elif [ $(lsof 2> /dev/null | grep -iE "$ffmpegcmdsub6" | grep -iEc "ffmpeg") -gt 0 ]; then
          echo -e "$ffmpegcmdsub6 ist noch von FFMPEG geöffnet, warten..."
        fi
        tmpcnt=30
        while [ "$(lsof +d $destDir 2> /dev/null | grep -iE "$ffmpegcmdsub6" | grep -iEc "ffmpeg")" -gt 0 -o "$(lsof 2> /dev/null | grep -iE "$p1" | grep -iEc "ffmpeg")" -gt 0 -a $tmpcnt -gt 0 ]; do
          lebendig ; sleep 30s ; tmpcnt=$(( $tmpcnt - 1 ))
        done
        [ $tmpcnt -eq 0 ] && ( echo -e "${SETFLRT}Abbruch, noch Dateien von FFMPEG geöffnet, $currDateTime$NOCOL" ; clrlogfiles ; exit 0 )
        tmpcnt=30
        ffprobe -show_streams $destDir$ffmpegcmdsub6 > $logDir$p1.ffprobe-log1 2> $logDir$p1.ffprobe-errlog1
        while [ "$(lsof $logDir$p1.ffprobe-errlog1 2> /dev/null)" != "" -a $tmpcnt -gt 0 ]; do
          lebendig ; sleep 2s ; tmpcnt=$(( $tmpcnt - 1 ))
        done
        [ $tmpcnt -eq 0 ] && ( echo -e "${SETFLRT}Abbruch, $logDir$p1.ffprobe-errlog1 noch geöffnet, $currDateTime$NOCOL" ; clrlogfiles ; exit 0 )
        sync
        einruecken
        # Log-Script sofort nach Ende der Konvertierung beenden:
        killPidSub "-e" "$logDir$p1.sh"
        clrFilesSub "$logDir$p1.sh"
        tmp_xbrval=($(cat $logDir$p1.ffprobe-log1 | grep -E 'bit_rate=' | sed -e 's/max_bit_rate=N\/A/ /g;s/max_bit_rate=[1-9][0-9][0-9][0-9][0-9][0-9][0-9]/ /g;s/max_bit_rate=[1-9][0-9][0-9][0-9][0-9][0-9]/ /g;s/max_bit_rate=[1-9][0-9][0-9][0-9][0-9]/ /g;s/max_bit_rate=[1-9][0-9][0-9][0-9]/ /g;s/bit_rate=//g'))
        tmp_xbrval0=${tmp_xbrval[0]}
        tmp_xbrval1=${tmp_xbrval[1]}
        tmp_xbrval2=${tmp_xbrval[2]}
        tmp_xbrval3=${tmp_xbrval[3]}
        tmp_xbrval4=${tmp_xbrval[4]}
        tmp_xbrval5=${tmp_xbrval[5]}
        if [ "$tmp_xbrval0" == "N/A" ]; then tmp_xbrval0=9999
        else
          [ "$tmp_xbrval0" != "" -a  "$tmp_xbrval0" != "9999" ] && tmp_xbrval0=$(( $tmp_xbrval0 / 1000 )) || tmp_xbrval0=9999
        fi
        if [ "$tmp_xbrval1" == "N/A" ]; then tmp_xbrval1=9999
        else
          [ "$tmp_xbrval1" != "" -a  "$tmp_xbrval1" != "9999" ] && tmp_xbrval1=$(( $tmp_xbrval1 / 1000 )) || tmp_xbrval1=9999
        fi
        if [ "$tmp_xbrval2" == "N/A" ]; then tmp_xbrval2=9999
        else
          [ "$tmp_xbrval2" != "" -a  "$tmp_xbrval2" != "9999" ] && tmp_xbrval2=$(( $tmp_xbrval2 / 1000 )) || tmp_xbrval2=9999
        fi
        if [ "$tmp_xbrval3" == "N/A" ]; then tmp_xbrval3=9999
        else
          [ "$tmp_xbrval3" != "" -a  "$tmp_xbrval3" != "9999" ] && tmp_xbrval3=$(( $tmp_xbrval3 / 1000 )) || tmp_xbrval3=9999
        fi
        if [ "$tmp_xbrval4" == "N/A" ]; then tmp_xbrval4=9999
        else
          [ "$tmp_xbrval4" != "" -a  "$tmp_xbrval4" != "9999" ] && tmp_xbrval4=$(( $tmp_xbrval4 / 1000 )) || tmp_xbrval4=9999
        fi
        if [ "$tmp_xbrval5" == "N/A" ]; then tmp_xbrval5=9999
        else
          [ "$tmp_xbrval5" != "" -a  "$tmp_xbrval5" != "9999" ] && tmp_xbrval5=$(( $tmp_xbrval5 / 1000 )) || tmp_xbrval5=9999
        fi
        tmp_durval1=$(cat $logDir$p1.ffprobe-log1 | grep -E 'duration=' | sed -e 's/duration=//g;s/\.[0-9]*/ +/g')  
        tmp_durval1=$(echo $tmp_durval1)
        tmp_durval1=$(echo $tmp_durval1 | sed 's/ +$//')
        tmp_durval1str=$(echo $tmp_durval1 | sed 's/+/x/g;s/ //g')
        tmp_durval1str="_${tmp_durval1str}_statt_${tval_delta}s"
        tmp_durval2=$(cat $logDir$p1.ffprobe-log1 | grep -c 'duration=')
        tmp_durval=$(echo "scale=0 ; ($tmp_durval1) / $tmp_durval2" | bc)
        tmp_dstr="$(cat $logDir$p1.ffprobe-errlog1 | grep -E 'Duration:')"
        tmp_dur_h=$(echo "$tmp_dstr" | cut -c13-14)
        tmp_dur_m=$(echo "$tmp_dstr" | cut -c16-17)
        tmp_dur_s=$(echo "$tmp_dstr" | cut -c19-20)
        tmp_brstr=${tmp_dstr#*, bitrate: }
        tmp_brval=$(echo "$tmp_brstr" | cut -c1-5)
        tmp_brval=${tmp_brval/ kb/}
        tmp_brval=${tmp_brval/ k/}
        tmp_brval=${tmp_brval/ Mb/}
        tmp_brval=${tmp_brval/ M/}
        tmp_brval=${tmp_brval/ /}
        tmp_brstr=$(echo "$tmp_brstr" | cut -c5-)
        tmp_brstr=${tmp_brstr/ /}
        [ "$tmp_dur_h" == "" ] && $(tmp_dur_h=0 ; tmp_dur_m=0 ; tmp_dur_s=0)
        [ "$tmp_brval" == "" ] && $(tmp_brval=0 ; tmp_brstr="---")
        if [ "$tmp_brstr" == "Mb/s" ]; then tmp_brstr="kb/s"; tmp_brval=$(( $tmp_brval * 1024 )); fi
        addsp="                    "
        numVideo=$(cat $logDir$p1.ffprobe-errlog1 | grep -c " Video: " )
        tmp_vstr=$(cat $logDir$p1.ffprobe-errlog1 | grep -E " Video: " )
        findStream "$tmp_vstr"
        tmp_vstr="$tmp_xstr"    # ein Videostream
        numAudio=$(cat $logDir$p1.ffprobe-errlog1 | grep -c " Audio: " )
        tmp_astr=$(cat $logDir$p1.ffprobe-errlog1 | grep -E " Audio: " )
        findStream "$tmp_astr"
        tmp_astr="$tmp_xstr"    # ein oder mehrere Audiostreams
        addsp="$addsp   "
        numSubtitle=$(cat $logDir$p1.ffprobe-errlog1 | grep -c " Subtitle: " )
        tmp_sstr=$(cat $logDir$p1.ffprobe-errlog1 | grep -E " Subtitle: " )
        findStream "$tmp_sstr"
        tmp_sstr="$tmp_xstr"
        tmp_dur=$(( $(( 10#$tmp_dur_h * 3600 )) + $(( 10#$tmp_dur_m * 60 )) + 10#$tmp_dur_s ))
        echoUndLog "$NOCOL$TRENNER\n *  ${SETBL}Status und einfache Prüfung (Filmdaten, enthaltene Streams):"
        tmpstr="${SETBL}Spieldauer der konvertierten Datei (h:m:s): $NOCOL$tmp_dur_h:$tmp_dur_m:$tmp_dur_s = ${tmp_dur}s"
        tmp_delta2=$(( $tval_delta_i - $tmp_dur ))
        tmp_durx=$(( $tmp_durval - $tmp_dur ))
        namenzusatz=""
        if [ "$aoutd" != "libopus" ]; then
          if [ $tmp_durx -lt -2 ] || [ $tmp_durx -gt 2 ]; then
            echoUndLog "$NOCOL      ${tmpstr} ${SETRT}, mindestens ein Stream hat >2s abweichende Dauer, #0...<n>: (($NOCOL$tmp_durval1$SETRT)/$tmp_durval2) != $tmp_durval / $tmp_dur.$NOCOL"
            namenzusatz="_Streamlaengenalarm$tmp_durval1str"
            Konvertierergebnis=$(( $Konvertierergebnis | 256 ))
          elif [ $tmp_delta2 -eq 0 ]; then
            echoUndLog "$NOCOL      ${tmpstr} ${SETGN}stimmt mit der erwarteten Dauer $NOCOL${tval_delta_i}s$SETGN überein, #0...<n>: (($NOCOL$tmp_durval1$SETGN)/$tmp_durval2).$NOCOL"
          elif [ $tmp_delta2 -le 1 ] && [ $tmp_delta2 -ge -1 ]; then
            echoUndLog "$NOCOL      ${tmpstr} ${SETGN}stimmt ±1s mit der erwarteten Dauer $NOCOL${tval_delta_i}s$SETGN überein, #0...<n>: (($NOCOL$tmp_durval1$SETGN)/$tmp_durval2).$NOCOL"
            Konvertierergebnis=$(( $Konvertierergebnis | 2 ))
          elif [ $tmp_dur -eq 0 ]; then
            echoUndLog "$NOCOL      $SETBGAL${SETRT}Konvertierung abgebrochen!$SETBGSW$NOCOL"
            namenzusatz="_Streamkonvertierfehler"
            Konvertierergebnis=$(( $Konvertierergebnis | 256 ))
          else
            echoUndLog "$NOCOL      ${tmpstr} $SETBGAL${SETRT}stimmt nicht mit der erwarteten Dauer $SETBGSW$NOCOL${tval_delta_i}s$SETBGAL$SETRT überein, #0...<n>: (($NOCOL$tmp_durval1$SETBGAL$SETRT)/$tmp_durval2)!$SETBGSW$NOCOL"
            namenzusatz="_Streamlaengenalarm$tmp_durval1str"
            Konvertierergebnis=$(( $Konvertierergebnis | 256 ))
          fi
          if [ $tmp_dur -gt 0 ]; then
            [ $numVideo -eq 0 -o "$numVideo" == "" ] && $( namenzusatz="_Video_fehlt" ; Konvertierergebnis=$(( $Konvertierergebnis | 1024 )) ; logStream "Video" "$tmp_vstr" )
            [ $numAudio -eq 0 -o "$numAudio" == "" ] && $( namenzusatz="_Audio_fehlt" ; Konvertierergebnis=$(( $Konvertierergebnis | 512 )) ; logStream "Audio" "$tmp_astr" )
            [ $numSubtitle -eq 0 -a 1 -eq $enasubtitle ] && $( namenzusatz="_Subtitle_fehlt" ; Konvertierergebnis=$(( $Konvertierergebnis | 4 )) ; logStream "Subtitle" "$tmp_sstr" )
          fi
        fi
        if [ $Konvertierergebnis -ge 256 ] && [ $tmp_dur -eq 0 ]; then
          AufrufStr3add=""
          #if [ $istAacLatmLc -eq 1 -o $istAacLatmLc -eq 2 ]; then
          #  wartezeit=5
          #  [ $istAacLatmLc -eq 1 ] && AufrufStr3add="$AufrufStr3add-aac" || AufrufStr3add="$AufrufStr3add-ac3"
          #  if [ "$durchsucheUnterverzeichnisseOrig" != "$durchsucheUnterverzeichnisseOrig" ]; then
          #    AufrufStr3add="$AufrufStr3add -d $durchsucheUnterverzeichnisse"
          #    if [ $durchsucheUnterverzeichnisse -ge 1 ] && [ "$vorherFindeDupOrig" != "$vorherFindeDup" ]; then AufrufStr3add="$AufrufStr3add $vorherFindeDup"; fi
          #  fi
          #  errstr="                $SETBGWW${SETRTD} ${AufrufStr3% -i *} \\ $SETBGSW$NOCOL\n                $SETBGWW${SETRTD} -i ${AufrufStr3##* -i } $SETBGSW$NOCOL\n\n                $SETBGWW${SETRTD} Konvertierung erfolglos, vermutlich Problem beim Lesen/Kopieren des Audio-Streams,$SETGND in ${wartezeit}s noch mal versuchen mit Parameter \"$AufrufStr3add\"... $SETBGSW$NOCOL"
          #  AufrufStr3="$AufrufStr3 $AufrufStr3add"
          #  echoUndLog "$errstr" ; echo -e "$errstr" >> ${logDir}filmschnitt.log
          #  rm -f "$destDir$ffmpegcmdsub6"
          #  clrFilesSub "$logDir$p1.ffprobe-log1"
          #  clrFilesSub "$logDir$p1.ffprobe-errlog1"
          #  clrlogfiles
          #  [ $(( ${#fullCmdLineStr} + 10 )) -ge $chkcols ] && echo -e "$SETBGWW$SETBLD${fullCmdLineStr% -i *} \\ $SETBL$SETBGSW$NOCOL\n$SETBGWW$SETBLD-i ${fullCmdLineStr##* -i } $NOCOL$SETBGSW" || echo -e "$SETBGWW$SETBLD$fullCmdLineStr $NOCOL$SETBGSW"
          #  #echo -e "                $SETBGWW${SETGND} ${AufrufStr3% -i *} \\ $SETBGSW$NOCOL\n                $SETBGWW${SETGND} -i ${AufrufStr3##* -i } $SETBGSW$NOCOL"
          #  [ $(( ${#fullCmdLineStr} + 10 )) -ge $chkcols ] && (echo -e "$SETBGWW$SETBLD${fullCmdLineStr% -i *} \\ $SETBL$SETBGSW$NOCOL\n$SETBGWW$SETBLD-i ${fullCmdLineStr##* -i } $NOCOL$SETBGSW" >> ${logDir}filmschnitt.log) || (echo -e "$SETBGWW$SETBLD$fullCmdLineStr $NOCOL$SETBGSW" >> ${logDir}filmschnitt.log)
          #  #echo -e "                $SETBGWW${SETGND} ${AufrufStr3% -i *} \\ $SETBGSW$NOCOL\n                $SETBGWW${SETGND} -i ${AufrufStr3##* -i } $SETBGSW$NOCOL" >> ${logDir}filmschnitt.log
          #  sleep ${wartezeit}s
          #  exec $AufrufStr3
          #  exit 0
          #else
            if [  $b_EAC3 -gt 0 -o $istAacLatmLc -eq 1 -o $istAacLatmLc -eq 2 ]; then add_str=" (Audio-Decoding-Problem)"; else add_str=" (Ursache unbekannt)"; fi
            #errstr="$NOCOL$SETBGWW$SETRTD'$SETBGSW$NOCOL$p1$SETBGWW$SETRTD'$SETBGSW\n${SETBGWW}Konvertierung erfolglos$add_str, Originaldatei wird als nicht konvertierbar markiert.$SETBGSW$NOCOL\r"
            #echoUndLog "$errstr" #; echo -e "$errstr" >> ${logDir}filmschnitt.log
            #rm -f "$destDir$ffmpegcmdsub6"
            errstr="$NOCOL$SETBGWW$SETRTD'$SETBGSW$NOCOL$p1$SETBGWW$SETRTD'$SETBGSW\n${SETBGWW}Konvertierung erfolglos$add_str, Originaldatei und vermutlich fehlerhaftes Ergebnis wird nach $offenDir verschoben.$SETBGSW$NOCOL\r"
            echoUndLog "$errstr" #; echo -e "$errstr" >> ${logDir}filmschnitt.log
            [ -d "$srcDir$offenDir" ] || mkdir -p "$srcDir$offenDir"
            [ -d "$destDir$offenDir" ] || mkdir -p "$destDir$offenDir"
            mvNachOffen "$p1" "$srcDir"
            [ -e "$destDir$ffmpegcmdsub6" -a $(getSizeTime "$destDir$ffmpegcmdsub6") -gt 10 ] && mvNachOffen "$ffmpegcmdsub6" "$destDir"
            [ -e "$destDir$ffmpegcmdsub6" -a $(getSizeTime "$destDir$ffmpegcmdsub6") -le 10 ] && rm -f "$destDir$ffmpegcmdsub6"
            sleep 1s
          #fi
          #echo -e "$TRENNER\n$SETBL${SETBGWS}ffmpegcmd:$NOCOL$ffmpegcmdstr$NOCOL\n$KonvStr\n$TRENNER"
          p1chk=$(echo "${p1%.*}" | sed -e 's/-nicht_konvertierbar$//;s/_nicht_konvertierbar$//')
          if [ "$backupDir1K$p1chk" == "$backupDir1K${p1%.*}" ]; then mvUmbenennen $p1 $backupDir1K "${p1chk}_nicht_konvertierbar.${p1##*.}"
          else echo -e "$SETBL    $backupDir1K$p1 ist schon entsprechend benannt.$NOCOL"
          fi
          if [ "$backupDir2K$p1chk" == "$backupDir2K${p1%.*}" ]; then mvUmbenennen $p1 $backupDir2K "${p1chk}_nicht_konvertierbar.${p1##*.}"
          else echo -e "$SETBL    $backupDir2K$p1 ist schon entsprechend benannt.$NOCOL"
          fi
          if [ "$backupDir3K$p1chk" == "$backupDir3K${p1%.*}" ]; then mvUmbenennen $p1 $backupDir3K "${p1chk}_nicht_konvertierbar.${p1##*.}"
          else echo -e "$SETBL    $backupDir3K$p1 ist schon entsprechend benannt.$NOCOL"
          fi
          if [ "$backupDir1S$p1chk" == "$backupDir1S${p1%.*}" ]; then mvUmbenennen $p1 $backupDir1S "${p1chk}_nicht_konvertierbar.${p1##*.}"
          else echo -e "$SETBL    $backupDir1S$p1 ist schon entsprechend benannt.$NOCOL"
          fi
          if [ "$backupDir2S$p1chk" == "$backupDir2S${p1%.*}" ]; then mvUmbenennen $p1 $backupDir2S "${p1chk}_nicht_konvertierbar.${p1##*.}"
          else echo -e "$SETBL    $backupDir2S$p1 ist schon entsprechend benannt.$NOCOL"
          fi
          if [ "$backupDir3S$p1chk" == "$backupDir3S${p1%.*}" ]; then mvUmbenennen $p1 $backupDir3S "${p1chk}_nicht_konvertierbar.${p1##*.}"
          else echo -e "$SETBL    $backupDir3S$p1 ist schon entsprechend benannt.$NOCOL"
          fi
          if [ "$srcDir" != "$backupDir1K" ] && [ "$srcDir" != "$backupDir2K" ] && [ "$srcDir" != "$backupDir3K" ] && [ "$srcDir" != "$backupDir1S" ] && [ "$srcDir" != "$backupDir2S" ] && [ "$srcDir" != "$backupDir3S" ]; then
            if [ "$srcDir$p1chk" == "$srcDir${p1%.*}" ]; then mvUmbenennen $p1 $srcDir "${p1chk}_nicht_konvertierbar.${p1##*.}"
            else echo -e "$SETBL    $srcDir$p1 ist schon entsprechend benannt.$NOCOL"
            fi
          fi
          echo -e "$NOCOL$TRENNER"
          sleep 1s
          clrFilesSub "$logDir$p1.ffprobe-log1"
          clrFilesSub "$logDir$p1.ffprobe-errlog1"
          if [ $durchsucheUnterverzeichnisse -ge 1 ]; then
            echoUndLog "$SETBGGR${SETRT}Konvertierung fehlerhaft!$SETBGSW$NOCOL\n$SETBGGR${SETRT}Suche Dateien mit ähnlichem Namen in realen Pfaden (\"$nameAnf2\"):$SETBGSW$NOCOL"
            numNamAnf=0
            tmpNameAnf2="$nameAnf2"
            findDup
            if [ "$p1neu" != "" ]; then
              prepName $p1orig
              [ "$tmpNameAnf2" != "$nameAnf2" ] && (echoUndLog "$SETBGGR${SETRT}Suche Dateien mit ähnlichem Namen in realen Pfaden (\"$nameAnf2\"):$SETBGSW$NOCOL"; findDup)
            fi
            echo -e "\n$NOCOL$TRENNER"
          fi
          getDateTime
          convstop=$currDateTime
          echo -e "$NOCOL$convstart${SETAL2}, Abbruch: $SETBGWS$SETBLD$convstop$SETBGSW$NOCOL\n$TRENNER"
          clrlogfiles
          exit 0
        else
          if [ "$aoutd" != "libopus" ] && [ $tmp_durx -lt -2 -o $tmp_durx -gt 2 ] && [ "$namenzusatz" != "" ]; then
            echoUndLog "$NOCOL *  $SETBGWW$SETRTD'$SETBGSW$NOCOL$p1$SETBGWW$SETRTD'\n    Konvertierung wahrscheinlich erfolglos, Ergebnis wird umbenannt.$SETBGSW$NOCOL"
            echo -e -n " - mv "
            mv "$destDir$ffmpegcmdsub6" "$destDir${ffmpegcmdsub6%.*}$namenzusatz.${ffmpegcmdsub6##*.}" > /dev/null 2> /dev/null
          fi
          min_br2=$(( $(( $min_br * 7 )) / 10 )) # < 100%  ... 70% => Warnung Stufe 1
          min_br3=$(( $(( $min_br * 5 )) / 10 )) # < 70%  ... 50% => Warnung Stufe 2
          min_br4=$(( $(( $min_br * 3 )) / 10 )) # < 50%  ... 30% => Warnung Stufe 3, < 30% => Fehler/Abbruch
          # !!! wegen evtl. unbekannter abr/vbr bleibt min_br* aktiv !!! ############
          min_vbr0=$(( $(( $min_vbr * 5 )) / 4 )) # > 125%
          min_vbr2=$(( $(( $min_vbr * 8 )) / 10 )) # < 100% ... 80% => Warnung Stufe 1
          min_vbr3=$(( $(( $min_vbr * 6 )) / 10 )) # < 80%  ... 60% => Warnung Stufe 2
          min_vbr4=$(( $(( $min_vbr * 4 )) / 10 )) # < 60%  ... 40% => Warnung Stufe 3, < 40% => Fehler/Abbruch
          if [ $voutd == "vp9" -o $voutd == "av1" ]; then
            echoUndLog "$NOCOL      ${SETBL}Gesamtbitrate: $NOCOL$tmp_brval$tmp_brstr\n$NOCOL      ${SETBL}Prüfung des Video-Streams ($NOCOL$voutd$SETBL) wird z.Z. nicht unterstützt."
          elif [ "$tmp_xbrval0" != "" -a "$tmp_xbrval0" != "N/A" -a "$tmp_xbrval0" != "9999" -a "$tmp_xbrval0" != "????" ]; then
            tmp_brstr1="$NOCOL      ${SETBL}Gesamtbitrate: $NOCOL$tmp_brval$tmp_brstr\n$NOCOL      ${SETBL}Videobitrate: $NOCOL$(echo $tmp_xbrval0 | sed 's/9999/????/g')$tmp_brstr"
            if [ $tmp_xbrval0 -lt $min_vbr4 ]; then
              echoUndLog "$tmp_brstr1$SETBGWS$SETRTD$SETBGWS$SETRTD ist zu niedrig ($NOCOL${min_vbr}kb/s$SETBGWS$SETRTD)!$SETBGSW$NOCOL"
              [ $nameErwBitrate -gt 0 ] && namenzusatz="${namenzusatz}_Bitratenfehler_V-$tmp_xbrval0-G${tmp_brval}kbps"
              Konvertierergebnis=$(( $Konvertierergebnis | 128 ))
            elif [ $tmp_xbrval0 -lt $min_vbr3 ]; then
              echoUndLog "$tmp_brstr1$SETFLRT ist erheblich niedriger, als erwartet ($NOCOL${min_vbr}kb/s$SETFLRT)!$NOCOL"
              #[ $nameErwBitrate -gt 0 ] && namenzusatz="${namenzusatz}_Bitratenalarm_V-${tmp_xbrval0}kbps"
              Konvertierergebnis=$(( $Konvertierergebnis | 64 ))
            elif [ $tmp_xbrval0 -lt $min_vbr2 ]; then
              echoUndLog "$tmp_brstr1$SETRT ist deutlich niedriger, als erwartet (< $NOCOL${min_vbr}kb/s$SETRT)!$NOCOL"
              #[ $nameErwBitrateAll -gt 0 -a $nameErwBitrate -gt 0 ] && namenzusatz="${namenzusatz}_Bitratenwarnung_V"
              Konvertierergebnis=$(( $Konvertierergebnis | 16 ))
            elif [ $tmp_xbrval0 -lt $min_vbr ]; then
              echoUndLog "$tmp_brstr1$SETOR ist etwas niedriger, als erwartet (< $NOCOL${min_vbr}kb/s$SETOR).$NOCOL"
              #[ $nameErwBitrateAll -gt 0 -a $nameErwBitrate -gt 0 ] && namenzusatz="${namenzusatz}_Bitratenhinweis_V"
              Konvertierergebnis=$(( $Konvertierergebnis | 1 ))
            elif [ $tmp_xbrval0 -lt $min_vbr0 ]; then echoUndLog "$tmp_brstr1$SETGN liegt über dem erwarteten Mindestwert ($NOCOL${min_vbr}kb/s$SETGN).$NOCOL"
            else echoUndLog "$tmp_brstr1$SETGND liegt weit über dem erwarteten Mindestwert ($NOCOL${min_vbr}kb/s$SETGND).$NOCOL"
            fi
          else
            tmp_brstr1="$NOCOL      ${SETBL}Gesamtbitrate: $NOCOL$tmp_brval$tmp_brstr "
            if [ $tmp_brval -lt $min_br4 ]; then
              echoUndLog "$NOCOL        ${SETBL}$tmp_brstr1$SETBGWS$SETRTD$SETBGWS$SETRTD ist zu niedrig ($NOCOL${min_br}kb/s$SETBGWS$SETRTD)!$SETBGSW$NOCOL"
              [ $nameErwBitrate -gt 0 ] && namenzusatz="${namenzusatz}_Bitratenfehler_G-${tmp_brval}kbps"
              Konvertierergebnis=$(( $Konvertierergebnis | 128 ))
            elif [ $tmp_brval -lt $min_br3 ]; then
              echoUndLog "$tmp_brstr1$SETFLRT ist deutlich niedriger, als erwartet ($NOCOL${min_br}kb/s$SETFLRT)!$NOCOL"
              #[ $nameErwBitrate -gt 0 ] && namenzusatz="${namenzusatz}_Bitratenalarm_G-${tmp_brval}kbps"
              Konvertierergebnis=$(( $Konvertierergebnis | 64 ))
            elif [ $tmp_brval -lt $min_br2 ]; then
              echoUndLog "$tmp_brstr1$SETRT ist niedriger, als erwartet (< $NOCOL${min_br}kb/s$SETRT)!$NOCOL"
              #[ $nameErwBitrateAll -gt 0 -a $nameErwBitrate -gt 0 ] && namenzusatz="${namenzusatz}_Bitratenwarnung_G"
              Konvertierergebnis=$(( $Konvertierergebnis | 16 ))
            elif [ $tmp_brval -lt $min_br ]; then
              echoUndLog "$tmp_brstr1$SETOR ist etwas niedriger, als erwartet (< $NOCOL${min_br}kb/s$SETOR).$NOCOL"
              #[ $nameErwBitrateAll -gt 0 -a $nameErwBitrate -gt 0 ] && namenzusatz="${namenzusatz}_Bitratenhinweis_G"
              Konvertierergebnis=$(( $Konvertierergebnis | 1 ))
            else echoUndLog "$tmp_brstr1$SETGN liegt über dem erwarteten Mindestwert ($NOCOL${min_br}kb/s$SETGN).$NOCOL"
            fi
            echoUndLog "$NOCOL      ${SETBL}Videobitrate: $NOCOL$(echo $tmp_xbrval0 | sed 's/9999/????/g')$tmp_brstr"
          fi
          tmp_brstr="$NOCOL      ${SETBL}Audiobitraten #1...<n>: $NOCOL$(echo $tmp_xbrval1 | sed 's/9999/????/g') $(echo $tmp_xbrval2 | sed 's/9999//g') $(echo $tmp_xbrval3 | sed 's/9999//g') $(echo $tmp_xbrval4 | sed 's/9999//g') $(echo $tmp_xbrval5 | sed 's/9999//g')kb/s"
          if [ "$aoutd" == "libopus" ]; then
            echoUndLog "$NOCOL        ${SETBL}Prüfung der Audio-Streams ($NOCOL$aoutd$SETBL) wird z.Z. nicht unterstützt."
          elif [ "${VMOD,,}" != "copy" -a "${VMOD,,}" != "vcopy" ]; then
            if [ "$tmp_xbrval1" == "" -o "$tmp_xbrval1" == "9999" -o "$tmp_xbrval1" == "????" -o  "$tmp_xbrval1" == "N/A" ]; then
              echoUndLog "$tmp_brstr$SETFLRT Warnung: AUDIO1-Bitrate unbekannt!$NOCOL"
              #[ $nameErwBitrate -gt 0 ] && namenzusatz="${namenzusatz}_Audioalarm-A1"
              Konvertierergebnis=$(( $Konvertierergebnis | 16 ))
            elif [ "$tmp_xbrval1" != "" -a $tmp_xbrval1 -lt $min_abr ]; then
              if [ $tmp_xbrval1 -lt $(( $(( $min_abr * 2 )) / 3 )) ]; then
                echoUndLog "$tmp_brstr$SETFLRT Fehler: AUDIO1-Bitrate zu niedrig (< $NOCOL${min_abr}kb/s$SETFLRT)!$NOCOL"
                #[ $nameErwBitrate -gt 0 ] && namenzusatz="${namenzusatz}_Audioalarm_A1-${tmp_xbrval1}kbps"
                Konvertierergebnis=$(( $Konvertierergebnis | 16 ))
              else
                echoUndLog "$tmp_brstr$SETOR Warnung: AUDIO1-Bitrate niedriger, als erwartet (< $NOCOL${min_abr}kb/s$SETOR).$NOCOL"
                #[ $nameErwBitrateAll -gt 0 -a $nameErwBitrate -gt 0 ] && namenzusatz="${namenzusatz}_Audiowarnung_A1"
                Konvertierergebnis=$(( $Konvertierergebnis | 8 ))
              fi
            else
              if [ "$tmp_xbrval2" != "" -a $tmp_xbrval2 -lt $min_abr ]; then
                echoUndLog "$tmp_brstr$SETOR Warnung: AUDIO2-Bitrate niedriger, als erwartet (< $NOCOL${min_abr}kb/s$SETOR).$NOCOL"
                #[ $nameErwBitrate -gt 0 ] && namenzusatz="${namenzusatz}_Audiowarnung_A2"
                Konvertierergebnis=$(( $Konvertierergebnis | 8 ))
              elif [ "$tmp_xbrval3" != "" -a $tmp_xbrval3 -lt $min_abr ]; then
                echoUndLog "$tmp_brstr$SETOR Warnung: AUDIO3-Bitrate niedriger, als erwartet (< $NOCOL${min_abr}kb/s$SETOR).$NOCOL"
                #[ $nameErwBitrate -gt 0 ] && namenzusatz="${namenzusatz}_Audiowarnung_A3"
                Konvertierergebnis=$(( $Konvertierergebnis | 8 ))
              elif [ "$tmp_xbrval4" != "" -a $tmp_xbrval4 -lt $min_abr ]; then
                echoUndLog "$tmp_brstr$SETOR Warnung: AUDIO4-Bitrate niedriger, als erwartet (< $NOCOL${min_abr}kb/s$SETOR).$NOCOL"
                #[ $nameErwBitrate -gt 0 ] && namenzusatz="${namenzusatz}_Audiowarnung_A4"
                Konvertierergebnis=$(( $Konvertierergebnis | 8 ))
              elif [ "$tmp_xbrval5" != "" -a $tmp_xbrval5 -lt $min_abr ]; then
                echoUndLog "$tmp_brstr$SETOR Warnung: AUDIO5-Bitrate niedriger, als erwartet (< $NOCOL${min_abr}kb/s$SETOR).$NOCOL"
                #[ $nameErwBitrate -gt 0 ] && namenzusatz="${namenzusatz}_Audiowarnung_A5"
                Konvertierergebnis=$(( $Konvertierergebnis | 8 ))
              else echoUndLog "$tmp_brstr$SETGN liegt über dem erwarteten Mindestwert ($NOCOL${min_abr}kb/s$SETGN).$NOCOL"
              fi
            fi
          else echoUndLog "$tmp_brstr$NOCOL"
          fi
          if [ $Konvertierergebnis -ge 16 ]; then
            zusatztext="."
            if [ $nameErwBitrate -gt 0 ] && [ "$namenzusatz" != "" ]; then
              zusatztext=", Ergebnis wird mit Hinweis '$namenzusatz' versehen."
              mv $destDir$ffmpegcmdsub6 $destDir${ffmpegcmdsub6%.*}$namenzusatz.${ffmpegcmdsub6##*.} > /dev/null 2> /dev/null
            fi
            [ $Konvertierergebnis -ge 64 ] && echoUndLog "$NOCOL *  $SETRT'$NOCOL$p1$SETRT' Konvertierung eventuell erfolglos, manuell prüfen$zusatztext$SETBGSW$NOCOL"
          fi
          outsize=$(ls -l "$destDir$ffmpegcmdsub6" | awk 'END { print int($5/1048576)}')
          logStream "Video" "$tmp_vstr"
          logStream "Audio" "$tmp_astr"
          [ 1 -eq $enasubtitle ] && logStream "Subtitle" "$tmp_sstr"
          logcnt=$(ls -al $logDir*.ffprobe-log1 | grep -c ".ffprobe-log1")
          logcnt2=$(ls -al $logDir/$p1.ffprobe-log1 | grep -c ".ffprobe-log1")
          logcnt=$(( $logcnt - $logcnt2 ))
          if [ ! -d "$destDir" ]; then echo -e -n " *  ${SETBL}Verzeichnis $NOCOL$destDir$SETBL anlegen$SETAL1"; mkdir -p "$destDir" 2> /dev/null; SubOkErr $? 0
          fi
          ls "${destDir}dummy" &> /dev/null
          status1=0 ; status2=0 ; status3=0
          [ $bkup1sel -eq 1 ] && bkupXSub 1
          [ $bkup2sel -eq 1 ] && bkupXSub 2
          [ $bkup3sel -eq 1 ] && bkupXSub 3
          status=$(( $status1 + $status2 + $status3 ))
          if [ $bkupLimCnt -ge 1 ]; then
            bkupSubStr=""
            if [ $bkupCnt2 -eq $bkupLimCnt ] && [ $status -le 1 -o $status -gt 127 ]; then
              bkupSubStr="$NOCOL    ${TRENNER:4}\n *  ${ffmpegcmdsub6%.*}$namenzusatz.${ffmpegcmdsub6##*.}$SETBL wurde in Backup-Zwischenverzeichnisse verschoben"
              tmpstr="$NOCOL *  $SETOK\"$srcDir$p1\" nach dem Konvertieren (${SETOR}in Videoplayer prüfen!$SETOK) löschen$NOCOL"
              if [ $bkupRm1stLvl -gt 0 ] && [ $bkupCnt -gt 0 ]; then
                if [ $anzeigeBkupMsg -gt 1 ]; then echo -e -n "$bkupSubStr"; fi
                echo -e -n "$bkupSubStr" >> $logDir$p1.ffmpeg-tmp
                if [ $status != 0 ]; then
                  [ $anzeigeBkupMsg -gt 1 ] && SubErr
                else
                  sleep 1s
                  clrFilesSub "$destDir$ffmpegcmdsub6"
                  [ "$namenzusatz" != "" ] && clrFilesSub "$destDir${ffmpegcmdsub6%.*}$namenzusatz.${ffmpegcmdsub6##*.}"
                  [ $anzeigeBkupMsg -gt 1 ] && SubOkErr $? 0
                  sleep 1s
                fi
              else
                echo -e "${bkupSubStr}.$NOCOL" >> $logDir$p1.ffmpeg-tmp
                echo -e "${tmpstr}.$NOCOL" >> $logDir$p1.ffmpeg-tmp
              fi
            elif [ $status1 -le 128 -a $status2 -le 128 -a $status3 -le 128 ]; then #elif [ $status1 -le 128 ] && [ $status2 -le 128 ] && [ $status3 -le 128 ] && [ "$bkup1dir" == "" -o "$bkup2dir" == "" -o "$bkup3dir" == "" ]; then
              echo -e -n "$NOCOL    ${TRENNER:4}\n *  ${ffmpegcmdsub6%.*}$namenzusatz.${ffmpegcmdsub6##*.}$SETGE konnte nicht in die Backup-Zielverzeichnisse kopiert werden und wird darum in das Verzeichnis $NOCOL$offenDir$SETGE verschoben "
              [ -d "$destDir$offenDir" ] || mkdir -p "$destDir$offenDir"
              tmpp="$_dstD"
              tmpp="$(df -l $tmpp 2> /dev/null)"
              if [ "$tmpp" != "" ] || [ $rsyncstattcpmv -eq 0 ]; then # lokales FS => verwende mv
                echo -e -n " - mv "
                mv "$destDir${ffmpegcmdsub6%.*}$namenzusatz.${ffmpegcmdsub6##*.}" "$destDir$offenDir" 2> /dev/null # > /dev/null
              else
                echo -e -n " - rsync\n    "
                rsync -IPat --info=progress2 --remove-source-files "$destDir${ffmpegcmdsub6%.*}$namenzusatz.${ffmpegcmdsub6##*.}" "$destDir$offenDir" 2> /dev/null # > /dev/null
                echo -e -n "   "
              fi 
              SubOkErr $? 0
            fi
          elif [ $status1 -le 128 ] && [ $status2 -le 128 ] && [ $status3 -le 128 ] && [ "$bkup1dir" == "" -o "$bkup2dir" == "" -o "$bkup3dir" == "" ]; then
            echo -e -n "$NOCOL    ${TRENNER:4}\n *  ${ffmpegcmdsub6%.*}$namenzusatz.${ffmpegcmdsub6##*.}$SETGE konnte nicht in die Backup-Zielverzeichnisse kopiert werden und wird darum in das Verzeichnis $NOCOL$offenDir$SETGE verschoben "
            [ -d "$destDir$offenDir" ] || mkdir "$destDir$offenDir"
            tmpp="$_dstD"
            tmpp="$(df -l $tmpp 2> /dev/null)"
            if [ "$tmpp" != "" ] || [ $rsyncstattcpmv -eq 0 ]; then # lokales FS => verwende mv
              echo -e -n " - mv "
              mv "$destDir${ffmpegcmdsub6%.*}$namenzusatz.${ffmpegcmdsub6##*.}" "$destDir$offenDir" 2> /dev/null # > /dev/null
            else
              echo -e -n " - rsync\n    "
              rsync -IPat --info=progress2 --remove-source-files "$destDir${ffmpegcmdsub6%.*}$namenzusatz.${ffmpegcmdsub6##*.}" "$destDir$offenDir" 2> /dev/nul # > /dev/null
              echo -e -n "   "
            fi 
            SubOkErr $? 0
          fi
          if [ $nicht_Schieben -eq 0 ]; then
            #sleep 1s
            echo -e "$NOCOL$TRENNER"
            mvNachErledigt $p1 $srcDir
            mvNachErledigt $p1 $konvertierenDir$offenDir
            mvNachErledigt $p1 $schneidenDir$offenDir
            mvNachErledigt $p1 $backupDir1K
            mvNachErledigt $p1 $backupDir1K$pcfg2
            mvNachErledigt $p1 $backupDir1K$pcfg2$pcfg2_2
            mvNachErledigt $p1 $backupDir2K
            mvNachErledigt $p1 $backupDir2K$pcfg2
            mvNachErledigt $p1 $backupDir2K$pcfg2$pcfg2_2
            mvNachErledigt $p1 $backupDir3K
            mvNachErledigt $p1 $backupDir3K$pcfg2
            mvNachErledigt $p1 $backupDir3K$pcfg2$pcfg2_2
            mvNachErledigt $p1 $backupDir1S
            mvNachErledigt $p1 $backupDir1S$pcfg2
            mvNachErledigt $p1 $backupDir1S$pcfg2$pcfg2_2
            mvNachErledigt $p1 $backupDir2S
            mvNachErledigt $p1 $backupDir2S$pcfg2
            mvNachErledigt $p1 $backupDir2S$pcfg2$pcfg2_2
            mvNachErledigt $p1 $backupDir3S
            mvNachErledigt $p1 $backupDir3S$pcfg2
            mvNachErledigt $p1 $backupDir3S$pcfg2$pcfg2_2
          fi
          echo -e "$NOCOL$TRENNER"
          sync
        fi
        sleep 1s
        clrFilesSub "$logDir$p1.ffprobe-log1"
        clrFilesSub "$logDir$p1.ffprobe-errlog1"
        if [ $durchsucheUnterverzeichnisse -ne 0 ]; then 
          echoUndLog "$SETBGGR${SETBL}Konvertierung beendet, Ergebnis-Länge und -Qualität prüfen! $SETBGSW$NOCOL\n$SETBGGR${SETBL}Suche Dateien mit ähnlichem Namen in realen Pfaden (\"$nameAnf2\"):$SETBGSW$NOCOL"
        else 
          echoUndLog "$SETBGGR${SETBL}Konvertierung beendet, Ergebnis-Länge und -Qualität prüfen! $SETBGSW$NOCOL"
        fi
        numNamAnf=0
        tmpNameAnf2="$nameAnf2"
        sleep 2s
        findDup
        if [ "$p1neu" != "" ]; then
          prepName $p1orig
          [ "$tmpNameAnf2" != "$nameAnf2" ] && (echoUndLog "$SETBGGR${SETBL}Suche Dateien mit ähnlichem Namen in realen Pfaden (\"$nameAnf2\"):$SETBGSW$NOCOL"; findDup)
        fi
        echo -e "\n$NOCOL$TRENNER"
        getDateTime
        convstop=$currDateTime
        currDurMin=$(echo "scale=3 ; ( ( $tval_delta_i / $currDurSec ) / 60 )" | bc)
        [ $(echo "scale=0 ; ( $currDurMin / 1 )" | bc) -lt 1 ] && currDurMin="0$currDurMin"
        echoUndLog "$NOCOL$TRENNER\n$convstart$SETBL$SETBGGR, Stop: $SETBGWS$SETBLD$convstop$SETBGGR$SETBL, Dauer: $currDurSec Minuten, Gesamt-Konvertiergeschwindigkeit: $SETBGWS$SETBLD${currDurMin}x$NOCOL\n$TRENNER"
        if [ -f $logDir$p1.freq-log ] && [ "${pcfg,,}" != "-umbenennen" ]; then
          cpufcnt=$(cat $logDir$p1.freq-log | grep "CPU MHz=" | awk 'BEGIN {v1=0} {v1=v1+1} END {print v1}')
          getFTMinMedMax "CPU MHz=" 26 29 ; cpufval=$cputmpftval
          getFTMinMedMax "Temperatur=" 47 49 ; cputval1=${cputmpftval#*/}
          cpuld=$(cat $logDir$p1.freq-log | grep "(CPU-Last)" | cut -c96- | awk 'BEGIN {v1=0;v3=0;v4=0} {v1=v1+1;v3=v3+$1;if ($1>v4) v4=$1} END {v3=v3/v1;printf("%.1f/%.1f", v3, v4)}')
          clrlogfiles
          if [ $updLogInterval -eq 0 ]; then updLogInterval=60; fi
          clkTStat="$currDateTime - $SETBL$cpufcnt Messungen alle $updLogInterval Sekunden:\nCPU-Freqenz-Minimalwert/Durchschnittswert/Maximalwert: $NOCOL${cpufval}MHz$SETBL\nTemperatur-Durchschnitt/Maximalwert: $NOCOL$cputval1°C (Core), ${SETBL}FFMPEG-Pwr-Durchschnitt/Maximalwert: $NOCOL$cpuld%"
          echoUndLog "$clkTStat"
        else clrlogfiles
        fi
        if [ "$bkup1dir" != "" ] || [ "$bkup2dir" != "" ] || [ "$bkup3dir" != "" ]; then  
          sleep 3s
          [ $bkupCnt2 -ge $bkupLimCnt ] && clrFilesSub $destDir${ffmpegcmdsub6%.*}$namenzusatz.${ffmpegcmdsub6##*.}
          sleep 3s
        fi
        resstr="$(echo -e "$TRENNER")\n$(echo -e -n "$(splitstr "Schneiden/Konvertierung von " "\"$p1\" " "($sizetimep1)" "$SETBLD${SETBGWW}" "$SETBGSW$NOCOL" "...\n...")")\n$(echo -e -n "$(splitstr "nach " "\"${ffmpegcmdsub6%.*}$namenzusatz.${ffmpegcmdsub6##*.}\" " "(${outsize}MB) beendet" "$SETBLD${SETBGWW}" "$SETBGSW$NOCOL" "...\n...")")\n$TRENNER"
        echoUndLog "$resstr"
      fi
    fi
  fi
fi
clrlogfiles
exit 0
